class
	CE_POINTER [G]

inherit
	ANY -- to make its features accessible to clients
	INTERNAL
		export
			{NONE}all
		end
	DISPOSABLE
		export
			{NONE}all
		end

create
	make,
	make_shared,
	make_shared_with_count,
	make_cast

feature {CE_POINTER}

	make
		do
			pp := default_pointer.memory_calloc (1, {PLATFORM}.Pointer_bytes)
			count := {INTEGER_32}.Max_value
		end

	make_shared (a_c_pointer: POINTER)
		do
			make
			pp_store (pp, a_c_pointer)
		end

	make_shared_with_count (a_c_pointer: POINTER; a_count: INTEGER_32)
		do
			make_shared (a_c_pointer)
			count := a_count
		end

	make_cast (a_other: CE_POINTER [ANY])
		do
			make
			ce_gets (a_other)
		end

feature {CE_POINTER}

	element_type: TYPE [ANY]
		once ("OBJECT")
			Result := generating_type.generic_parameter_type (1)
		end

	element_id: INTEGER_32
		once ("OBJECT")
			    if element_type = {BOOLEAN} then    Result := Element_id_boolean
			elseif element_type = {INTEGER_8} then  Result := Element_id_integer_8
			elseif element_type = {INTEGER_16} then Result := Element_id_integer_16
			elseif element_type = {INTEGER_32} then Result := Element_id_integer_32
			elseif element_type = {INTEGER_64} then Result := Element_id_integer_64
			elseif element_type = {NATURAL_8} then  Result := Element_id_natural_8
			elseif element_type = {NATURAL_16} then Result := Element_id_natural_16
			elseif element_type = {NATURAL_32} then Result := Element_id_natural_32
			elseif element_type = {NATURAL_64} then Result := Element_id_natural_64
			elseif element_type = {REAL_32} then    Result := Element_id_real_32
			elseif element_type = {REAL_64} then    Result := Element_id_real_64
			elseif element_type = {REAL_96} then    Result := Element_id_real_96
			elseif element_type = {ANY} then        Result := Element_id_void
			elseif attached {TYPE [CE_ARRAY [ANY]]} element_type then
				Result := Element_id_array
			elseif attached {TYPE [CE_POINTER [ANY]]} element_type then
				Result := Element_id_pointer
			elseif attached {TYPE [CE_CLASS]} element_type then
				Result := Element_id_class
			else
				(create {EXCEPTIONS}).raise ("Unknown type: " + element_type.name)
			end
		end

	element_size: INTEGER_32
		local
			old_state: BOOLEAN
		once ("OBJECT")
			inspect
				element_id
			when Element_id_boolean    then Result := {PLATFORM}.Boolean_bytes
			when Element_id_integer_8  then Result := {PLATFORM}.Integer_8_bytes
			when Element_id_integer_16 then Result := {PLATFORM}.Integer_16_bytes
			when Element_id_integer_32 then Result := {PLATFORM}.Integer_32_bytes
			when Element_id_integer_64 then Result := {PLATFORM}.Integer_64_bytes
			when Element_id_natural_8  then Result := {PLATFORM}.Natural_8_bytes
			when Element_id_natural_16 then Result := {PLATFORM}.Natural_16_bytes
			when Element_id_natural_32 then Result := {PLATFORM}.Natural_32_bytes
			when Element_id_natural_64 then Result := {PLATFORM}.Natural_64_bytes
			when Element_id_real_32    then Result := {PLATFORM}.Real_32_bytes
			when Element_id_real_64    then Result := {PLATFORM}.Real_64_bytes
			when Element_id_real_96    then Result := {CE_UTILITIES}.Real_96_bytes
			when Element_id_void       then Result := 1
			when Element_id_pointer    then Result := {PLATFORM}.Pointer_bytes
			when Element_id_array then
				-- TODO: That's actually wrong. Should be sizeof(array).
				Result := {PLATFORM}.Pointer_bytes
			when Element_id_class then
				old_state := {ISE_RUNTIME}.check_assert (False)
				Result := ({CE_CLASS} #? new_instance_of (element_type.type_id)).sizeof
				old_state := {ISE_RUNTIME}.check_assert (old_state)
			end
		end

feature {NONE}

	pp: POINTER
			-- We need a manually allocated pointer to store our pointer in.
			-- Otherwise we can't create an address without having the GC messing with it.

	pp_store (a_pp, a_p: POINTER)
		external
			"C inline"
		alias
			"*(void **)$a_pp = $a_p;"
		end

	pp_read (a_pp: POINTER): POINTER
		external
			"C inline"
		alias
			"return *(void **)$a_pp;"
		end

	dispose
		do
			pp.memory_free
		end

feature {ANY}

	c_pointer: POINTER
		do
			Result := pp_read (pp)
		end

	ce_gets (a_other: CE_POINTER [ANY])
		do
			if a_other.count = {INTEGER_32}.Max_value then
				count := {INTEGER_32}.Max_value
			elseif attached {CE_ARRAY [ANY]} a_other as tmp then
				count := (tmp.total_count * a_other.element_size) // element_size
			else
				count := (a_other.count * a_other.element_size) // element_size
			end
			pp_store (pp, a_other.c_pointer)
		end

feature {ANY}

	count: INTEGER_32

	item: G assign set_item
			-- *Current
		do
			Result := i_th (0)
		end

	set_item (a_item: G)
		do
			set_i_th (a_item, 0)
		end

	i_th alias "[]" (a_index: INTEGER_32): G assign set_i_th
			-- Current[0]
		require
			non_void_target: c_pointer /= default_pointer
		local
			p: POINTER
			old_state: BOOLEAN
		do
			if (not {CE_SETTINGS}.Disable_bounds_checking) and then a_index >= count then
				(create {EXCEPTIONS}).raise ("{" + generating_type + "}.i_th: " + a_index.out + " is out of bounds")
			end
			inspect
				element_id
			when
				Element_id_boolean,
				Element_id_integer_8, Element_id_integer_16, Element_id_integer_32, Element_id_integer_64,
				Element_id_natural_8, Element_id_natural_16, Element_id_natural_32, Element_id_natural_64,
				Element_id_real_32, Element_id_real_64
			then
				($Result).memory_copy (c_pointer + (a_index * element_size), element_size)
			when Element_id_real_96 then
				({REAL_96} #? Result).c_pointer.memory_copy (c_pointer + (a_index * element_size), element_size)
			when Element_id_pointer then
				old_state := {ISE_RUNTIME}.check_assert (False)
				if attached {CE_POINTER [ANY]} new_instance_of (element_type.type_id) as tmp then
					($p).memory_copy (c_pointer + (a_index * element_size), element_size)
					tmp.make_shared (p)
					Result := ({G} #? tmp)
				end
				old_state := {ISE_RUNTIME}.check_assert (old_state)
			when Element_id_array then
				old_state := {ISE_RUNTIME}.check_assert (False)
				if attached {CE_ARRAY [ANY]} new_instance_of (element_type.type_id) as tmp then
					p := c_pointer + (a_index * element_size)
					tmp.make_shared_from_pointer (p) -- can we call make_shared here?
					Result := ({G} #? tmp)
				end
				old_state := {ISE_RUNTIME}.check_assert (old_state)
			when Element_id_class then
				old_state := {ISE_RUNTIME}.check_assert (False)
				if attached {CE_CLASS} new_instance_of (element_type.type_id) as tmp then
					p := c_pointer + (a_index * element_size)
					tmp.make_shared (p)
					Result := ({G} #? tmp)
				end
				old_state := {ISE_RUNTIME}.check_assert (old_state)
			end
		end

	set_i_th (a_item: G; a_index: INTEGER_32)
			-- Current[0] := foo
		require
			non_void_target: c_pointer /= default_pointer
		local
			p: POINTER
		do
			if (not {CE_SETTINGS}.Disable_bounds_checking) and then a_index >= count then
				(create {EXCEPTIONS}).raise ("{" + generating_type + "}.set_i_th: " + a_index.out + " is out of bounds")
			end
			inspect
				element_id
			when
				Element_id_boolean,
				Element_id_integer_8, Element_id_integer_16, Element_id_integer_32, Element_id_integer_64,
				Element_id_natural_8, Element_id_natural_16, Element_id_natural_32, Element_id_natural_64,
				Element_id_real_32, Element_id_real_64
			then
				(c_pointer + (a_index * element_size)).memory_copy ($a_item, element_size)
			when Element_id_real_96 then
				(c_pointer + (a_index * element_size)).memory_copy (({REAL_96} #? a_item).c_pointer, element_size)
			when Element_id_pointer then
				p := ({CE_POINTER [ANY]} #? a_item).c_pointer;
				(c_pointer + (a_index * element_size)).memory_copy ($p, element_size)
			when Element_id_array then
				p := ({CE_ARRAY [ANY]} #? a_item).c_pointer
				pp_store (pp, p + (a_index * element_size))
			when Element_id_class then
				p := ({CE_CLASS} #? a_item).c_pointer;
				(c_pointer + (a_index * element_size)).memory_copy (p, element_size)
			end
		end

	address: CE_POINTER [like Current]
			-- &Current
		do
			create Result.make_shared (pp)
		end

feature {ANY} -- pointer arithmetics

	plus alias "+" (i: INTEGER_32): CE_POINTER [G]
		local
			l_count: INTEGER_32
		do
			if count = {INTEGER_32}.Max_value then
				l_count := {INTEGER_32}.Max_value
			else
				l_count := count - i
			end
			create Result.make_shared_with_count (c_pointer + (i * element_size), l_count)
		end

	minus alias "-" (i: INTEGER_32): CE_POINTER [G]
		local
			l_count: INTEGER_32
		do
			if count = {INTEGER_32}.Max_value then
				l_count := {INTEGER_32}.Max_value
			else
				l_count := count + i
			end
			create Result.make_shared_with_count (c_pointer + (-1 * i * element_size), l_count)
		end

	distance (other: CE_POINTER [G]): INTEGER_32
		do
			Result := (c_pointer.to_integer_32 - other.c_pointer.to_integer_32) // element_size
		end

feature {NONE} -- element type id constants

	Element_id_boolean: NATURAL_8 = 1

	Element_id_integer_8: NATURAL_8 = 2

	Element_id_integer_16: NATURAL_8 = 3

	Element_id_integer_32: NATURAL_8 = 4

	Element_id_integer_64: NATURAL_8 = 5

	Element_id_natural_8: NATURAL_8 = 6

	Element_id_natural_16: NATURAL_8 = 7

	Element_id_natural_32: NATURAL_8 = 8

	Element_id_natural_64: NATURAL_8 = 9

	Element_id_real_32: NATURAL_8 = 10

	Element_id_real_64: NATURAL_8 = 11

	Element_id_real_96: NATURAL_8 = 12

	Element_id_void: NATURAL_8 = 13

	Element_id_pointer: NATURAL_8 = 14

	Element_id_array: NATURAL_8 = 15

	Element_id_class: NATURAL_8 = 16

invariant
	pp /= default_pointer

end
