class
	CE_ARRAY [G]

inherit
	CE_POINTER [G]
		rename
			make as make_from_pointer,
			make_shared as make_shared_from_pointer,
			address as address_from_pointer,
			plus as plus_from_pointer,
			minus as minus_from_pointer,
			ce_gets as ce_gets_bad
		redefine
			make_shared_from_pointer,
			element_type,
			element_id,
			i_th,
			set_i_th,
			dispose,
			ce_gets_bad
		end

create
	make,
	make_shared,
	make_shared_from_pointer

feature {CE_ARRAY, CE_POINTER}

	make (a_dimensions: ARRAY [INTEGER_32])
		do
			make_from_pointer

			count := a_dimensions[1]
			create subarray_dimensions.make_filled (0, 1, a_dimensions.count - 1)
			across 2 |..| a_dimensions.count as c loop
				subarray_dimensions[c.item - 1] := a_dimensions[c.item]
			end

			pp_store (pp, default_pointer.memory_calloc (total_count, element_size))
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER; a_dimensions: ARRAY [INTEGER_32])
		do
			make_from_pointer

			count := a_dimensions[1]
			create subarray_dimensions.make_filled (0, 1, a_dimensions.count - 1)
			across 2 |..| a_dimensions.count as c loop
				subarray_dimensions[c.item - 1] := a_dimensions[c.item]
			end

			pp_store (pp, a_c_pointer)
		end

	make_shared_from_pointer (a_c_pointer: POINTER)
		do
			if attached {TYPE [CE_ARRAY [ANY]]} generating_type.generic_parameter_type (1) then
				(create {EXCEPTIONS}).raise ("CE_ARRAY.make_shared_from_pointer called on a multi-dimension array: " + generating_type.name)
			else
				make_shared (a_c_pointer, <<{INTEGER_32}.Max_value>>)
			end
		end

feature {NONE}

	element_type: TYPE [ANY]
		once ("OBJECT")
			from
				Result := generating_type
			until
				Result.generic_parameter_count = 0 or
				not attached {TYPE [CE_ARRAY [ANY]]} Result
			loop
				Result := Result.generic_parameter_type (1)
			end
		end

	element_id: INTEGER_32
		once ("OBJECT")
			Result := Precursor
			if Result = Element_id_void or Result = Element_id_array then
				(create {EXCEPTIONS}).raise ("CE_ARRAY.element_id: invalid element_id")
			end
		end

feature {ANY}

	address: CE_POINTER [like Current]
			-- &Current
		do
			create Result.make_shared (c_pointer)
		end

feature {CE_POINTER}

	subarray_dimensions: ARRAY [INTEGER_32]

	total_count: INTEGER_32
			-- the number of all elements in the array (all dimensions)
		do
			Result := count
			across subarray_dimensions as c loop
				Result := Result * c.item
			end
		end

feature {ANY}

	i_th alias "[]" (a_index: INTEGER_32): G assign set_i_th
			-- Current[0]
		local
			old_state: BOOLEAN
			ret_array: CE_ARRAY [ANY]
		do
			if subarray_dimensions.count > 0 then
				old_state := {ISE_RUNTIME}.check_assert (False)
				ret_array ?= new_instance_of (generating_type.generic_parameter_type (1).type_id)
				ret_array.make_shared (c_pointer + a_index * (total_count // count) * element_size, subarray_dimensions)
				old_state := {ISE_RUNTIME}.check_assert (old_state)
				Result := ({G} #? ret_array)
			else
				Result := Precursor (a_index)
			end
		end

	set_i_th (a_item: G; a_index: INTEGER_32)
			-- Current[0] := foo
		do
			if subarray_dimensions.count > 0 then
				(create {EXCEPTIONS}).raise ("CE_ARRAY.set_i_th called on a multi-dimension array")
			else
				Precursor (a_item, a_index)
			end
		end

feature {ANY} -- pointer arithmetics

	plus alias "+" (i: INTEGER_32): CE_POINTER [G]
		do
			if subarray_dimensions.count > 0 then
				create Result.make_shared (c_pointer + (i * element_size * subarray_dimensions[1]))
			else
				Result := plus_from_pointer (i)
			end
		end

	minus alias "-" (i: INTEGER_32): CE_POINTER [G]
		do
			if subarray_dimensions.count > 0 then
				create Result.make_shared (c_pointer + (-1 * i * element_size * subarray_dimensions[1]))
			else
				Result := minus_from_pointer (i)
			end
		end

feature {NONE}

	is_allocated_memory: BOOLEAN

	dispose
		do
			if is_allocated_memory then
				c_pointer.memory_free
			end
			Precursor
		end

feature {NONE}

	ce_gets_bad (new: CE_POINTER [ANY])
		do
			(create {EXCEPTIONS}).raise ("CE_ARRAY.ce_gets_bad called")
		end

invariant
	c_pointer /= default_pointer
	subarray_dimensions /= Void

end
