class
	P_MARK

inherit
	LESS_DATA
		redefine
			default_create
		end
	S_UNISTD
	S_STDLIB
	S_STRING
	S_TERMIOS
	S_SYS_IOCTL
	S_TERMCAP
	S_STDIO
	S_CTYPE
	S_LOCALE
	S_FCNTL
	S_LIBIO
	S_SYS_STAT
	S_SIGNAL
	S_TIME
	S_BITS_ERRNO
	S_REGEX

feature {NONE} -- Initialization

	default_create
		do
			create marks.make (<<53>>)
			create sm.make
		end

feature {ANY} -- attributes

	marks: CE_ARRAY [PD_MARK]

	sm: PD_MARK

feature {ANY} -- routines

	init_mark
		local
			l_i: INTEGER_32
		do
			l_i := 0
			from
			until
				not (l_i < 53)
			loop
				marks[l_i].m_scrpos.pos := (-1).to_integer_32
				l_i := l_i + 1
			end
		end

	getumark (a_c: INTEGER_32): CE_POINTER [PD_MARK]
		local
			js: JUMP_STATE
		do
			if a_c >= 97 then
				if a_c <= 122 then
					Result := (marks + a_c - 97)
					js.return
				end
			end
			if not js.return_called and then (a_c >= 65) then
				if a_c <= 90 then
					Result := (marks + (a_c - 65) + 26)
					js.return
				end
			end
			if not js.return_called then
				output_inst.error (ce_string ("Invalid mark letter"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
				Result := (create {CE_POINTER [PD_MARK]}.make_shared (default_pointer))
			end
		end

	getmark (a_c: INTEGER_32): CE_POINTER [PD_MARK]
		local
			l_m: CE_POINTER [PD_MARK]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_m.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect a_c -- switch 1 head
					when 94 then l_pos := -1 + 1
					when 36 then l_pos := -1 + 2
					when 46 then l_pos := -1 + 3
					when 39 then l_pos := -1 + 4
					else l_pos := -1 + 5
					end
				when 1 then -- switch 1, case 94
					l_m.ce_gets (sm.address)
					l_m.item.m_scrpos.pos := (0).to_integer_32
					l_m.item.m_scrpos.ln := 0
					l_m.item.m_ifile := main_inst.curr_ifile
					l_pos := -1 + 7 -- switch 1; break
				when 2 then -- switch 1, case 36
					if (ch_inst.ch_end_seek).to_boolean then
						output_inst.error (ce_string ("Cannot seek to end of file"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
						Result := (create {CE_POINTER [PD_MARK]}.make_shared (default_pointer))
						js.return
					end
					if not js.return_called then
						l_m.ce_gets (sm.address)
						l_m.item.m_scrpos.pos := ch_inst.ch_tell
						l_m.item.m_scrpos.ln := screen_inst.sc_height - 1
						l_m.item.m_ifile := main_inst.curr_ifile
					end
					l_pos := -1 + 7 -- switch 1; break
				when 3 then -- switch 1, case 46
					l_m.ce_gets (sm.address)
					position_inst.get_scrpos (create {CE_POINTER [PD_SCRPOS]}.make_shared (l_m.c_pointer + {PD_MARK}.m_scrpos_offset))
					l_m.item.m_ifile := main_inst.curr_ifile
					l_pos := -1 + 7 -- switch 1; break
				when 4 then -- switch 1, case 39
					l_m.ce_gets (marks + 52)
					l_pos := -1 + 7 -- switch 1; break
				when 5 then -- switch 1, default:
					l_m.ce_gets (getumark (a_c))
					if (l_m.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
						l_pos := -1 + 7 -- switch 1; break
					end
				when 6 then
					if l_m.item.m_scrpos.pos = -1 then
						output_inst.error (ce_string ("Mark not set"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
						Result := (create {CE_POINTER [PD_MARK]}.make_shared (default_pointer))
						js.return
					end
					l_pos := -1 + 7 -- switch 1; break
				when 7 then -- switch 1, done:
					if not js.return_called then
						Result := l_m
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	badmark (a_c: INTEGER_32): INTEGER_32
		do
			Result := (((getmark (a_c)).c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32).to_integer
		end

	setmark (a_c: INTEGER_32)
		local
			l_m: CE_POINTER [PD_MARK]
			l_scrpos: PD_SCRPOS
			js: JUMP_STATE
		do
			create l_m.make
			create l_scrpos.make

			l_m.ce_gets (getumark (a_c))
			if (l_m.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				js.return
			end
			if not js.return_called then
				position_inst.get_scrpos (l_scrpos.address)
				l_m.item.m_scrpos := l_scrpos
				l_m.item.m_ifile := main_inst.curr_ifile
			end
		end

	lastmark
		local
			l_scrpos: PD_SCRPOS
			js: JUMP_STATE
		do
			create l_scrpos.make

			if (((ch_inst.ch_getflags)).bit_and (8)).to_boolean then
				js.return
			end
			if not js.return_called then
				position_inst.get_scrpos (l_scrpos.address)
				if l_scrpos.pos = -1 then
					js.return
				end
			end
			if not js.return_called then
				marks[52].m_scrpos := l_scrpos
				marks[52].m_ifile := main_inst.curr_ifile
			end
		end

	gomark (a_c: INTEGER_32)
		local
			l_m: CE_POINTER [PD_MARK]
			l_scrpos: PD_SCRPOS
			js: JUMP_STATE
		do
			create l_m.make
			create l_scrpos.make

			l_m.ce_gets (getmark (a_c))
			if (l_m.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				js.return
			end
			if not js.return_called then
				if (l_m.c_pointer.to_integer_32).to_natural_32 = ((marks + 52).c_pointer.to_integer_32).to_natural_32 then
					if l_m.item.m_scrpos.pos = -1 then
						l_m.item.m_ifile := main_inst.curr_ifile
						l_m.item.m_scrpos.pos := (0).to_integer_32
						l_m.item.m_scrpos.ln := main_inst.jump_sline
					end
				end
				l_scrpos.ce_gets (l_m.item.m_scrpos)
				if ((l_m.item.m_ifile).c_pointer.to_integer_32).to_natural_32 /= (main_inst.curr_ifile.c_pointer.to_integer_32).to_natural_32 then
					if (edit_inst.edit_ifile (l_m.item.m_ifile)).to_boolean then
						js.return
					end
				end
			end
			if not js.return_called then
				jump_inst.jump_loc (l_scrpos.pos, l_scrpos.ln)
			end
		end

	markpos (a_c: INTEGER_32): INTEGER_32
		local
			l_m: CE_POINTER [PD_MARK]
			js: JUMP_STATE
		do
			create l_m.make

			l_m.ce_gets (getmark (a_c))
			if (l_m.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := (-1).to_integer_32
				js.return
			end
			if not js.return_called and then (((l_m.item.m_ifile).c_pointer.to_integer_32).to_natural_32 /= (main_inst.curr_ifile.c_pointer.to_integer_32).to_natural_32) then
				output_inst.error (ce_string ("Mark not in current file"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
				Result := (-1).to_integer_32
				js.return
			end
			if not js.return_called then
				Result := (l_m.item.m_scrpos.pos)
			end
		end

	unmark (a_ifile: CE_POINTER [ANY])
		local
			l_i: INTEGER_32
		do
			l_i := 0
			from
			until
				not (l_i < 53)
			loop
				if ((marks[l_i].m_ifile).c_pointer.to_integer_32).to_natural_32 = (a_ifile.c_pointer.to_integer_32).to_natural_32 then
					marks[l_i].m_scrpos.pos := (-1).to_integer_32
				end
				l_i := l_i + 1
			end
		end

end
