class
	P_TAGS

inherit
	LESS_DATA
		redefine
			default_create
		end
	S_UNISTD
	S_STDLIB
	S_STRING
	S_TERMIOS
	S_SYS_IOCTL
	S_TERMCAP
	S_STDIO
	S_CTYPE
	S_LOCALE
	S_FCNTL
	S_LIBIO
	S_SYS_STAT
	S_SIGNAL
	S_TIME
	S_BITS_ERRNO
	S_REGEX

feature {NONE} -- Initialization

	default_create
		do
			create tags.make
			create taglist.make
			create curtag.make
		end

feature {ANY} -- attributes

	tags: CE_POINTER [INTEGER_8]

	total: INTEGER_32 assign set_total

	set_total (a_total: INTEGER_32)
		do
			total := a_total
		end

	curseq: INTEGER_32 assign set_curseq

	set_curseq (a_curseq: INTEGER_32)
		do
			curseq := a_curseq
		end

	taglist: PD_TAGLIST

	curtag: CE_POINTER [PD_TAG]

	circular: INTEGER_32 assign set_circular

	set_circular (a_circular: INTEGER_32)
		do
			circular := a_circular
		end

feature {ANY} -- routines

	cleantags
		local
			l_tp: CE_POINTER [PD_TAG]
			js: JUMP_STATE
		do
			create l_tp.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_tp.ce_gets (taglist.tl_first)
				if (l_tp.c_pointer.to_integer_32).to_natural_32 = ((taglist.address).c_pointer.to_integer_32).to_natural_32 then
					js.break
				end
				if not js.loop_jumped then
					(l_tp.item.next).item.prev := l_tp.item.prev;
					(l_tp.item.prev).item.next := l_tp.item.next
					free (create {CE_POINTER [ANY]}.make_cast (l_tp))
				end
			end
			js.reset
			curtag.ce_gets (create {CE_POINTER [PD_TAG]}.make_shared (default_pointer))
			curseq := 0
			total := curseq
		end

	maketagent (a_name: CE_POINTER [INTEGER_8]; a_file: CE_POINTER [INTEGER_8]; a_linenum: INTEGER_32; a_pattern: CE_POINTER [INTEGER_8]; a_endline: INTEGER_32): CE_POINTER [PD_TAG]
		local
			l_tp: CE_POINTER [PD_TAG]
		do
			create l_tp.make

			l_tp.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((main_inst.ecalloc (({PD_TAG}.sizeof.to_natural_32).to_integer_32, {NATURAL_32} 1))))
			l_tp.item.tag_file := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (a_file)) + {NATURAL_32} 1).to_integer_32, Integer_8_bytes.to_natural_32)))
			strcpy (l_tp.item.tag_file, a_file).do_nothing
			l_tp.item.tag_linenum := a_linenum
			l_tp.item.tag_endline := a_endline.to_integer_8
			if (a_pattern.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				l_tp.item.tag_pattern := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			else
				l_tp.item.tag_pattern := create {CE_POINTER [INTEGER_8]}.make_cast ((main_inst.ecalloc (((strlen (a_pattern)) + {NATURAL_32} 1).to_integer_32, Integer_8_bytes.to_natural_32)))
				strcpy (l_tp.item.tag_pattern, a_pattern).do_nothing
			end
			Result := l_tp
		end

	gettagtype: INTEGER_32
		local
			l_f: INTEGER_32
			js: JUMP_STATE
		do
			if (strcmp (tags, ce_string ("GTAGS"))) = 0 then
				Result := 2
				js.return
			end
			if not js.return_called then
				if (strcmp (tags, ce_string ("GRTAGS"))) = 0 then
					Result := 3
					js.return
				end
			end
			if not js.return_called then
				if (strcmp (tags, ce_string ("GSYMS"))) = 0 then
					Result := 4
					js.return
				end
			end
			if not js.return_called then
				if (strcmp (tags, ce_string ("GPATH"))) = 0 then
					Result := 5
					js.return
				end
			end
			if not js.return_called then
				if (strcmp (tags, ce_string ("-"))) = 0 then
					Result := 1
					js.return
				end
			end
			if not js.return_called then
				l_f := open ([tags, 0])
				if l_f >= 0 then
					close (l_f).do_nothing
					Result := 0
					js.return
				end
			end
			if not js.return_called then
				Result := 2
			end
		end

	findtag (a_tag: CE_POINTER [INTEGER_8])
		local
			l_type: INTEGER_32
			l_result: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_type := (gettagtype)
					if l_type = 0 then
						l_result := findctag (a_tag)
					else
						l_result := findgtag (a_tag, l_type)
					end
					inspect l_result -- switch 1 head
					when 0 then l_pos := -1 + 1
					when 4 then l_pos := -1 + 1
					when 1 then l_pos := -1 + 2
					when 2 then l_pos := -1 + 3
					when 3 then l_pos := -1 + 4
					else l_pos := -1 + 5
					end
				when 1 then -- switch 1, case 0 | switch 1, case 4
					l_pos := -1 + 5 -- switch 1; break
				when 2 then -- switch 1, case 1
					output_inst.error (ce_string ("No tags file"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 2
					output_inst.error (ce_string ("No such tag in tags file"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, case 3
					output_inst.error (ce_string ("unknown tag type"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	tagsearch: INTEGER_32
		local
			js: JUMP_STATE
		do
			if (curtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := (-1).to_integer_32
				js.return
			end
			if not js.return_called then
				if curtag.item.tag_linenum /= 0 then
					Result := (gtagsearch)
				else
					Result := (ctagsearch)
				end
			end
		end

	nexttag (a_n: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_n: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tmp: INTEGER_32
			js: JUMP_STATE
		do
			create l_tagfile.make

			l_n := a_n
			l_tagfile.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			from
			until
				js.loop_left
			loop
				js.reset
				l_tmp := l_n
				l_n := l_n - 1
				if not (l_tmp > 0) then
					js.break
				end
				if not js.loop_jumped then
					l_tagfile.ce_gets (nextgtag)
				end
			end
			js.reset
			Result := l_tagfile
		end

	prevtag (a_n: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_n: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tmp: INTEGER_32
			js: JUMP_STATE
		do
			create l_tagfile.make

			l_n := a_n
			l_tagfile.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			from
			until
				js.loop_left
			loop
				js.reset
				l_tmp := l_n
				l_n := l_n - 1
				if not (l_tmp > 0) then
					js.break
				end
				if not js.loop_jumped then
					l_tagfile.ce_gets (prevgtag)
				end
			end
			js.reset
			Result := l_tagfile
		end

	ntags: INTEGER_32
		do
			Result := total
		end

	curr_tag: INTEGER_32
		do
			Result := curseq
		end

	findctag (a_tag: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_p: CE_POINTER [INTEGER_8]
			l_f: CE_POINTER [SD_IO_FILE]
			l_taglen: INTEGER_32
			l_taglinenum: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tagpattern: CE_POINTER [INTEGER_8]
			l_tagendline: INTEGER_32
			l_search_char: INTEGER_32
			l_err: INTEGER_32
			l_tline: CE_ARRAY [INTEGER_8]
			l_tp: CE_POINTER [PD_TAG]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___3: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_p.make
			create l_f.make
			create l_tagfile.make
			create l_tagpattern.make
			create l_tline.make (<<512>>)
			create l_tp.make
			create l_tmp___1.make
			create l_tmp___3.make

			l_p.ce_gets (filename_inst.shell_unquote (tags))
			l_f.ce_gets (fopen (l_p, ce_string ("r")))
			free (create {CE_POINTER [ANY]}.make_cast (l_p))
			if (l_f.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := ({PC_TAG_RESULT}.TAG_NOFILE)
				js.return
			end
			if not js.return_called then
				cleantags
				total := 0
				l_taglen := (strlen (a_tag)).to_integer_32
				from
				until
					js.loop_left
				loop
					js.reset
					if ((fgets (l_tline, (Integer_8_bytes.to_natural_32 * (512).to_natural_32).to_integer_32, l_f)).c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
						js.break
					end
					if not js.loop_jumped and then ((l_tline[0]).to_integer_32 = 33) then
						js.continue
					end
					if not js.loop_jumped then
						if (strncmp (a_tag, l_tline, l_taglen.to_natural_32)) /= 0 then
							js.continue
						elseif (l_tline[l_taglen]).to_integer_32 /= 32 then
							if (l_tline[l_taglen]).to_integer_32 /= 9 then
								js.continue
							end
						end
					end
					if not js.loop_jumped then
						l_tagpattern.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
						l_p.ce_gets (main_inst.skipsp (l_tline + l_taglen))
						if (l_p[0]).to_integer_32 = 0 then
							js.continue
						end
					end
					if not js.loop_jumped then
						l_tagfile.ce_gets (l_p)
						from
						until
							js.loop_left
						loop
							js.reset
							if (l_p[0]).to_integer_32 = 32 then
								js.break
							elseif (l_p[0]).to_integer_32 = 9 then
								js.break
							elseif (l_p[0]).to_integer_32 = 0 then
								js.break
							end
							if not js.loop_jumped then
								l_p.ce_gets (l_p + 1)
							end
						end
						js.reset
						l_tmp___1.ce_gets (l_p)
						l_p.ce_gets (l_p + 1)
						l_tmp___1[0] := (('%/0/').code).to_integer_8
						l_p.ce_gets (main_inst.skipsp (l_p))
						if (l_p[0]).to_integer_32 = 0 then
							js.continue
						end
					end
					if not js.loop_jumped then
						l_tagendline := 0
						l_taglinenum := (option_inst.getnum (l_p.address, create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), create {CE_POINTER [INTEGER_32]}.make_shared ($l_err))).to_integer_32
						if l_err.to_boolean then
							l_taglinenum := (0).to_integer_32
							l_tmp___3.ce_gets (l_p)
							l_p.ce_gets (l_p + 1)
							l_search_char := (l_tmp___3[0]).to_integer_32
							if (l_p[0]).to_integer_32 = 94 then
								l_p.ce_gets (l_p + 1)
							end
							l_tagpattern.ce_gets (l_p)
							from
							until
								js.loop_left
							loop
								js.reset
								if (l_p[0]).to_integer_32 /= l_search_char then
									if (l_p[0]).to_integer_32 = 0 then
										js.break
									end
								else
									js.break
								end
								if not js.loop_jumped then
									if (l_p[0]).to_integer_32 = 92 then
										l_p.ce_gets (l_p + 1)
									end
									l_p.ce_gets (l_p + 1)
								end
							end
							js.reset
							l_tagendline := ((l_p[-1]).to_integer_32 = 36).to_integer
							if l_tagendline.to_boolean then
								l_p.ce_gets (l_p - 1)
							end
							l_p[0] := (('%/0/').code).to_integer_8
						end
						l_tp.ce_gets (maketagent (a_tag, l_tagfile, l_taglinenum, l_tagpattern, l_tagendline))
						l_tp.item.next := taglist.tl_first
						l_tp.item.prev := create {CE_POINTER [PD_TAG]}.make_cast ((taglist.address));
						(taglist.tl_first).item.prev := l_tp
						taglist.tl_first := l_tp
						total := total + 1
					end
				end
				js.reset
				fclose (l_f).do_nothing
				if total = 0 then
					Result := ({PC_TAG_RESULT}.TAG_NOTAG)
					js.return
				end
			end
			if not js.return_called then
				curtag.ce_gets (taglist.tl_first)
				curseq := 1
				Result := ({PC_TAG_RESULT}.TAG_FOUND)
			end
		end

	edit_tagfile: INTEGER_32
		local
			js: JUMP_STATE
		do
			if (curtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := 1
				js.return
			end
			if not js.return_called then
				Result := (edit_inst.edit (curtag.item.tag_file))
			end
		end

	ctagsearch: INTEGER_32
		local
			l_pos2: INTEGER_32
			l_linepos: INTEGER_32
			l_linenum: INTEGER_32
			l_len: INTEGER_32
			l_line: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_line.make

			l_pos2 := (0).to_integer_32
			l_linenum := linenum_inst.find_linenum (l_pos2)
			from
			until
				js.loop_left
			loop
				js.reset
				if ((ch_inst.sigs).bit_and (3)).to_boolean then
					Result := (-1).to_integer_32
					js.return
				end
				if not js.return_called then
					l_linepos := l_pos2
					l_pos2 := line_inst.forw_raw_line (l_pos2, l_line.address)
					if l_linenum /= 0 then
						l_linenum := l_linenum + 1
					end
					if l_pos2 = -1 then
						output_inst.error (ce_string ("Tag not found"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
						Result := (-1).to_integer_32
						js.return
					end
				end
				if not js.return_called then
					if line_inst.linenums.to_boolean then
						linenum_inst.add_lnum (l_linenum, l_pos2)
					end
					l_len := (strlen (curtag.item.tag_pattern)).to_integer_32
					if (strncmp (curtag.item.tag_pattern, l_line, l_len.to_natural_32)) = 0 then
						if not (curtag.item.tag_endline).to_boolean then
							curtag.item.tag_linenum := linenum_inst.find_linenum (l_linepos)
							js.break
						elseif (l_line[l_len]).to_integer_32 = 0 then
							curtag.item.tag_linenum := linenum_inst.find_linenum (l_linepos)
							js.break
						elseif (l_line[l_len]).to_integer_32 = 13 then
							curtag.item.tag_linenum := linenum_inst.find_linenum (l_linepos)
							js.break
						end
					end
				end
			end
			js.reset
			if not js.return_called then
				Result := l_linepos
			end
		end

	findgtag (a_tag: CE_POINTER [INTEGER_8]; a_type: INTEGER_32): INTEGER_32
		local
			l_buf___1: CE_ARRAY [INTEGER_8]
			l_fp: CE_POINTER [SD_IO_FILE]
			l_tp: CE_POINTER [PD_TAG]
			l_command: CE_ARRAY [INTEGER_8]
			l_flag: CE_POINTER [INTEGER_8]
			l_qtag: CE_POINTER [INTEGER_8]
			l_cmd: CE_POINTER [INTEGER_8]
			l_name: CE_POINTER [INTEGER_8]
			l_file: CE_POINTER [INTEGER_8]
			l_line: CE_POINTER [INTEGER_8]
			l_c: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_buf___1.make (<<256>>)
			create l_fp.make
			create l_tp.make
			create l_command.make (<<512>>)
			create l_flag.make
			create l_qtag.make
			create l_cmd.make
			create l_name.make
			create l_file.make
			create l_line.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_type /= 1 then
						if (a_tag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 12 -- skip if block
					end
				when 1 then -- if:
						cleantags
						total := 0
						if a_type /= 1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_fp.ce_gets ((create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer)))
						tags.ce_gets (ce_string ("tags"))
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 4 then -- else:
						l_cmd.ce_gets ((decode_inst.lgetenv (ce_string ("LESSGLOBALTAGS"))))
						if (l_cmd.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							js.return
						elseif (l_cmd[0]).to_integer_32 = 0 then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 11 -- skip if block
						end
				when 5 then -- if:
						inspect a_type -- switch 1 head
						when 2 then l_pos := -1 + 6
						when 3 then l_pos := -1 + 7
						when 4 then l_pos := -1 + 8
						when 5 then l_pos := -1 + 9
						else l_pos := -1 + 10
						end
				when 6 then -- switch 1, case 2
						l_flag.ce_gets (ce_string (""))
						l_pos := -1 + 11 -- switch 1; break
				when 7 then -- switch 1, case 3
						l_flag.ce_gets (ce_string ("r"))
						l_pos := -1 + 11 -- switch 1; break
				when 8 then -- switch 1, case 4
						l_flag.ce_gets (ce_string ("s"))
						l_pos := -1 + 11 -- switch 1; break
				when 9 then -- switch 1, case 5
						l_flag.ce_gets (ce_string ("P"))
						l_pos := -1 + 11 -- switch 1; break
				when 10 then -- switch 1, default:
						Result := ({PC_TAG_RESULT}.TAG_NOTYPE)
						js.return
				when 11 then -- switch 1, done: | if done
						if not js.return_called then
							l_qtag.ce_gets (filename_inst.shell_quote (a_tag))
							if (l_qtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
								l_qtag.ce_gets (a_tag)
							end
							sprintf ([l_command, ce_string ("%%s -x%%s %%s"), l_cmd, l_flag, l_qtag]).do_nothing
							if (l_qtag.c_pointer.to_integer_32).to_natural_32 /= (a_tag.c_pointer.to_integer_32).to_natural_32 then
								free (create {CE_POINTER [ANY]}.make_cast (l_qtag))
							end
							l_fp.ce_gets (popen (l_command, ce_string ("r")))
						end
				when 12 then -- if done | if done
						if not js.return_called and then ((l_fp.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32) then
							from
							until
								js.loop_left
							loop
								js.reset
								if ((fgets (l_buf___1, (Integer_8_bytes.to_natural_32 * (256).to_natural_32).to_integer_32, l_fp))).c_pointer = default_pointer then
									js.break
								end
								if not js.loop_jumped then
									if ch_inst.sigs.to_boolean then
										if (l_fp.c_pointer.to_integer_32).to_natural_32 /= (((create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer))).c_pointer.to_integer_32).to_natural_32 then
											pclose (l_fp).do_nothing
										end
										Result := ({PC_TAG_RESULT}.TAG_INTR)
										js.return
									end
									if not js.return_called then
										if (l_buf___1[((strlen (l_buf___1)) - {NATURAL_32} 1).to_integer_32]).to_integer_32 = 10 then
											l_buf___1[((strlen (l_buf___1)) - {NATURAL_32} 1).to_integer_32] := (0).to_integer_8
										else
											from
											until
												js.loop_left
											loop
												js.reset
												l_c := fgetc (l_fp)
												if l_c /= 10 then
													if l_c = -1 then
														js.break
													end
												else
													js.break
												end
											end
											js.reset
										end
										if (getentry (l_buf___1, l_name.address, l_file.address, l_line.address)).to_boolean then
											js.break
										end
										if not js.loop_jumped then
											l_tp.ce_gets (maketagent (l_name, l_file, (atoi (l_line)).to_integer_32, create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), 0))
											l_tp.item.next := taglist.tl_first
											l_tp.item.prev := create {CE_POINTER [PD_TAG]}.make_cast ((taglist.address));
											(taglist.tl_first).item.prev := l_tp
											taglist.tl_first := l_tp
											total := total + 1
										end
									end
								end
							end
							js.reset
							if not js.return_called and then ((l_fp.c_pointer.to_integer_32).to_natural_32 /= (((create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer))).c_pointer.to_integer_32).to_natural_32) then
								if (pclose (l_fp)).to_boolean then
									curtag.ce_gets (create {CE_POINTER [PD_TAG]}.make_shared (default_pointer))
									curseq := 0
									total := curseq
									Result := ({PC_TAG_RESULT}.TAG_NOFILE)
									js.return
								end
							end
						end
						if not js.return_called then
							l_tp.ce_gets (taglist.tl_first)
							if (l_tp.c_pointer.to_integer_32).to_natural_32 = ((taglist.address).c_pointer.to_integer_32).to_natural_32 then
								Result := ({PC_TAG_RESULT}.TAG_NOTAG)
								js.return
							end
						end
						if not js.return_called then
							curtag.ce_gets (l_tp)
							curseq := 1
							Result := ({PC_TAG_RESULT}.TAG_FOUND)
						end
				when 13 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	nextgtag: CE_POINTER [INTEGER_8]
		local
			l_tp: CE_POINTER [PD_TAG]
			js: JUMP_STATE
		do
			create l_tp.make

			if (curtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_tp.ce_gets (curtag.item.next)
				if (l_tp.c_pointer.to_integer_32).to_natural_32 = ((taglist.address).c_pointer.to_integer_32).to_natural_32 then
					if not circular.to_boolean then
						Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
						js.return
					end
					if not js.return_called then
						curtag.ce_gets (taglist.tl_first)
						curseq := 1
					end
				else
					curtag.ce_gets (l_tp)
					curseq := curseq + 1
				end
			end
			if not js.return_called then
				Result := (curtag.item.tag_file)
			end
		end

	prevgtag: CE_POINTER [INTEGER_8]
		local
			l_tp: CE_POINTER [PD_TAG]
			js: JUMP_STATE
		do
			create l_tp.make

			if (curtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_tp.ce_gets (curtag.item.prev)
				if (l_tp.c_pointer.to_integer_32).to_natural_32 = ((taglist.address).c_pointer.to_integer_32).to_natural_32 then
					if not circular.to_boolean then
						Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
						js.return
					end
					if not js.return_called then
						curtag.ce_gets (taglist.tl_last)
						curseq := total
					end
				else
					curtag.ce_gets (l_tp)
					curseq := curseq - 1
				end
			end
			if not js.return_called then
				Result := (curtag.item.tag_file)
			end
		end

	gtagsearch: INTEGER_32
		local
			js: JUMP_STATE
		do
			if (curtag.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
				Result := (-1).to_integer_32
				js.return
			end
			if not js.return_called then
				Result := (linenum_inst.find_pos (curtag.item.tag_linenum))
			end
		end

	getentry (a_buf___1: CE_POINTER [INTEGER_8]; a_tag: CE_POINTER [CE_POINTER [INTEGER_8]]; a_file: CE_POINTER [CE_POINTER [INTEGER_8]]; a_line: CE_POINTER [CE_POINTER [INTEGER_8]]): INTEGER_32
		local
			l_p: CE_POINTER [INTEGER_8]
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_tmp___7: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_p.make
			create l_tmp___0.make
			create l_tmp___7.make

			l_p.ce_gets (a_buf___1)
			a_tag.item := l_p
			from
			until
				js.loop_left
			loop
				js.reset
				if (l_p[0]).to_boolean then
					if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					l_p.ce_gets (l_p + 1)
				end
			end
			js.reset
			if (l_p[0]).to_integer_32 = 0 then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				l_tmp___0.ce_gets (l_p)
				l_p.ce_gets (l_p + 1)
				l_tmp___0[0] := (0).to_integer_8
				from
				until
					js.loop_left
				loop
					js.reset
					if (l_p[0]).to_boolean then
						if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_p.ce_gets (l_p + 1)
					end
				end
				js.reset
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
					from
					until
						js.loop_left
					loop
						js.reset
						if (l_p[0]).to_boolean then
							if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
								js.break
							end
						else
							js.break
						end
						if not js.loop_jumped then
							l_p.ce_gets (l_p + 1)
						end
					end
					js.reset
					from
					until
						js.loop_left
					loop
						js.reset
						if (l_p[0]).to_boolean then
							if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
								js.break
							end
						else
							js.break
						end
						if not js.loop_jumped then
							l_p.ce_gets (l_p + 1)
						end
					end
					js.reset
				end
				if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				a_line.item := l_p
				a_line.item := l_p
				from
				until
					js.loop_left
				loop
					js.reset
					if (l_p[0]).to_boolean then
						if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_p.ce_gets (l_p + 1)
					end
				end
				js.reset
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				l_tmp___7.ce_gets (l_p)
				l_p.ce_gets (l_p + 1)
				l_tmp___7[0] := (0).to_integer_8
				from
				until
					js.loop_left
				loop
					js.reset
					if (l_p[0]).to_boolean then
						if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_p.ce_gets (l_p + 1)
					end
				end
				js.reset
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				a_file.item := l_p
				a_file.item := l_p
				from
				until
					js.loop_left
				loop
					js.reset
					if (l_p[0]).to_boolean then
						if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_p.ce_gets (l_p + 1)
					end
				end
				js.reset
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				l_p[0] := (0).to_integer_8
				if (strlen (a_tag.item)).to_boolean then
					if (strlen (a_line.item)).to_boolean then
						if (strlen (a_file.item)).to_boolean then
							if (atoi (a_line.item)) > 0 then
								Result := 0
								js.return
							end
						end
					end
				end
			end
			if not js.return_called then
				Result := (-1)
			end
		end

end
