class
	CE_VA_LIST

create
	make,
	make_shared,
	make_copy

feature {NONE}

	make (a_args: TUPLE; a_va_start_index: INTEGER_32)
		do
			if a_va_start_index <= a_args.count and then attached {CE_VA_LIST} a_args.item (a_va_start_index) as tmp then
				-- When external code calls a "vararg" Eiffel routine, the varargs are already
				-- put into a CE_VA_LIST and are accessible through `shared_c_pointer'.
				make_copy (tmp)
			else
				args := a_args
				va_start_index := a_va_start_index
				index := va_start_index
			end
		end

	make_shared (a_c_pointer: POINTER)
		do
			shared_c_pointer := a_c_pointer
		end

	make_copy (a_other: CE_VA_LIST)
		do
			args := a_other.args
			va_start_index := a_other.va_start_index
			index := a_other.index
			shared_c_pointer := a_other.shared_c_pointer
		end

feature {ANY} -- non-shared data

	args: TUPLE

	va_start_index, index: INTEGER_32

feature {CE_VA_LIST} -- shared data

	shared_c_pointer: POINTER

feature {ANY}

	integer_32_item: INTEGER_32
		do
			if args /= Void then
				inspect
					args.item_code (index)
				when {TUPLE}.Integer_32_code then
					Result := args.integer_32_item (index)
				when {TUPLE}.Natural_32_code then
					Result := args.natural_32_item (index).to_integer_32
				end
				index := index + 1
			else
				Result := c_integer_32_item ($shared_c_pointer)
			end
		end

	integer_64_item: INTEGER_64
		require
			args /= Void
		do
			inspect
				args.item_code (index)
			when {TUPLE}.Integer_64_code then
				Result := args.integer_64_item (index)
			end
			index := index + 1
		end

	natural_32_item: NATURAL_32
		require
			args /= Void
		do
			inspect
				args.item_code (index)
			when {TUPLE}.Integer_32_code then
				Result := args.integer_32_item (index).to_natural_32
			when {TUPLE}.Natural_32_code then
				Result := args.natural_32_item (index)
			end
			index := index + 1
		end

	natural_64_item: NATURAL_64
		require
			args /= Void
		do
			inspect
				args.item_code (index)
			when {TUPLE}.Integer_64_code then
				Result := args.integer_64_item (index).to_natural_64
			when {TUPLE}.Natural_64_code then
				Result := args.natural_64_item (index)
			end
			index := index + 1
		end

	real_64_item: REAL_64
		require
			args /= Void
		do
			inspect
				args.item_code (index)
			when {TUPLE}.Real_64_code then
				Result := args.real_64_item (index)
			end
			index := index + 1
		end

	real_96_item: REAL_96
		require
			args /= Void
		do
			inspect
				args.item_code (index)
			when {TUPLE}.Reference_code then
				if attached {REAL_96} args.reference_item (index) as tmp then
					Result := tmp
				else
					(create {EXCEPTIONS}).raise (args.reference_item (index).generator)
				end
			end
			index := index + 1
		end

	pointer_item: CE_POINTER [ANY]
		require
			args /= Void implies args.is_reference_item (index)
		do
			if args /= Void then
				Result ?= args.item (index)
				index := index + 1
			else
				create Result.make_shared (c_pointer_item ($shared_c_pointer))
			end
		ensure
			Result /= Void
		end

feature {ANY}

	c_pointer: POINTER
		do
			if args /= Void then
				Result := va_list + byte_offset[index - va_start_index + 1]
			else
				Result := shared_c_pointer
			end
		end

feature {NONE} -- c_pointer helpers

	va_list: POINTER
		require
			args /= Void
		local
			arg_index: INTEGER_32
		once ("OBJECT")
			if byte_offset[byte_offset.count] > 0 then
				from
					Result := Result.memory_alloc (byte_offset[byte_offset.count])
					arg_index := va_start_index
				until
					arg_index > args.count
				loop
					inspect
						args.item_code (arg_index)
					when {TUPLE}.Integer_8_code then  (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (integer_8_pointer (args, arg_index), {PLATFORM}.Integer_8_bytes)
					when {TUPLE}.Integer_16_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (integer_16_pointer (args, arg_index), {PLATFORM}.Integer_16_bytes)
					when {TUPLE}.Integer_32_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (integer_32_pointer (args, arg_index), {PLATFORM}.Integer_32_bytes)
					when {TUPLE}.Integer_64_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (integer_64_pointer (args, arg_index), {PLATFORM}.Integer_64_bytes)
					when {TUPLE}.Natural_8_code then  (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (natural_8_pointer (args, arg_index), {PLATFORM}.Natural_8_bytes)
					when {TUPLE}.Natural_16_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (natural_16_pointer (args, arg_index), {PLATFORM}.Natural_16_bytes)
					when {TUPLE}.Natural_32_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (natural_32_pointer (args, arg_index), {PLATFORM}.Natural_32_bytes)
					when {TUPLE}.Natural_64_code then (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (natural_64_pointer (args, arg_index), {PLATFORM}.Natural_64_bytes)
					when {TUPLE}.Real_32_code then    (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (real_32_pointer (args, arg_index), {PLATFORM}.Real_32_bytes)
					when {TUPLE}.Real_64_code then    (Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (real_64_pointer (args, arg_index), {PLATFORM}.Real_64_bytes)
					when {TUPLE}.reference_code then
						if attached {CE_POINTER [ANY]} args.item (arg_index) as tmp then
							(Result + byte_offset[arg_index - va_start_index + 1]).memory_copy (tmp.address.c_pointer, {PLATFORM}.Pointer_bytes)
						else
							(create {EXCEPTIONS}).raise ("CE_VA_LIST.va_list")
						end
					end
					arg_index := arg_index + 1
				end
			end
		end

	byte_offset: ARRAY [INTEGER]
		require
			args /= Void
		local
			arg_index: INTEGER_32
			array_index: INTEGER_32
		once ("OBJECT")
			from
				create Result.make_filled (0, 1, args.count - va_start_index + 2)
				arg_index := va_start_index
				array_index := 2
			until
				arg_index > args.count
			loop
				inspect
					args.item_code (arg_index)
				when {TUPLE}.Integer_8_code then  Result[array_index] := Result[array_index - 1] + {PLATFORM}.Integer_8_bytes
				when {TUPLE}.Integer_16_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Integer_16_bytes
				when {TUPLE}.Integer_32_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Integer_32_bytes
				when {TUPLE}.Integer_64_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Integer_64_bytes
				when {TUPLE}.Natural_8_code then  Result[array_index] := Result[array_index - 1] + {PLATFORM}.Natural_8_bytes
				when {TUPLE}.Natural_16_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Natural_16_bytes
				when {TUPLE}.Natural_32_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Natural_32_bytes
				when {TUPLE}.Natural_64_code then Result[array_index] := Result[array_index - 1] + {PLATFORM}.Natural_64_bytes
				when {TUPLE}.Real_32_code then    Result[array_index] := Result[array_index - 1] + {PLATFORM}.Real_32_bytes
				when {TUPLE}.Real_64_code then    Result[array_index] := Result[array_index - 1] + {PLATFORM}.Real_64_bytes
				when {TUPLE}.reference_code then  Result[array_index] := Result[array_index - 1] + {PLATFORM}.Pointer_bytes
				end
				arg_index := arg_index + 1
				array_index := array_index + 1
			end
		end

	integer_8_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_integer_8_item(eif_access($t), $i);"
		end

	integer_16_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_integer_16_item(eif_access($t), $i);"
		end

	integer_32_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_integer_32_item(eif_access($t), $i);"
		end

	integer_64_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_integer_64_item(eif_access($t), $i);"
		end

	natural_8_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_natural_8_item(eif_access($t), $i);"
		end

	natural_16_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_natural_16_item(eif_access($t), $i);"
		end

	natural_32_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_natural_32_item(eif_access($t), $i);"
		end

	natural_64_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_natural_64_item(eif_access($t), $i);"
		end

	real_32_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_real_32_item(eif_access($t), $i);"
		end

	real_64_pointer (t: TUPLE i: INTEGER): POINTER
		external
			"C inline"
		alias
			"return &eif_real_64_item(eif_access($t), $i);"
		end

feature {NONE}

	c_integer_32_item (a_va_list: POINTER): INTEGER_32
		external
			"C inline use <stdarg.h>"
		alias
			"return va_arg(*(va_list *)$a_va_list, int);"
		end

	c_pointer_item (a_va_list: POINTER): POINTER
		external
			"C inline use <stdarg.h>"
		alias
			"return va_arg(*(va_list *)$a_va_list, void *);"
		end

end
