class
	P_ERROR

inherit
	LINKS_DATA
		redefine
			default_create
		end
	S_SYS_SELECT
	S_STDLIB
	S_UNISTD
	S_STRING
	S_BITS_ERRNO
	S_SYS_SOCKET
	S_STDIO
	S_FCNTL
	S_NETINET_IN
	S_SYS_STAT
	S_SYS_UTSNAME
	S_NETDB
	S_SIGNAL
	S_PWD
	S_GRP
	S_TIME
	S_DIRENT
	S_TERMIOS
	S_SYS_IOCTL
	S_SYS_TIME
	S_SYS_WAIT
	S_UTIME

feature {NONE} -- Initialization

	default_create
		do
			create dummy_ptr.make
			create memory_list.make
			create errbuf.make (<<4096>>)
		end

feature {ANY} -- attributes

	dummy_val: INTEGER_8 assign set_dummy_val
		do
			Result := dummy_val_address[0]
		end

	set_dummy_val (a_dummy_val: INTEGER_8)
		do
			dummy_val_address[0] := a_dummy_val
		end

	dummy_val_address: CE_POINTER [INTEGER_8]
		once ("OBJECT")
			create Result.make_shared (default_pointer.memory_calloc (1, Integer_8_bytes))
		end

	dummy_ptr: CE_POINTER [INTEGER_8]

	mem_amount: INTEGER_32 assign set_mem_amount

	set_mem_amount (a_mem_amount: INTEGER_32)
		do
			mem_amount := a_mem_amount
		end

	last_mem_amount: INTEGER_32 assign set_last_mem_amount

	set_last_mem_amount (a_last_mem_amount: INTEGER_32)
		do
			last_mem_amount := a_last_mem_amount
		end

	memory_list: PD_LIST_HEAD

	errbuf: CE_ARRAY [NATURAL_8]

feature {ANY} -- routines

	do_not_optimize_here (a_p: CE_POINTER [ANY])
		do
			dummy_ptr[0] := (0).to_integer_8
		end

	force_dump
		do
			Io.error.put_string ("%N%/27/[1mForcing core dump%/27/[0m%N")
			Io.standard_default.flush
			Io.error.flush
			kbd_inst.fatal_tty_exit
			raise (11).do_nothing
		end

	check_memory_leaks
		local
			l_r: INTEGER_32
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_tmp: CE_POINTER [INTEGER_8]
		do
			create l_ah.make
			create l_tmp.make

			if mem_amount.to_boolean then
				fprintf ([(create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("%N%/27/[1mMemory leak by %%ld bytes%/27/[0m%N"), mem_amount]).do_nothing
				Io.error.put_string ("%NList of blocks: ")
				l_r := 0
				l_ah.ce_gets (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast (memory_list.next))
				from
				until
					(l_ah.c_pointer.to_integer_32).to_natural_32 = ((memory_list.address).c_pointer.to_integer_32).to_natural_32
				loop
					if l_r.to_boolean then
						l_tmp.ce_gets (ce_string (", "))
					else
						l_tmp.ce_gets (ce_string (""))
					end
					fprintf ([(create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("%%s%%p:%%d @ %%s:%%d"), l_tmp, create {CE_POINTER [INTEGER_8]}.make_cast (l_ah) + ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32, l_ah.item.size, l_ah.item.file, l_ah.item.line]).do_nothing
					l_r := 1
					if ((l_ah.item.comment).c_pointer /= default_pointer) then
						Io.error.put_string (":%"" + eif_string (create {CE_POINTER [INTEGER_8]}.make_shared ((l_ah.item.comment).c_pointer)) + "%"")
					end
					l_ah.ce_gets (l_ah.item.next)
				end
				Io.error.put_string ("%N")
				force_dump
			end
		end

	er (a_b: INTEGER_32; a_m: CE_POINTER [NATURAL_8]; a_l: CE_VA_LIST)
		local
			l_l: CE_VA_LIST
		do
			create l_l.make_copy (a_l)
			if a_b.to_boolean then
				Io.error.put_string ("")
			end
			vfprintf ((create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), create {CE_POINTER [INTEGER_8]}.make_cast (a_m), l_l).do_nothing
			Io.error.put_string ("%N")
			sleep ({NATURAL_32} 1).do_nothing
		end

	error (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make (a_args, 2)
			er (1, a_args.a_m, l_l)
		end

	int_error (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make (a_args, 2)
			sprintf ([create {CE_POINTER [INTEGER_8]}.make_cast (errbuf), ce_string ("%/27/[1mINTERNAL ERROR%/27/[0m at %%s:%%d: "), links_inst.errfile, links_inst.errline]).do_nothing
			strcat (create {CE_POINTER [INTEGER_8]}.make_cast (errbuf), create {CE_POINTER [INTEGER_8]}.make_cast (a_args.a_m)).do_nothing
			er (1, errbuf, l_l)
			force_dump
		end

	debug_msg (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make (a_args, 2)
			sprintf ([create {CE_POINTER [INTEGER_8]}.make_cast (errbuf), ce_string ("DEBUG MESSAGE at %%s:%%d: "), links_inst.errfile, links_inst.errline]).do_nothing
			strcat (create {CE_POINTER [INTEGER_8]}.make_cast (errbuf), create {CE_POINTER [INTEGER_8]}.make_cast (a_args.a_m)).do_nothing
			er (0, errbuf, l_l)
		end

	debug_mem_alloc (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_size: NATURAL_32
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			js: JUMP_STATE
		do
			create l_p.make
			create l_ah.make

			l_size := a_size
			if not l_size.to_boolean then
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer + -1))
				js.return
			end
			if not js.return_called then
				if l_size > {NATURAL_32} 2147483647 then
					from
					until
						False
					loop
						error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("error.c"), 106])
						kbd_inst.fatal_tty_exit
						exit (4)
					end
				end
				mem_amount := (mem_amount.to_natural_32 + l_size.to_natural_32).to_integer_32
				l_size := l_size + ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288))
				l_p.ce_gets (malloc (l_size))
				if (l_p).c_pointer = default_pointer then
					error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: out of memory (malloc returned NULL)"))])
					kbd_inst.fatal_tty_exit
					exit (4)
					Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				l_ah.ce_gets (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p))
				l_p.ce_gets (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_p) + ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32)))
				l_ah.item.size := (l_size - ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288))).to_integer_32
				l_ah.item.file := a_file
				l_ah.item.line := a_line
				l_ah.item.comment := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer)
				from
				until
					js.loop_left
				loop
					js.reset
					do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((memory_list.address)))
					l_ah.item.next := (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast ((memory_list.address))).item.next
					l_ah.item.prev := create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast ((memory_list.address));
					(create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast ((memory_list.address))).item.next := l_ah;
					(l_ah.item.next).item.prev := l_ah
					do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((memory_list.address)))
					js.break
				end
				js.reset
				Result := l_p
			end
		end

	debug_mem_free (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_p: CE_POINTER [ANY])
		local
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			js: JUMP_STATE
		do
			create l_p.make
			create l_ah.make

			l_p.ce_gets (a_p)
			if (l_p.c_pointer.to_integer_32).to_natural_32 = (-1).to_natural_32 then
				js.return
			end
			if not js.return_called and then ((l_p).c_pointer = default_pointer) then
				links_inst.errfile.ce_gets (a_file)
				links_inst.errline := a_line
				int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("mem_free(NULL)"))])
				js.return
			end
			if not js.return_called then
				l_p.ce_gets (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_p) - ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32)))
				l_ah.ce_gets (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p))
				do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_ah));
				(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_ah.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_ah.item.prev);
				(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_ah.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_ah.item.next)
				do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_ah))
				if ((l_ah.item.comment).c_pointer /= default_pointer) then
					free (create {CE_POINTER [ANY]}.make_cast (l_ah.item.comment))
				end
				mem_amount := mem_amount - (l_ah.item.size).to_integer_32
				free (l_p)
			end
		end

	debug_mem_realloc (a_file: CE_POINTER [NATURAL_8]; a_line: INTEGER_32; a_p: CE_POINTER [ANY]; a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [ANY]
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			js: JUMP_STATE
		do
			create l_p.make
			create l_ah.make

			l_p.ce_gets (a_p)
			if (l_p.c_pointer.to_integer_32).to_natural_32 = (-1).to_natural_32 then
				Result := (debug_mem_alloc (a_file, a_line, a_size))
				js.return
			end
			if not js.return_called and then ((l_p).c_pointer = default_pointer) then
				links_inst.errfile.ce_gets (a_file)
				links_inst.errline := a_line
				int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("mem_realloc(NULL, %%d)")), a_size])
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called and then (not a_size.to_boolean) then
				debug_mem_free (a_file, a_line, l_p)
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer + -1))
				js.return
			end
			if not js.return_called then
				if a_size > {NATURAL_32} 2147483647 then
					from
					until
						False
					loop
						error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("error.c"), 167])
						kbd_inst.fatal_tty_exit
						exit (4)
					end
				end
				l_p.ce_gets (realloc (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_p) - ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32)), a_size + ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288))))
				if (l_p).c_pointer = default_pointer then
					error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: out of memory (realloc returned NULL)"))])
					kbd_inst.fatal_tty_exit
					exit (4)
					Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				l_ah.ce_gets (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast (l_p))
				mem_amount := (mem_amount.to_natural_32 + (a_size - (l_ah.item.size).to_natural_32).to_natural_32).to_integer_32
				l_ah.item.size := a_size.to_integer_32;
				(l_ah.item.prev).item.next := l_ah;
				(l_ah.item.next).item.prev := l_ah
				Result := (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (l_p) + ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32)))
			end
		end

	set_mem_comment (a_p: CE_POINTER [ANY]; a_c: CE_POINTER [NATURAL_8]; a_l: INTEGER_32)
		local
			l_ah: CE_POINTER [PD_ALLOC_HEADER]
			l_tmp: CE_POINTER [NATURAL_8]
		do
			create l_ah.make
			create l_tmp.make

			l_ah.ce_gets (create {CE_POINTER [PD_ALLOC_HEADER]}.make_cast ((create {CE_POINTER [INTEGER_8]}.make_cast (a_p) - ((({PD_ALLOC_HEADER}.sizeof.to_natural_32 + {NATURAL_32} 7)).bit_and ({NATURAL_32} 4294967288)).to_integer_32)))
			if ((l_ah.item.comment).c_pointer /= default_pointer) then
				free (create {CE_POINTER [ANY]}.make_cast (l_ah.item.comment))
			end
			l_tmp.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast ((malloc ((a_l + 1).to_natural_32))))
			l_ah.item.comment := l_tmp
			if ((l_tmp).c_pointer /= default_pointer) then
				memcpy (create {CE_POINTER [ANY]}.make_cast (l_ah.item.comment), create {CE_POINTER [ANY]}.make_cast (a_c), a_l.to_natural_32).do_nothing
				l_ah.item.comment[a_l] := (0).to_natural_8
			end
		end

end
