class
	P_MD5

inherit
	LIBCURL_DATA
		redefine
			default_create
		end
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {NONE} -- Initialization

	default_create
		do
			create PADDING.make (<<64>>)
			create Curl_HMAC_MD5.make (<<1>>)
		end

feature {ANY} -- attributes

	PADDING: CE_ARRAY [NATURAL_8]

	Curl_HMAC_MD5: CE_ARRAY [PD_HMAC_PARAMS]

feature {ANY} -- routines

	MD5_Init (a_context: CE_POINTER [PD_MD5_CTX])
		local
			l_tmp: NATURAL_32
		do
			l_tmp := (0).to_natural_32
			a_context.item.count[1] := l_tmp
			a_context.item.count[0] := l_tmp
			a_context.item.state[0] := (1732584193).to_natural_32
			a_context.item.state[1] := {NATURAL_32} 4023233417
			a_context.item.state[2] := {NATURAL_32} 2562383102
			a_context.item.state[3] := (271733878).to_natural_32
		end

	MD5_Init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent MD5_Init)
		end

	MD5_Update (a_context: CE_POINTER [PD_MD5_CTX]; a_input: CE_POINTER [NATURAL_8]; a_inputLen: NATURAL_32)
		local
			l_i: NATURAL_32
			l_bufindex: NATURAL_32
			l_partLen: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_bufindex := (((a_context.item.count[0]).bit_shift_right (3))).bit_and ({NATURAL_32} 63)
			l_tmp := a_context.item.count[0] + ((a_inputLen).bit_shift_left (3))
			a_context.item.count[0] := l_tmp
			if l_tmp < (a_inputLen).bit_shift_left (3) then
				a_context.item.count[1] := (a_context.item.count[1]) + 1
			end
			a_context.item.count[1] := a_context.item.count[1] + ((a_inputLen).bit_shift_right (29))
			l_partLen := {NATURAL_32} 64 - l_bufindex
			if a_inputLen >= l_partLen then
				memcpy (create {CE_POINTER [ANY]}.make_cast ((a_context.item.buffer + l_bufindex.to_integer_32)), create {CE_POINTER [ANY]}.make_cast (a_input), l_partLen).do_nothing
				MD5Transform (create {CE_POINTER [NATURAL_32]}.make_cast ((a_context.item.state)), create {CE_POINTER [NATURAL_8]}.make_cast ((a_context.item.buffer)))
				l_i := l_partLen
				from
				until
					not (l_i + {NATURAL_32} 63 < a_inputLen)
				loop
					MD5Transform (create {CE_POINTER [NATURAL_32]}.make_cast ((a_context.item.state)), (a_input + l_i.to_integer_32))
					l_i := l_i + ({NATURAL_32} 64)
				end
				l_bufindex := {NATURAL_32} 0
			else
				l_i := {NATURAL_32} 0
			end
			memcpy (create {CE_POINTER [ANY]}.make_cast ((a_context.item.buffer + l_bufindex.to_integer_32)), create {CE_POINTER [ANY]}.make_cast ((a_input + l_i.to_integer_32)), a_inputLen - l_i).do_nothing
		end

	MD5_Update_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent MD5_Update)
		end

	MD5_Final (a_digest: CE_POINTER [NATURAL_8]; a_context: CE_POINTER [PD_MD5_CTX])
		local
			l_bits: CE_ARRAY [NATURAL_8]
			l_count: NATURAL_32
			l_padLen: NATURAL_32
		do
			create l_bits.make (<<8>>)

			Encode (l_bits, a_context.item.count, {NATURAL_32} 8)
			l_count := (((a_context.item.count[0]).bit_shift_right (3))).bit_and ({NATURAL_32} 63)
			if l_count < {NATURAL_32} 56 then
				l_padLen := {NATURAL_32} 56 - l_count
			else
				l_padLen := {NATURAL_32} 120 - l_count
			end
			MD5_Update (a_context, PADDING, l_padLen)
			MD5_Update (a_context, create {CE_POINTER [NATURAL_8]}.make_cast (l_bits), {NATURAL_32} 8)
			Encode (a_digest, a_context.item.state, {NATURAL_32} 16)
			memset (create {CE_POINTER [ANY]}.make_cast (a_context), 0, {PD_MD5_CTX}.sizeof.to_natural_32).do_nothing
		end

	MD5_Final_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent MD5_Final)
		end

	MD5Transform (a_state___3: CE_POINTER [NATURAL_32]; a_block: CE_POINTER [NATURAL_8])
		local
			l_a: NATURAL_32
			l_b: NATURAL_32
			l_c: NATURAL_32
			l_d: NATURAL_32
			l_x: CE_ARRAY [NATURAL_32]
		do
			create l_x.make (<<16>>)

			l_a := a_state___3[0]
			l_b := a_state___3[1]
			l_c := a_state___3[2]
			l_d := a_state___3[3]
			Decode (l_x, a_block, {NATURAL_32} 64)
			l_a := l_a + ((((((l_b).bit_and (l_c))).bit_or ((((l_b).bit_not).bit_and (l_d)))) + l_x[0]) + {NATURAL_32} 3614090360)
			l_a := (((l_a).bit_shift_left (7))).bit_or (((l_a).bit_shift_right (25)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_b))).bit_or ((((l_a).bit_not).bit_and (l_c)))) + l_x[1]) + {NATURAL_32} 3905402710)
			l_d := (((l_d).bit_shift_left (12))).bit_or (((l_d).bit_shift_right (20)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_a))).bit_or ((((l_d).bit_not).bit_and (l_b)))) + l_x[2]) + {NATURAL_32} 606105819)
			l_c := (((l_c).bit_shift_left (17))).bit_or (((l_c).bit_shift_right (15)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_d))).bit_or ((((l_c).bit_not).bit_and (l_a)))) + l_x[3]) + {NATURAL_32} 3250441966)
			l_b := (((l_b).bit_shift_left (22))).bit_or (((l_b).bit_shift_right (10)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_c))).bit_or ((((l_b).bit_not).bit_and (l_d)))) + l_x[4]) + {NATURAL_32} 4118548399)
			l_a := (((l_a).bit_shift_left (7))).bit_or (((l_a).bit_shift_right (25)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_b))).bit_or ((((l_a).bit_not).bit_and (l_c)))) + l_x[5]) + {NATURAL_32} 1200080426)
			l_d := (((l_d).bit_shift_left (12))).bit_or (((l_d).bit_shift_right (20)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_a))).bit_or ((((l_d).bit_not).bit_and (l_b)))) + l_x[6]) + {NATURAL_32} 2821735955)
			l_c := (((l_c).bit_shift_left (17))).bit_or (((l_c).bit_shift_right (15)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_d))).bit_or ((((l_c).bit_not).bit_and (l_a)))) + l_x[7]) + {NATURAL_32} 4249261313)
			l_b := (((l_b).bit_shift_left (22))).bit_or (((l_b).bit_shift_right (10)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_c))).bit_or ((((l_b).bit_not).bit_and (l_d)))) + l_x[8]) + {NATURAL_32} 1770035416)
			l_a := (((l_a).bit_shift_left (7))).bit_or (((l_a).bit_shift_right (25)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_b))).bit_or ((((l_a).bit_not).bit_and (l_c)))) + l_x[9]) + {NATURAL_32} 2336552879)
			l_d := (((l_d).bit_shift_left (12))).bit_or (((l_d).bit_shift_right (20)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_a))).bit_or ((((l_d).bit_not).bit_and (l_b)))) + l_x[10]) + {NATURAL_32} 4294925233)
			l_c := (((l_c).bit_shift_left (17))).bit_or (((l_c).bit_shift_right (15)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_d))).bit_or ((((l_c).bit_not).bit_and (l_a)))) + l_x[11]) + {NATURAL_32} 2304563134)
			l_b := (((l_b).bit_shift_left (22))).bit_or (((l_b).bit_shift_right (10)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_c))).bit_or ((((l_b).bit_not).bit_and (l_d)))) + l_x[12]) + {NATURAL_32} 1804603682)
			l_a := (((l_a).bit_shift_left (7))).bit_or (((l_a).bit_shift_right (25)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_b))).bit_or ((((l_a).bit_not).bit_and (l_c)))) + l_x[13]) + {NATURAL_32} 4254626195)
			l_d := (((l_d).bit_shift_left (12))).bit_or (((l_d).bit_shift_right (20)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_a))).bit_or ((((l_d).bit_not).bit_and (l_b)))) + l_x[14]) + {NATURAL_32} 2792965006)
			l_c := (((l_c).bit_shift_left (17))).bit_or (((l_c).bit_shift_right (15)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_d))).bit_or ((((l_c).bit_not).bit_and (l_a)))) + l_x[15]) + {NATURAL_32} 1236535329)
			l_b := (((l_b).bit_shift_left (22))).bit_or (((l_b).bit_shift_right (10)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_d))).bit_or (((l_c).bit_and ((l_d).bit_not)))) + l_x[1]) + {NATURAL_32} 4129170786)
			l_a := (((l_a).bit_shift_left (5))).bit_or (((l_a).bit_shift_right (27)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_c))).bit_or (((l_b).bit_and ((l_c).bit_not)))) + l_x[6]) + {NATURAL_32} 3225465664)
			l_d := (((l_d).bit_shift_left (9))).bit_or (((l_d).bit_shift_right (23)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_b))).bit_or (((l_a).bit_and ((l_b).bit_not)))) + l_x[11]) + {NATURAL_32} 643717713)
			l_c := (((l_c).bit_shift_left (14))).bit_or (((l_c).bit_shift_right (18)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_a))).bit_or (((l_d).bit_and ((l_a).bit_not)))) + l_x[0]) + {NATURAL_32} 3921069994)
			l_b := (((l_b).bit_shift_left (20))).bit_or (((l_b).bit_shift_right (12)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_d))).bit_or (((l_c).bit_and ((l_d).bit_not)))) + l_x[5]) + {NATURAL_32} 3593408605)
			l_a := (((l_a).bit_shift_left (5))).bit_or (((l_a).bit_shift_right (27)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_c))).bit_or (((l_b).bit_and ((l_c).bit_not)))) + l_x[10]) + {NATURAL_32} 38016083)
			l_d := (((l_d).bit_shift_left (9))).bit_or (((l_d).bit_shift_right (23)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_b))).bit_or (((l_a).bit_and ((l_b).bit_not)))) + l_x[15]) + {NATURAL_32} 3634488961)
			l_c := (((l_c).bit_shift_left (14))).bit_or (((l_c).bit_shift_right (18)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_a))).bit_or (((l_d).bit_and ((l_a).bit_not)))) + l_x[4]) + {NATURAL_32} 3889429448)
			l_b := (((l_b).bit_shift_left (20))).bit_or (((l_b).bit_shift_right (12)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_d))).bit_or (((l_c).bit_and ((l_d).bit_not)))) + l_x[9]) + {NATURAL_32} 568446438)
			l_a := (((l_a).bit_shift_left (5))).bit_or (((l_a).bit_shift_right (27)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_c))).bit_or (((l_b).bit_and ((l_c).bit_not)))) + l_x[14]) + {NATURAL_32} 3275163606)
			l_d := (((l_d).bit_shift_left (9))).bit_or (((l_d).bit_shift_right (23)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_b))).bit_or (((l_a).bit_and ((l_b).bit_not)))) + l_x[3]) + {NATURAL_32} 4107603335)
			l_c := (((l_c).bit_shift_left (14))).bit_or (((l_c).bit_shift_right (18)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_a))).bit_or (((l_d).bit_and ((l_a).bit_not)))) + l_x[8]) + {NATURAL_32} 1163531501)
			l_b := (((l_b).bit_shift_left (20))).bit_or (((l_b).bit_shift_right (12)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_and (l_d))).bit_or (((l_c).bit_and ((l_d).bit_not)))) + l_x[13]) + {NATURAL_32} 2850285829)
			l_a := (((l_a).bit_shift_left (5))).bit_or (((l_a).bit_shift_right (27)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_and (l_c))).bit_or (((l_b).bit_and ((l_c).bit_not)))) + l_x[2]) + {NATURAL_32} 4243563512)
			l_d := (((l_d).bit_shift_left (9))).bit_or (((l_d).bit_shift_right (23)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_and (l_b))).bit_or (((l_a).bit_and ((l_b).bit_not)))) + l_x[7]) + {NATURAL_32} 1735328473)
			l_c := (((l_c).bit_shift_left (14))).bit_or (((l_c).bit_shift_right (18)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_and (l_a))).bit_or (((l_d).bit_and ((l_a).bit_not)))) + l_x[12]) + {NATURAL_32} 2368359562)
			l_b := (((l_b).bit_shift_left (20))).bit_or (((l_b).bit_shift_right (12)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_xor (l_c))).bit_xor (l_d)) + l_x[5]) + {NATURAL_32} 4294588738)
			l_a := (((l_a).bit_shift_left (4))).bit_or (((l_a).bit_shift_right (28)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_xor (l_b))).bit_xor (l_c)) + l_x[8]) + {NATURAL_32} 2272392833)
			l_d := (((l_d).bit_shift_left (11))).bit_or (((l_d).bit_shift_right (21)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_xor (l_a))).bit_xor (l_b)) + l_x[11]) + {NATURAL_32} 1839030562)
			l_c := (((l_c).bit_shift_left (16))).bit_or (((l_c).bit_shift_right (16)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_xor (l_d))).bit_xor (l_a)) + l_x[14]) + {NATURAL_32} 4259657740)
			l_b := (((l_b).bit_shift_left (23))).bit_or (((l_b).bit_shift_right (9)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_xor (l_c))).bit_xor (l_d)) + l_x[1]) + {NATURAL_32} 2763975236)
			l_a := (((l_a).bit_shift_left (4))).bit_or (((l_a).bit_shift_right (28)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_xor (l_b))).bit_xor (l_c)) + l_x[4]) + {NATURAL_32} 1272893353)
			l_d := (((l_d).bit_shift_left (11))).bit_or (((l_d).bit_shift_right (21)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_xor (l_a))).bit_xor (l_b)) + l_x[7]) + {NATURAL_32} 4139469664)
			l_c := (((l_c).bit_shift_left (16))).bit_or (((l_c).bit_shift_right (16)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_xor (l_d))).bit_xor (l_a)) + l_x[10]) + {NATURAL_32} 3200236656)
			l_b := (((l_b).bit_shift_left (23))).bit_or (((l_b).bit_shift_right (9)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_xor (l_c))).bit_xor (l_d)) + l_x[13]) + {NATURAL_32} 681279174)
			l_a := (((l_a).bit_shift_left (4))).bit_or (((l_a).bit_shift_right (28)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_xor (l_b))).bit_xor (l_c)) + l_x[0]) + {NATURAL_32} 3936430074)
			l_d := (((l_d).bit_shift_left (11))).bit_or (((l_d).bit_shift_right (21)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_xor (l_a))).bit_xor (l_b)) + l_x[3]) + {NATURAL_32} 3572445317)
			l_c := (((l_c).bit_shift_left (16))).bit_or (((l_c).bit_shift_right (16)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_xor (l_d))).bit_xor (l_a)) + l_x[6]) + {NATURAL_32} 76029189)
			l_b := (((l_b).bit_shift_left (23))).bit_or (((l_b).bit_shift_right (9)))
			l_b := l_b + l_c
			l_a := l_a + ((((((l_b).bit_xor (l_c))).bit_xor (l_d)) + l_x[9]) + {NATURAL_32} 3654602809)
			l_a := (((l_a).bit_shift_left (4))).bit_or (((l_a).bit_shift_right (28)))
			l_a := l_a + l_b
			l_d := l_d + ((((((l_a).bit_xor (l_b))).bit_xor (l_c)) + l_x[12]) + {NATURAL_32} 3873151461)
			l_d := (((l_d).bit_shift_left (11))).bit_or (((l_d).bit_shift_right (21)))
			l_d := l_d + l_a
			l_c := l_c + ((((((l_d).bit_xor (l_a))).bit_xor (l_b)) + l_x[15]) + {NATURAL_32} 530742520)
			l_c := (((l_c).bit_shift_left (16))).bit_or (((l_c).bit_shift_right (16)))
			l_c := l_c + l_d
			l_b := l_b + ((((((l_c).bit_xor (l_d))).bit_xor (l_a)) + l_x[2]) + {NATURAL_32} 3299628645)
			l_b := (((l_b).bit_shift_left (23))).bit_or (((l_b).bit_shift_right (9)))
			l_b := l_b + l_c
			l_a := l_a + ((((l_c).bit_xor (((l_b).bit_or ((l_d).bit_not)))) + l_x[0]) + {NATURAL_32} 4096336452)
			l_a := (((l_a).bit_shift_left (6))).bit_or (((l_a).bit_shift_right (26)))
			l_a := l_a + l_b
			l_d := l_d + ((((l_b).bit_xor (((l_a).bit_or ((l_c).bit_not)))) + l_x[7]) + {NATURAL_32} 1126891415)
			l_d := (((l_d).bit_shift_left (10))).bit_or (((l_d).bit_shift_right (22)))
			l_d := l_d + l_a
			l_c := l_c + ((((l_a).bit_xor (((l_d).bit_or ((l_b).bit_not)))) + l_x[14]) + {NATURAL_32} 2878612391)
			l_c := (((l_c).bit_shift_left (15))).bit_or (((l_c).bit_shift_right (17)))
			l_c := l_c + l_d
			l_b := l_b + ((((l_d).bit_xor (((l_c).bit_or ((l_a).bit_not)))) + l_x[5]) + {NATURAL_32} 4237533241)
			l_b := (((l_b).bit_shift_left (21))).bit_or (((l_b).bit_shift_right (11)))
			l_b := l_b + l_c
			l_a := l_a + ((((l_c).bit_xor (((l_b).bit_or ((l_d).bit_not)))) + l_x[12]) + {NATURAL_32} 1700485571)
			l_a := (((l_a).bit_shift_left (6))).bit_or (((l_a).bit_shift_right (26)))
			l_a := l_a + l_b
			l_d := l_d + ((((l_b).bit_xor (((l_a).bit_or ((l_c).bit_not)))) + l_x[3]) + {NATURAL_32} 2399980690)
			l_d := (((l_d).bit_shift_left (10))).bit_or (((l_d).bit_shift_right (22)))
			l_d := l_d + l_a
			l_c := l_c + ((((l_a).bit_xor (((l_d).bit_or ((l_b).bit_not)))) + l_x[10]) + {NATURAL_32} 4293915773)
			l_c := (((l_c).bit_shift_left (15))).bit_or (((l_c).bit_shift_right (17)))
			l_c := l_c + l_d
			l_b := l_b + ((((l_d).bit_xor (((l_c).bit_or ((l_a).bit_not)))) + l_x[1]) + {NATURAL_32} 2240044497)
			l_b := (((l_b).bit_shift_left (21))).bit_or (((l_b).bit_shift_right (11)))
			l_b := l_b + l_c
			l_a := l_a + ((((l_c).bit_xor (((l_b).bit_or ((l_d).bit_not)))) + l_x[8]) + {NATURAL_32} 1873313359)
			l_a := (((l_a).bit_shift_left (6))).bit_or (((l_a).bit_shift_right (26)))
			l_a := l_a + l_b
			l_d := l_d + ((((l_b).bit_xor (((l_a).bit_or ((l_c).bit_not)))) + l_x[15]) + {NATURAL_32} 4264355552)
			l_d := (((l_d).bit_shift_left (10))).bit_or (((l_d).bit_shift_right (22)))
			l_d := l_d + l_a
			l_c := l_c + ((((l_a).bit_xor (((l_d).bit_or ((l_b).bit_not)))) + l_x[6]) + {NATURAL_32} 2734768916)
			l_c := (((l_c).bit_shift_left (15))).bit_or (((l_c).bit_shift_right (17)))
			l_c := l_c + l_d
			l_b := l_b + ((((l_d).bit_xor (((l_c).bit_or ((l_a).bit_not)))) + l_x[13]) + {NATURAL_32} 1309151649)
			l_b := (((l_b).bit_shift_left (21))).bit_or (((l_b).bit_shift_right (11)))
			l_b := l_b + l_c
			l_a := l_a + ((((l_c).bit_xor (((l_b).bit_or ((l_d).bit_not)))) + l_x[4]) + {NATURAL_32} 4149444226)
			l_a := (((l_a).bit_shift_left (6))).bit_or (((l_a).bit_shift_right (26)))
			l_a := l_a + l_b
			l_d := l_d + ((((l_b).bit_xor (((l_a).bit_or ((l_c).bit_not)))) + l_x[11]) + {NATURAL_32} 3174756917)
			l_d := (((l_d).bit_shift_left (10))).bit_or (((l_d).bit_shift_right (22)))
			l_d := l_d + l_a
			l_c := l_c + ((((l_a).bit_xor (((l_d).bit_or ((l_b).bit_not)))) + l_x[2]) + {NATURAL_32} 718787259)
			l_c := (((l_c).bit_shift_left (15))).bit_or (((l_c).bit_shift_right (17)))
			l_c := l_c + l_d
			l_b := l_b + ((((l_d).bit_xor (((l_c).bit_or ((l_a).bit_not)))) + l_x[9]) + {NATURAL_32} 3951481745)
			l_b := (((l_b).bit_shift_left (21))).bit_or (((l_b).bit_shift_right (11)))
			l_b := l_b + l_c
			a_state___3[0] := a_state___3[0] + l_a
			a_state___3[1] := a_state___3[1] + l_b
			a_state___3[2] := a_state___3[2] + l_c
			a_state___3[3] := a_state___3[3] + l_d
			memset (create {CE_POINTER [ANY]}.make_cast (l_x), 0, (Natural_32_bytes.to_natural_32 * (16).to_natural_32)).do_nothing
		end

	Encode (a_output: CE_POINTER [NATURAL_8]; a_input: CE_POINTER [NATURAL_32]; a_len: NATURAL_32)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
		do
			l_i := {NATURAL_32} 0
			l_j := {NATURAL_32} 0
			from
			until
				not (l_j < a_len)
			loop
				a_output[l_j.to_integer_32] := ((a_input[l_i.to_integer_32]).bit_and ({NATURAL_32} 255)).to_natural_8
				a_output[(l_j + {NATURAL_32} 1).to_integer_32] := ((((a_input[l_i.to_integer_32]).bit_shift_right (8))).bit_and ({NATURAL_32} 255)).to_natural_8
				a_output[(l_j + {NATURAL_32} 2).to_integer_32] := ((((a_input[l_i.to_integer_32]).bit_shift_right (16))).bit_and ({NATURAL_32} 255)).to_natural_8
				a_output[(l_j + {NATURAL_32} 3).to_integer_32] := ((((a_input[l_i.to_integer_32]).bit_shift_right (24))).bit_and ({NATURAL_32} 255)).to_natural_8
				l_i := l_i + 1
				l_j := l_j + ({NATURAL_32} 4)
			end
		end

	Decode (a_output: CE_POINTER [NATURAL_32]; a_input: CE_POINTER [NATURAL_8]; a_len: NATURAL_32)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
		do
			l_i := {NATURAL_32} 0
			l_j := {NATURAL_32} 0
			from
			until
				not (l_j < a_len)
			loop
				a_output[l_i.to_integer_32] := ((((((a_input[l_j.to_integer_32]).to_natural_32).bit_or ((((a_input[(l_j + {NATURAL_32} 1).to_integer_32]).to_natural_32).bit_shift_left (8))))).bit_or ((((a_input[(l_j + {NATURAL_32} 2).to_integer_32]).to_natural_32).bit_shift_left (16))))).bit_or ((((a_input[(l_j + {NATURAL_32} 3).to_integer_32]).to_natural_32).bit_shift_left (24)))
				l_i := l_i + 1
				l_j := l_j + ({NATURAL_32} 4)
			end
		end

	Curl_md5it (a_outbuffer: CE_POINTER [NATURAL_8]; a_input: CE_POINTER [NATURAL_8])
		local
			l_ctx: PD_MD5_CTX
		do
			create l_ctx.make

			MD5_Init (l_ctx.address)
			MD5_Update (l_ctx.address, a_input, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (a_input))))
			MD5_Final (a_outbuffer, l_ctx.address)
		end

end
