class
	P_MUL_FFT

inherit
	LIBGMP_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_STDIO
	S_STDLIB
	S_SIGNAL
	S_STRING
	S_CTYPE
	S_LANGINFO
	S_LIBIO
	S_OBSTACK

feature {NONE} -- Initialization

	default_create
		do
			create mpn_fft_table3.make (<<2, 146>>)
			create fft_tab.make
			create tab.make
		end

feature {ANY} -- attributes

	mpn_fft_table3: CE_ARRAY [CE_ARRAY [PD_FFT_TABLE_NK]]

	fft_tab: CE_POINTER [PD_FFT_TABLE_NK]

	tab: CE_POINTER [PD_FFT_TABLE_NK]

feature {ANY} -- routines

	gmpn_fft_best_k (a_n: INTEGER_32; a_sqr: INTEGER_32): INTEGER_32
		local
			l_tab_n: INTEGER_32
			l_thres: INTEGER_32
			l_last_k: INTEGER_32
			js: JUMP_STATE
		do
			fft_tab.ce_gets (mpn_fft_table3[a_sqr])
			l_last_k := (fft_tab.item.k).to_integer_32
			tab.ce_gets (fft_tab + 1)
			from
			until
				js.loop_left
			loop
				js.reset
				l_tab_n := (tab.item.n).to_integer_32
				l_thres := (l_tab_n).bit_shift_left (l_last_k)
				if a_n <= l_thres then
					js.break
				end
				if not js.loop_jumped then
					l_last_k := (tab.item.k).to_integer_32
					tab.ce_gets (tab + 1)
				end
			end
			js.reset
			Result := l_last_k
		end

	gmpn_fft_next_size (a_pl: INTEGER_32; a_k: INTEGER_32): INTEGER_32
		local
			l_pl: INTEGER_32
		do
			l_pl := a_pl
			l_pl := 1 + (((l_pl - 1)).bit_shift_right (a_k))
			Result := ((l_pl).bit_shift_left (a_k))
		end

	mpn_fft_initl (a_l: CE_POINTER [CE_POINTER [INTEGER_32]]; a_k: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_K: INTEGER_32
			l_li: CE_POINTER [INTEGER_32]
			js: JUMP_STATE
		do
			create l_li.make;

			(a_l[0])[0] := 0
			l_i := 1
			l_K := 1
			from
			until
				not (not js.loop_left and then (l_i <= a_k))
			loop
				l_li.ce_gets (a_l[l_i])
				l_j := 0
				from
				until
					not (l_j < l_K)
				loop
					l_li[l_j] := 2 * (a_l[(l_i - 1)])[l_j]
					l_li[(l_K + l_j)] := 1 + l_li[l_j]
					l_j := l_j + 1
				end
				l_i := l_i + 1
				l_K := l_K * (2)
			end
		end

	mpn_fft_mul_2exp_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_d: NATURAL_32; a_n: INTEGER_32)
		local
			l_r: CE_POINTER [NATURAL_32]
			l_d: NATURAL_32
			l_sh: INTEGER_32
			l_cc: NATURAL_32
			l_rd: NATURAL_32
			l_d2: CE_POINTER [NATURAL_32]
			l_s: CE_POINTER [NATURAL_32]
			l_n: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			l_tmp___0: CE_POINTER [NATURAL_32]
			l_ptr_dummy: CE_POINTER [NATURAL_32]
			l_ptr_dummy___0: CE_POINTER [NATURAL_32]
			l_d___0: CE_POINTER [NATURAL_32]
			l_s___0: CE_POINTER [NATURAL_32]
			l_n___0: INTEGER_32
			l_tmp___1: CE_POINTER [NATURAL_32]
			l_tmp___2: CE_POINTER [NATURAL_32]
			l_tmp___3: NATURAL_32
			js: JUMP_STATE
		do
			create l_r.make
			create l_d2.make
			create l_s.make
			create l_tmp.make
			create l_tmp___0.make
			create l_ptr_dummy.make
			create l_ptr_dummy___0.make
			create l_d___0.make
			create l_s___0.make
			create l_tmp___1.make
			create l_tmp___2.make

			l_r.ce_gets (a_r)
			l_d := a_d
			l_sh := (l_d \\ {NATURAL_32} 32).to_integer_32
			l_d := l_d // ({NATURAL_32} 32)
			if l_d.to_natural_32 >= a_n.to_natural_32 then
				l_d := (l_d.to_natural_32 - a_n.to_natural_32).to_natural_32
				if l_sh /= 0 then
					gmp_inst.gmpn_lshift (l_r, (a_a + a_n) - l_d.to_integer_32, (l_d + {NATURAL_32} 1).to_integer_32, l_sh.to_natural_32).do_nothing
					l_rd := l_r[l_d.to_integer_32]
					l_cc := lshiftc_inst.gmpn_lshiftc (l_r + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_sh.to_natural_32)
				else
					from
					until
						js.loop_left
					loop
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							gmp_inst.gmpn_copyi (l_r, (a_a + a_n) - l_d.to_integer_32, l_d.to_integer_32)
							js.break
						end
						js.reset
						js.break
					end
					js.reset
					l_rd := a_a[a_n]
					from
					until
						js.loop_left
					loop
						js.reset
						l_d2.ce_gets (l_r + l_d.to_integer_32)
						l_s.ce_gets (a_a)
						l_n := (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32
						from
						until
							js.loop_left
						loop
							js.reset
							l_tmp.ce_gets (l_d2)
							l_d2.ce_gets (l_d2 + 1)
							l_tmp___0.ce_gets (l_s)
							l_s.ce_gets (l_s + 1)
							l_tmp.item := (((l_tmp___0.item).bit_not).bit_and ({NATURAL_32} 4294967295))
							l_n := l_n - 1
							if not l_n.to_boolean then
								js.break
							end
						end
						js.reset
						js.break
					end
					js.reset
					l_cc := (0).to_natural_32
				end
				l_r[a_n] := (0).to_natural_32
				l_cc := l_cc + 1
				from
				until
					js.loop_left
				loop
					js.reset
					asm95 (l_ptr_dummy.address, l_r.address, create {CE_POINTER [NATURAL_32]}.make_shared ($l_cc))
					js.break
				end
				js.reset
				l_rd := l_rd + 1
				if l_rd = {NATURAL_32} 0 then
					l_cc := (1).to_natural_32
				else
					l_cc := l_rd
				end
				l_r.ce_gets ((l_r + l_d.to_integer_32) + (l_rd = {NATURAL_32} 0).to_integer)
				from
				until
					js.loop_left
				loop
					js.reset
					asm96 (l_ptr_dummy___0.address, l_r.address, create {CE_POINTER [NATURAL_32]}.make_shared ($l_cc))
					js.break
				end
				js.reset
			else
				if l_sh /= 0 then
					lshiftc_inst.gmpn_lshiftc (l_r, (a_a + a_n) - l_d.to_integer_32, (l_d + {NATURAL_32} 1).to_integer_32, l_sh.to_natural_32).do_nothing
					l_rd := (l_r[l_d.to_integer_32]).bit_not
					l_cc := gmp_inst.gmpn_lshift (l_r + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_sh.to_natural_32)
				else
					from
					until
						js.loop_left
					loop
						js.reset
						l_d___0.ce_gets (l_r)
						l_s___0.ce_gets ((a_a + a_n) - l_d.to_integer_32)
						l_n___0 := (l_d + {NATURAL_32} 1).to_integer_32
						from
						until
							js.loop_left
						loop
							js.reset
							l_tmp___1.ce_gets (l_d___0)
							l_d___0.ce_gets (l_d___0 + 1)
							l_tmp___2.ce_gets (l_s___0)
							l_s___0.ce_gets (l_s___0 + 1)
							l_tmp___1.item := (((l_tmp___2.item).bit_not).bit_and ({NATURAL_32} 4294967295))
							l_n___0 := l_n___0 - 1
							if not l_n___0.to_boolean then
								js.break
							end
						end
						js.reset
						js.break
					end
					js.reset
					l_rd := a_a[a_n]
					from
					until
						js.loop_left
					loop
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							gmp_inst.gmpn_copyi (l_r + l_d.to_integer_32, a_a, (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32)
							js.break
						end
						js.reset
						js.break
					end
					js.reset
					l_cc := (0).to_natural_32
				end
				if l_d /= {NATURAL_32} 0 then
					l_tmp___3 := l_cc
					l_cc := l_cc - 1
					if l_tmp___3 = {NATURAL_32} 0 then
						l_cc := gmp_inst.gmpn_add_1 (l_r, l_r, a_n, (1).to_natural_32)
					end
					l_cc := (gmp_inst.gmpn_sub_1 (l_r, l_r, l_d.to_integer_32, l_cc)) + {NATURAL_32} 1
				end
				l_r[a_n] := (-(gmp_inst.gmpn_sub_1 (l_r + l_d.to_integer_32, (l_r + l_d.to_integer_32), (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_cc)).to_integer_32).to_natural_32
				l_r[a_n] := l_r[a_n] - (gmp_inst.gmpn_sub_1 (l_r + l_d.to_integer_32, (l_r + l_d.to_integer_32), (a_n.to_natural_32 - l_d.to_natural_32).to_integer_32, l_rd))
				if ((l_r[a_n]).bit_and ((({NATURAL_32} 4294967295).bit_xor ((({NATURAL_32} 4294967295).bit_shift_right (1)))))).to_boolean then
					l_r[a_n] := gmp_inst.gmpn_add_1 (l_r, l_r, a_n, (1).to_natural_32)
				end
			end
		end

	mpn_fft_add_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_b: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_c: NATURAL_32
			l_x___2: NATURAL_32
			l_ptr_dummy: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_ptr_dummy.make

			l_c := ((a_a[a_n] + a_b[a_n]) + (gmp_inst.gmpn_add_n (a_r, a_a, a_b, a_n.to_integer_32)))
			l_x___2 := ((l_c - {NATURAL_32} 1)).bit_and ((-(l_c /= {NATURAL_32} 0).to_integer).to_integer.to_natural_32)
			a_r[a_n] := l_c - l_x___2
			from
			until
				js.loop_left
			loop
				js.reset
				asm97 (l_ptr_dummy.address, a_r.address, create {CE_POINTER [NATURAL_32]}.make_shared ($l_x___2))
				js.break
			end
			js.reset
		end

	mpn_fft_sub_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_b: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_c: NATURAL_32
			l_x___2: NATURAL_32
			l_ptr_dummy: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_ptr_dummy.make

			l_c := ((a_a[a_n] - a_b[a_n]) - (gmp_inst.gmpn_sub_n (a_r, a_a, a_b, a_n.to_integer_32)))
			l_x___2 := ((-l_c.to_integer_32).to_natural_32).bit_and ((-(((l_c).bit_and ((({NATURAL_32} 4294967295).bit_xor ((({NATURAL_32} 4294967295).bit_shift_right (1)))))) /= {NATURAL_32} 0).to_integer).to_integer.to_natural_32)
			a_r[a_n] := l_x___2 + l_c
			from
			until
				js.loop_left
			loop
				js.reset
				asm98 (l_ptr_dummy.address, a_r.address, create {CE_POINTER [NATURAL_32]}.make_shared ($l_x___2))
				js.break
			end
			js.reset
		end

	mpn_fft_fft (a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_ll: CE_POINTER [CE_POINTER [INTEGER_32]]; a_omega: INTEGER_32; a_n: INTEGER_32; a_inc: INTEGER_32; a_tp: CE_POINTER [NATURAL_32])
		local
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_cy: NATURAL_32
			l_j: INTEGER_32
			l_lk: CE_POINTER [INTEGER_32]
			js: JUMP_STATE
		do
			create l_Ap.make
			create l_lk.make

			l_Ap.ce_gets (a_Ap)
			if a_K = 2 then
				from
				until
					js.loop_left
				loop
					js.reset
					from
					until
						js.loop_left
					loop
						js.reset
						gmp_inst.gmpn_copyi (a_tp, l_Ap[0], a_n + 1)
						js.break
					end
					js.reset
					js.break
				end
				js.reset
				gmp_inst.gmpn_add_n (l_Ap[0], l_Ap[0], l_Ap[a_inc], a_n + 1).do_nothing
				l_cy := gmp_inst.gmpn_sub_n (l_Ap[a_inc], a_tp, l_Ap[a_inc], a_n + 1)
				if (l_Ap[0])[a_n] > {NATURAL_32} 1 then
					(l_Ap[0])[a_n] := {NATURAL_32} 1 - (gmp_inst.gmpn_sub_1 (l_Ap[0], l_Ap[0], a_n, (l_Ap[0])[a_n] - {NATURAL_32} 1))
				end
				if l_cy.to_boolean then
					(l_Ap[a_inc])[a_n] := gmp_inst.gmpn_add_1 (l_Ap[a_inc], l_Ap[a_inc], a_n, ((l_Ap[a_inc])[a_n]).bit_not + {NATURAL_32} 1)
				end
			else
				l_lk.ce_gets (a_ll.item)
				mpn_fft_fft (l_Ap, (a_K).bit_shift_right (1), a_ll - 1, 2 * a_omega, a_n, a_inc * 2, a_tp)
				mpn_fft_fft (l_Ap + a_inc, (a_K).bit_shift_right (1), a_ll - 1, 2 * a_omega, a_n, a_inc * 2, a_tp)
				l_j := 0
				from
				until
					not (l_j.to_integer_32 < (a_K).bit_shift_right (1))
				loop
					mpn_fft_mul_2exp_modF (a_tp, l_Ap[a_inc], ((l_lk[0]).to_integer_32 * a_omega).to_natural_32, a_n)
					mpn_fft_sub_modF (l_Ap[a_inc], l_Ap[0], a_tp, a_n.to_integer_32)
					mpn_fft_add_modF (l_Ap[0], l_Ap[0], a_tp, a_n.to_integer_32)
					l_j := l_j + 1
					l_lk.ce_gets (l_lk + (2))
					l_Ap.ce_gets (l_Ap + (2 * a_inc))
				end
			end
		end

	mpn_fft_normalize (a_ap: CE_POINTER [NATURAL_32]; a_n: INTEGER_32)
		local
			l_ptr_dummy: CE_POINTER [NATURAL_32]
			l_dst: CE_POINTER [NATURAL_32]
			l_n: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_ptr_dummy.make
			create l_dst.make
			create l_tmp.make

			if a_ap[a_n] /= {NATURAL_32} 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					asm99 (l_ptr_dummy.address, a_ap.address)
					js.break
				end
				js.reset
				if a_ap[a_n] = {NATURAL_32} 0 then
					from
					until
						js.loop_left
					loop
						js.reset
						if a_n /= 0 then
							l_dst.ce_gets (a_ap)
							l_n := a_n
							from
							until
								js.loop_left
							loop
								js.reset
								l_tmp.ce_gets (l_dst)
								l_dst.ce_gets (l_dst + 1)
								l_tmp.item := (0).to_natural_32
								l_n := l_n - 1
								if not l_n.to_boolean then
									js.break
								end
							end
							js.reset
						end
						js.break
					end
					js.reset
					a_ap[a_n] := (1).to_natural_32
				else
					a_ap[a_n] := (0).to_natural_32
				end
			end
		end

	mpn_fft_mul_modF_K (a_ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_bp: CE_POINTER [CE_POINTER [NATURAL_32]]; a_n: INTEGER_32; a_K: INTEGER_32)
		local
			l_ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_bp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_i: INTEGER_32
			l_sqr: INTEGER_32
			l_tmp_marker: CE_POINTER [PD_TMP_REENTRANT_T]
			l_k: INTEGER_32
			l_K2: INTEGER_32
			l_nprime2: INTEGER_32
			l_Nprime22: INTEGER_32
			l_M2: INTEGER_32
			l_maxLK: INTEGER_32
			l_l: INTEGER_32
			l_Mp2: INTEGER_32
			l_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]
			l_Ap2: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_Bp2: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_A: CE_POINTER [NATURAL_32]
			l_B: CE_POINTER [NATURAL_32]
			l_T: CE_POINTER [NATURAL_32]
			l_K3: NATURAL_32
			l_tmp___0: INTEGER_32
			l_tmp___4: CE_POINTER [ANY]
			l_tmp___9: CE_POINTER [ANY]
			l_tmp___14: CE_POINTER [ANY]
			l_tmp___19: CE_POINTER [ANY]
			l_tmp___24: CE_POINTER [ANY]
			l_tmp___29: CE_POINTER [ANY]
			l_cy: NATURAL_32
			l_a2: CE_POINTER [NATURAL_32]
			l_b2: CE_POINTER [NATURAL_32]
			l_tp: CE_POINTER [NATURAL_32]
			l_tpn: CE_POINTER [NATURAL_32]
			l_cc: NATURAL_32
			l_n2: INTEGER_32
			l_tmp___34: CE_POINTER [ANY]
			l_tmp___36: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_tmp___37: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_tmp___41: INTEGER_32
			l_tmp___42: INTEGER_32
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_ap.make
			create l_bp.make
			create l_tmp_marker.make
			create l_fft_l.make
			create l_Ap2.make
			create l_Bp2.make
			create l_A.make
			create l_B.make
			create l_T.make
			create l_tmp___4.make
			create l_tmp___9.make
			create l_tmp___14.make
			create l_tmp___19.make
			create l_tmp___24.make
			create l_tmp___29.make
			create l_a2.make
			create l_b2.make
			create l_tp.make
			create l_tpn.make
			create l_tmp___34.make
			create l_tmp___36.make
			create l_tmp___37.make

			l_ap.ce_gets (a_ap)
			l_bp.ce_gets (a_bp)
			l_sqr := ((l_ap.c_pointer.to_integer_32).to_natural_32 = (l_bp.c_pointer.to_integer_32).to_natural_32).to_integer
			l_tmp_marker.ce_gets (create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer))
			if l_sqr.to_boolean then
				l_tmp___42 := 472
			else
				l_tmp___42 := 565
			end
			if a_n >= l_tmp___42.to_integer_32 then
				l_k := gmpn_fft_best_k (a_n, l_sqr)
				l_K2 := (1).bit_shift_left (l_k)
				from
				until
					js.loop_left
				loop
					js.reset
					if ((a_n).bit_and ((l_K2 - 1).to_integer_32)) /= 0 then
						assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 437, ce_string ("(n & (K2 - 1)) == 0"))
					end
					js.break
				end
				js.reset
				if l_K2 > 32 then
					l_maxLK := l_K2
				else
					l_maxLK := 32
				end
				l_M2 := ((a_n * 32).bit_shift_right (l_k)).to_integer_32
				l_l := ((a_n).bit_shift_right (l_k)).to_integer_32
				l_Nprime22 := ((((2 * l_M2 + l_k) + 2) + l_maxLK) // l_maxLK) * l_maxLK
				l_nprime2 := l_Nprime22 // 32
				if l_sqr.to_boolean then
					l_tmp___0 := 472
				else
					l_tmp___0 := 565
				end
				if l_nprime2 >= l_tmp___0 then
					from
					until
						js.loop_left
					loop
						js.reset
						l_K3 := ((1).bit_shift_left ((gmpn_fft_best_k (l_nprime2.to_integer_32, l_sqr)))).to_natural_32
						if ((l_nprime2.to_natural_32).bit_and ((l_K3 - {NATURAL_32} 1))) = {NATURAL_32} 0 then
							js.break
						end
						if not js.loop_jumped then
							l_nprime2 := ((((l_nprime2.to_natural_32 + l_K3) - {NATURAL_32} 1)).bit_and ((-l_K3.to_integer_32).to_natural_32)).to_integer_32
							l_Nprime22 := l_nprime2 * 32
						end
					end
					js.reset
				end
				from
				until
					js.loop_left
				loop
					js.reset
					if not (l_nprime2.to_integer_32 < a_n) then
						assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 459, ce_string ("nprime2 < n"))
					end
					js.break
				end
				js.reset
				l_Mp2 := (l_Nprime22).bit_shift_right (l_k)
				if (builtin_expect ((l_K2.to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___4.ce_gets ((l_aaf.new (l_K2.to_natural_32 * Pointer_bytes.to_natural_32)))
				else
					l_tmp___4.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_K2.to_natural_32 * Pointer_bytes.to_natural_32)))
				end
				l_Ap2.ce_gets (create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___4))
				if (builtin_expect ((l_K2.to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___9.ce_gets ((l_aaf.new (l_K2.to_natural_32 * Pointer_bytes.to_natural_32)))
				else
					l_tmp___9.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_K2.to_natural_32 * Pointer_bytes.to_natural_32)))
				end
				l_Bp2.ce_gets (create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___9))
				if (builtin_expect ((((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * Natural_32_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___14.ce_gets ((l_aaf.new (((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * Natural_32_bytes.to_natural_32)))
				else
					l_tmp___14.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((2 * (l_nprime2 + 1)).bit_shift_left (l_k)).to_natural_32 * Natural_32_bytes.to_natural_32)))
				end
				l_A.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___14))
				if (builtin_expect (((2 * (l_nprime2 + 1)).to_natural_32 * Natural_32_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___19.ce_gets ((l_aaf.new ((2 * (l_nprime2 + 1)).to_natural_32 * Natural_32_bytes.to_natural_32)))
				else
					l_tmp___19.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, (2 * (l_nprime2 + 1)).to_natural_32 * Natural_32_bytes.to_natural_32)))
				end
				l_T.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___19))
				l_B.ce_gets (l_A + (((l_nprime2 + 1)).bit_shift_left (l_k)))
				if (builtin_expect (((l_k + 1).to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___24.ce_gets ((l_aaf.new ((l_k + 1).to_natural_32 * Pointer_bytes.to_natural_32)))
				else
					l_tmp___24.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, (l_k + 1).to_natural_32 * Pointer_bytes.to_natural_32)))
				end
				l_fft_l.ce_gets (create {CE_POINTER [CE_POINTER [INTEGER_32]]}.make_cast (l_tmp___24))
				l_i := 0
				from
				until
					not (l_i <= l_k)
				loop
					if (builtin_expect ((((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
						l_tmp___29.ce_gets ((l_aaf.new (((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32)))
					else
						l_tmp___29.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32)))
					end
					l_fft_l[l_i] := create {CE_POINTER [INTEGER_32]}.make_cast (l_tmp___29)
					l_i := l_i + 1
				end
				mpn_fft_initl (l_fft_l, l_k)
				l_i := 0
				from
				until
					not (l_i < a_K)
				loop
					mpn_fft_normalize (l_ap.item, a_n)
					if not l_sqr.to_boolean then
						mpn_fft_normalize (l_bp.item, a_n)
					end
					mpn_mul_fft_decompose (l_A, l_Ap2, l_K2, l_nprime2, l_ap.item, (((l_l).bit_shift_left (l_k)) + 1).to_integer_32, l_l, l_Mp2, l_T)
					if not l_sqr.to_boolean then
						mpn_mul_fft_decompose (l_B, l_Bp2, l_K2, l_nprime2, l_bp.item, (((l_l).bit_shift_left (l_k)) + 1).to_integer_32, l_l, l_Mp2, l_T)
					end
					l_cy := mpn_mul_fft_internal (l_ap.item, a_n, l_k, l_Ap2, l_Bp2, l_A, l_B, l_nprime2.to_integer_32, l_l.to_integer_32, l_Mp2.to_integer_32, l_fft_l, l_T, l_sqr)
					l_ap.item[a_n] := l_cy
					l_i := l_i + 1
					l_ap.ce_gets (l_ap + 1)
					l_bp.ce_gets (l_bp + 1)
				end
			else
				l_n2 := (2 * a_n).to_integer_32
				if (builtin_expect ((l_n2.to_natural_32 * Natural_32_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___34.ce_gets ((l_aaf.new (l_n2.to_natural_32 * Natural_32_bytes.to_natural_32)))
				else
					l_tmp___34.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_n2.to_natural_32 * Natural_32_bytes.to_natural_32)))
				end
				l_tp.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___34))
				l_tpn.ce_gets (l_tp + a_n)
				l_i := 0
				from
				until
					not (not js.loop_left and then (l_i < a_K))
				loop
					l_tmp___36.ce_gets (l_ap)
					l_ap.ce_gets (l_ap + 1)
					l_a2.ce_gets (l_tmp___36.item)
					l_tmp___37.ce_gets (l_bp)
					l_bp.ce_gets (l_bp + 1)
					l_b2.ce_gets (l_tmp___37.item)
					if l_sqr.to_boolean then
						sqr_inst.gmpn_sqr (l_tp, l_a2, a_n)
					else
						mul_n_inst.gmpn_mul_n (l_tp, l_b2, l_a2, a_n)
					end
					if l_a2[a_n] /= {NATURAL_32} 0 then
						l_cc := gmp_inst.gmpn_add_n (l_tpn, l_tpn, l_b2, a_n)
					else
						l_cc := (0).to_natural_32
					end
					if l_b2[a_n] /= {NATURAL_32} 0 then
						l_cc := l_cc + ((gmp_inst.gmpn_add_n (l_tpn, l_tpn, l_a2, a_n)) + l_a2[a_n])
					end
					if l_cc /= {NATURAL_32} 0 then
						l_cc := gmp_inst.gmpn_add_1 (l_tp, l_tp, l_n2.to_integer_32, l_cc)
					end
					if (gmp_inst.gmpn_sub_n (l_a2, l_tp, l_tpn, a_n)).to_boolean then
						if (gmp_inst.gmpn_add_1 (l_a2, l_a2, a_n, (1).to_natural_32)).to_boolean then
							l_tmp___41 := 1
						else
							l_tmp___41 := 0
						end
					else
						l_tmp___41 := 0
					end
					l_a2[a_n] := l_tmp___41.to_natural_32
					l_i := l_i + 1
				end
			end
			from
			until
				js.loop_left
			loop
				js.reset
				if (builtin_expect (((l_tmp_marker.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l_tmp_marker)
				end
				js.break
			end
			js.reset
		end

	mpn_fft_fftinv (a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_omega: INTEGER_32; a_n: INTEGER_32; a_tp: CE_POINTER [NATURAL_32])
		local
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_cy: NATURAL_32
			l_j: INTEGER_32
			l_K2: INTEGER_32
			js: JUMP_STATE
		do
			create l_Ap.make

			l_Ap.ce_gets (a_Ap)
			if a_K = 2 then
				from
				until
					js.loop_left
				loop
					js.reset
					from
					until
						js.loop_left
					loop
						js.reset
						gmp_inst.gmpn_copyi (a_tp, l_Ap[0], a_n + 1)
						js.break
					end
					js.reset
					js.break
				end
				js.reset
				gmp_inst.gmpn_add_n (l_Ap[0], l_Ap[0], l_Ap[1], a_n + 1).do_nothing
				l_cy := gmp_inst.gmpn_sub_n (l_Ap[1], a_tp, l_Ap[1], a_n + 1)
				if (l_Ap[0])[a_n] > {NATURAL_32} 1 then
					(l_Ap[0])[a_n] := {NATURAL_32} 1 - (gmp_inst.gmpn_sub_1 (l_Ap[0], l_Ap[0], a_n, (l_Ap[0])[a_n] - {NATURAL_32} 1))
				end
				if l_cy.to_boolean then
					(l_Ap[1])[a_n] := gmp_inst.gmpn_add_1 (l_Ap[1], l_Ap[1], a_n, ((l_Ap[1])[a_n]).bit_not + {NATURAL_32} 1)
				end
			else
				l_K2 := (a_K).bit_shift_right (1)
				mpn_fft_fftinv (l_Ap, l_K2, 2 * a_omega, a_n, a_tp)
				mpn_fft_fftinv (l_Ap + l_K2, l_K2, 2 * a_omega, a_n, a_tp)
				l_j := 0
				from
				until
					not (l_j < l_K2)
				loop
					mpn_fft_mul_2exp_modF (a_tp, l_Ap[l_K2], (l_j.to_integer_32 * a_omega).to_natural_32, a_n)
					mpn_fft_sub_modF (l_Ap[l_K2], l_Ap[0], a_tp, a_n.to_integer_32)
					mpn_fft_add_modF (l_Ap[0], l_Ap[0], a_tp, a_n.to_integer_32)
					l_j := l_j + 1
					l_Ap.ce_gets (l_Ap + 1)
				end
			end
		end

	mpn_fft_div_2exp_modF (a_r: CE_POINTER [NATURAL_32]; a_a: CE_POINTER [NATURAL_32]; a_k: INTEGER_32; a_n: INTEGER_32)
		local
			l_i: INTEGER_32
		do
			l_i := ((2 * a_n) * 32 - a_k.to_integer_32).to_integer_32
			mpn_fft_mul_2exp_modF (a_r, a_a, l_i.to_natural_32, a_n)
			mpn_fft_normalize (a_r, a_n)
		end

	mpn_fft_norm_modF (a_rp: CE_POINTER [NATURAL_32]; a_n: INTEGER_32; a_ap: CE_POINTER [NATURAL_32]; a_an: INTEGER_32): INTEGER_32
		local
			l_l: INTEGER_32
			l_m: INTEGER_32
			l_cc: NATURAL_32
			l_rpn: INTEGER_32
			js: JUMP_STATE
		do
			l_m := a_an - 2 * a_n
			if l_m > 0 then
				l_l := a_n
				l_cc := gmp_inst.gmpn_add_n (a_rp, a_ap, (a_ap + 2 * a_n), l_m)
				l_rpn := (gmp_inst.gmpn_add_1 (a_rp + l_m, (a_ap + l_m), a_n - l_m, l_cc)).to_integer_32
			else
				l_l := a_an - a_n
				from
				until
					js.loop_left
				loop
					js.reset
					from
					until
						js.loop_left
					loop
						js.reset
						gmp_inst.gmpn_copyi (a_rp, a_ap, a_n)
						js.break
					end
					js.reset
					js.break
				end
				js.reset
				l_rpn := 0
			end
			l_cc := gmp_inst.gmpn_sub_n (a_rp, a_rp, (a_ap + a_n), l_l)
			l_rpn := (l_rpn.to_natural_32 - (gmp_inst.gmpn_sub_1 (a_rp + l_l, (a_rp + l_l), a_n - l_l, l_cc))).to_integer_32
			if l_rpn < 0 then
				l_rpn := (gmp_inst.gmpn_add_1 (a_rp, a_rp, a_n, (1).to_natural_32)).to_integer_32
			end
			Result := l_rpn
		end

	mpn_mul_fft_decompose (a_A: CE_POINTER [NATURAL_32]; a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_K: INTEGER_32; a_nprime: INTEGER_32; a_n: CE_POINTER [NATURAL_32]; a_nl: INTEGER_32; a_l: INTEGER_32; a_Mp: INTEGER_32; a_T: CE_POINTER [NATURAL_32])
		local
			l_A: CE_POINTER [NATURAL_32]
			l_n: CE_POINTER [NATURAL_32]
			l_nl: INTEGER_32
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_tmp: CE_POINTER [NATURAL_32]
			l_Kl: INTEGER_32
			l_tmp_marker: CE_POINTER [PD_TMP_REENTRANT_T]
			l_dif: INTEGER_32
			l_cy: INTEGER_32
			l_tmp___3: CE_POINTER [ANY]
			l_subp: INTEGER_32
			l_dst: CE_POINTER [NATURAL_32]
			l_n2: INTEGER_32
			l_tmp___14: CE_POINTER [NATURAL_32]
			l_dst___0: CE_POINTER [NATURAL_32]
			l_n___0: INTEGER_32
			l_tmp___15: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_A.make
			create l_n.make
			create l_tmp.make
			create l_tmp_marker.make
			create l_tmp___3.make
			create l_dst.make
			create l_tmp___14.make
			create l_dst___0.make
			create l_tmp___15.make

			l_A.ce_gets (a_A)
			l_n.ce_gets (a_n)
			l_nl := a_nl
			l_Kl := (a_K * a_l).to_integer_32
			l_tmp_marker.ce_gets (create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer))
			if l_nl > l_Kl then
				l_dif := l_nl - l_Kl
				if (builtin_expect (((l_Kl + 1).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___3.ce_gets ((l_aaf.new (((l_Kl + 1).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				else
					l_tmp___3.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((l_Kl + 1).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				end
				l_tmp.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___3))
				if l_dif > l_Kl then
					l_subp := 0
					l_cy := (gmp_inst.gmpn_sub_n (l_tmp, l_n, l_n + l_Kl, l_Kl)).to_integer_32
					l_n.ce_gets (l_n + (2 * l_Kl))
					l_dif := l_dif - l_Kl
					from
					until
						not (l_dif > l_Kl)
					loop
						if l_subp.to_boolean then
							l_cy := (l_cy.to_natural_32 + (gmp_inst.gmpn_sub_n (l_tmp, l_tmp, l_n, l_Kl))).to_integer_32
						else
							l_cy := (l_cy.to_natural_32 - (gmp_inst.gmpn_add_n (l_tmp, l_tmp, l_n, l_Kl))).to_integer_32
						end
						l_subp := (l_subp).bit_xor ((1))
						l_n.ce_gets (l_n + l_Kl)
						l_dif := l_dif - l_Kl
					end
					if l_subp.to_boolean then
						l_cy := (l_cy.to_natural_32 + (gmp_inst.gmpn_sub (l_tmp, l_tmp, l_Kl, l_n, l_dif))).to_integer_32
					else
						l_cy := (l_cy.to_natural_32 - (gmp_inst.gmpn_add (l_tmp, l_tmp, l_Kl, l_n, l_dif))).to_integer_32
					end
					if l_cy >= 0 then
						l_cy := (gmp_inst.gmpn_add_1 (l_tmp, l_tmp, l_Kl, l_cy.to_natural_32)).to_integer_32
					else
						l_cy := (gmp_inst.gmpn_sub_1 (l_tmp, l_tmp, l_Kl, (-l_cy).to_natural_32)).to_integer_32
					end
				else
					l_cy := (gmp_inst.gmpn_sub (l_tmp, l_n, l_Kl, l_n + l_Kl, l_dif)).to_integer_32
					l_cy := (gmp_inst.gmpn_add_1 (l_tmp, l_tmp, l_Kl, l_cy.to_natural_32)).to_integer_32
				end
				l_tmp[l_Kl] := l_cy.to_natural_32
				l_nl := l_Kl + 1
				l_n.ce_gets (l_tmp)
			end
			l_i := 0
			from
			until
				not (not js.loop_left and then (l_i < a_K))
			loop
				a_Ap[l_i] := l_A
				if l_nl > 0 then
					if a_l.to_integer_32 <= l_nl then
						if l_i < a_K - 1 then
							l_j := a_l
						else
							l_j := l_nl.to_integer_32
						end
					else
						l_j := l_nl.to_integer_32
					end
					l_nl := l_nl - l_j.to_integer_32
					from
					until
						js.loop_left
					loop
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							gmp_inst.gmpn_copyi (a_T, l_n, l_j.to_integer_32)
							js.break
						end
						js.reset
						js.break
					end
					js.reset
					from
					until
						js.loop_left
					loop
						js.reset
						if (a_nprime + 1) - l_j /= 0 then
							l_dst.ce_gets (a_T + l_j)
							l_n2 := ((a_nprime + 1) - l_j).to_integer_32
							from
							until
								js.loop_left
							loop
								js.reset
								l_tmp___14.ce_gets (l_dst)
								l_dst.ce_gets (l_dst + 1)
								l_tmp___14.item := (0).to_natural_32
								l_n2 := l_n2 - 1
								if not l_n2.to_boolean then
									js.break
								end
							end
							js.reset
						end
						js.break
					end
					js.reset
					l_n.ce_gets (l_n + a_l)
					mpn_fft_mul_2exp_modF (l_A, a_T, (l_i * a_Mp).to_natural_32, a_nprime.to_integer_32)
				else
					from
					until
						js.loop_left
					loop
						js.reset
						if a_nprime + 1 /= 0 then
							l_dst___0.ce_gets (l_A)
							l_n___0 := (a_nprime + 1).to_integer_32
							from
							until
								js.loop_left
							loop
								js.reset
								l_tmp___15.ce_gets (l_dst___0)
								l_dst___0.ce_gets (l_dst___0 + 1)
								l_tmp___15.item := (0).to_natural_32
								l_n___0 := l_n___0 - 1
								if not l_n___0.to_boolean then
									js.break
								end
							end
							js.reset
						end
						js.break
					end
					js.reset
				end
				l_A.ce_gets (l_A + (a_nprime + 1))
				l_i := l_i + 1
			end
			from
			until
				js.loop_left
			loop
				js.reset
				if l_nl /= 0 then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 699, ce_string ("nl == 0"))
				end
				js.break
			end
			js.reset
			from
			until
				js.loop_left
			loop
				js.reset
				if (builtin_expect (((l_tmp_marker.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l_tmp_marker)
				end
				js.break
			end
			js.reset
		end

	mpn_mul_fft_internal (a_op: CE_POINTER [NATURAL_32]; a_pl: INTEGER_32; a_k: INTEGER_32; a_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]; a_Bp: CE_POINTER [CE_POINTER [NATURAL_32]]; a_A: CE_POINTER [NATURAL_32]; a_B: CE_POINTER [NATURAL_32]; a_nprime: INTEGER_32; a_l: INTEGER_32; a_Mp: INTEGER_32; a_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]; a_T: CE_POINTER [NATURAL_32]; a_sqr: INTEGER_32): NATURAL_32
		local
			l_K: INTEGER_32
			l_i: INTEGER_32
			l_pla: INTEGER_32
			l_lo: INTEGER_32
			l_sh: INTEGER_32
			l_j: INTEGER_32
			l_p: CE_POINTER [NATURAL_32]
			l_cc: NATURAL_32
			l_tmp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_dst: CE_POINTER [NATURAL_32]
			l_n: INTEGER_32
			l_tmp___0: CE_POINTER [NATURAL_32]
			l_dst___0: CE_POINTER [NATURAL_32]
			l_n___0: INTEGER_32
			l_tmp___1: CE_POINTER [NATURAL_32]
			l_n2: CE_POINTER [NATURAL_32]
			js: JUMP_STATE
		do
			create l_p.make
			create l_tmp.make
			create l_dst.make
			create l_tmp___0.make
			create l_dst___0.make
			create l_tmp___1.make
			create l_n2.make

			l_K := (1).bit_shift_left (a_k)
			mpn_fft_fft (a_Ap, l_K.to_integer_32, a_fft_l + a_k, 2 * a_Mp, a_nprime, (1).to_integer_32, a_T)
			if not a_sqr.to_boolean then
				mpn_fft_fft (a_Bp, l_K.to_integer_32, a_fft_l + a_k, 2 * a_Mp, a_nprime, (1).to_integer_32, a_T)
			end
			if a_sqr.to_boolean then
				l_tmp.ce_gets (a_Ap)
			else
				l_tmp.ce_gets (a_Bp)
			end
			mpn_fft_mul_modF_K (a_Ap, l_tmp, a_nprime, l_K)
			mpn_fft_fftinv (a_Ap, l_K, 2 * a_Mp, a_nprime, a_T)
			a_Bp[0] := (a_T + a_nprime) + 1
			mpn_fft_div_2exp_modF (a_Bp[0], a_Ap[0], a_k, a_nprime)
			l_i := 1
			from
			until
				not (l_i < l_K)
			loop
				a_Bp[l_i] := a_Ap[(l_i - 1)]
				mpn_fft_div_2exp_modF (a_Bp[l_i], a_Ap[l_i], (a_k.to_integer_32 + (l_K - l_i).to_integer_32 * a_Mp).to_integer_32, a_nprime)
				l_i := l_i + 1
			end
			from
			until
				js.loop_left
			loop
				js.reset
				if a_nprime + 1 /= 0 then
					l_dst.ce_gets (a_T)
					l_n := a_nprime + 1
					from
					until
						js.loop_left
					loop
						js.reset
						l_tmp___0.ce_gets (l_dst)
						l_dst.ce_gets (l_dst + 1)
						l_tmp___0.item := (0).to_natural_32
						l_n := l_n - 1
						if not l_n.to_boolean then
							js.break
						end
					end
					js.reset
				end
				js.break
			end
			js.reset
			l_pla := ((a_l * (l_K - 1).to_integer_32 + a_nprime) + 1).to_integer_32
			l_p.ce_gets (a_B)
			from
			until
				js.loop_left
			loop
				js.reset
				if l_pla /= 0 then
					l_dst___0.ce_gets (l_p)
					l_n___0 := l_pla.to_integer_32
					from
					until
						js.loop_left
					loop
						js.reset
						l_tmp___1.ce_gets (l_dst___0)
						l_dst___0.ce_gets (l_dst___0 + 1)
						l_tmp___1.item := (0).to_natural_32
						l_n___0 := l_n___0 - 1
						if not l_n___0.to_boolean then
							js.break
						end
					end
					js.reset
				end
				js.break
			end
			js.reset
			l_cc := (0).to_natural_32
			l_i := l_K - 1
			l_lo := (a_l * l_i.to_integer_32 + a_nprime).to_integer_32
			l_sh := (a_l * l_i.to_integer_32).to_integer_32
			from
			until
				not (l_i >= 0)
			loop
				l_n2.ce_gets (l_p + l_sh)
				l_j := ((l_K - l_i)).bit_and ((l_K - 1))
				if (gmp_inst.gmpn_add_n (l_n2, l_n2, a_Bp[l_j], a_nprime + 1)).to_boolean then
					l_cc := l_cc + (gmp_inst.gmpn_add_1 ((l_n2 + a_nprime) + 1, ((l_n2 + a_nprime) + 1), ((l_pla - l_sh).to_integer_32 - a_nprime) - 1, (1).to_natural_32))
				end
				a_T[2 * a_l] := (l_i + 1).to_natural_32
				if (gmp_inst.gmpn_cmp (a_Bp[l_j], a_T, a_nprime + 1)) > 0 then
					l_cc := l_cc - (gmp_inst.gmpn_sub_1 (l_n2, l_n2, (l_pla - l_sh).to_integer_32, (1).to_natural_32))
					l_cc := l_cc - (gmp_inst.gmpn_sub_1 (l_p + l_lo, (l_p + l_lo), (l_pla - l_lo).to_integer_32, (1).to_natural_32))
				end
				l_i := l_i - 1
				l_lo := (l_lo.to_integer_32 - a_l).to_integer_32
				l_sh := (l_sh.to_integer_32 - a_l).to_integer_32
			end
			if l_cc = {NATURAL_32} 4294967295 then
				l_cc := gmp_inst.gmpn_add_1 ((l_p + l_pla) - a_pl, ((l_p + l_pla) - a_pl), a_pl, (1).to_natural_32)
				if l_cc.to_boolean then
					gmp_inst.gmpn_sub_1 (((l_p + l_pla) - a_pl) - 1, (((l_p + l_pla) - a_pl) - 1), a_pl + 1, (1).to_natural_32).do_nothing
					gmp_inst.gmpn_sub_1 ((l_p + l_pla) - 1, ((l_p + l_pla) - 1), (1).to_integer_32, (1).to_natural_32).do_nothing
				end
			elseif l_cc = {NATURAL_32} 1 then
				if l_pla.to_integer_32 >= 2 * a_pl then
					from
					until
						js.loop_left
					loop
						js.reset
						l_cc := gmp_inst.gmpn_add_1 ((l_p + l_pla) - 2 * a_pl, ((l_p + l_pla) - 2 * a_pl), 2 * a_pl, l_cc)
						if not l_cc.to_boolean then
							js.break
						end
					end
					js.reset
				else
					l_cc := gmp_inst.gmpn_sub_1 ((l_p + l_pla) - a_pl, ((l_p + l_pla) - a_pl), a_pl, l_cc)
				end
			else
			end
			Result := (mpn_fft_norm_modF (a_op, a_pl, l_p, l_pla.to_integer_32)).to_natural_32
		end

	mpn_mul_fft_lcm (a_a: NATURAL_32; a_k: NATURAL_32): NATURAL_32
		local
			l_a: NATURAL_32
			l_k: NATURAL_32
			l_l: NATURAL_32
			js: JUMP_STATE
		do
			l_a := a_a
			l_k := a_k
			l_l := l_k.to_natural_32
			from
			until
				js.loop_left
			loop
				js.reset
				if l_a \\ {NATURAL_32} 2 = {NATURAL_32} 0 then
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					l_a := (l_a).bit_shift_right ((1))
					l_k := l_k - 1
				end
			end
			js.reset
			Result := ((l_a).bit_shift_left (l_l.to_integer_32))
		end

	gmpn_mul_fft (a_op: CE_POINTER [NATURAL_32]; a_pl: INTEGER_32; a_n: CE_POINTER [NATURAL_32]; a_nl: INTEGER_32; a_m: CE_POINTER [NATURAL_32]; a_ml: INTEGER_32; a_k: INTEGER_32): NATURAL_32
		local
			l_K: INTEGER_32
			l_maxLK: INTEGER_32
			l_i: INTEGER_32
			l_N: INTEGER_32
			l_Nprime: INTEGER_32
			l_nprime2: INTEGER_32
			l_M: INTEGER_32
			l_Mp: INTEGER_32
			l_l: INTEGER_32
			l_Ap: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_Bp: CE_POINTER [CE_POINTER [NATURAL_32]]
			l_A: CE_POINTER [NATURAL_32]
			l_T: CE_POINTER [NATURAL_32]
			l_B: CE_POINTER [NATURAL_32]
			l_fft_l: CE_POINTER [CE_POINTER [INTEGER_32]]
			l_sqr: INTEGER_32
			l_tmp: INTEGER_32
			l_h: NATURAL_32
			l_tmp_marker: CE_POINTER [PD_TMP_REENTRANT_T]
			l_tmp___4: CE_POINTER [ANY]
			l_tmp___9: CE_POINTER [ANY]
			l_K2: NATURAL_32
			l_tmp___13: INTEGER_32
			l_tmp___17: CE_POINTER [ANY]
			l_tmp___22: CE_POINTER [ANY]
			l_tmp___27: CE_POINTER [ANY]
			l_pla: INTEGER_32
			l_tmp___32: CE_POINTER [ANY]
			l_tmp___37: CE_POINTER [ANY]
			l_tmp___42: CE_POINTER [ANY]
			l_tmp___47: CE_POINTER [ANY]
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_Ap.make
			create l_Bp.make
			create l_A.make
			create l_T.make
			create l_B.make
			create l_fft_l.make
			create l_tmp_marker.make
			create l_tmp___4.make
			create l_tmp___9.make
			create l_tmp___17.make
			create l_tmp___22.make
			create l_tmp___27.make
			create l_tmp___32.make
			create l_tmp___37.make
			create l_tmp___42.make
			create l_tmp___47.make

			if (a_n.c_pointer.to_integer_32).to_natural_32 = (a_m.c_pointer.to_integer_32).to_natural_32 then
				if a_nl = a_ml then
					l_tmp := 1
				else
					l_tmp := 0
				end
			else
				l_tmp := 0
			end
			l_sqr := l_tmp
			from
			until
				js.loop_left
			loop
				js.reset
				if (gmpn_fft_next_size (a_pl, a_k)) /= a_pl then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 824, ce_string ("__gmpn_fft_next_size (pl, k) == pl"))
				end
				js.break
			end
			js.reset
			l_tmp_marker.ce_gets (create {CE_POINTER [PD_TMP_REENTRANT_T]}.make_shared (default_pointer))
			l_N := a_pl * 32
			if (builtin_expect (((a_k + 1).to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
				l_tmp___4.ce_gets ((l_aaf.new ((a_k + 1).to_natural_32 * Pointer_bytes.to_natural_32)))
			else
				l_tmp___4.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, (a_k + 1).to_natural_32 * Pointer_bytes.to_natural_32)))
			end
			l_fft_l.ce_gets (create {CE_POINTER [CE_POINTER [INTEGER_32]]}.make_cast (l_tmp___4))
			l_i := 0
			from
			until
				not (l_i <= a_k)
			loop
				if (builtin_expect ((((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___9.ce_gets ((l_aaf.new (((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32)))
				else
					l_tmp___9.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((1).bit_shift_left (l_i)).to_natural_32 * Integer_32_bytes.to_natural_32)))
				end
				l_fft_l[l_i] := create {CE_POINTER [INTEGER_32]}.make_cast (l_tmp___9)
				l_i := l_i + 1
			end
			mpn_fft_initl (l_fft_l, a_k)
			l_K := (1).bit_shift_left (a_k)
			l_M := (l_N).bit_shift_right (a_k)
			l_l := 1 + (l_M - 1) // 32
			l_maxLK := (mpn_mul_fft_lcm ({NATURAL_32} 32, a_k.to_natural_32)).to_integer_32
			l_Nprime := (1 + ((2 * l_M + a_k.to_integer_32) + 2) // l_maxLK.to_integer_32) * l_maxLK.to_integer_32
			l_nprime2 := l_Nprime // 32
			if l_sqr.to_boolean then
				l_tmp___13 := 472
			else
				l_tmp___13 := 565
			end
			if l_nprime2 >= l_tmp___13.to_integer_32 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_K2 := ((1).bit_shift_left ((gmpn_fft_best_k (l_nprime2, l_sqr)))).to_natural_32
					if ((l_nprime2.to_natural_32).bit_and ((l_K2 - {NATURAL_32} 1))) = {NATURAL_32} 0 then
						js.break
					end
					if not js.loop_jumped then
						l_nprime2 := ((((l_nprime2.to_natural_32 + l_K2) - {NATURAL_32} 1)).bit_and ((-l_K2.to_integer_32).to_natural_32)).to_integer_32
						l_Nprime := l_nprime2 * 32
					end
				end
				js.reset
			end
			from
			until
				js.loop_left
			loop
				js.reset
				if not (l_nprime2 < a_pl) then
					assert_inst.gmp_assert_fail (ce_string ("mul_fft.c"), 857, ce_string ("nprime < pl"))
				end
				js.break
			end
			js.reset
			if (builtin_expect (((2 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
				l_tmp___17.ce_gets ((l_aaf.new (((2 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
			else
				l_tmp___17.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((2 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
			end
			l_T.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___17))
			l_Mp := (l_Nprime).bit_shift_right (a_k)
			if (builtin_expect (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
				l_tmp___22.ce_gets ((l_aaf.new (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
			else
				l_tmp___22.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
			end
			l_A.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___22))
			if (builtin_expect ((l_K.to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
				l_tmp___27.ce_gets ((l_aaf.new (l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
			else
				l_tmp___27.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
			end
			l_Ap.ce_gets (create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___27))
			mpn_mul_fft_decompose (l_A, l_Ap, l_K, l_nprime2.to_integer_32, a_n, a_nl, l_l.to_integer_32, l_Mp.to_integer_32, l_T)
			if l_sqr.to_boolean then
				l_pla := (l_l * (l_K - 1).to_integer_32 + l_nprime2) + 1
				if (builtin_expect ((l_pla.to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___32.ce_gets ((l_aaf.new ((l_pla.to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				else
					l_tmp___32.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, (l_pla.to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				end
				l_B.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___32))
				if (builtin_expect ((l_K.to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___37.ce_gets ((l_aaf.new (l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
				else
					l_tmp___37.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
				end
				l_Bp.ce_gets (create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___37))
			else
				if (builtin_expect (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___42.ce_gets ((l_aaf.new (((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				else
					l_tmp___42.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, ((l_K.to_integer_32 * (l_nprime2 + 1)).to_natural_32 * (Natural_32_bytes.to_natural_32).to_natural_32).to_natural_32)))
				end
				l_B.ce_gets (create {CE_POINTER [NATURAL_32]}.make_cast (l_tmp___42))
				if (builtin_expect ((l_K.to_natural_32 * Pointer_bytes.to_natural_32 < {NATURAL_32} 65536).to_integer, 1)).to_boolean then
					l_tmp___47.ce_gets ((l_aaf.new (l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
				else
					l_tmp___47.ce_gets ((tal_reent_inst.gmp_tmp_reentrant_alloc (l_tmp_marker.address, l_K.to_natural_32 * Pointer_bytes.to_natural_32)))
				end
				l_Bp.ce_gets (create {CE_POINTER [CE_POINTER [NATURAL_32]]}.make_cast (l_tmp___47))
				mpn_mul_fft_decompose (l_B, l_Bp, l_K, l_nprime2.to_integer_32, a_m, a_ml, l_l.to_integer_32, l_Mp.to_integer_32, l_T)
			end
			l_h := mpn_mul_fft_internal (a_op, a_pl, a_k, l_Ap, l_Bp, l_A, l_B, l_nprime2, l_l, l_Mp, l_fft_l, l_T, l_sqr)
			from
			until
				js.loop_left
			loop
				js.reset
				if (builtin_expect (((l_tmp_marker.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32).to_integer, 0)).to_boolean then
					tal_reent_inst.gmp_tmp_reentrant_free (l_tmp_marker)
				end
				js.break
			end
			js.reset
			Result := l_h
		end

end
