class
	P_BESSEL_J

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_sf_bessel_j0_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_sin_result: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
		do
			create l_sin_result.make

			l_ax := (fabs (a_x))
			if l_ax < 0.5 then
				l_y := a_x * a_x
				l_c1___2 := -1.0 / 6.0
				l_c2___0 := 1.0 / 120.0
				l_c3___3 := -1.0 / 5040.0
				l_c4___2 := 1.0 / 362880.0
				l_c5___1 := -1.0 / 39916800.0
				l_c6___2 := 1.0 / 6227020800.0
				a_result.item.val := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * l_c6___2)))))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				l_stat := (trig_inst.gsl_sf_sin_e (a_x, l_sin_result.address))
				a_result.item.val := l_sin_result.val / a_x
				a_result.item.err := fabs (l_sin_result.err / a_x)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat
			end
		end

	gsl_sf_bessel_j1_e2 (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_sum: REAL_64
			l_cos_result: PD_GSL_SF_RESULT_STRUCT
			l_sin_result: PD_GSL_SF_RESULT_STRUCT
			l_stat_cos: INTEGER_32
			l_stat_sin: INTEGER_32
			l_cos_x: REAL_64
			l_sin_x: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___6: REAL_64
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
			js: JUMP_STATE
		do
			create l_cos_result.make
			create l_sin_result.make

			l_ax := (fabs (a_x))
			if a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_ax < 3.1 * 2.2250738585072014e-308 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_j.c"), 77, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_ax < 0.25 then
				l_y := a_x * a_x
				l_c1___2 := -1.0 / 10.0
				l_c2___0 := 1.0 / 280.0
				l_c3___3 := -1.0 / 15120.0
				l_c4___2 := 1.0 / 1330560.0
				l_c5___1 := -1.0 / 172972800.0
				l_sum := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * l_c5___1))))
				a_result.item.val := (a_x / 3.0) * l_sum
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				l_stat_cos := (trig_inst.gsl_sf_cos_e (a_x, l_cos_result.address))
				l_stat_sin := (trig_inst.gsl_sf_sin_e (a_x, l_sin_result.address))
				l_cos_x := l_cos_result.val
				l_sin_x := l_sin_result.val
				a_result.item.val := (l_sin_x / a_x - l_cos_x) / a_x
				l_tmp___3 := fabs (l_sin_result.err / a_x)
				l_tmp___4 := fabs (l_cos_result.err)
				a_result.item.err := (l_tmp___3 + l_tmp___4) / (fabs (a_x))
				l_tmp___6 := fabs (l_sin_x / (a_x * a_x))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (l_tmp___6 + (fabs (l_cos_x / a_x))))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_cos /= 0 then
					l_tmp___10 := l_stat_cos
				else
					if l_stat_sin /= 0 then
						l_tmp___9 := l_stat_sin
					else
						l_tmp___9 := 0
					end
					l_tmp___10 := l_tmp___9
				end
				Result := l_tmp___10
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_j2_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_y: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_c8___0: REAL_64
			l_c9: REAL_64
			l_sum: REAL_64
			l_cos_result: PD_GSL_SF_RESULT_STRUCT
			l_sin_result: PD_GSL_SF_RESULT_STRUCT
			l_stat_cos: INTEGER_32
			l_stat_sin: INTEGER_32
			l_cos_x: REAL_64
			l_sin_x: REAL_64
			l_f: REAL_64
			l_tmp___3: REAL_64
			l_tmp___5: REAL_64
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			js: JUMP_STATE
		do
			create l_cos_result.make
			create l_sin_result.make

			l_ax := (fabs (a_x))
			if a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_ax < 4.0 * 1.4916681462400413e-154 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_j.c"), 119, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_ax < 1.3 then
				l_y := a_x * a_x
				l_c1___2 := -1.0 / 14.0
				l_c2___0 := 1.0 / 504.0
				l_c3___3 := -1.0 / 33264.0
				l_c4___2 := 1.0 / 3459456.0
				l_c5___1 := -1.0 / (518918400).to_double
				l_c6___2 := 1.0 / 105859353600.0
				l_c7___0 := -1.0 / 28158588057600.0
				l_c8___0 := 1.0 / 9461285587353600.0
				l_c9 := -1.0 / 3916972233164390400.0
				l_sum := 1.0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * (l_c6___2 + l_y * (l_c7___0 + l_y * (l_c8___0 + l_y * l_c9))))))))
				a_result.item.val := (l_y / 15.0) * l_sum
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				l_stat_cos := (trig_inst.gsl_sf_cos_e (a_x, l_cos_result.address))
				l_stat_sin := (trig_inst.gsl_sf_sin_e (a_x, l_sin_result.address))
				l_cos_x := l_cos_result.val
				l_sin_x := l_sin_result.val
				l_f := 3.0 / (a_x * a_x) - 1.0
				a_result.item.val := (l_f * l_sin_x - (3.0 * l_cos_x) / a_x) / a_x
				l_tmp___3 := fabs ((l_f * l_sin_result.err) / a_x)
				a_result.item.err := l_tmp___3 + (fabs (((3.0 * l_cos_result.err) / a_x) / a_x))
				l_tmp___5 := fabs ((l_f * l_sin_x) / a_x)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (l_tmp___5 + 3.0 * (fabs (l_cos_x / (a_x * a_x)))))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_cos /= 0 then
					l_tmp___9 := l_stat_cos
				else
					if l_stat_sin /= 0 then
						l_tmp___8 := l_stat_sin
					else
						l_tmp___8 := 0
					end
					l_tmp___9 := l_tmp___8
				end
				Result := l_tmp___9
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_jl_e (a_l: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_b___0: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			l_pre: REAL_64
			l_b___1: PD_GSL_SF_RESULT_STRUCT
			l_status___0: INTEGER_32
			l_pre___0: REAL_64
			l_b___2: PD_GSL_SF_RESULT_STRUCT
			l_status___1: INTEGER_32
			l_pre___1: REAL_64
			l_b___3: PD_GSL_SF_RESULT_STRUCT
			l_status___2: INTEGER_32
			l_pre___2: REAL_64
			l_sgn: REAL_64
			l_ratio: REAL_64
			l_stat_CF1: INTEGER_32
			l_jellp1: REAL_64
			l_jell: REAL_64
			l_jellm1: REAL_64
			l_ell: INTEGER_32
			l_j0_result: PD_GSL_SF_RESULT_STRUCT
			l_stat_j0: INTEGER_32
			l_pre___3: REAL_64
			l_tmp___20: INTEGER_32
			l_tmp___21: INTEGER_32
			l_j1_result: PD_GSL_SF_RESULT_STRUCT
			l_stat_j1: INTEGER_32
			l_pre___4: REAL_64
			l_tmp___25: INTEGER_32
			l_tmp___26: INTEGER_32
			l_tmp___27: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_b___0.make
			create l_b___1.make
			create l_b___2.make
			create l_b___3.make
			create l_j0_result.make
			create l_j1_result.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_l < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___0
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 3 then -- else:
						if not (a_x < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___0:
						from
						until
							js.loop_left
						loop
							js.reset
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_j.c"), 158, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 6 then -- else:
						if a_x /= 0.0 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if a_l > 0 then
							a_result.item.val := 0.0
						else
							a_result.item.val := 1.0
						end
						a_result.item.err := 0.0
						Result := 0
						js.return
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 9 then -- else:
						if a_l /= 0 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						Result := (gsl_sf_bessel_j0_e (a_x, a_result))
						js.return
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 12 then -- else:
						if a_l /= 1 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						Result := (gsl_sf_bessel_j1_e2 (a_x, a_result))
						js.return
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 15 then -- else:
						if a_l /= 2 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						Result := (gsl_sf_bessel_j2_e (a_x, a_result))
						js.return
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 18 then -- else:
						if not (a_x * a_x < (10.0 * (a_l.to_double + 0.5)) / 2.7182818284590452354) then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						l_status := (bessel_inst.gsl_sf_bessel_IJ_taylor_e (a_l.to_double + 0.5, a_x, -1, 50, 2.2204460492503131e-16, l_b___0.address))
						l_pre := (sqrt ((0.5 * 3.14159265358979323846) / a_x))
						a_result.item.val := l_pre * l_b___0.val
						a_result.item.err := l_pre * l_b___0.err
						a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
						Result := l_status
						js.return
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 21 then -- else:
						if not (1.2207031250000000e-04 * a_x > (a_l * a_l + a_l).to_double + 1.0) then
							l_pos := -1 + 24 -- skip if block
						end
				when 22 then -- if:
						l_status___0 := (bessel_inst.gsl_sf_bessel_Jnu_asympx_e (a_l.to_double + 0.5, a_x, l_b___1.address))
						l_pre___0 := (sqrt ((0.5 * 3.14159265358979323846) / a_x))
						a_result.item.val := l_pre___0 * l_b___1.val
						a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_pre___0 * l_b___1.err
						Result := l_status___0
						js.return
				when 23 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 24 then -- else:
						if not (a_l.to_double > 1.0 / 2.4607833005759251e-03) then
							l_pos := -1 + 27 -- skip if block
						end
				when 25 then -- if:
						l_status___1 := (bessel_olver_inst.gsl_sf_bessel_Jnu_asymp_Olver_e (a_l.to_double + 0.5, a_x, l_b___2.address))
						l_pre___1 := (sqrt ((0.5 * 3.14159265358979323846) / a_x))
						a_result.item.val := l_pre___1 * l_b___2.val
						a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_pre___1 * l_b___2.err
						Result := l_status___1
						js.return
				when 26 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 27 then -- else:
						if not (a_x > 1000.0) then
							l_pos := -1 + 30 -- skip if block
						end
				when 28 then -- if:
						if a_x > (a_l * a_l).to_double then
							l_status___2 := (bessel_inst.gsl_sf_bessel_Jnu_asympx_e (a_l.to_double + 0.5, a_x, l_b___3.address))
							l_pre___2 := (sqrt ((0.5 * 3.14159265358979323846) / a_x))
							a_result.item.val := l_pre___2 * l_b___3.val
							a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)) + l_pre___2 * l_b___3.err
							Result := l_status___2
							js.return
						else
							l_pos := -1 + 30 -- goto _L
						end
				when 29 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 30 then -- else: | _L:
						l_stat_CF1 := (bessel_inst.gsl_sf_bessel_J_CF1 (a_l.to_double + 0.5, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_ratio), create {CE_POINTER [REAL_64]}.make_shared ($l_sgn)))
						l_jellp1 := 1.4901161193847656e-08 * l_ratio
						l_jell := 1.4901161193847656e-08
						l_ell := a_l
						from
						until
							not (l_ell > 0)
						loop
							l_jellm1 := -l_jellp1 + ((2 * l_ell + 1).to_double / a_x) * l_jell
							l_jellp1 := l_jell
							l_jell := l_jellm1
							l_ell := l_ell - 1
						end
						l_tmp___27 := fabs (l_jell)
						if l_tmp___27 > (fabs (l_jellp1)) then
							l_stat_j0 := (gsl_sf_bessel_j0_e (a_x, l_j0_result.address))
							l_pre___3 := 1.4901161193847656e-08 / l_jell
							a_result.item.val := l_j0_result.val * l_pre___3
							a_result.item.err := l_j0_result.err * (fabs (l_pre___3))
							a_result.item.err := a_result.item.err + (((4.0 * 2.2204460492503131e-16) * (0.5 * a_l.to_double + 1.0)) * (fabs (a_result.item.val)))
							if l_stat_j0 /= 0 then
								l_tmp___21 := l_stat_j0
							else
								if l_stat_CF1 /= 0 then
									l_tmp___20 := l_stat_CF1
								else
									l_tmp___20 := 0
								end
								l_tmp___21 := l_tmp___20
							end
							Result := l_tmp___21
							js.return
						else
							l_stat_j1 := (gsl_sf_bessel_j1_e2 (a_x, l_j1_result.address))
							l_pre___4 := 1.4901161193847656e-08 / l_jellp1
							a_result.item.val := l_j1_result.val * l_pre___4
							a_result.item.err := l_j1_result.err * (fabs (l_pre___4))
							a_result.item.err := a_result.item.err + (((4.0 * 2.2204460492503131e-16) * (0.5 * a_l.to_double + 1.0)) * (fabs (a_result.item.val)))
							if l_stat_j1 /= 0 then
								l_tmp___26 := l_stat_j1
							else
								if l_stat_CF1 /= 0 then
									l_tmp___25 := l_stat_CF1
								else
									l_tmp___25 := 0
								end
								l_tmp___26 := l_tmp___25
							end
							Result := l_tmp___26
							js.return
						end
				when 31 then -- if done | if done | if done | if done | if done | if done | if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 32 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_jl_array (a_lmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_j: INTEGER_32
			l_j___0: INTEGER_32
			l_r_jellp1: PD_GSL_SF_RESULT_STRUCT
			l_r_jell: PD_GSL_SF_RESULT_STRUCT
			l_stat_0: INTEGER_32
			l_stat_1: INTEGER_32
			l_jellp1: REAL_64
			l_jell: REAL_64
			l_jellm1: REAL_64
			l_ell: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_r_jellp1.make
			create l_r_jell.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_lmax < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_x < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						l_j := 0
						from
						until
							not (l_j <= a_lmax)
						loop
							a_result_array[l_j] := 0.0
							l_j := l_j + 1
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_j.c"), 254, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x = 0.0 then
							l_j___0 := 1
							from
							until
								not (l_j___0 <= a_lmax)
							loop
								a_result_array[l_j___0] := 0.0
								l_j___0 := l_j___0 + 1
							end
							a_result_array[0] := 1.0
							Result := 0
							js.return
						else
							l_stat_0 := (gsl_sf_bessel_jl_e (a_lmax + 1, a_x, l_r_jellp1.address))
							l_stat_1 := (gsl_sf_bessel_jl_e (a_lmax, a_x, l_r_jell.address))
							l_jellp1 := l_r_jellp1.val
							l_jell := l_r_jell.val
							a_result_array[a_lmax] := l_jell
							l_ell := a_lmax
							from
							until
								not (l_ell >= 1)
							loop
								l_jellm1 := -l_jellp1 + ((2 * l_ell + 1).to_double / a_x) * l_jell
								l_jellp1 := l_jell
								l_jell := l_jellm1
								a_result_array[(l_ell - 1)] := l_jellm1
								l_ell := l_ell - 1
							end
							if l_stat_0 /= 0 then
								l_tmp___2 := l_stat_0
							else
								if l_stat_1 /= 0 then
									l_tmp___1 := l_stat_1
								else
									l_tmp___1 := 0
								end
								l_tmp___2 := l_tmp___1
							end
							Result := l_tmp___2
							js.return
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_jl_steed_array (a_lmax: INTEGER_32; a_x: REAL_64; a_jl_x: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_j: INTEGER_32
			l_j___0: INTEGER_32
			l_inv_fact: REAL_64
			l_x_l: REAL_64
			l_l: INTEGER_32
			l_x_inv: REAL_64
			l_W: REAL_64
			l_F___0: REAL_64
			l_FP: REAL_64
			l_B: REAL_64
			l_end: REAL_64
			l_D: REAL_64
			l_del: REAL_64
			l_tmp: REAL_64
			l_XP2: REAL_64
			l_PL: REAL_64
			l_L2: INTEGER_32
			l_LP: INTEGER_32
			l_L___0: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_lmax < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_x < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						l_j := 0
						from
						until
							not (l_j <= a_lmax)
						loop
							a_jl_x[l_j] := 0.0
							l_j := l_j + 1
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_j.c"), 292, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x = 0.0 then
							l_j___0 := 1
							from
							until
								not (l_j___0 <= a_lmax)
							loop
								a_jl_x[l_j___0] := 0.0
								l_j___0 := l_j___0 + 1
							end
							a_jl_x[0] := 1.0
							Result := 0
							js.return
						elseif a_x < 2.0 * 1.2207031250000000e-04 then
							l_inv_fact := 1.0
							l_x_l := 1.0
							l_l := 0
							from
							until
								not (l_l <= a_lmax)
							loop
								a_jl_x[l_l] := l_x_l * l_inv_fact
								a_jl_x[l_l] := a_jl_x[l_l] * (1.0 - ((0.5 * a_x) * a_x) / (2.0 * l_l.to_double + 3.0))
								l_inv_fact := l_inv_fact / (2.0 * l_l.to_double + 3.0)
								l_x_l := l_x_l * a_x
								l_l := l_l + 1
							end
							Result := 0
							js.return
						else
							l_x_inv := 1.0 / a_x
							l_W := 2.0 * l_x_inv
							l_F___0 := 1.0
							l_FP := (a_lmax.to_double + 1.0) * l_x_inv
							l_B := 2.0 * l_FP + l_x_inv
							l_end := l_B + 20000.0 * l_W
							l_D := 1.0 / l_B
							l_del := -l_D
							l_FP := l_FP + l_del
							from
							until
								js.loop_left
							loop
								js.reset
								l_B := l_B + l_W
								l_D := 1.0 / (l_B - l_D)
								l_del := l_del * (l_B * l_D - 1.)
								l_FP := l_FP + l_del
								if l_D < 0.0 then
									l_F___0 := -l_F___0
								end
								if l_B > l_end then
									from
									until
										js.loop_left
									loop
										js.reset
										error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_j.c"), 334, 11)
										Result := 11
										js.return
										if not js.return_called then
											js.break
										end
									end
									js.reset
								end
								if not js.return_called then
									l_tmp := fabs (l_del)
									if not (l_tmp >= (fabs (l_FP)) * 2.2204460492503131e-16) then
										js.break
									end
								end
							end
							js.reset
							if not js.return_called then
								l_FP := l_FP * l_F___0
								if a_lmax > 0 then
									l_XP2 := l_FP
									l_PL := a_lmax.to_double * l_x_inv
									l_L2 := a_lmax
									a_jl_x[a_lmax] := l_F___0
									l_LP := 1
									from
									until
										not (l_LP <= a_lmax)
									loop
										a_jl_x[(l_L2 - 1)] := l_PL * a_jl_x[l_L2] + l_XP2
										l_FP := l_PL * a_jl_x[(l_L2 - 1)] - a_jl_x[l_L2]
										l_XP2 := l_FP
										l_PL := l_PL - l_x_inv
										l_L2 := l_L2 - 1
										l_LP := l_LP + 1
									end
									l_F___0 := a_jl_x[0]
								end
								l_W := l_x_inv / (hypot (l_FP, l_F___0))
								a_jl_x[0] := l_W * l_F___0
								if a_lmax > 0 then
									l_L___0 := 1
									from
									until
										not (l_L___0 <= a_lmax)
									loop
										a_jl_x[l_L___0] := a_jl_x[l_L___0] * l_W
										l_L___0 := l_L___0 + 1
									end
								end
								Result := 0
								js.return
							end
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_j0 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_j0_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_j0_e(x, &result)"), ce_string ("bessel_j.c"), 379, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_bessel_j12 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_j1_e2 (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_j1_e(x, &result)"), ce_string ("bessel_j.c"), 384, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_bessel_j2 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_j2_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_j2_e(x, &result)"), ce_string ("bessel_j.c"), 389, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_bessel_jl (a_l: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_jl_e (a_l, a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_jl_e(l, x, &result)"), ce_string ("bessel_j.c"), 394, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
