class
	P_BESSEL_J1

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create bj1_data.make (<<12>>)
			create bj1_cs.make
		end

feature {ANY} -- attributes

	bj1_data: CE_ARRAY [REAL_64]

	bj1_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	gsl_sf_bessel_J1_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_y: REAL_64
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_z: REAL_64
			l_ca: PD_GSL_SF_RESULT_STRUCT
			l_ct: PD_GSL_SF_RESULT_STRUCT
			l_sp: PD_GSL_SF_RESULT_STRUCT
			l_stat_ca: INTEGER_32
			l_stat_ct: INTEGER_32
			l_stat_sp: INTEGER_32
			l_sqrty: REAL_64
			l_ampl: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
			js: JUMP_STATE
		do
			create l_c___1.make
			create l_ca.make
			create l_ct.make
			create l_sp.make

			l_y := (fabs (a_x))
			if l_y = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_y < 2.0 * 2.2250738585072014e-308 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_J1.c"), 86, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_y < (2.0 * 1.41421356237309504880) * 1.4901161193847656e-08 then
				a_result.item.val := 0.5 * a_x
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_y < 4.0 then
				cheb_eval_inst.cheb_eval_e ((bj1_cs.address), (0.125 * l_y) * l_y - 1.0, l_c___1.address).do_nothing
				a_result.item.val := a_x * (0.25 + l_c___1.val)
				a_result.item.err := fabs (a_x * l_c___1.err)
				Result := 0
				js.return
			else
				l_z := 32.0 / (l_y * l_y) - 1.0
				l_stat_ca := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bm1_cs.address, l_z, l_ca.address))
				l_stat_ct := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bth1_cs.address, l_z, l_ct.address))
				l_stat_sp := (bessel_inst.gsl_sf_bessel_sin_pi4_e (l_y, l_ct.val / l_y, l_sp.address))
				l_sqrty := (sqrt (l_y))
				l_ampl := (0.75 + l_ca.val) / l_sqrty
				if a_x < 0.0 then
					l_tmp___4 := -l_ampl
				else
					l_tmp___4 := l_ampl
				end
				a_result.item.val := l_tmp___4 * l_sp.val
				l_tmp___5 := fabs (l_sp.val)
				a_result.item.err := (l_tmp___5 * l_ca.err) / l_sqrty + (fabs (l_ampl)) * l_sp.err
				a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
				if l_stat_ca /= 0 then
					l_tmp___10 := l_stat_ca
				else
					if l_stat_ct /= 0 then
						l_tmp___9 := l_stat_ct
					else
						if l_stat_sp /= 0 then
							l_tmp___8 := l_stat_sp
						else
							l_tmp___8 := 0
						end
						l_tmp___9 := l_tmp___8
					end
					l_tmp___10 := l_tmp___9
				end
				Result := l_tmp___10
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_J1 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_J1_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_J1_e(x, &result)"), ce_string ("bessel_J1.c"), 127, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
