class
	P_BESSEL_K1

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create bk1_data.make (<<11>>)
			create bk1_cs.make
			create ak1_data.make (<<17>>)
			create ak1_cs.make
			create ak12_data.make (<<14>>)
			create ak12_cs.make
		end

feature {ANY} -- attributes

	bk1_data: CE_ARRAY [REAL_64]

	bk1_cs: PD_CHEB_SERIES_STRUCT

	ak1_data: CE_ARRAY [REAL_64]

	ak1_cs: PD_CHEB_SERIES_STRUCT

	ak12_data: CE_ARRAY [REAL_64]

	ak12_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	gsl_sf_bessel_K1_scaled_e2 (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lx: REAL_64
			l_ex: REAL_64
			l_stat_I1: INTEGER_32
			l_I1: PD_GSL_SF_RESULT_STRUCT
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_sx: REAL_64
			l_c___2: PD_GSL_SF_RESULT_STRUCT
			l_sx___0: REAL_64
			l_c___3: PD_GSL_SF_RESULT_STRUCT
			js: JUMP_STATE
		do
			create l_I1.make
			create l_c___1.make
			create l_c___2.make
			create l_c___3.make

			if a_x <= 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_K1.c"), 136, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_x < 2.0 * 2.2250738585072014e-308 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("bessel_K1.c"), 139, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_x <= 2.0 then
				l_lx := (log (a_x))
				l_ex := (exp (a_x))
				cheb_eval_inst.cheb_eval_e ((bk1_cs.address), (0.5 * a_x) * a_x - 1.0, l_c___1.address).do_nothing
				l_stat_I1 := bessel_i1_inst.gsl_sf_bessel_I1_e (a_x, l_I1.address)
				a_result.item.val := l_ex * ((l_lx - 0.69314718055994530942) * l_I1.val + (0.75 + l_c___1.val) / a_x)
				a_result.item.err := l_ex * (l_c___1.err / a_x + (fabs (l_lx)) * l_I1.err)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat_I1
				js.return
			elseif a_x <= 8.0 then
				l_sx := (sqrt (a_x))
				cheb_eval_inst.cheb_eval_e ((ak1_cs.address), (16.0 / a_x - 5.0) / 3.0, l_c___2.address).do_nothing
				a_result.item.val := (1.25 + l_c___2.val) / l_sx
				a_result.item.err := l_c___2.err / l_sx
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			else
				l_sx___0 := (sqrt (a_x))
				cheb_eval_inst.cheb_eval_e ((ak12_cs.address), 16.0 / a_x - 1.0, l_c___3.address).do_nothing
				a_result.item.val := (1.25 + l_c___3.val) / l_sx___0
				a_result.item.err := l_c___3.err / l_sx___0
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_K1_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lx: REAL_64
			l_stat_I1: INTEGER_32
			l_I1: PD_GSL_SF_RESULT_STRUCT
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_K1_scaled: PD_GSL_SF_RESULT_STRUCT
			l_stat_K1: INTEGER_32
			l_stat_e: INTEGER_32
			l_tmp___8: REAL_64
			l_tmp___10: INTEGER_32
			l_tmp___11: INTEGER_32
			js: JUMP_STATE
		do
			create l_I1.make
			create l_c___1.make
			create l_K1_scaled.make

			if a_x <= 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_K1.c"), 180, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_x < 2.0 * 2.2250738585072014e-308 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("bessel_K1.c"), 183, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_x <= 2.0 then
				l_lx := (log (a_x))
				cheb_eval_inst.cheb_eval_e ((bk1_cs.address), (0.5 * a_x) * a_x - 1.0, l_c___1.address).do_nothing
				l_stat_I1 := bessel_i1_inst.gsl_sf_bessel_I1_e (a_x, l_I1.address)
				a_result.item.val := (l_lx - 0.69314718055994530942) * l_I1.val + (0.75 + l_c___1.val) / a_x
				a_result.item.err := l_c___1.err / a_x + (fabs (l_lx)) * l_I1.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat_I1
				js.return
			else
				l_stat_K1 := (gsl_sf_bessel_K1_scaled_e2 (a_x, l_K1_scaled.address))
				l_stat_e := (exp_inst.gsl_sf_exp_mult_err_e (-a_x, 0.0, l_K1_scaled.val, l_K1_scaled.err, a_result))
				l_tmp___8 := fabs (a_result.item.val)
				a_result.item.err := l_tmp___8 * (2.2204460492503131e-16 * (fabs (a_x)) + l_K1_scaled.err / l_K1_scaled.val)
				if l_stat_e /= 0 then
					l_tmp___11 := l_stat_e
				else
					if l_stat_K1 /= 0 then
						l_tmp___10 := l_stat_K1
					else
						l_tmp___10 := 0
					end
					l_tmp___11 := l_tmp___10
				end
				Result := l_tmp___11
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_K1_scaled2 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_K1_scaled_e2 (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_K1_scaled_e(x, &result)"), ce_string ("bessel_K1.c"), 214, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_bessel_K1 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_K1_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_K1_e(x, &result)"), ce_string ("bessel_K1.c"), 219, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
