class
	P_BESSEL_KN

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	bessel_Kn_scaled_small_x (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_k: INTEGER_32
			l_y: REAL_64
			l_ln_x_2: REAL_64
			l_ex: REAL_64
			l_ln_nm1_fact: PD_GSL_SF_RESULT_STRUCT
			l_k_term: REAL_64
			l_term1: REAL_64
			l_sum1: REAL_64
			l_ln_pre1: REAL_64
			l_term2: REAL_64
			l_sum2: REAL_64
			l_pre2: REAL_64
			l_KMAX: INTEGER_32
			l_psi_n: PD_GSL_SF_RESULT_STRUCT
			l_npk_fact: PD_GSL_SF_RESULT_STRUCT
			l_yk: REAL_64
			l_k_fact: REAL_64
			l_psi_kp1: REAL_64
			l_psi_npkp1: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_tmp___5: REAL_64
			js: JUMP_STATE
		do
			create l_ln_nm1_fact.make
			create l_psi_n.make
			create l_npk_fact.make

			l_y := (0.25 * a_x) * a_x
			l_ln_x_2 := (log (0.5 * a_x))
			l_ex := (exp (a_x))
			gamma_inst.gsl_sf_lnfact_e ((a_n - 1).to_natural_32, l_ln_nm1_fact.address).do_nothing
			l_ln_pre1 := (-a_n).to_double * l_ln_x_2 + l_ln_nm1_fact.val
			if l_ln_pre1 > 7.0978271289338397e+02 - 3.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_Kn.c"), 54, 16)
					Result := 16
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_sum1 := 1.0
				l_k_term := 1.0
				l_k := 1
				from
				until
					not (l_k <= a_n - 1)
				loop
					l_k_term := l_k_term * (-l_y / (l_k * (a_n - l_k)).to_double)
					l_sum1 := l_sum1 + l_k_term
					l_k := l_k + 1
				end
				l_term1 := (0.5 * (exp (l_ln_pre1))) * l_sum1
				l_pre2 := 0.5 * (exp (a_n.to_double * l_ln_x_2))
				if l_pre2 > 0.0 then
					l_KMAX := 20
					l_yk := 1.0
					l_k_fact := 1.0
					l_psi_kp1 := -0.57721566490153286060651209008
					psi_inst.gsl_sf_psi_int_e (a_n, l_psi_n.address).do_nothing
					gamma_inst.gsl_sf_fact_e (a_n.to_natural_32, l_npk_fact.address).do_nothing
					l_psi_npkp1 := l_psi_n.val + 1.0 / a_n.to_double
					l_sum2 := ((l_psi_kp1 + l_psi_npkp1) - 2.0 * l_ln_x_2) / l_npk_fact.val
					l_k := 1
					from
					until
						not (l_k < l_KMAX)
					loop
						l_psi_kp1 := l_psi_kp1 + (1.0 / l_k.to_double)
						l_psi_npkp1 := l_psi_npkp1 + (1.0 / (a_n + l_k).to_double)
						l_k_fact := l_k_fact * l_k.to_double
						l_npk_fact.val := l_npk_fact.val * (a_n + l_k).to_double
						l_yk := l_yk * l_y
						l_k_term := (l_yk * ((l_psi_kp1 + l_psi_npkp1) - 2.0 * l_ln_x_2)) / (l_k_fact * l_npk_fact.val)
						l_sum2 := l_sum2 + l_k_term
						l_k := l_k + 1
					end
					if ((a_n).bit_and (1)).to_boolean then
						l_tmp___3 := -1.0
					else
						l_tmp___3 := 1.0
					end
					l_term2 := (l_tmp___3 * l_pre2) * l_sum2
				else
					l_term2 := 0.0
				end
				a_result.item.val := l_ex * (l_term1 + l_term2)
				l_tmp___4 := fabs (l_ln_pre1)
				l_tmp___5 := fabs (l_term1)
				a_result.item.err := (l_ex * 2.2204460492503131e-16) * (l_tmp___4 * l_tmp___5 + (fabs (l_term2)))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
			end
		end

	gsl_sf_bessel_Kn_scaled_e (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n: INTEGER_32
			l_two_over_x: REAL_64
			l_r_b_jm1: PD_GSL_SF_RESULT_STRUCT
			l_r_b_j: PD_GSL_SF_RESULT_STRUCT
			l_stat_0: INTEGER_32
			l_stat_1: INTEGER_32
			l_b_jm1: REAL_64
			l_b_j: REAL_64
			l_b_jp1: REAL_64
			l_j: INTEGER_32
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			l_tmp___14: REAL_64
			js: JUMP_STATE
		do
			create l_r_b_jm1.make
			create l_r_b_j.make

			l_n := a_n
			l_n := abs (l_n)
			if a_x <= 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Kn.c"), 109, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_n = 0 then
				Result := (bessel_k0_inst.gsl_sf_bessel_K0_scaled_e2 (a_x, a_result))
				js.return
			elseif l_n = 1 then
				Result := (bessel_k1_inst.gsl_sf_bessel_K1_scaled_e2 (a_x, a_result))
				js.return
			elseif a_x <= 5.0 then
				Result := (bessel_Kn_scaled_small_x (l_n, a_x, a_result))
				js.return
			elseif 6.0554544523933429e-06 * a_x > 0.25 * (l_n * l_n + 1).to_double then
				Result := (bessel_inst.gsl_sf_bessel_Knu_scaled_asympx_e (l_n.to_double, a_x, a_result))
				js.return
			else
				if 0.29 / (l_n * l_n).to_double < 0.5 / ((l_n * l_n).to_double + a_x * a_x) then
					l_tmp___14 := 0.29 / (l_n * l_n).to_double
				else
					l_tmp___14 := 0.5 / ((l_n * l_n).to_double + a_x * a_x)
				end
				if l_tmp___14 < 6.0554544523933429e-06 then
					Result := (bessel_inst.gsl_sf_bessel_Knu_scaled_asymp_unif_e (l_n.to_double, a_x, a_result))
					js.return
				else
					l_two_over_x := 2.0 / a_x
					l_stat_0 := (bessel_k0_inst.gsl_sf_bessel_K0_scaled_e2 (a_x, l_r_b_jm1.address))
					l_stat_1 := (bessel_k1_inst.gsl_sf_bessel_K1_scaled_e2 (a_x, l_r_b_j.address))
					l_b_jm1 := l_r_b_jm1.val
					l_b_j := l_r_b_j.val
					l_j := 1
					from
					until
						not (l_j < l_n)
					loop
						l_b_jp1 := l_b_jm1 + (l_j.to_double * l_two_over_x) * l_b_j
						l_b_jm1 := l_b_j
						l_b_j := l_b_jp1
						l_j := l_j + 1
					end
					a_result.item.val := l_b_j
					l_tmp___8 := fabs (l_b_j)
					l_tmp___9 := fabs (l_r_b_jm1.err / l_r_b_jm1.val)
					a_result.item.err := l_n.to_double * (l_tmp___8 * (l_tmp___9 + (fabs (l_r_b_j.err / l_r_b_j.val))))
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
					if l_stat_0 /= 0 then
						l_tmp___13 := l_stat_0
					else
						if l_stat_1 /= 0 then
							l_tmp___12 := l_stat_1
						else
							l_tmp___12 := 0
						end
						l_tmp___13 := l_tmp___12
					end
					Result := l_tmp___13
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_Kn_e (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_status: INTEGER_32
			l_ex: REAL_64
		do
			l_status := (gsl_sf_bessel_Kn_scaled_e (a_n, a_x, a_result))
			l_ex := (exp (-a_x))
			a_result.item.val := a_result.item.val * l_ex
			a_result.item.err := a_result.item.err * l_ex
			a_result.item.err := a_result.item.err + ((a_x * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			Result := l_status
		end

	gsl_sf_bessel_Kn_scaled_array (a_nmin: INTEGER_32; a_nmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_j: INTEGER_32
			l_b___0: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
			l_two_over_x: REAL_64
			l_r_Knm1: PD_GSL_SF_RESULT_STRUCT
			l_r_Kn: PD_GSL_SF_RESULT_STRUCT
			l_stat_0: INTEGER_32
			l_stat_1: INTEGER_32
			l_stat___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_Knp1: REAL_64
			l_Kn: REAL_64
			l_Knm1: REAL_64
			l_n: INTEGER_32
			l_j___0: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_b___0.make
			create l_r_Knm1.make
			create l_r_Kn.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_nmin < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if not (a_nmax < a_nmin) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if not (a_x <= 0.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						l_j := 0
						from
						until
							not (l_j <= a_nmax - a_nmin)
						loop
							a_result_array[l_j] := 0.0
							l_j := l_j + 1
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Kn.c"), 171, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_nmax = 0 then
							l_stat := (bessel_k0_inst.gsl_sf_bessel_K0_scaled_e2 (a_x, l_b___0.address))
							a_result_array[0] := l_b___0.val
							Result := l_stat
							js.return
						else
							l_two_over_x := 2.0 / a_x
							l_stat_0 := (gsl_sf_bessel_Kn_scaled_e (a_nmin, a_x, l_r_Knm1.address))
							l_stat_1 := (gsl_sf_bessel_Kn_scaled_e (a_nmin + 1, a_x, l_r_Kn.address))
							if l_stat_0 /= 0 then
								l_tmp___3 := l_stat_0
							else
								if l_stat_1 /= 0 then
									l_tmp___2 := l_stat_1
								else
									l_tmp___2 := 0
								end
								l_tmp___3 := l_tmp___2
							end
							l_stat___0 := l_tmp___3
							l_Kn := l_r_Kn.val
							l_Knm1 := l_r_Knm1.val
							l_n := a_nmin + 1
							from
							until
								not (not js.loop_left and then (l_n <= a_nmax + 1))
							loop
								if l_Knm1 < 1.7976931348623157e+308 then
									a_result_array[((l_n - 1) - a_nmin)] := l_Knm1
									l_Knp1 := l_Knm1 + (l_n.to_double * l_two_over_x) * l_Kn
									l_Knm1 := l_Kn
									l_Kn := l_Knp1
								else
									l_j___0 := l_n
									from
									until
										not (l_j___0 <= a_nmax + 1)
									loop
										a_result_array[((l_j___0 - 1) - a_nmin)] := 0.0
										l_j___0 := l_j___0 + 1
									end
									from
									until
										js.loop_left
									loop
										js.reset
										error_inst.gsl_error (ce_string ("overflow"), ce_string ("bessel_Kn.c"), 208, 16)
										Result := 16
										js.return
										if not js.return_called then
											js.break
										end
									end
									js.reset
								end
								if not js.return_called then
									l_n := l_n + 1
								end
							end
							if not js.return_called then
								Result := l_stat___0
								js.return
							end
						end
				when 10 then -- if done | if done
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_Kn_array (a_nmin: INTEGER_32; a_nmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_status: INTEGER_32
			l_ex: REAL_64
			l_i: INTEGER_32
		do
			l_status := (gsl_sf_bessel_Kn_scaled_array (a_nmin, a_nmax, a_x, a_result_array))
			l_ex := (exp (-a_x))
			l_i := 0
			from
			until
				not (l_i <= a_nmax - a_nmin)
			loop
				a_result_array[l_i] := a_result_array[l_i] * l_ex
				l_i := l_i + 1
			end
			Result := l_status
		end

	gsl_sf_bessel_Kn_scaled (a_n: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_Kn_scaled_e (a_n, a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Kn_scaled_e(n, x, &result)"), ce_string ("bessel_Kn.c"), 234, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_bessel_Kn (a_n: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_bessel_Kn_e (a_n, a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Kn_e(n, x, &result)"), ce_string ("bessel_Kn.c"), 239, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
