class
	P_BESSEL_TEMME

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create g1_dat.make (<<14>>)
			create g1_cs.make
			create g2_dat.make (<<15>>)
			create g2_cs.make
		end

feature {ANY} -- attributes

	g1_dat: CE_ARRAY [REAL_64]

	g1_cs: PD_CHEB_SERIES_STRUCT

	g2_dat: CE_ARRAY [REAL_64]

	g2_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	gsl_sf_temme_gamma (a_nu: REAL_64; a_g_1pnu: CE_POINTER [REAL_64]; a_g_1mnu: CE_POINTER [REAL_64]; a_g1: CE_POINTER [REAL_64]; a_g2: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_anu: REAL_64
			l_x: REAL_64
			l_r_g1: PD_GSL_SF_RESULT_STRUCT
			l_r_g2: PD_GSL_SF_RESULT_STRUCT
		do
			create l_r_g1.make
			create l_r_g2.make

			l_anu := (fabs (a_nu))
			l_x := 4.0 * l_anu - 1.0
			cheb_eval_inst.cheb_eval_e ((g1_cs.address), l_x, l_r_g1.address).do_nothing
			cheb_eval_inst.cheb_eval_e ((g2_cs.address), l_x, l_r_g2.address).do_nothing
			a_g1.item := l_r_g1.val
			a_g2.item := l_r_g2.val
			a_g_1mnu.item := 1.0 / (l_r_g2.val + a_nu * l_r_g1.val)
			a_g_1pnu.item := 1.0 / (l_r_g2.val - a_nu * l_r_g1.val)
			Result := 0
		end

	gsl_sf_bessel_Y_temme (a_nu: REAL_64; a_x: REAL_64; a_Ynu: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_Ynup1: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_max_iter: INTEGER_32
			l_half_x: REAL_64
			l_ln_half_x: REAL_64
			l_half_x_nu: REAL_64
			l_pi_nu: REAL_64
			l_alpha: REAL_64
			l_sigma: REAL_64
			l_sinrat: REAL_64
			l_tmp___3: REAL_64
			l_sinhrat: REAL_64
			l_tmp___7: REAL_64
			l_sinhalf: REAL_64
			l_tmp___11: REAL_64
			l_sin_sqr: REAL_64
			l_sum0: REAL_64
			l_sum1: REAL_64
			l_fk: REAL_64
			l_pk: REAL_64
			l_qk: REAL_64
			l_hk: REAL_64
			l_ck: REAL_64
			l_k: INTEGER_32
			l_stat_iter: INTEGER_32
			l_g_1pnu: REAL_64
			l_g_1mnu: REAL_64
			l_g1: REAL_64
			l_g2: REAL_64
			l_stat_g: INTEGER_32
			l_del0: REAL_64
			l_del1: REAL_64
			l_gk: REAL_64
			l_tmp___15: REAL_64
			l_tmp___19: INTEGER_32
			l_tmp___20: INTEGER_32
			js: JUMP_STATE
		do
			l_max_iter := 15000
			l_half_x := 0.5 * a_x
			l_ln_half_x := (log (l_half_x))
			l_half_x_nu := (exp (a_nu * l_ln_half_x))
			l_pi_nu := 3.14159265358979323846 * a_nu
			l_alpha := l_pi_nu / 2.0
			l_sigma := -a_nu * l_ln_half_x
			if (fabs (l_pi_nu)) < 2.2204460492503131e-16 then
				l_tmp___3 := 1.0
			else
				l_tmp___3 := l_pi_nu / (sin (l_pi_nu))
			end
			l_sinrat := l_tmp___3
			if (fabs (l_sigma)) < 2.2204460492503131e-16 then
				l_tmp___7 := 1.0
			else
				l_tmp___7 := (sinh (l_sigma)) / l_sigma
			end
			l_sinhrat := l_tmp___7
			if (fabs (l_alpha)) < 2.2204460492503131e-16 then
				l_tmp___11 := 1.0
			else
				l_tmp___11 := (sin (l_alpha)) / l_alpha
			end
			l_sinhalf := l_tmp___11
			l_sin_sqr := ((((a_nu * 3.14159265358979323846) * 3.14159265358979323846) * 0.5) * l_sinhalf) * l_sinhalf
			l_k := 0
			l_stat_g := (gsl_sf_temme_gamma (a_nu, create {CE_POINTER [REAL_64]}.make_shared ($l_g_1pnu), create {CE_POINTER [REAL_64]}.make_shared ($l_g_1mnu), create {CE_POINTER [REAL_64]}.make_shared ($l_g1), create {CE_POINTER [REAL_64]}.make_shared ($l_g2)))
			l_fk := ((2.0 / 3.14159265358979323846) * l_sinrat) * ((cosh (l_sigma)) * l_g1 - (l_sinhrat * l_ln_half_x) * l_g2)
			l_pk := ((1.0 / 3.14159265358979323846) / l_half_x_nu) * l_g_1pnu
			l_qk := ((1.0 / 3.14159265358979323846) * l_half_x_nu) * l_g_1mnu
			l_hk := l_pk
			l_ck := 1.0
			l_sum0 := l_fk + l_sin_sqr * l_qk
			l_sum1 := l_pk
			from
			until
				not (not js.loop_left and then (l_k < l_max_iter))
			loop
				js.reset
				l_k := l_k + 1
				l_fk := ((l_k.to_double * l_fk + l_pk) + l_qk) / ((l_k * l_k).to_double - a_nu * a_nu)
				l_ck := l_ck * ((-l_half_x * l_half_x) / l_k.to_double)
				l_pk := l_pk / (l_k.to_double - a_nu)
				l_qk := l_qk / (l_k.to_double + a_nu)
				l_gk := l_fk + l_sin_sqr * l_qk
				l_hk := (-l_k).to_double * l_gk + l_pk
				l_del0 := l_ck * l_gk
				l_del1 := l_ck * l_hk
				l_sum0 := l_sum0 + l_del0
				l_sum1 := l_sum1 + l_del1
				l_tmp___15 := fabs (l_del0)
				if l_tmp___15 < (0.5 * (1.0 + (fabs (l_sum0)))) * 2.2204460492503131e-16 then
					js.break
				end
			end
			js.reset
			a_Ynu.item.val := -l_sum0
			a_Ynu.item.err := ((2.0 + 0.5 * l_k.to_double) * 2.2204460492503131e-16) * (fabs (a_Ynu.item.val))
			a_Ynup1.item.val := (-l_sum1 * 2.0) / a_x
			a_Ynup1.item.err := ((2.0 + 0.5 * l_k.to_double) * 2.2204460492503131e-16) * (fabs (a_Ynup1.item.val))
			if l_k >= l_max_iter then
				l_stat_iter := 11
			else
				l_stat_iter := 0
			end
			if l_stat_iter /= 0 then
				l_tmp___20 := l_stat_iter
			else
				if l_stat_g /= 0 then
					l_tmp___19 := l_stat_g
				else
					l_tmp___19 := 0
				end
				l_tmp___20 := l_tmp___19
			end
			Result := l_tmp___20
		end

	gsl_sf_bessel_K_scaled_temme (a_nu: REAL_64; a_x: REAL_64; a_K_nu: CE_POINTER [REAL_64]; a_K_nup1: CE_POINTER [REAL_64]; a_Kp_nu: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_max_iter: INTEGER_32
			l_half_x: REAL_64
			l_ln_half_x: REAL_64
			l_half_x_nu: REAL_64
			l_pi_nu: REAL_64
			l_sigma: REAL_64
			l_sinrat: REAL_64
			l_tmp___3: REAL_64
			l_sinhrat: REAL_64
			l_tmp___7: REAL_64
			l_ex: REAL_64
			l_sum0: REAL_64
			l_sum1: REAL_64
			l_fk: REAL_64
			l_pk: REAL_64
			l_qk: REAL_64
			l_hk: REAL_64
			l_ck: REAL_64
			l_k: INTEGER_32
			l_stat_iter: INTEGER_32
			l_g_1pnu: REAL_64
			l_g_1mnu: REAL_64
			l_g1: REAL_64
			l_g2: REAL_64
			l_stat_g: INTEGER_32
			l_del0: REAL_64
			l_del1: REAL_64
			l_tmp___12: REAL_64
			l_tmp___14: INTEGER_32
			l_tmp___15: INTEGER_32
			js: JUMP_STATE
		do
			l_max_iter := 15000
			l_half_x := 0.5 * a_x
			l_ln_half_x := (log (l_half_x))
			l_half_x_nu := (exp (a_nu * l_ln_half_x))
			l_pi_nu := 3.14159265358979323846 * a_nu
			l_sigma := -a_nu * l_ln_half_x
			if (fabs (l_pi_nu)) < 2.2204460492503131e-16 then
				l_tmp___3 := 1.0
			else
				l_tmp___3 := l_pi_nu / (sin (l_pi_nu))
			end
			l_sinrat := l_tmp___3
			if (fabs (l_sigma)) < 2.2204460492503131e-16 then
				l_tmp___7 := 1.0
			else
				l_tmp___7 := (sinh (l_sigma)) / l_sigma
			end
			l_sinhrat := l_tmp___7
			l_ex := (exp (a_x))
			l_k := 0
			l_stat_g := (gsl_sf_temme_gamma (a_nu, create {CE_POINTER [REAL_64]}.make_shared ($l_g_1pnu), create {CE_POINTER [REAL_64]}.make_shared ($l_g_1mnu), create {CE_POINTER [REAL_64]}.make_shared ($l_g1), create {CE_POINTER [REAL_64]}.make_shared ($l_g2)))
			l_fk := l_sinrat * ((cosh (l_sigma)) * l_g1 - (l_sinhrat * l_ln_half_x) * l_g2)
			l_pk := (0.5 / l_half_x_nu) * l_g_1pnu
			l_qk := (0.5 * l_half_x_nu) * l_g_1mnu
			l_hk := l_pk
			l_ck := 1.0
			l_sum0 := l_fk
			l_sum1 := l_hk
			from
			until
				not (not js.loop_left and then (l_k < l_max_iter))
			loop
				js.reset
				l_k := l_k + 1
				l_fk := ((l_k.to_double * l_fk + l_pk) + l_qk) / ((l_k * l_k).to_double - a_nu * a_nu)
				l_ck := l_ck * ((l_half_x * l_half_x) / l_k.to_double)
				l_pk := l_pk / (l_k.to_double - a_nu)
				l_qk := l_qk / (l_k.to_double + a_nu)
				l_hk := (-l_k).to_double * l_fk + l_pk
				l_del0 := l_ck * l_fk
				l_del1 := l_ck * l_hk
				l_sum0 := l_sum0 + l_del0
				l_sum1 := l_sum1 + l_del1
				l_tmp___12 := fabs (l_del0)
				if l_tmp___12 < (0.5 * (fabs (l_sum0))) * 2.2204460492503131e-16 then
					js.break
				end
			end
			js.reset
			a_K_nu.item := l_sum0 * l_ex
			a_K_nup1.item := ((l_sum1 * 2.0) / a_x) * l_ex
			a_Kp_nu.item := -a_K_nup1.item + (a_nu / a_x) * a_K_nu.item
			if l_k = l_max_iter then
				l_stat_iter := 11
			else
				l_stat_iter := 0
			end
			if l_stat_iter /= 0 then
				l_tmp___15 := l_stat_iter
			else
				if l_stat_g /= 0 then
					l_tmp___14 := l_stat_g
				else
					l_tmp___14 := 0
				end
				l_tmp___15 := l_tmp___14
			end
			Result := l_tmp___15
		end

end
