class
	P_BINOMIAL

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_ran_binomial_knuth (a_r___3: CE_POINTER [PD_GSL_RNG]; a_p: REAL_64; a_n: NATURAL_32): NATURAL_32
		local
			l_p: REAL_64
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_k: NATURAL_32
			l_X: REAL_64
			l_u: REAL_64
		do
			l_p := a_p
			l_n := a_n
			l_k := {NATURAL_32} 0
			from
			until
				not (l_n > {NATURAL_32} 10)
			loop
				l_a___5 := {NATURAL_32} 1 + l_n // {NATURAL_32} 2
				l_b___0 := ({NATURAL_32} 1 + l_n) - l_a___5
				l_X := beta_inst.gsl_ran_beta (a_r___3, l_a___5.to_real_64, l_b___0.to_real_64)
				if l_X >= l_p then
					l_n := l_a___5 - {NATURAL_32} 1
					l_p := l_p / l_X
				else
					l_k := l_k + l_a___5
					l_n := l_b___0 - {NATURAL_32} 1
					l_p := (l_p - l_X) / ((1).to_double - l_X)
				end
			end
			l_i := {NATURAL_32} 0
			from
			until
				not (l_i < l_n)
			loop
				l_u := (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3))
				if l_u < l_p then
					l_k := l_k + 1
				end
				l_i := l_i + 1
			end
			Result := l_k
		end

	gsl_ran_binomial_pdf (a_k: NATURAL_32; a_p: REAL_64; a_n: NATURAL_32): REAL_64
		local
			l_P___2: REAL_64
			l_ln_Cnk: REAL_64
			l_tmp___0: REAL_64
		do
			if a_k > a_n then
				Result := (0).to_double
			else
				if a_p = (0).to_double then
					if a_k = {NATURAL_32} 0 then
						l_P___2 := (1).to_double
					else
						l_P___2 := (0).to_double
					end
				elseif a_p = (1).to_double then
					if a_k = a_n then
						l_P___2 := (1).to_double
					else
						l_P___2 := (0).to_double
					end
				else
					l_ln_Cnk := (gamma_inst.gsl_sf_lnchoose (a_n, a_k))
					l_tmp___0 := log (a_p)
					l_P___2 := (l_ln_Cnk + a_k.to_real_64 * l_tmp___0) + (a_n - a_k).to_real_64 * (log1p (-a_p))
					l_P___2 := exp (l_P___2)
				end
				Result := l_P___2
			end
		end

	gsl_cdf_binomial_P (a_k: NATURAL_32; a_p: REAL_64; a_n: NATURAL_32): REAL_64
		local
			l_P___2: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_p > 1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_p < 0.0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("p < 0 or p > 1"), ce_string ("binomial.c"), 67, 1)
							Result := (builtin_nanf (ce_string (""))).to_double
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if done
						if not js.return_called then
							if a_k >= a_n then
								l_P___2 := 1.0
							else
								l_a___5 := a_k.to_real_64 + 1.0
								l_b___0 := a_n.to_real_64 - a_k.to_real_64
								l_P___2 := beta_inst.gsl_cdf_beta_Q (a_p, l_a___5, l_b___0)
							end
							Result := l_P___2
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_cdf_binomial_Q (a_k: NATURAL_32; a_p: REAL_64; a_n: NATURAL_32): REAL_64
		local
			l_Q___1: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_p > 1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_p < 0.0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("p < 0 or p > 1"), ce_string ("binomial.c"), 93, 1)
							Result := (builtin_nanf (ce_string (""))).to_double
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if done
						if not js.return_called then
							if a_k >= a_n then
								l_Q___1 := 0.0
							else
								l_a___5 := a_k.to_real_64 + 1.0
								l_b___0 := a_n.to_real_64 - a_k.to_real_64
								l_Q___1 := beta_inst.gsl_cdf_beta_P (a_p, l_a___5, l_b___0)
							end
							Result := l_Q___1
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
