class
	P_BRENT

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create brent_type.make
			create gsl_min_fminimizer_brent.make
			create brent_type___0.make
			create gsl_root_fsolver_brent.make
		end

feature {ANY} -- attributes

	brent_type: PD_GSL_MIN_FMINIMIZER_TYPE

	gsl_min_fminimizer_brent: CE_POINTER [PD_GSL_MIN_FMINIMIZER_TYPE]

	brent_type___0: PD_GSL_ROOT_FSOLVER_TYPE

	gsl_root_fsolver_brent: CE_POINTER [PD_GSL_ROOT_FSOLVER_TYPE]

feature {ANY} -- routines

	brent_init (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_x_minimum: REAL_64; a_f_minimum: REAL_64; a_x_lower: REAL_64; a_f_lower: REAL_64; a_x_upper: REAL_64; a_f_upper: REAL_64): INTEGER_32
		local
			l_x_minimum: REAL_64
			l_f_minimum: REAL_64
			l_f_lower: REAL_64
			l_f_upper: REAL_64
			l_state: CE_POINTER [PD_BRENT_STATE_T]
			l_golden: REAL_64
			l_v: REAL_64
			l_w: REAL_64
			l_f_vw: REAL_64
			js: JUMP_STATE
		do
			create l_state.make

			l_x_minimum := a_x_minimum
			l_f_minimum := a_f_minimum
			l_f_lower := a_f_lower
			l_f_upper := a_f_upper
			l_state.ce_gets (create {CE_POINTER [PD_BRENT_STATE_T]}.make_cast (a_vstate))
			l_golden := 0.3819660
			l_v := a_x_lower + l_golden * (a_x_upper - a_x_lower)
			l_w := l_v
			l_x_minimum := (0).to_double
			l_f_minimum := (0).to_double
			l_f_lower := (0).to_double
			l_f_upper := (0).to_double
			l_state.item.v := l_v
			l_state.item.w := l_w
			l_state.item.d := (0).to_double
			l_state.item.e := (0).to_double
			from
			until
				js.loop_left
			loop
				js.reset
				l_f_vw := (a_f.item.function).real_64_item ([l_v, a_f.item.params])
				if not (infnan_inst.gsl_finite (l_f_vw)).to_boolean then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("computed function value is infinite or NaN"), ce_string ("brent.c"), 69, 9)
						Result := 9
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
				if not js.return_called then
					js.break
				end
			end
			js.reset
			if not js.return_called then
				l_state.item.f_v := l_f_vw
				l_state.item.f_w := l_f_vw
				Result := 0
			end
		end

	brent_init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent brent_init)
		end

	brent_iterate (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_x_minimum: CE_POINTER [REAL_64]; a_f_minimum: CE_POINTER [REAL_64]; a_x_lower: CE_POINTER [REAL_64]; a_f_lower: CE_POINTER [REAL_64]; a_x_upper: CE_POINTER [REAL_64]; a_f_upper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_BRENT_STATE_T]
			l_x_left: REAL_64
			l_x_right: REAL_64
			l_z: REAL_64
			l_d: REAL_64
			l_e: REAL_64
			l_u: REAL_64
			l_f_u: REAL_64
			l_v: REAL_64
			l_w: REAL_64
			l_f_v: REAL_64
			l_f_w: REAL_64
			l_f_z: REAL_64
			l_golden: REAL_64
			l_w_lower: REAL_64
			l_w_upper: REAL_64
			l_tolerance: REAL_64
			l_p: REAL_64
			l_q___3: REAL_64
			l_r___3: REAL_64
			l_midpoint: REAL_64
			l_t2: REAL_64
			l_tmp___1: REAL_64
			l_tmp___3: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_state.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state.ce_gets (create {CE_POINTER [PD_BRENT_STATE_T]}.make_cast (a_vstate))
					l_x_left := a_x_lower.item
					l_x_right := a_x_upper.item
					l_z := a_x_minimum.item
					l_d := l_state.item.e
					l_e := l_state.item.d
					l_v := l_state.item.v
					l_w := l_state.item.w
					l_f_v := l_state.item.f_v
					l_f_w := l_state.item.f_w
					l_f_z := a_f_minimum.item
					l_golden := 0.3819660
					l_w_lower := l_z - l_x_left
					l_w_upper := l_x_right - l_z
					l_tolerance := 1.4901161193847656e-08 * (fabs (l_z))
					l_p := (0).to_double
					l_q___3 := (0).to_double
					l_r___3 := (0).to_double
					l_midpoint := 0.5 * (l_x_left + l_x_right)
					if (fabs (l_e)) > l_tolerance then
						l_r___3 := (l_z - l_w) * (l_f_z - l_f_v)
						l_q___3 := (l_z - l_v) * (l_f_z - l_f_w)
						l_p := (l_z - l_v) * l_q___3 - (l_z - l_w) * l_r___3
						l_q___3 := (2).to_double * (l_q___3 - l_r___3)
						if l_q___3 > (0).to_double then
							l_p := -l_p
						else
							l_q___3 := -l_q___3
						end
						l_r___3 := l_e
						l_e := l_d
					end
					l_tmp___1 := fabs (l_p)
					if not (l_tmp___1 < (fabs ((0.5 * l_q___3) * l_r___3))) then
						l_pos := -1 + 13 -- skip if block
					end
				when 1 then -- if:
						if not (l_p < l_q___3 * l_w_lower) then
							l_pos := -1 + 11 -- skip if block
						end
				when 2 then -- if:
						if not (l_p < l_q___3 * l_w_upper) then
							l_pos := -1 + 9 -- skip if block
						end
				when 3 then -- if:
						l_t2 := (2).to_double * l_tolerance
						l_d := l_p / l_q___3
						l_u := l_z + l_d
						if not (l_u - l_x_left < l_t2) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if not (l_x_right - l_u < l_t2) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						if l_z < l_midpoint then
							l_d := l_tolerance
						else
							l_d := -l_tolerance
						end
				when 8 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_pos := -1 + 13 -- goto _L___1
				when 10 then -- if done | if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_pos := -1 + 13 -- goto _L___1
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 14
				when 13 then -- else: | _L___1:
						if l_z < l_midpoint then
							l_e := l_x_right - l_z
						else
							l_e := -(l_z - l_x_left)
						end
						l_d := l_golden * l_e
				when 14 then
					if (fabs (l_d)) >= l_tolerance then
						l_u := l_z + l_d
					else
						if l_d > (0).to_double then
							l_tmp___3 := l_tolerance
						else
							l_tmp___3 := -l_tolerance
						end
						l_u := l_z + l_tmp___3
					end
					l_state.item.e := l_e
					l_state.item.d := l_d
					from
					until
						js.loop_left
					loop
						js.reset
						l_f_u := (a_f.item.function).real_64_item ([l_u, a_f.item.params])
						if not (infnan_inst.gsl_finite (l_f_u)).to_boolean then
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("computed function value is infinite or NaN"), ce_string ("brent.c"), 159, 9)
								Result := 9
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						if l_f_u <= l_f_z then
							if l_u < l_z then
								a_x_upper.item := l_z
								a_f_upper.item := l_f_z
							else
								a_x_lower.item := l_z
								a_f_lower.item := l_f_z
							end
							l_state.item.v := l_w
							l_state.item.f_v := l_f_w
							l_state.item.w := l_z
							l_state.item.f_w := l_f_z
							a_x_minimum.item := l_u
							a_f_minimum.item := l_f_u
							Result := 0
							js.return
						else
							if l_u < l_z then
								a_x_lower.item := l_u
								a_f_lower.item := l_f_u
							else
								a_x_upper.item := l_u
								a_f_upper.item := l_f_u
							end
							if l_f_u <= l_f_w then
								l_state.item.v := l_w
								l_state.item.f_v := l_f_w
								l_state.item.w := l_u
								l_state.item.f_w := l_f_u
								Result := 0
								js.return
							elseif l_w = l_z then
								l_state.item.v := l_w
								l_state.item.f_v := l_f_w
								l_state.item.w := l_u
								l_state.item.f_w := l_f_u
								Result := 0
								js.return
							elseif l_f_u <= l_f_v then
								l_state.item.v := l_u
								l_state.item.f_v := l_f_u
								Result := 0
								js.return
							elseif l_v = l_z then
								l_state.item.v := l_u
								l_state.item.f_v := l_f_u
								Result := 0
								js.return
							elseif l_v = l_w then
								l_state.item.v := l_u
								l_state.item.f_v := l_f_u
								Result := 0
								js.return
							end
						end
					end
					if not js.return_called then
						Result := 0
					end
				when 15 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	brent_iterate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent brent_iterate)
		end

	brent_init___0 (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_root: CE_POINTER [REAL_64]; a_x_lower: REAL_64; a_x_upper: REAL_64): INTEGER_32
		local
			l_state: CE_POINTER [PD_BRENT_STATE_T2]
			l_f_lower: REAL_64
			l_f_upper: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_state.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state.ce_gets (create {CE_POINTER [PD_BRENT_STATE_T2]}.make_cast (a_vstate))
					a_root.item := 0.5 * (a_x_lower + a_x_upper)
					from
					until
						js.loop_left
					loop
						js.reset
						l_f_lower := (a_f.item.function).real_64_item ([a_x_lower, a_f.item.params])
						if not (infnan_inst.gsl_finite (l_f_lower)).to_boolean then
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("function value is not finite"), ce_string ("brent.c"), 57, 9)
								Result := 9
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						from
						until
							js.loop_left
						loop
							js.reset
							l_f_upper := (a_f.item.function).real_64_item ([a_x_upper, a_f.item.params])
							if not (infnan_inst.gsl_finite (l_f_upper)).to_boolean then
								from
								until
									js.loop_left
								loop
									js.reset
									error_inst.gsl_error (ce_string ("function value is not finite"), ce_string ("brent.c"), 58, 9)
									Result := 9
									js.return
									if not js.return_called then
										js.break
									end
								end
								js.reset
							end
							if not js.return_called then
								js.break
							end
						end
						js.reset
					end
					if js.return_called then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						l_state.item.a := a_x_lower
						l_state.item.fa := l_f_lower
						l_state.item.b := a_x_upper
						l_state.item.fb := l_f_upper
						l_state.item.c := a_x_upper
						l_state.item.fc := l_f_upper
						l_state.item.d := a_x_upper - a_x_lower
						l_state.item.e := a_x_upper - a_x_lower
						if not (l_f_lower < 0.0) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_f_upper < 0.0 then
							l_pos := -1 + 6 -- goto _L
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 4 then -- else: | _L___0:
						if not (l_f_lower > 0.0) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if not (l_f_upper > 0.0) then
							l_pos := -1 + 7 -- skip if block
						end
				when 6 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("endpoints do not straddle y=0"), ce_string ("brent.c"), 74, 4)
							Result := 4
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 7 then -- if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	brent_init___0_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent brent_init___0)
		end

	brent_iterate___0 (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_root: CE_POINTER [REAL_64]; a_x_lower: CE_POINTER [REAL_64]; a_x_upper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_BRENT_STATE_T2]
			l_tol: REAL_64
			l_m___4: REAL_64
			l_ac_equal: INTEGER_32
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_c___1: REAL_64
			l_fa: REAL_64
			l_fb: REAL_64
			l_fc: REAL_64
			l_d: REAL_64
			l_e: REAL_64
			l_tmp: REAL_64
			l_p: REAL_64
			l_q___3: REAL_64
			l_r___3: REAL_64
			l_s: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___11: REAL_64
			l_tmp___13: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_state.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state.ce_gets (create {CE_POINTER [PD_BRENT_STATE_T2]}.make_cast (a_vstate))
					l_ac_equal := 0
					l_a___5 := l_state.item.a
					l_b___0 := l_state.item.b
					l_c___1 := l_state.item.c
					l_fa := l_state.item.fa
					l_fb := l_state.item.fb
					l_fc := l_state.item.fc
					l_d := l_state.item.d
					l_e := l_state.item.e
					if not (l_fb < (0).to_double) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_fc < (0).to_double then
							l_ac_equal := 1
							l_c___1 := l_a___5
							l_fc := l_fa
							l_d := l_b___0 - l_a___5
							l_e := l_b___0 - l_a___5
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						if l_fb > (0).to_double then
							if l_fc > (0).to_double then
								l_ac_equal := 1
								l_c___1 := l_a___5
								l_fc := l_fa
								l_d := l_b___0 - l_a___5
								l_e := l_b___0 - l_a___5
							end
						end
				when 4 then
					l_tmp := fabs (l_fc)
					if l_tmp < (fabs (l_fb)) then
						l_ac_equal := 1
						l_a___5 := l_b___0
						l_b___0 := l_c___1
						l_c___1 := l_a___5
						l_fa := l_fb
						l_fb := l_fc
						l_fc := l_fa
					end
					l_tol := (0.5 * 2.2204460492503131e-16) * (fabs (l_b___0))
					l_m___4 := 0.5 * (l_c___1 - l_b___0)
					if l_fb = (0).to_double then
						a_root.item := l_b___0
						a_x_lower.item := l_b___0
						a_x_upper.item := l_b___0
						Result := 0
						js.return
					end
					if not js.return_called then
						if (fabs (l_m___4)) <= l_tol then
							a_root.item := l_b___0
							if l_b___0 < l_c___1 then
								a_x_lower.item := l_b___0
								a_x_upper.item := l_c___1
							else
								a_x_lower.item := l_c___1
								a_x_upper.item := l_b___0
							end
							Result := 0
							js.return
						end
					end
					if not js.return_called then
						if (fabs (l_e)) < l_tol then
							l_d := l_m___4
							l_e := l_m___4
						else
							l_tmp___11 := fabs (l_fa)
							if l_tmp___11 <= (fabs (l_fb)) then
								l_d := l_m___4
								l_e := l_m___4
							else
								l_s := l_fb / l_fa
								if l_ac_equal.to_boolean then
									l_p := ((2).to_double * l_m___4) * l_s
									l_q___3 := (1).to_double - l_s
								else
									l_q___3 := l_fa / l_fc
									l_r___3 := l_fb / l_fc
									l_p := l_s * ((((2).to_double * l_m___4) * l_q___3) * (l_q___3 - l_r___3) - (l_b___0 - l_a___5) * (l_r___3 - (1).to_double))
									l_q___3 := ((l_q___3 - (1).to_double) * (l_r___3 - (1).to_double)) * (l_s - (1).to_double)
								end
								if l_p > (0).to_double then
									l_q___3 := -l_q___3
								else
									l_p := -l_p
								end
								l_tmp___8 := fabs (l_tol * l_q___3)
								if ((3).to_double * l_m___4) * l_q___3 - l_tmp___8 < (fabs (l_e * l_q___3)) then
									l_tmp___7 := ((3).to_double * l_m___4) * l_q___3 - (fabs (l_tol * l_q___3))
								else
									l_tmp___7 := (fabs (l_e * l_q___3))
								end
								if (2).to_double * l_p < l_tmp___7 then
									l_e := l_d
									l_d := l_p / l_q___3
								else
									l_d := l_m___4
									l_e := l_m___4
								end
							end
						end
						l_a___5 := l_b___0
						l_fa := l_fb
						if (fabs (l_d)) > l_tol then
							l_b___0 := l_b___0 + l_d
						else
							if l_m___4 > (0).to_double then
								l_tmp___13 := l_tol
							else
								l_tmp___13 := -l_tol
							end
							l_b___0 := l_b___0 + l_tmp___13
						end
						from
						until
							js.loop_left
						loop
							js.reset
							l_fb := (a_f.item.function).real_64_item ([l_b___0, a_f.item.params])
							if not (infnan_inst.gsl_finite (l_fb)).to_boolean then
								from
								until
									js.loop_left
								loop
									js.reset
									error_inst.gsl_error (ce_string ("function value is not finite"), ce_string ("brent.c"), 202, 9)
									Result := 9
									js.return
									if not js.return_called then
										js.break
									end
								end
								js.reset
							end
							if not js.return_called then
								js.break
							end
						end
						js.reset
					end
					if js.return_called then
						l_pos := -1 + 10 -- skip if block
					end
				when 5 then -- if:
						l_state.item.a := l_a___5
						l_state.item.b := l_b___0
						l_state.item.c := l_c___1
						l_state.item.d := l_d
						l_state.item.e := l_e
						l_state.item.fa := l_fa
						l_state.item.fb := l_fb
						l_state.item.fc := l_fc
						a_root.item := l_b___0
						if not (l_fb < (0).to_double) then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if:
						if l_fc < (0).to_double then
							l_c___1 := l_a___5
						else
							l_pos := -1 + 8 -- goto _L___0
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else: | _L___0:
						if l_fb > (0).to_double then
							if l_fc > (0).to_double then
								l_c___1 := l_a___5
							end
						end
				when 9 then
						if l_b___0 < l_c___1 then
							a_x_lower.item := l_b___0
							a_x_upper.item := l_c___1
						else
							a_x_lower.item := l_c___1
							a_x_upper.item := l_b___0
						end
						Result := 0
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	brent_iterate___0_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent brent_iterate___0)
		end

end
