class
	P_CLAUSEN

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create aclaus_data.make (<<15>>)
			create aclaus_cs.make
		end

feature {ANY} -- attributes

	aclaus_data: CE_ARRAY [REAL_64]

	aclaus_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	gsl_sf_clausen_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: REAL_64
			l_x_cut: REAL_64
			l_sgn: REAL_64
			l_status_red: INTEGER_32
			l_p0: REAL_64
			l_p1: REAL_64
			l_t: REAL_64
			l_result_c: PD_GSL_SF_RESULT_STRUCT
		do
			create l_result_c.make

			l_x := a_x
			l_x_cut := 3.14159265358979323846 * 1.4901161193847656e-08
			l_sgn := 1.0
			if l_x < 0.0 then
				l_x := -l_x
				l_sgn := -1.0
			end
			l_status_red := trig_inst.gsl_sf_angle_restrict_pos_e (create {CE_POINTER [REAL_64]}.make_shared ($l_x))
			if l_x > 3.14159265358979323846 then
				l_p0 := 6.28125
				l_p1 := 0.19353071795864769253e-02
				l_x := (l_p0 - l_x) + l_p1
				l_sgn := -l_sgn
			end
			if l_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
			elseif l_x < l_x_cut then
				a_result.item.val := l_x * (1.0 - (log (l_x)))
				a_result.item.err := l_x * 2.2204460492503131e-16
			else
				l_t := 2.0 * ((l_x * l_x) / (3.14159265358979323846 * 3.14159265358979323846) - 0.5)
				cheb_eval_inst.cheb_eval_e ((aclaus_cs.address), l_t, l_result_c.address).do_nothing
				a_result.item.val := l_x * (l_result_c.val - (log (l_x)))
				a_result.item.err := l_x * (l_result_c.err + 2.2204460492503131e-16)
			end
			a_result.item.val := a_result.item.val * l_sgn
			Result := l_status_red
		end

	gsl_sf_clausen (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_clausen_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_clausen_e(x, &result)"), ce_string ("clausen.c"), 110, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
