class
	P_COULOMB

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	C0sq (a_eta: REAL_64): REAL_64
		local
			l_twopieta: REAL_64
			l_scale: PD_GSL_SF_RESULT_STRUCT
		do
			create l_scale.make

			l_twopieta := (2.0 * 3.14159265358979323846) * a_eta
			if (fabs (a_eta)) < 2.2204460492503131e-16 then
				Result := (1.0)
			elseif l_twopieta > 7.0978271289338397e+02 then
				Result := (0.0)
			else
				exp_inst.gsl_sf_expm1_e (l_twopieta, l_scale.address).do_nothing
				Result := (l_twopieta / l_scale.val)
			end
		end

	CLeta (a_L: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ln1: PD_GSL_SF_RESULT_STRUCT
			l_ln2: PD_GSL_SF_RESULT_STRUCT
			l_sgn: REAL_64
			l_arg_val: REAL_64
			l_arg_err: REAL_64
			l_p1: PD_GSL_SF_RESULT_STRUCT
			l_tmp___0: REAL_64
		do
			create l_ln1.make
			create l_ln2.make
			create l_p1.make

			l_sgn := 1.0
			if (fabs (a_eta / (a_L + 1.0))) < 2.2204460492503131e-16 then
				gamma_inst.gsl_sf_lngamma_e (a_L + 1.0, l_ln1.address).do_nothing
			else
				gamma_inst.gsl_sf_lngamma_complex_e (a_L + 1.0, a_eta, l_ln1.address, l_p1.address).do_nothing
			end
			gamma_inst.gsl_sf_lngamma_e (2.0 * (a_L + 1.0), l_ln2.address).do_nothing
			if a_L < -1.0 then
				l_sgn := -l_sgn
			end
			l_arg_val := ((a_L * 0.69314718055994530942 - (0.5 * a_eta) * 3.14159265358979323846) + l_ln1.val) - l_ln2.val
			l_arg_err := l_ln1.err + l_ln2.err
			l_tmp___0 := fabs (a_L * 0.69314718055994530942)
			l_arg_err := l_arg_err + (2.2204460492503131e-16 * (l_tmp___0 + (fabs ((0.5 * a_eta) * 3.14159265358979323846))))
			Result := (exp_inst.gsl_sf_exp_err_e (l_arg_val, l_arg_err, a_result))
		end

	gsl_sf_coulomb_CL_e (a_lam: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_lam <= -1.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("coulomb.c"), 110, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif (fabs (a_lam)) < 2.2204460492503131e-16 then
				a_result.item.val := sqrt ((C0sq (a_eta)))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * a_result.item.val
				Result := 0
				js.return
			else
				Result := (CLeta (a_lam, a_eta, a_result))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_coulomb_CL_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_cl: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_k: INTEGER_32
			l_cl_0: PD_GSL_SF_RESULT_STRUCT
			l_L: REAL_64
		do
			create l_cl_0.make

			gsl_sf_coulomb_CL_e (a_lam_min, a_eta, l_cl_0.address).do_nothing
			a_cl[0] := l_cl_0.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_L := a_lam_min + l_k.to_double
				a_cl[l_k] := (a_cl[(l_k - 1)] * (hypot (l_L, a_eta))) / (l_L * (2.0 * l_L + 1.0))
				l_k := l_k + 1
			end
			Result := 0
		end

	coulomb_connection (a_lam: REAL_64; a_eta: REAL_64; a_cos_phi: CE_POINTER [REAL_64]; a_sin_phi: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_eps: REAL_64
			l_tpl: REAL_64
			l_dth: REAL_64
			l_X: REAL_64
			l_tmp___1: REAL_64
			l_phi: REAL_64
			js: JUMP_STATE
		do
			if a_eta > (-(-7.0839641853226408e+02) / 2.0) * 3.14159265358979323846 - 1.0 then
				a_cos_phi.item := 1.0
				a_sin_phi.item := 0.0
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 242, 15)
					Result := 15
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_eta > -(-3.6043653389117154e+01) / (4.0 * 3.14159265358979323846) then
				l_eps := 2.0 * (exp ((-2.0 * 3.14159265358979323846) * a_eta))
				l_tpl := (tan (3.14159265358979323846 * a_lam))
				l_dth := (l_eps * l_tpl) / (l_tpl * l_tpl + 1.0)
				a_cos_phi.item := -1.0 + (0.5 * l_dth) * l_dth
				a_sin_phi.item := -l_dth
				Result := 0
				js.return
			else
				l_tmp___1 := tanh (3.14159265358979323846 * a_eta)
				l_X := l_tmp___1 / (tan (3.14159265358979323846 * a_lam))
				l_phi := -(atan (l_X)) - (a_lam + 0.5) * 3.14159265358979323846
				a_cos_phi.item := cos (l_phi)
				a_sin_phi.item := sin (l_phi)
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	coulomb_FG_series (a_lam: REAL_64; a_eta: REAL_64; a_x: REAL_64; a_F___0: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_G: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_max_iter: INTEGER_32
			l_ClamA: PD_GSL_SF_RESULT_STRUCT
			l_ClamB: PD_GSL_SF_RESULT_STRUCT
			l_stat_A: INTEGER_32
			l_stat_B: INTEGER_32
			l_tlp1: REAL_64
			l_pow_x: REAL_64
			l_cos_phi_lam: REAL_64
			l_sin_phi_lam: REAL_64
			l_uA_mm2: REAL_64
			l_uA_mm1: REAL_64
			l_uA_m: REAL_64
			l_uB_mm2: REAL_64
			l_uB_mm1: REAL_64
			l_uB_m: REAL_64
			l_A_sum: REAL_64
			l_B_sum: REAL_64
			l_A_abs_del_prev: REAL_64
			l_B_abs_del_prev: REAL_64
			l_FA: PD_GSL_SF_RESULT_STRUCT
			l_FB: PD_GSL_SF_RESULT_STRUCT
			l_m___4: INTEGER_32
			l_stat_conn: INTEGER_32
			l_abs_dA: REAL_64
			l_abs_dB: REAL_64
			l_max_abs_dA: REAL_64
			l_tmp___7: REAL_64
			l_max_abs_dB: REAL_64
			l_tmp___8: REAL_64
			l_abs_A: REAL_64
			l_abs_B: REAL_64
			l_tmp___11: REAL_64
			l_tmp___13: REAL_64
			l_tmp___15: REAL_64
			l_tmp___17: INTEGER_32
			l_tmp___18: INTEGER_32
			js: JUMP_STATE
		do
			create l_ClamA.make
			create l_ClamB.make
			create l_FA.make
			create l_FB.make

			l_max_iter := 800
			l_stat_A := (CLeta (a_lam, a_eta, l_ClamA.address))
			l_stat_B := (CLeta (-a_lam - 1.0, a_eta, l_ClamB.address))
			l_tlp1 := 2.0 * a_lam + 1.0
			l_pow_x := (pow (a_x, a_lam))
			l_uA_mm2 := 1.0
			l_uA_mm1 := (a_x * a_eta) / (a_lam + 1.0)
			l_uB_mm2 := 1.0
			l_uB_mm1 := (-a_x * a_eta) / a_lam
			l_A_sum := l_uA_mm2 + l_uA_mm1
			l_B_sum := l_uB_mm2 + l_uB_mm1
			l_A_abs_del_prev := (fabs (l_A_sum))
			l_B_abs_del_prev := (fabs (l_B_sum))
			l_m___4 := 2
			l_stat_conn := (coulomb_connection (a_lam, a_eta, create {CE_POINTER [REAL_64]}.make_shared ($l_cos_phi_lam), create {CE_POINTER [REAL_64]}.make_shared ($l_sin_phi_lam)))
			if l_stat_conn = 15 then
				a_F___0.item.val := 0.0
				a_F___0.item.err := 0.0
				from
				until
					js.loop_left
				loop
					js.reset
					a_G.item.val := (builtin_inff).to_double
					a_G.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("coulomb.c"), 308, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				from
				until
					not (not js.loop_left and then (l_m___4 < l_max_iter))
				loop
					js.reset
					l_uA_m := (a_x * ((2.0 * a_eta) * l_uA_mm1 - a_x * l_uA_mm2)) / (l_m___4.to_double * (l_m___4.to_double + l_tlp1))
					l_uB_m := (a_x * ((2.0 * a_eta) * l_uB_mm1 - a_x * l_uB_mm2)) / (l_m___4.to_double * (l_m___4.to_double - l_tlp1))
					l_A_sum := l_A_sum + l_uA_m
					l_B_sum := l_B_sum + l_uB_m
					l_abs_dA := fabs (l_uA_m)
					l_abs_dB := fabs (l_uB_m)
					if l_m___4 > 15 then
						if l_abs_dA > l_A_abs_del_prev then
							l_tmp___7 := l_abs_dA
						else
							l_tmp___7 := l_A_abs_del_prev
						end
						l_max_abs_dA := l_tmp___7
						if l_abs_dB > l_B_abs_del_prev then
							l_tmp___8 := l_abs_dB
						else
							l_tmp___8 := l_B_abs_del_prev
						end
						l_max_abs_dB := l_tmp___8
						l_abs_A := (fabs (l_A_sum))
						l_abs_B := (fabs (l_B_sum))
						if l_max_abs_dA / (l_max_abs_dA + l_abs_A) < 4.0 * 2.2204460492503131e-16 then
							if l_max_abs_dB / (l_max_abs_dB + l_abs_B) < 4.0 * 2.2204460492503131e-16 then
								js.break
							end
						end
					end
					if not js.loop_jumped then
						l_A_abs_del_prev := l_abs_dA
						l_B_abs_del_prev := l_abs_dB
						l_uA_mm2 := l_uA_mm1
						l_uA_mm1 := l_uA_m
						l_uB_mm2 := l_uB_mm1
						l_uB_mm1 := l_uB_m
						l_m___4 := l_m___4 + 1
					end
				end
				js.reset
				l_FA.val := ((l_A_sum * l_ClamA.val) * l_pow_x) * a_x
				l_tmp___11 := fabs (l_A_sum)
				l_FA.err := ((l_tmp___11 * l_ClamA.err) * l_pow_x) * a_x + (2.0 * 2.2204460492503131e-16) * (fabs (l_FA.val))
				l_FB.val := (l_B_sum * l_ClamB.val) / l_pow_x
				l_tmp___13 := fabs (l_B_sum)
				l_FB.err := (l_tmp___13 * l_ClamB.err) / l_pow_x + (2.0 * 2.2204460492503131e-16) * (fabs (l_FB.val))
				a_F___0.item.val := l_FA.val
				a_F___0.item.err := l_FA.err
				a_G.item.val := (l_FA.val * l_cos_phi_lam - l_FB.val) / l_sin_phi_lam
				l_tmp___15 := fabs (l_cos_phi_lam)
				a_G.item.err := (l_FA.err * l_tmp___15 + l_FB.err) / (fabs (l_sin_phi_lam))
				if l_m___4 >= l_max_iter then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 356, 11)
						Result := 11
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				else
					if l_stat_A /= 0 then
						l_tmp___18 := l_stat_A
					else
						if l_stat_B /= 0 then
							l_tmp___17 := l_stat_B
						else
							l_tmp___17 := 0
						end
						l_tmp___18 := l_tmp___17
					end
					Result := l_tmp___18
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	coulomb_FG0_series (a_eta: REAL_64; a_x: REAL_64; a_F___0: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_G: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_max_iter: INTEGER_32
			l_x2___1: REAL_64
			l_tex: REAL_64
			l_C0: PD_GSL_SF_RESULT_STRUCT
			l_stat_CL: INTEGER_32
			l_r1pie: PD_GSL_SF_RESULT_STRUCT
			l_psi_stat: INTEGER_32
			l_u_mm2: REAL_64
			l_u_mm1: REAL_64
			l_u_m: REAL_64
			l_v_mm2: REAL_64
			l_v_mm1: REAL_64
			l_v_m: REAL_64
			l_u_sum: REAL_64
			l_v_sum: REAL_64
			l_u_abs_del_prev: REAL_64
			l_v_abs_del_prev: REAL_64
			l_m___4: INTEGER_32
			l_u_sum_err: REAL_64
			l_v_sum_err: REAL_64
			l_ln2x: REAL_64
			l_abs_du: REAL_64
			l_abs_dv: REAL_64
			l_m_mm1: REAL_64
			l_max_abs_du: REAL_64
			l_tmp___6: REAL_64
			l_max_abs_dv: REAL_64
			l_tmp___7: REAL_64
			l_abs_u: REAL_64
			l_abs_v: REAL_64
			l_tmp___13: REAL_64
			l_tmp___14: REAL_64
			l_tmp___15: REAL_64
			l_tmp___17: REAL_64
			l_tmp___20: INTEGER_32
			l_tmp___21: INTEGER_32
			js: JUMP_STATE
		do
			create l_C0.make
			create l_r1pie.make

			l_max_iter := 800
			l_x2___1 := a_x * a_x
			l_tex := (2.0 * a_eta) * a_x
			l_stat_CL := (CLeta (0.0, a_eta, l_C0.address))
			l_psi_stat := (psi_inst.gsl_sf_psi_1piy_e (a_eta, l_r1pie.address))
			l_u_mm2 := 0.0
			l_u_mm1 := a_x
			l_v_mm2 := 1.0
			l_v_mm1 := l_tex * ((2.0 * 0.57721566490153286060651209008 - 1.0) + l_r1pie.val)
			l_u_sum := l_u_mm2 + l_u_mm1
			l_v_sum := l_v_mm2 + l_v_mm1
			l_u_abs_del_prev := (fabs (l_u_sum))
			l_v_abs_del_prev := (fabs (l_v_sum))
			l_m___4 := 2
			l_u_sum_err := (2.0 * 2.2204460492503131e-16) * (fabs (l_u_sum))
			l_v_sum_err := (2.0 * 2.2204460492503131e-16) * (fabs (l_v_sum))
			l_ln2x := (log (2.0 * a_x))
			from
			until
				not (not js.loop_left and then (l_m___4 < l_max_iter))
			loop
				js.reset
				l_m_mm1 := l_m___4.to_double * (l_m___4.to_double - 1.0)
				l_u_m := (l_tex * l_u_mm1 - l_x2___1 * l_u_mm2) / l_m_mm1
				l_v_m := ((l_tex * l_v_mm1 - l_x2___1 * l_v_mm2) - ((2.0 * a_eta) * (2 * l_m___4 - 1).to_double) * l_u_m) / l_m_mm1
				l_u_sum := l_u_sum + l_u_m
				l_v_sum := l_v_sum + l_v_m
				l_abs_du := fabs (l_u_m)
				l_abs_dv := fabs (l_v_m)
				l_u_sum_err := l_u_sum_err + ((2.0 * 2.2204460492503131e-16) * l_abs_du)
				l_v_sum_err := l_v_sum_err + ((2.0 * 2.2204460492503131e-16) * l_abs_dv)
				if l_m___4 > 15 then
					if l_abs_du > l_u_abs_del_prev then
						l_tmp___6 := l_abs_du
					else
						l_tmp___6 := l_u_abs_del_prev
					end
					l_max_abs_du := l_tmp___6
					if l_abs_dv > l_v_abs_del_prev then
						l_tmp___7 := l_abs_dv
					else
						l_tmp___7 := l_v_abs_del_prev
					end
					l_max_abs_dv := l_tmp___7
					l_abs_u := (fabs (l_u_sum))
					l_abs_v := (fabs (l_v_sum))
					if l_max_abs_du / (l_max_abs_du + l_abs_u) < 40.0 * 2.2204460492503131e-16 then
						if l_max_abs_dv / (l_max_abs_dv + l_abs_v) < 40.0 * 2.2204460492503131e-16 then
							js.break
						end
					end
				end
				if not js.loop_jumped then
					l_u_abs_del_prev := l_abs_du
					l_v_abs_del_prev := l_abs_dv
					l_u_mm2 := l_u_mm1
					l_u_mm1 := l_u_m
					l_v_mm2 := l_v_mm1
					l_v_mm1 := l_v_m
					l_m___4 := l_m___4 + 1
				end
			end
			js.reset
			a_F___0.item.val := l_C0.val * l_u_sum
			a_F___0.item.err := l_C0.err * (fabs (l_u_sum))
			a_F___0.item.err := a_F___0.item.err + ((fabs (l_C0.val)) * l_u_sum_err)
			a_F___0.item.err := a_F___0.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_F___0.item.val)))
			a_G.item.val := (l_v_sum + ((2.0 * a_eta) * l_u_sum) * l_ln2x) / l_C0.val
			l_tmp___13 := fabs (l_v_sum)
			l_tmp___14 := fabs (((2.0 * a_eta) * l_u_sum) * l_ln2x)
			l_tmp___15 := fabs (l_C0.val)
			a_G.item.err := ((l_tmp___13 + l_tmp___14) / l_tmp___15) * (fabs (l_C0.err / l_C0.val))
			l_tmp___17 := fabs (((2.0 * a_eta) * l_u_sum_err) * l_ln2x)
			a_G.item.err := a_G.item.err + ((l_v_sum_err + l_tmp___17) / (fabs (l_C0.val)))
			a_G.item.err := a_G.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_G.item.val)))
			if l_m___4 = l_max_iter then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 440, 11)
					Result := 11
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				if l_psi_stat /= 0 then
					l_tmp___21 := l_psi_stat
				else
					if l_stat_CL /= 0 then
						l_tmp___20 := l_stat_CL
					else
						l_tmp___20 := 0
					end
					l_tmp___21 := l_tmp___20
				end
				Result := l_tmp___21
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	coulomb_FGmhalf_series (a_eta: REAL_64; a_x: REAL_64; a_F___0: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_G: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_max_iter: INTEGER_32
			l_rx: REAL_64
			l_x2___1: REAL_64
			l_tex: REAL_64
			l_Cmhalf: PD_GSL_SF_RESULT_STRUCT
			l_stat_CL: INTEGER_32
			l_u_mm2: REAL_64
			l_u_mm1: REAL_64
			l_u_m: REAL_64
			l_v_mm2: REAL_64
			l_v_mm1: REAL_64
			l_v_m: REAL_64
			l_f_sum: REAL_64
			l_g_sum: REAL_64
			l_tmp1: REAL_64
			l_rpsi_1pe: PD_GSL_SF_RESULT_STRUCT
			l_rpsi_1p2e: PD_GSL_SF_RESULT_STRUCT
			l_m___4: INTEGER_32
			l_m2___3: REAL_64
			l_tmp___1: REAL_64
			l_tmp___3: REAL_64
			l_tmp___6: REAL_64
			l_tmp___7: REAL_64
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			js: JUMP_STATE
		do
			create l_Cmhalf.make
			create l_rpsi_1pe.make
			create l_rpsi_1p2e.make

			l_max_iter := 800
			l_rx := (sqrt (a_x))
			l_x2___1 := a_x * a_x
			l_tex := (2.0 * a_eta) * a_x
			l_stat_CL := (CLeta (-0.5, a_eta, l_Cmhalf.address))
			l_u_mm2 := 1.0
			l_u_mm1 := l_tex * l_u_mm2
			l_m___4 := 2
			psi_inst.gsl_sf_psi_1piy_e (a_eta, l_rpsi_1pe.address).do_nothing
			psi_inst.gsl_sf_psi_1piy_e (2.0 * a_eta, l_rpsi_1p2e.address).do_nothing
			l_v_mm2 := ((2.0 * 0.57721566490153286060651209008 - 0.69314718055994530942) - l_rpsi_1pe.val) + 2.0 * l_rpsi_1p2e.val
			l_v_mm1 := l_tex * (l_v_mm2 - 2.0 * l_u_mm2)
			l_f_sum := l_u_mm2 + l_u_mm1
			l_g_sum := l_v_mm2 + l_v_mm1
			from
			until
				not (not js.loop_left and then (l_m___4 < l_max_iter))
			loop
				js.reset
				l_m2___3 := (l_m___4 * l_m___4).to_double
				l_u_m := (l_tex * l_u_mm1 - l_x2___1 * l_u_mm2) / l_m2___3
				l_v_m := ((l_tex * l_v_mm1 - l_x2___1 * l_v_mm2) - (2.0 * l_m___4.to_double) * l_u_m) / l_m2___3
				l_f_sum := l_f_sum + l_u_m
				l_g_sum := l_g_sum + l_v_m
				if l_f_sum /= 0.0 then
					if l_g_sum /= 0.0 then
						l_tmp___1 := fabs (l_u_m / l_f_sum)
						if l_tmp___1 + (fabs (l_v_m / l_g_sum)) < 10.0 * 2.2204460492503131e-16 then
							js.break
						end
					end
				end
				if not js.loop_jumped then
					l_u_mm2 := l_u_mm1
					l_u_mm1 := l_u_m
					l_v_mm2 := l_v_mm1
					l_v_mm1 := l_v_m
					l_m___4 := l_m___4 + 1
				end
			end
			js.reset
			a_F___0.item.val := (l_Cmhalf.val * l_rx) * l_f_sum
			l_tmp___3 := fabs (l_rx * l_f_sum)
			a_F___0.item.err := l_Cmhalf.err * l_tmp___3 + (2.0 * 2.2204460492503131e-16) * (fabs (a_F___0.item.val))
			l_tmp1 := l_f_sum * (log (a_x))
			a_G.item.val := (-l_rx * (l_tmp1 + l_g_sum)) / l_Cmhalf.val
			l_tmp___6 := fabs (l_rx)
			l_tmp___7 := fabs (l_tmp1)
			l_tmp___8 := fabs (l_g_sum)
			l_tmp___9 := fabs (l_Cmhalf.val)
			a_G.item.err := ((l_tmp___6 * (l_tmp___7 + l_tmp___8)) / l_tmp___9) * (fabs (l_Cmhalf.err / l_Cmhalf.val))
			if l_m___4 = l_max_iter then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 503, 11)
					Result := 11
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				Result := l_stat_CL
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	coulomb_F_recur (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_F_lam_max: REAL_64; a_Fp_lam_max: REAL_64; a_F_lam_min: CE_POINTER [REAL_64]; a_Fp_lam_min: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_x_inv: REAL_64
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam_max: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
		do
			l_x_inv := 1.0 / a_x
			l_fcl := a_F_lam_max
			l_fpl := a_Fp_lam_max
			l_lam_max := a_lam_min + a_kmax___0.to_double
			l_lam := l_lam_max
			l_k := a_kmax___0 - 1
			from
			until
				not (l_k >= 0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
				l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
				l_fcl := l_fc_lm1
				l_lam := l_lam - (1.0)
				l_k := l_k - 1
			end
			a_F_lam_min.item := l_fcl
			a_Fp_lam_min.item := l_fpl
			Result := 0
		end

	coulomb_G_recur (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_G_lam_min: REAL_64; a_Gp_lam_min: REAL_64; a_G_lam_max: CE_POINTER [REAL_64]; a_Gp_lam_max: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_x_inv: REAL_64
			l_gcl: REAL_64
			l_gpl: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_gcl1: REAL_64
		do
			l_x_inv := 1.0 / a_x
			l_gcl := a_G_lam_min
			l_gpl := a_Gp_lam_min
			l_lam := a_lam_min + 1.0
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_gcl1 := (l_sl * l_gcl - l_gpl) / l_rl
				l_gpl := l_rl * l_gcl - l_sl * l_gcl1
				l_gcl := l_gcl1
				l_lam := l_lam + (1.0)
				l_k := l_k + 1
			end
			a_G_lam_max.item := l_gcl
			a_Gp_lam_max.item := l_gpl
			Result := 0
		end

	coulomb_CF1 (a_lambda: REAL_64; a_eta: REAL_64; a_x: REAL_64; a_fcl_sign: CE_POINTER [REAL_64]; a_result: CE_POINTER [REAL_64]; a_count: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_CF1_small: REAL_64
			l_CF1_abort: REAL_64
			l_CF1_acc: REAL_64
			l_x_inv: REAL_64
			l_px: REAL_64
			l_pk: REAL_64
			l_F___0: REAL_64
			l_D: REAL_64
			l_C: REAL_64
			l_df: REAL_64
			l_pk1: REAL_64
			l_ek: REAL_64
			l_rk2: REAL_64
			l_tk: REAL_64
			js: JUMP_STATE
		do
			l_CF1_small := 1.e-30
			l_CF1_abort := 1.0e+05
			l_CF1_acc := 2.0 * 2.2204460492503131e-16
			l_x_inv := 1.0 / a_x
			l_px := (a_lambda + 1.0) + l_CF1_abort
			l_pk := a_lambda + 1.0
			l_F___0 := a_eta / l_pk + l_pk * l_x_inv
			a_fcl_sign.item := 1.0
			a_count.item := 0
			if (fabs (l_F___0)) < l_CF1_small then
				l_F___0 := l_CF1_small
			end
			l_D := 0.0
			l_C := l_F___0
			from
			until
				js.loop_left
			loop
				js.reset
				l_pk1 := l_pk + 1.0
				l_ek := a_eta / l_pk
				l_rk2 := 1.0 + l_ek * l_ek
				l_tk := (l_pk + l_pk1) * (l_x_inv + l_ek / l_pk1)
				l_D := l_tk - l_rk2 * l_D
				l_C := l_tk - l_rk2 / l_C
				if (fabs (l_C)) < l_CF1_small then
					l_C := l_CF1_small
				end
				if (fabs (l_D)) < l_CF1_small then
					l_D := l_CF1_small
				end
				l_D := 1.0 / l_D
				l_df := l_D * l_C
				l_F___0 := l_F___0 * l_df
				if l_D < 0.0 then
					a_fcl_sign.item := -a_fcl_sign.item
				end
				l_pk := l_pk1
				if l_pk > l_px then
					a_result.item := l_F___0
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 639, 10)
						Result := 10
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
				if not js.return_called then
					a_count.item := (a_count.item) + 1
					if not ((fabs (l_df - 1.0)) > l_CF1_acc) then
						js.break
					end
				end
			end
			js.reset
			if not js.return_called then
				a_result.item := l_F___0
				Result := 0
			end
		end

	coulomb_CF2 (a_lambda: REAL_64; a_eta: REAL_64; a_x: REAL_64; a_result_P: CE_POINTER [REAL_64]; a_result_Q: CE_POINTER [REAL_64]; a_count: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_status: INTEGER_32
			l_CF2_acc: REAL_64
			l_CF2_abort: REAL_64
			l_wi: REAL_64
			l_x_inv: REAL_64
			l_e2mm1: REAL_64
			l_ar: REAL_64
			l_ai: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_dr: REAL_64
			l_di: REAL_64
			l_dp: REAL_64
			l_dq: REAL_64
			l_A___0: REAL_64
			l_B: REAL_64
			l_C: REAL_64
			l_D: REAL_64
			l_pk: REAL_64
			l_P___2: REAL_64
			l_Q___1: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			js: JUMP_STATE
		do
			l_status := 0
			l_CF2_acc := 4.0 * 2.2204460492503131e-16
			l_CF2_abort := 2.0e+05
			l_wi := 2.0 * a_eta
			l_x_inv := 1.0 / a_x
			l_e2mm1 := a_eta * a_eta + a_lambda * (a_lambda + 1.0)
			l_ar := -l_e2mm1
			l_ai := a_eta
			l_br := 2.0 * (a_x - a_eta)
			l_bi := 2.0
			l_dr := l_br / (l_br * l_br + l_bi * l_bi)
			l_di := -l_bi / (l_br * l_br + l_bi * l_bi)
			l_dp := -l_x_inv * (l_ar * l_di + l_ai * l_dr)
			l_dq := l_x_inv * (l_ar * l_dr - l_ai * l_di)
			l_pk := 0.0
			l_P___2 := 0.0
			l_Q___1 := 1.0 - a_eta * l_x_inv
			a_count.item := 0
			from
			until
				js.loop_left
			loop
				js.reset
				l_P___2 := l_P___2 + l_dp
				l_Q___1 := l_Q___1 + l_dq
				l_pk := l_pk + (2.0)
				l_ar := l_ar + l_pk
				l_ai := l_ai + l_wi
				l_bi := l_bi + (2.0)
				l_D := (l_ar * l_dr - l_ai * l_di) + l_br
				l_di := (l_ai * l_dr + l_ar * l_di) + l_bi
				l_C := 1.0 / (l_D * l_D + l_di * l_di)
				l_dr := l_C * l_D
				l_di := -l_C * l_di
				l_A___0 := (l_br * l_dr - l_bi * l_di) - 1.
				l_B := l_bi * l_dr + l_br * l_di
				l_C := l_dp * l_A___0 - l_dq * l_B
				l_dq := l_dp * l_B + l_dq * l_A___0
				l_dp := l_C
				if l_pk > l_CF2_abort then
					l_status := 10
					js.break
				end
				if not js.loop_jumped then
					a_count.item := (a_count.item) + 1
					l_tmp := fabs (l_dp)
					l_tmp___0 := fabs (l_dq)
					l_tmp___1 := fabs (l_P___2)
					if not (l_tmp + l_tmp___0 > (l_tmp___1 + (fabs (l_Q___1))) * l_CF2_acc) then
						js.break
					end
				end
			end
			js.reset
			if l_Q___1 < (l_CF2_abort * 2.2204460492503131e-16) * (fabs (l_P___2)) then
				l_status := 17
			end
			a_result_P.item := l_P___2
			a_result_Q.item := l_Q___1
			Result := l_status
		end

	coulomb_jwkb (a_lam: REAL_64; a_eta: REAL_64; a_x: REAL_64; a_fjwkb: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_gjwkb: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_exponent: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_llp1: REAL_64
			l_llp1_eff: REAL_64
			l_tmp: REAL_64
			l_rho_ghalf: REAL_64
			l_sinh_arg: REAL_64
			l_sinh_inv: REAL_64
			l_phi: REAL_64
			l_tmp___4: REAL_64
			l_zeta_half: REAL_64
			l_prefactor: REAL_64
			l_F___0: REAL_64
			l_G: REAL_64
			l_F_exp: REAL_64
			l_G_exp: REAL_64
			l_airy_scale_exp: REAL_64
			l_ai: PD_GSL_SF_RESULT_STRUCT
			l_bi: PD_GSL_SF_RESULT_STRUCT
			js: JUMP_STATE
		do
			create l_ai.make
			create l_bi.make

			l_llp1 := a_lam * (a_lam + 1.0) + 6.0 / 35.0
			if l_llp1 > 0.0 then
				l_tmp := l_llp1
			else
				l_tmp := 0.0
			end
			l_llp1_eff := l_tmp
			l_rho_ghalf := (sqrt (a_x * (2.0 * a_eta - a_x) + l_llp1_eff))
			l_sinh_arg := ((sqrt (l_llp1_eff / (a_eta * a_eta + l_llp1_eff))) * l_rho_ghalf) / a_x
			l_sinh_inv := (log (l_sinh_arg + (hypot (1.0, l_sinh_arg))))
			l_tmp___4 := atan2 (l_rho_ghalf, a_x - a_eta)
			l_phi := (fabs ((l_rho_ghalf - a_eta * l_tmp___4) - (sqrt (l_llp1_eff)) * l_sinh_inv))
			l_zeta_half := (pow ((3.0 * l_phi) / 2.0, 1.0 / 3.0))
			l_prefactor := (sqrt (((3.14159265358979323846 * l_phi) * a_x) / (6.0 * l_rho_ghalf)))
			l_F___0 := (l_prefactor * 3.0) / l_zeta_half
			l_G := (l_prefactor * 3.0) / l_zeta_half
			l_airy_scale_exp := l_phi
			airy_inst.gsl_sf_airy_Ai_scaled_e (l_zeta_half * l_zeta_half, (0).to_natural_32, l_ai.address).do_nothing
			airy_inst.gsl_sf_airy_Bi_scaled_e (l_zeta_half * l_zeta_half, (0).to_natural_32, l_bi.address).do_nothing
			l_F___0 := l_F___0 * (l_ai.val)
			l_G := l_G * (l_bi.val)
			l_F_exp := (log (l_F___0)) - l_airy_scale_exp
			l_G_exp := (log (l_G)) + l_airy_scale_exp
			if l_G_exp >= 7.0978271289338397e+02 then
				a_fjwkb.item.val := l_F___0
				a_gjwkb.item.val := l_G
				a_fjwkb.item.err := 1.0e-3 * (fabs (l_F___0))
				a_gjwkb.item.err := 1.0e-3 * (fabs (l_G))
				a_exponent.item := l_airy_scale_exp
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 858, 16)
					Result := 16
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				a_fjwkb.item.val := exp (l_F_exp)
				a_gjwkb.item.val := exp (l_G_exp)
				a_fjwkb.item.err := 1.0e-3 * (fabs (a_fjwkb.item.val))
				a_gjwkb.item.err := 1.0e-3 * (fabs (a_gjwkb.item.val))
				a_exponent.item := 0.0
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_coulomb_wave_FG_e (a_eta: REAL_64; a_x: REAL_64; a_lam_F: REAL_64; a_k_lam_G: INTEGER_32; a_F___0: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_Fp: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_G: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_Gp: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_exp_F: CE_POINTER [REAL_64]; a_exp_G: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_lam_G: REAL_64
			l_C0: PD_GSL_SF_RESULT_STRUCT
			l_tmp: REAL_64
			l_SMALL: REAL_64
			l_N: INTEGER_32
			l_span: INTEGER_32
			l_tmp___1: INTEGER_32
			l_lam_min: REAL_64
			l_F_lam_F: REAL_64
			l_Fp_lam_F: REAL_64
			l_G_lam_G: REAL_64
			l_Gp_lam_G: REAL_64
			l_F_lam_F_err: REAL_64
			l_Fp_lam_F_err: REAL_64
			l_Fp_over_F_lam_F: REAL_64
			l_F_sign_lam_F: REAL_64
			l_F_lam_min_unnorm: REAL_64
			l_Fp_lam_min_unnorm: REAL_64
			l_Fp_over_F_lam_min: REAL_64
			l_F_lam_min: PD_GSL_SF_RESULT_STRUCT
			l_G_lam_min: PD_GSL_SF_RESULT_STRUCT
			l_Gp_lam_min: PD_GSL_SF_RESULT_STRUCT
			l_F_scale: REAL_64
			l_Gerr_frac: REAL_64
			l_F_scale_frac_err: REAL_64
			l_F_unnorm_frac_err: REAL_64
			l_CF1_count: INTEGER_32
			l_stat_CF1: INTEGER_32
			l_stat_ser: INTEGER_32
			l_stat_Fr: INTEGER_32
			l_stat_Gr: INTEGER_32
			l_tmp___8: REAL_64
			l_tmp___12: INTEGER_32
			l_tmp___15: REAL_64
			l_tmp___21: INTEGER_32
			l_tmp___22: INTEGER_32
			l_tmp___23: INTEGER_32
			l_tmp___24: INTEGER_32
			l_F_lam_F___0: PD_GSL_SF_RESULT_STRUCT
			l_G_lam_F: PD_GSL_SF_RESULT_STRUCT
			l_F_lam_G: PD_GSL_SF_RESULT_STRUCT
			l_G_lam_G___0: PD_GSL_SF_RESULT_STRUCT
			l_exp_lam_F: REAL_64
			l_exp_lam_G: REAL_64
			l_stat_lam_F: INTEGER_32
			l_stat_lam_G: INTEGER_32
			l_stat_CF1_lam_F: INTEGER_32
			l_stat_CF1_lam_G: INTEGER_32
			l_CF1_count___0: INTEGER_32
			l_Fp_over_F_lam_F___0: REAL_64
			l_Fp_over_F_lam_G: REAL_64
			l_F_sign_lam_F___0: REAL_64
			l_F_sign_lam_G: REAL_64
			l_tmp___28: REAL_64
			l_tmp___30: INTEGER_32
			l_tmp___31: INTEGER_32
			l_SMALL___0: REAL_64
			l_C: REAL_64
			l_N___0: INTEGER_32
			l_lam_0: REAL_64
			l_tmp___34: INTEGER_32
			l_lam_min___0: REAL_64
			l_tmp___35: REAL_64
			l_F_lam_F___1: REAL_64
			l_Fp_lam_F___0: REAL_64
			l_G_lam_G___1: REAL_64
			l_Gp_lam_G___0: REAL_64
			l_F_lam_min_unnorm___0: REAL_64
			l_Fp_lam_min_unnorm___0: REAL_64
			l_F_lam_min___0: REAL_64
			l_Fp_lam_min: REAL_64
			l_G_lam_min___0: REAL_64
			l_Gp_lam_min___0: REAL_64
			l_Fp_over_F_lam_F___1: REAL_64
			l_Fp_over_F_lam_min___0: REAL_64
			l_F_sign_lam_F___1: REAL_64
			l_F_sign_lam_min: REAL_64
			l_P_lam_min: REAL_64
			l_Q_lam_min: REAL_64
			l_alpha: REAL_64
			l_gamma___0: REAL_64
			l_F_scale___0: REAL_64
			l_CF1_count___1: INTEGER_32
			l_CF2_count: INTEGER_32
			l_stat_CF1___0: INTEGER_32
			l_stat_CF2: INTEGER_32
			l_stat_Fr___0: INTEGER_32
			l_stat_Gr___0: INTEGER_32
			l_F_recur_count: INTEGER_32
			l_G_recur_count: INTEGER_32
			l_err_amplify: REAL_64
			l_tmp___42: INTEGER_32
			l_tmp___43: INTEGER_32
			l_tmp___44: INTEGER_32
			l_tmp___45: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_C0.make
			create l_F_lam_min.make
			create l_G_lam_min.make
			create l_Gp_lam_min.make
			create l_F_lam_F___0.make
			create l_G_lam_F.make
			create l_F_lam_G.make
			create l_G_lam_G___0.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_lam_G := a_lam_F - a_k_lam_G.to_double
					if not (a_x < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L___2
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 25
				when 3 then -- else:
						if not (a_lam_F <= -0.5) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L___2
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 25
				when 6 then -- else:
						if not (l_lam_G <= -0.5) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L___2:
						from
						until
							js.loop_left
						loop
							js.reset
							a_F___0.item.val := 0.0
							a_F___0.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_Fp.item.val := 0.0
							a_Fp.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_G.item.val := 0.0
							a_G.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_Gp.item.val := 0.0
							a_Gp.item.err := 0.0
							js.break
						end
						js.reset
						a_exp_F.item := 0.0
						a_exp_G.item := 0.0
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("coulomb.c"), 912, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 25
				when 9 then -- else:
						if a_x /= 0.0 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						CLeta (0.0, a_eta, l_C0.address).do_nothing
						from
						until
							js.loop_left
						loop
							js.reset
							a_F___0.item.val := 0.0
							a_F___0.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_Fp.item.val := 0.0
							a_Fp.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_G.item.val := 0.0
							a_G.item.err := 0.0
							js.break
						end
						js.reset
						from
						until
							js.loop_left
						loop
							js.reset
							a_Gp.item.val := 0.0
							a_Gp.item.err := 0.0
							js.break
						end
						js.reset
						a_exp_F.item := 0.0
						a_exp_G.item := 0.0
						if a_lam_F = 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								a_Fp.item.val := l_C0.val
								a_Fp.item.err := l_C0.err
								js.break
							end
							js.reset
						end
						if l_lam_G = 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								a_Gp.item.val := 1.0 / l_C0.val
								l_tmp := fabs (l_C0.err / l_C0.val)
								a_Gp.item.err := l_tmp / (fabs (l_C0.val))
								js.break
							end
							js.reset
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("coulomb.c"), 929, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 25
				when 12 then -- else:
						if not (a_x < 1.2) then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if (2.0 * 3.14159265358979323846) * a_eta < 0.9 * -(-7.0839641853226408e+02) then
							if (fabs (a_eta * a_x)) < 10.0 then
								l_SMALL := 1.4901161193847656e-08
								l_N := (a_lam_F + 0.5).truncated_to_integer
								if a_k_lam_G > l_N then
									l_tmp___1 := a_k_lam_G
								else
									l_tmp___1 := l_N
								end
								l_span := l_tmp___1
								l_lam_min := a_lam_F - l_N.to_double
								l_stat_CF1 := (coulomb_CF1 (a_lam_F, a_eta, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_F_sign_lam_F), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_over_F_lam_F), create {CE_POINTER [INTEGER_32]}.make_shared ($l_CF1_count)))
								l_F_lam_F := l_SMALL
								l_Fp_lam_F := l_Fp_over_F_lam_F * l_F_lam_F
								if l_span /= 0 then
									l_stat_Fr := coulomb_F_recur (l_lam_min, l_span, a_eta, a_x, l_F_lam_F, l_Fp_lam_F, create {CE_POINTER [REAL_64]}.make_shared ($l_F_lam_min_unnorm), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_lam_min_unnorm))
								else
									l_F_lam_min_unnorm := l_F_lam_F
									l_Fp_lam_min_unnorm := l_Fp_lam_F
									l_stat_Fr := 0
								end
								if l_lam_min = -0.5 then
									l_stat_ser := coulomb_FGmhalf_series (a_eta, a_x, l_F_lam_min.address, l_G_lam_min.address)
								elseif l_lam_min = 0.0 then
									l_stat_ser := coulomb_FG0_series (a_eta, a_x, l_F_lam_min.address, l_G_lam_min.address)
								elseif l_lam_min = 0.5 then
									a_F___0.item.val := l_F_lam_F
									a_F___0.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_F___0.item.val))
									a_Fp.item.val := l_Fp_lam_F
									a_Fp.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_Fp.item.val))
									a_G.item.val := l_G_lam_G
									a_G.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_G.item.val))
									a_Gp.item.val := l_Gp_lam_G
									a_Gp.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_Gp.item.val))
									a_exp_F.item := 0.0
									a_exp_G.item := 0.0
									from
									until
										js.loop_left
									loop
										js.reset
										error_inst.gsl_error (ce_string ("error"), ce_string ("coulomb.c"), 1000, 7)
										Result := 7
										js.return
										if not js.return_called then
											js.break
										end
									end
									js.reset
								else
									l_stat_ser := coulomb_FG_series (l_lam_min, a_eta, a_x, l_F_lam_min.address, l_G_lam_min.address)
								end
								if not js.return_called then
									l_Fp_over_F_lam_min := l_Fp_lam_min_unnorm / l_F_lam_min_unnorm
									l_Gp_lam_min.val := l_Fp_over_F_lam_min * l_G_lam_min.val - 1.0 / l_F_lam_min.val
									l_Gp_lam_min.err := (fabs (l_Fp_over_F_lam_min)) * l_G_lam_min.err
									l_tmp___8 := fabs (1.0 / l_F_lam_min.val)
									l_Gp_lam_min.err := l_Gp_lam_min.err + (l_tmp___8 * (fabs (l_F_lam_min.err / l_F_lam_min.val)))
									l_F_scale := l_F_lam_min.val / l_F_lam_min_unnorm
									l_F_scale_frac_err := fabs (l_F_lam_min.err / l_F_lam_min.val)
									l_F_unnorm_frac_err := (2.0 * 2.2204460492503131e-16) * ((l_CF1_count + l_span) + 1).to_double
									l_F_lam_F := l_F_lam_F * l_F_scale
									l_F_lam_F_err := (fabs (l_F_lam_F)) * (l_F_unnorm_frac_err + l_F_scale_frac_err)
									l_Fp_lam_F := l_Fp_lam_F * l_F_scale
									l_Fp_lam_F_err := (fabs (l_Fp_lam_F)) * (l_F_unnorm_frac_err + l_F_scale_frac_err)
									if l_N - a_k_lam_G > 0 then
										l_tmp___12 := l_N - a_k_lam_G
									else
										l_tmp___12 := 0
									end
									l_stat_Gr := coulomb_G_recur (l_lam_min, l_tmp___12, a_eta, a_x, l_G_lam_min.val, l_Gp_lam_min.val, create {CE_POINTER [REAL_64]}.make_shared ($l_G_lam_G), create {CE_POINTER [REAL_64]}.make_shared ($l_Gp_lam_G))
									a_F___0.item.val := l_F_lam_F
									a_F___0.item.err := l_F_lam_F_err
									a_F___0.item.err := a_F___0.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (l_F_lam_F)))
									a_Fp.item.val := l_Fp_lam_F
									a_Fp.item.err := l_Fp_lam_F_err
									a_Fp.item.err := a_Fp.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (l_Fp_lam_F)))
									l_tmp___15 := fabs (l_G_lam_min.err / l_G_lam_min.val)
									l_Gerr_frac := l_tmp___15 + (fabs (l_Gp_lam_min.err / l_Gp_lam_min.val))
									a_G.item.val := l_G_lam_G
									a_G.item.err := l_Gerr_frac * (fabs (l_G_lam_G))
									a_G.item.err := a_G.item.err + (((2.0 * (l_CF1_count + 1).to_double) * 2.2204460492503131e-16) * (fabs (a_G.item.val)))
									a_Gp.item.val := l_Gp_lam_G
									a_Gp.item.err := l_Gerr_frac * (fabs (a_Gp.item.val))
									a_Gp.item.err := a_Gp.item.err + (((2.0 * (l_CF1_count + 1).to_double) * 2.2204460492503131e-16) * (fabs (a_Gp.item.val)))
									a_exp_F.item := 0.0
									a_exp_G.item := 0.0
									if l_stat_ser /= 0 then
										l_tmp___24 := l_stat_ser
									else
										if l_stat_CF1 /= 0 then
											l_tmp___23 := l_stat_CF1
										else
											if l_stat_Fr /= 0 then
												l_tmp___22 := l_stat_Fr
											else
												if l_stat_Gr /= 0 then
													l_tmp___21 := l_stat_Gr
												else
													l_tmp___21 := 0
												end
												l_tmp___22 := l_tmp___21
											end
											l_tmp___23 := l_tmp___22
										end
										l_tmp___24 := l_tmp___23
									end
									Result := l_tmp___24
									js.return
								end
							else
								l_pos := -1 + 15 -- goto _L___1
							end
						else
							l_pos := -1 + 15 -- goto _L___1
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 25
				when 15 then -- else: | _L___1:
						if not (a_x < 2.0 * a_eta) then
							l_pos := -1 + 24 -- skip if block
						end
				when 16 then -- if:
						l_stat_lam_F := coulomb_jwkb (a_lam_F, a_eta, a_x, l_F_lam_F___0.address, l_G_lam_F.address, create {CE_POINTER [REAL_64]}.make_shared ($l_exp_lam_F))
						if a_k_lam_G = 0 then
							l_stat_lam_G := l_stat_lam_F
							l_F_lam_G.ce_gets (l_F_lam_F___0)
							l_G_lam_G___0.ce_gets (l_G_lam_F)
							l_exp_lam_G := l_exp_lam_F
						else
							l_stat_lam_G := coulomb_jwkb (l_lam_G, a_eta, a_x, l_F_lam_G.address, l_G_lam_G___0.address, create {CE_POINTER [REAL_64]}.make_shared ($l_exp_lam_G))
						end
						l_stat_CF1_lam_F := coulomb_CF1 (a_lam_F, a_eta, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_F_sign_lam_F___0), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_over_F_lam_F___0), create {CE_POINTER [INTEGER_32]}.make_shared ($l_CF1_count___0))
						if a_k_lam_G = 0 then
							l_stat_CF1_lam_G := l_stat_CF1_lam_F
							l_F_sign_lam_G := l_F_sign_lam_F___0
							l_Fp_over_F_lam_G := l_Fp_over_F_lam_F___0
						else
							l_stat_CF1_lam_G := coulomb_CF1 (l_lam_G, a_eta, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_F_sign_lam_G), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_over_F_lam_G), create {CE_POINTER [INTEGER_32]}.make_shared ($l_CF1_count___0))
						end
						a_F___0.item.val := l_F_lam_F___0.val
						a_F___0.item.err := l_F_lam_F___0.err
						a_G.item.val := l_G_lam_G___0.val
						a_G.item.err := l_G_lam_G___0.err
						a_Fp.item.val := l_Fp_over_F_lam_F___0 * l_F_lam_F___0.val
						a_Fp.item.err := (fabs (l_Fp_over_F_lam_F___0)) * l_F_lam_F___0.err
						a_Fp.item.err := a_Fp.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_Fp.item.val)))
						a_Gp.item.val := l_Fp_over_F_lam_G * l_G_lam_G___0.val - 1.0 / l_F_lam_G.val
						a_Gp.item.err := (fabs (l_Fp_over_F_lam_G)) * l_G_lam_G___0.err
						l_tmp___28 := fabs (1.0 / l_F_lam_G.val)
						a_Gp.item.err := a_Gp.item.err + (l_tmp___28 * (fabs (l_F_lam_G.err / l_F_lam_G.val)))
						a_exp_F.item := l_exp_lam_F
						a_exp_G.item := l_exp_lam_G
						if l_stat_lam_F /= 16 then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if:
						l_pos := -1 + 20 -- goto _L
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 23
				when 19 then -- else:
						if l_stat_lam_G /= 16 then
							l_pos := -1 + 22 -- skip if block
						end
				when 20 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("overflow"), ce_string ("coulomb.c"), 1107, 16)
							Result := 16
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 21 then -- if block done, skip else block:
						l_pos := -1 + 23
				when 22 then -- else:
						if l_stat_lam_F /= 0 then
							l_tmp___31 := l_stat_lam_F
						else
							if l_stat_lam_G /= 0 then
								l_tmp___30 := l_stat_lam_G
							else
								l_tmp___30 := 0
							end
							l_tmp___31 := l_tmp___30
						end
						Result := l_tmp___31
						js.return
				when 23 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 25
				when 24 then -- else:
						l_SMALL___0 := 1.4901161193847656e-08
						l_C := (sqrt (1.0 + (4.0 * a_x) * (a_x - 2.0 * a_eta)))
						l_N___0 := (ceil ((a_lam_F - l_C) + 0.5)).truncated_to_integer
						if l_N___0 > 0 then
							l_tmp___34 := l_N___0
						else
							l_tmp___34 := 0
						end
						l_lam_0 := a_lam_F - l_tmp___34.to_double
						if l_lam_0 < l_lam_G then
							l_tmp___35 := l_lam_0
						else
							l_tmp___35 := l_lam_G
						end
						l_lam_min___0 := l_tmp___35
						l_stat_CF1___0 := (coulomb_CF1 (a_lam_F, a_eta, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_F_sign_lam_F___1), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_over_F_lam_F___1), create {CE_POINTER [INTEGER_32]}.make_shared ($l_CF1_count___1)))
						l_F_lam_F___1 := l_F_sign_lam_F___1 * l_SMALL___0
						l_Fp_lam_F___0 := l_Fp_over_F_lam_F___1 * l_F_lam_F___1
						if a_k_lam_G > l_N___0 then
							l_F_recur_count := a_k_lam_G
						else
							l_F_recur_count := l_N___0
						end
						l_stat_Fr___0 := coulomb_F_recur (l_lam_min___0, l_F_recur_count, a_eta, a_x, l_F_lam_F___1, l_Fp_lam_F___0, create {CE_POINTER [REAL_64]}.make_shared ($l_F_lam_min_unnorm___0), create {CE_POINTER [REAL_64]}.make_shared ($l_Fp_lam_min_unnorm___0))
						l_Fp_over_F_lam_min___0 := l_Fp_lam_min_unnorm___0 / l_F_lam_min_unnorm___0
						l_stat_CF2 := coulomb_CF2 (l_lam_min___0, a_eta, a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_P_lam_min), create {CE_POINTER [REAL_64]}.make_shared ($l_Q_lam_min), create {CE_POINTER [INTEGER_32]}.make_shared ($l_CF2_count))
						l_alpha := l_Fp_over_F_lam_min___0 - l_P_lam_min
						l_gamma___0 := l_alpha / l_Q_lam_min
						if l_F_lam_min_unnorm___0 >= 0.0 then
							l_F_sign_lam_min := (1).to_double
						else
							l_F_sign_lam_min := (-1).to_double
						end
						l_F_lam_min___0 := l_F_sign_lam_min / (sqrt ((l_alpha * l_alpha) / l_Q_lam_min + l_Q_lam_min))
						l_Fp_lam_min := l_Fp_over_F_lam_min___0 * l_F_lam_min___0
						l_G_lam_min___0 := l_gamma___0 * l_F_lam_min___0
						l_Gp_lam_min___0 := (l_P_lam_min * l_gamma___0 - l_Q_lam_min) * l_F_lam_min___0
						l_F_scale___0 := l_F_lam_min___0 / l_F_lam_min_unnorm___0
						l_F_lam_F___1 := l_F_lam_F___1 * l_F_scale___0
						l_Fp_lam_F___0 := l_Fp_lam_F___0 * l_F_scale___0
						if l_N___0 - a_k_lam_G > 0 then
							l_G_recur_count := l_N___0 - a_k_lam_G
						else
							l_G_recur_count := 0
						end
						l_stat_Gr___0 := coulomb_G_recur (l_lam_min___0, l_G_recur_count, a_eta, a_x, l_G_lam_min___0, l_Gp_lam_min___0, create {CE_POINTER [REAL_64]}.make_shared ($l_G_lam_G___1), create {CE_POINTER [REAL_64]}.make_shared ($l_Gp_lam_G___0))
						l_err_amplify := ((((l_CF1_count___1 + l_CF2_count) + l_F_recur_count) + l_G_recur_count) + 1).to_double
						a_F___0.item.val := l_F_lam_F___1
						a_F___0.item.err := ((8.0 * l_err_amplify) * 2.2204460492503131e-16) * (fabs (a_F___0.item.val))
						a_Fp.item.val := l_Fp_lam_F___0
						a_Fp.item.err := ((8.0 * l_err_amplify) * 2.2204460492503131e-16) * (fabs (a_Fp.item.val))
						a_G.item.val := l_G_lam_G___1
						a_G.item.err := ((8.0 * l_err_amplify) * 2.2204460492503131e-16) * (fabs (a_G.item.val))
						a_Gp.item.val := l_Gp_lam_G___0
						a_Gp.item.err := ((8.0 * l_err_amplify) * 2.2204460492503131e-16) * (fabs (a_Gp.item.val))
						a_exp_F.item := 0.0
						a_exp_G.item := 0.0
						if l_stat_CF1___0 /= 0 then
							l_tmp___45 := l_stat_CF1___0
						else
							if l_stat_CF2 /= 0 then
								l_tmp___44 := l_stat_CF2
							else
								if l_stat_Fr___0 /= 0 then
									l_tmp___43 := l_stat_Fr___0
								else
									if l_stat_Gr___0 /= 0 then
										l_tmp___42 := l_stat_Gr___0
									else
										l_tmp___42 := 0
									end
									l_tmp___43 := l_tmp___42
								end
								l_tmp___44 := l_tmp___43
							end
							l_tmp___45 := l_tmp___44
						end
						Result := l_tmp___45
						js.return
				when 25 then -- if done | if done | if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 26 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_coulomb_wave_F_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_fc_array: CE_POINTER [REAL_64]; a_F_exp: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_k: INTEGER_32
			l_f_0: PD_GSL_SF_RESULT_STRUCT
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: PD_GSL_SF_RESULT_STRUCT
			l_Fp: PD_GSL_SF_RESULT_STRUCT
			l_G: PD_GSL_SF_RESULT_STRUCT
			l_Gp: PD_GSL_SF_RESULT_STRUCT
			l_G_exp: REAL_64
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k___0: INTEGER_32
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
		do
			create l_f_0.make
			create l_F___0.make
			create l_Fp.make
			create l_G.make
			create l_Gp.make

			if a_x = 0.0 then
				a_F_exp.item := 0.0
				l_k := 0
				from
				until
					not (l_k <= a_kmax___0)
				loop
					a_fc_array[l_k] := 0.0
					l_k := l_k + 1
				end
				if a_lam_min = 0.0 then
					CLeta (0.0, a_eta, l_f_0.address).do_nothing
					a_fc_array[0] := l_f_0.val
				end
				Result := 0
			else
				l_x_inv := 1.0 / a_x
				l_lam_max := a_lam_min + a_kmax___0.to_double
				l_stat_FG := (gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, 0, l_F___0.address, l_Fp.address, l_G.address, l_Gp.address, a_F_exp, create {CE_POINTER [REAL_64]}.make_shared ($l_G_exp)))
				l_fcl := l_F___0.val
				l_fpl := l_Fp.val
				l_lam := l_lam_max
				a_fc_array[a_kmax___0] := l_F___0.val
				l_k___0 := a_kmax___0 - 1
				from
				until
					not (l_k___0 >= 0)
				loop
					l_el := a_eta / l_lam
					l_rl := (hypot (1.0, l_el))
					l_sl := l_el + l_lam * l_x_inv
					l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
					a_fc_array[l_k___0] := l_fc_lm1
					l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
					l_fcl := l_fc_lm1
					l_lam := l_lam - (1.0)
					l_k___0 := l_k___0 - 1
				end
				Result := l_stat_FG
			end
		end

	gsl_sf_coulomb_wave_FG_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_fc_array: CE_POINTER [REAL_64]; a_gc_array: CE_POINTER [REAL_64]; a_F_exp: CE_POINTER [REAL_64]; a_G_exp: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: PD_GSL_SF_RESULT_STRUCT
			l_Fp: PD_GSL_SF_RESULT_STRUCT
			l_G: PD_GSL_SF_RESULT_STRUCT
			l_Gp: PD_GSL_SF_RESULT_STRUCT
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_gcl: REAL_64
			l_gpl: REAL_64
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
			l_el___0: REAL_64
			l_rl___0: REAL_64
			l_sl___0: REAL_64
			l_gcl1: REAL_64
		do
			create l_F___0.make
			create l_Fp.make
			create l_G.make
			create l_Gp.make

			l_x_inv := 1.0 / a_x
			l_lam_max := a_lam_min + a_kmax___0.to_double
			l_stat_FG := (gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, a_kmax___0, l_F___0.address, l_Fp.address, l_G.address, l_Gp.address, a_F_exp, a_G_exp))
			l_fcl := l_F___0.val
			l_fpl := l_Fp.val
			l_lam := l_lam_max
			a_fc_array[a_kmax___0] := l_F___0.val
			l_k := a_kmax___0 - 1
			from
			until
				not (l_k >= 0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
				a_fc_array[l_k] := l_fc_lm1
				l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
				l_fcl := l_fc_lm1
				l_lam := l_lam - (1.0)
				l_k := l_k - 1
			end
			l_gcl := l_G.val
			l_gpl := l_Gp.val
			l_lam := a_lam_min + 1.0
			a_gc_array[0] := l_G.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_el___0 := a_eta / l_lam
				l_rl___0 := (hypot (1.0, l_el___0))
				l_sl___0 := l_el___0 + l_lam * l_x_inv
				l_gcl1 := (l_sl___0 * l_gcl - l_gpl) / l_rl___0
				a_gc_array[l_k] := l_gcl1
				l_gpl := l_rl___0 * l_gcl - l_sl___0 * l_gcl1
				l_gcl := l_gcl1
				l_lam := l_lam + (1.0)
				l_k := l_k + 1
			end
			Result := l_stat_FG
		end

	gsl_sf_coulomb_wave_FGp_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_fc_array: CE_POINTER [REAL_64]; a_fcp_array: CE_POINTER [REAL_64]; a_gc_array: CE_POINTER [REAL_64]; a_gcp_array: CE_POINTER [REAL_64]; a_F_exp: CE_POINTER [REAL_64]; a_G_exp: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: PD_GSL_SF_RESULT_STRUCT
			l_Fp: PD_GSL_SF_RESULT_STRUCT
			l_G: PD_GSL_SF_RESULT_STRUCT
			l_Gp: PD_GSL_SF_RESULT_STRUCT
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_gcl: REAL_64
			l_gpl: REAL_64
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
			l_el___0: REAL_64
			l_rl___0: REAL_64
			l_sl___0: REAL_64
			l_gcl1: REAL_64
		do
			create l_F___0.make
			create l_Fp.make
			create l_G.make
			create l_Gp.make

			l_x_inv := 1.0 / a_x
			l_lam_max := a_lam_min + a_kmax___0.to_double
			l_stat_FG := (gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, a_kmax___0, l_F___0.address, l_Fp.address, l_G.address, l_Gp.address, a_F_exp, a_G_exp))
			l_fcl := l_F___0.val
			l_fpl := l_Fp.val
			l_lam := l_lam_max
			a_fc_array[a_kmax___0] := l_F___0.val
			a_fcp_array[a_kmax___0] := l_Fp.val
			l_k := a_kmax___0 - 1
			from
			until
				not (l_k >= 0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
				a_fc_array[l_k] := l_fc_lm1
				l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
				a_fcp_array[l_k] := l_fpl
				l_fcl := l_fc_lm1
				l_lam := l_lam - (1.0)
				l_k := l_k - 1
			end
			l_gcl := l_G.val
			l_gpl := l_Gp.val
			l_lam := a_lam_min + 1.0
			a_gc_array[0] := l_G.val
			a_gcp_array[0] := l_Gp.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_el___0 := a_eta / l_lam
				l_rl___0 := (hypot (1.0, l_el___0))
				l_sl___0 := l_el___0 + l_lam * l_x_inv
				l_gcl1 := (l_sl___0 * l_gcl - l_gpl) / l_rl___0
				a_gc_array[l_k] := l_gcl1
				l_gpl := l_rl___0 * l_gcl - l_sl___0 * l_gcl1
				a_gcp_array[l_k] := l_gpl
				l_gcl := l_gcl1
				l_lam := l_lam + (1.0)
				l_k := l_k + 1
			end
			Result := l_stat_FG
		end

	gsl_sf_coulomb_wave_sphF_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_x: REAL_64; a_fc_array: CE_POINTER [REAL_64]; a_F_exp: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_k: INTEGER_32
			l_k___0: INTEGER_32
			l_stat_F: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_lam_min < -0.5) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("coulomb.c"), 1387, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x < 10.0 / 1.7976931348623157e+308 then
							l_k := 0
							from
							until
								not (l_k <= a_kmax___0)
							loop
								a_fc_array[l_k] := 0.0
								l_k := l_k + 1
							end
							if a_lam_min = 0.0 then
								a_fc_array[0] := sqrt ((C0sq (a_eta)))
							end
							a_F_exp.item := 0.0
							if a_x = 0.0 then
								Result := 0
								js.return
							else
								from
								until
									js.loop_left
								loop
									js.reset
									error_inst.gsl_error (ce_string ("underflow"), ce_string ("coulomb.c"), 1401, 15)
									Result := 15
									js.return
									if not js.return_called then
										js.break
									end
								end
								js.reset
							end
						else
							l_stat_F := (gsl_sf_coulomb_wave_F_array (a_lam_min, a_kmax___0, a_eta, a_x, a_fc_array, a_F_exp))
							l_k___0 := 0
							from
							until
								not (l_k___0 <= a_kmax___0)
							loop
								a_fc_array[l_k___0] := a_fc_array[l_k___0] / a_x
								l_k___0 := l_k___0 + 1
							end
							Result := l_stat_F
							js.return
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
