class
	P_COVARIANCE_SOURCE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	compute_long_double_covariance (a_data1: CE_POINTER [REAL_96]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_96]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := (a_data1[(l_i * a_stride1).to_integer_32] - real_96_from_real_64 (a_mean1))
				l_delta2 := (a_data2[(l_i * a_stride2).to_integer_32] - real_96_from_real_64 (a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_long_double_covariance_m (a_data1: CE_POINTER [REAL_96]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_96]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_long_double_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_long_double_covariance (a_data1: CE_POINTER [REAL_96]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_96]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_long_double_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_long_double_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_long_double_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_long_double_correlation (a_data1: CE_POINTER [REAL_96]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_96]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := a_data1[({NATURAL_32} 0).to_integer_32]
			l_mean_y := a_data2[({NATURAL_32} 0).to_integer_32]
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := (a_data1[(l_i * a_stride1).to_integer_32] - l_mean_x)
				l_delta_y := (a_data2[(l_i * a_stride2).to_integer_32] - l_mean_y)
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_covariance (a_data1: CE_POINTER [REAL_64]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_64]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 ((a_data1[(l_i * a_stride1).to_integer_32] - a_mean1))
				l_delta2 := real_96_from_real_64 ((a_data2[(l_i * a_stride2).to_integer_32] - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_covariance_m (a_data1: CE_POINTER [REAL_64]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_64]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_covariance (a_data1: CE_POINTER [REAL_64]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_64]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_correlation (a_data1: CE_POINTER [REAL_64]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_64]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_real_64 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_real_64 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_real_64 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_real_64 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_float_covariance (a_data1: CE_POINTER [REAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_double - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_double - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_float_covariance_m (a_data1: CE_POINTER [REAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_float_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_float_covariance (a_data1: CE_POINTER [REAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_float_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_float_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_float_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_float_correlation (a_data1: CE_POINTER [REAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [REAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_real_32 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_real_32 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_real_32 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_real_32 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_ulong_covariance (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_real_64 - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_real_64 - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_ulong_covariance_m (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_ulong_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_ulong_covariance (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_ulong_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_ulong_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_ulong_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_ulong_correlation (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_natural_32 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_natural_32 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_natural_32 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_natural_32 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_long_covariance (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_double - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_double - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_long_covariance_m (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_long_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_long_covariance (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_long_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_long_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_long_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_long_correlation (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_integer_32 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_integer_32 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_integer_32 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_integer_32 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_uint_covariance (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_real_64 - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_real_64 - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_uint_covariance_m (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_uint_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_uint_covariance (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_uint_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_uint_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_uint_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_uint_correlation (a_data1: CE_POINTER [NATURAL_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_natural_32 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_natural_32 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_natural_32 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_natural_32 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_int_covariance (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_double - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_double - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_int_covariance_m (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_int_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_int_covariance (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_int_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_int_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_int_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_int_correlation (a_data1: CE_POINTER [INTEGER_32]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_32]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_integer_32 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_integer_32 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_integer_32 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_integer_32 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_ushort_covariance (a_data1: CE_POINTER [NATURAL_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_16]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_real_64 - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_real_64 - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_ushort_covariance_m (a_data1: CE_POINTER [NATURAL_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_16]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_ushort_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_ushort_covariance (a_data1: CE_POINTER [NATURAL_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_16]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_ushort_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_ushort_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_ushort_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_ushort_correlation (a_data1: CE_POINTER [NATURAL_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_16]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_natural_16 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_natural_16 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_natural_16 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_natural_16 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_short_covariance (a_data1: CE_POINTER [INTEGER_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_16]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_double - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_double - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_short_covariance_m (a_data1: CE_POINTER [INTEGER_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_16]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_short_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_short_covariance (a_data1: CE_POINTER [INTEGER_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_16]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_short_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_short_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_short_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_short_correlation (a_data1: CE_POINTER [INTEGER_16]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_16]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_integer_16 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_integer_16 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_integer_16 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_integer_16 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_uchar_covariance (a_data1: CE_POINTER [NATURAL_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_8]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_real_64 - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_real_64 - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_uchar_covariance_m (a_data1: CE_POINTER [NATURAL_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_8]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_uchar_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_uchar_covariance (a_data1: CE_POINTER [NATURAL_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_8]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_uchar_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_uchar_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_uchar_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_uchar_correlation (a_data1: CE_POINTER [NATURAL_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [NATURAL_8]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_natural_8 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_natural_8 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_natural_8 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_natural_8 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

	compute_char_covariance (a_data1: CE_POINTER [INTEGER_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_8]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_96
			l_i: NATURAL_32
			l_delta1: REAL_96
			l_delta2: REAL_96
		do
			l_covariance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta1 := real_96_from_real_64 (((a_data1[(l_i * a_stride1).to_integer_32]).to_double - a_mean1))
				l_delta2 := real_96_from_real_64 (((a_data2[(l_i * a_stride2).to_integer_32]).to_double - a_mean2))
				l_covariance := l_covariance + ((l_delta1 * l_delta2 - l_covariance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_covariance.truncated_to_real_64
		end

	gsl_stats_char_covariance_m (a_data1: CE_POINTER [INTEGER_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_8]; a_stride2: NATURAL_32; a_n: NATURAL_32; a_mean1: REAL_64; a_mean2: REAL_64): REAL_64
		local
			l_covariance: REAL_64
		do
			l_covariance := (compute_char_covariance (a_data1, a_stride1, a_data2, a_stride2, a_n, a_mean1, a_mean2))
			Result := (l_covariance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_char_covariance (a_data1: CE_POINTER [INTEGER_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_8]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_mean1: REAL_64
			l_mean2: REAL_64
		do
			l_mean1 := (mean_source_inst.gsl_stats_char_mean (a_data1, a_stride1, a_n))
			l_mean2 := (mean_source_inst.gsl_stats_char_mean (a_data2, a_stride2, a_n))
			Result := (gsl_stats_char_covariance_m (a_data1, a_stride1, a_data2, a_stride2, a_n, l_mean1, l_mean2))
		end

	gsl_stats_char_correlation (a_data1: CE_POINTER [INTEGER_8]; a_stride1: NATURAL_32; a_data2: CE_POINTER [INTEGER_8]; a_stride2: NATURAL_32; a_n: NATURAL_32): REAL_64
		local
			l_i: NATURAL_32
			l_sum_xsq: REAL_96
			l_sum_ysq: REAL_96
			l_sum_cross: REAL_96
			l_ratio: REAL_96
			l_delta_x: REAL_96
			l_delta_y: REAL_96
			l_mean_x: REAL_96
			l_mean_y: REAL_96
			l_r___3: REAL_96
			l_tmp: REAL_64
		do
			l_sum_xsq := real_96_from_real_64 (0.0)
			l_sum_ysq := real_96_from_real_64 (0.0)
			l_sum_cross := real_96_from_real_64 (0.0)
			l_mean_x := real_96_from_integer_8 (a_data1[({NATURAL_32} 0).to_integer_32])
			l_mean_y := real_96_from_integer_8 (a_data2[({NATURAL_32} 0).to_integer_32])
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_ratio := real_96_from_real_64 ((l_i.to_real_64 / (l_i.to_real_64 + 1.0)))
				l_delta_x := real_96_from_integer_8 (a_data1[(l_i * a_stride1).to_integer_32]) - l_mean_x
				l_delta_y := real_96_from_integer_8 (a_data2[(l_i * a_stride2).to_integer_32]) - l_mean_y
				l_sum_xsq := l_sum_xsq + ((l_delta_x * l_delta_x) * l_ratio)
				l_sum_ysq := l_sum_ysq + ((l_delta_y * l_delta_y) * l_ratio)
				l_sum_cross := l_sum_cross + ((l_delta_x * l_delta_y) * l_ratio)
				l_mean_x := l_mean_x + (l_delta_x / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_mean_y := l_mean_y + (l_delta_y / real_96_from_real_64 ((l_i.to_real_64 + 1.0)))
				l_i := l_i + 1
			end
			l_tmp := sqrt (l_sum_xsq.truncated_to_real_64)
			l_r___3 := l_sum_cross / real_96_from_real_64 ((l_tmp * (sqrt (l_sum_ysq.truncated_to_real_64))))
			Result := l_r___3.truncated_to_real_64
		end

end
