class
	P_DFT_SOURCE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_dft_complex_forward (a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_dft_complex_transform (a_data, a_stride, a_n, a_result, l_sign))
			Result := l_status
		end

	gsl_dft_complex_backward (a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_dft_complex_transform (a_data, a_stride, a_n, a_result, l_sign))
			Result := l_status
		end

	gsl_dft_complex_inverse (a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_64
			l_i: NATURAL_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_dft_complex_transform (a_data, a_stride, a_n, a_result, l_sign))
			l_norm___0 := 1.0 / a_n.to_real_64
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
				a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
				l_i := l_i + 1
			end
			Result := l_status
		end

	gsl_dft_complex_transform (a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_64]; a_sign: INTEGER_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_exponent: NATURAL_32
			l_d_theta: REAL_64
			l_sum_real: REAL_64
			l_sum_imag: REAL_64
			l_theta: REAL_64
			l_w_real: REAL_64
			l_w_imag: REAL_64
			l_data_real: REAL_64
			l_data_imag: REAL_64
			js: JUMP_STATE
		do
			l_d_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / a_n.to_real_64
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < a_n))
			loop
				l_sum_real := (0).to_double
				l_sum_imag := (0).to_double
				l_exponent := (0).to_natural_32
				l_j := (0).to_natural_32
				from
				until
					not (l_j < a_n)
				loop
					l_theta := l_d_theta * l_exponent.to_real_64
					l_w_real := (cos (l_theta))
					l_w_imag := (sin (l_theta))
					l_data_real := a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
					l_data_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
					l_sum_real := l_sum_real + (l_w_real * l_data_real - l_w_imag * l_data_imag)
					l_sum_imag := l_sum_imag + (l_w_real * l_data_imag + l_w_imag * l_data_real)
					l_exponent := (l_exponent + l_i) \\ a_n
					l_j := l_j + 1
				end
				a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_sum_real
				a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_sum_imag
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_dft_complex_float_forward (a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_32]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := -1
			l_status := (gsl_dft_complex_float_transform (a_data, a_stride, a_n, a_result, l_sign))
			Result := l_status
		end

	gsl_dft_complex_float_backward (a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_32]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_dft_complex_float_transform (a_data, a_stride, a_n, a_result, l_sign))
			Result := l_status
		end

	gsl_dft_complex_float_inverse (a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_32]): INTEGER_32
		local
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_norm___0: REAL_32
			l_i: NATURAL_32
		do
			l_sign := {PC_GSL_FFT_DIRECTION}.gsl_fft_backward
			l_status := (gsl_dft_complex_float_transform (a_data, a_stride, a_n, a_result, l_sign))
			l_norm___0 := {REAL_32} 1.0 / a_n.to_real_32
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] * l_norm___0
				a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] * l_norm___0
				l_i := l_i + 1
			end
			Result := l_status
		end

	gsl_dft_complex_float_transform (a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32; a_result: CE_POINTER [REAL_32]; a_sign: INTEGER_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_exponent: NATURAL_32
			l_d_theta: REAL_64
			l_sum_real: REAL_32
			l_sum_imag: REAL_32
			l_theta: REAL_64
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_data_real: REAL_32
			l_data_imag: REAL_32
			js: JUMP_STATE
		do
			l_d_theta := ((2.0 * a_sign.to_double) * 3.14159265358979323846) / a_n.to_real_64
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < a_n))
			loop
				l_sum_real := (0).to_real
				l_sum_imag := (0).to_real
				l_exponent := (0).to_natural_32
				l_j := (0).to_natural_32
				from
				until
					not (l_j < a_n)
				loop
					l_theta := l_d_theta * l_exponent.to_real_64
					l_w_real := (cos (l_theta)).truncated_to_real
					l_w_imag := (sin (l_theta)).truncated_to_real
					l_data_real := a_data[(({NATURAL_32} 2 * a_stride) * l_j).to_integer_32]
					l_data_imag := a_data[(({NATURAL_32} 2 * a_stride) * l_j + {NATURAL_32} 1).to_integer_32]
					l_sum_real := l_sum_real + (l_w_real * l_data_real - l_w_imag * l_data_imag)
					l_sum_imag := l_sum_imag + (l_w_real * l_data_imag + l_w_imag * l_data_real)
					l_exponent := (l_exponent + l_i) \\ a_n
					l_j := l_j + 1
				end
				a_result[(({NATURAL_32} 2 * a_stride) * l_i).to_integer_32] := l_sum_real
				a_result[(({NATURAL_32} 2 * a_stride) * l_i + {NATURAL_32} 1).to_integer_32] := l_sum_imag
				l_i := l_i + 1
			end
			Result := 0
		end

end
