class
	P_ERFC

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create P___0.make (<<6>>)
			create Q7.make (<<7>>)
			create erfc_xlt1_data.make (<<20>>)
			create erfc_xlt1_cs.make
			create erfc_x15_data.make (<<25>>)
			create erfc_x15_cs.make
			create erfc_x510_data.make (<<20>>)
			create erfc_x510_cs.make
		end

feature {ANY} -- attributes

	P___0: CE_ARRAY [REAL_64]

	Q7: CE_ARRAY [REAL_64]

	erfc_xlt1_data: CE_ARRAY [REAL_64]

	erfc_xlt1_cs: PD_CHEB_SERIES_STRUCT

	erfc_x15_data: CE_ARRAY [REAL_64]

	erfc_x15_cs: PD_CHEB_SERIES_STRUCT

	erfc_x510_data: CE_ARRAY [REAL_64]

	erfc_x510_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	erfc8_sum (a_x: REAL_64): REAL_64
		local
			l_num: REAL_64
			l_den: REAL_64
			l_i: INTEGER_32
		do
			l_num := 0.0
			l_den := 0.0
			l_num := P___0[5]
			l_i := 4
			from
			until
				not (l_i >= 0)
			loop
				l_num := a_x * l_num + P___0[l_i]
				l_i := l_i - 1
			end
			l_den := Q7[6]
			l_i := 5
			from
			until
				not (l_i >= 0)
			loop
				l_den := a_x * l_den + Q7[l_i]
				l_i := l_i - 1
			end
			Result := (l_num / l_den)
		end

	erfc8 (a_x: REAL_64): REAL_64
		local
			l_e: REAL_64
		do
			l_e := erfc8_sum (a_x)
			l_e := l_e * (exp (-a_x * a_x))
			Result := l_e
		end

	log_erfc8 (a_x: REAL_64): REAL_64
		local
			l_e: REAL_64
		do
			l_e := erfc8_sum (a_x)
			l_e := (log (l_e)) - a_x * a_x
			Result := l_e
		end

	erfseries (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_coef: REAL_64
			l_e: REAL_64
			l_del: REAL_64
			l_k: INTEGER_32
		do
			l_coef := a_x
			l_e := l_coef
			l_k := 1
			from
			until
				not (l_k < 30)
			loop
				l_coef := l_coef * ((-a_x * a_x) / l_k.to_double)
				l_del := l_coef / (2.0 * l_k.to_double + 1.0)
				l_e := l_e + l_del
				l_k := l_k + 1
			end
			a_result.item.val := (2.0 / 1.77245385090551602729816748334) * l_e
			a_result.item.err := (2.0 / 1.77245385090551602729816748334) * ((fabs (l_del)) + 2.2204460492503131e-16)
			Result := 0
		end

	gsl_sf_erfc_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_e_val: REAL_64
			l_e_err: REAL_64
			l_t: REAL_64
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_ex2: REAL_64
			l_t___0: REAL_64
			l_c___2: PD_GSL_SF_RESULT_STRUCT
			l_exterm: REAL_64
			l_t___1: REAL_64
			l_c___3: PD_GSL_SF_RESULT_STRUCT
		do
			create l_c___1.make
			create l_c___2.make
			create l_c___3.make

			l_ax := (fabs (a_x))
			if l_ax <= 1.0 then
				l_t := 2.0 * l_ax - 1.0
				cheb_eval_inst.cheb_eval_e ((erfc_xlt1_cs.address), l_t, l_c___1.address).do_nothing
				l_e_val := l_c___1.val
				l_e_err := l_c___1.err
			elseif l_ax <= 5.0 then
				l_ex2 := (exp (-a_x * a_x))
				l_t___0 := 0.5 * (l_ax - 3.0)
				cheb_eval_inst.cheb_eval_e ((erfc_x15_cs.address), l_t___0, l_c___2.address).do_nothing
				l_e_val := l_ex2 * l_c___2.val
				l_e_err := l_ex2 * (l_c___2.err + (2.0 * (fabs (a_x))) * 2.2204460492503131e-16)
			elseif l_ax < 10.0 then
				l_exterm := (exp (-a_x * a_x)) / l_ax
				l_t___1 := (2.0 * l_ax - 15.0) / 5.0
				cheb_eval_inst.cheb_eval_e ((erfc_x510_cs.address), l_t___1, l_c___3.address).do_nothing
				l_e_val := l_exterm * l_c___3.val
				l_e_err := l_exterm * ((l_c___3.err + (2.0 * (fabs (a_x))) * 2.2204460492503131e-16) + 2.2204460492503131e-16)
			else
				l_e_val := erfc8 (l_ax)
				l_e_err := ((a_x * a_x + 1.0) * 2.2204460492503131e-16) * (fabs (l_e_val))
			end
			if a_x < 0.0 then
				a_result.item.val := 2.0 - l_e_val
				a_result.item.err := l_e_err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			else
				a_result.item.val := l_e_val
				a_result.item.err := l_e_err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			end
			Result := 0
		end

	gsl_sf_log_erfc_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_y: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_c8___0: REAL_64
			l_c9: REAL_64
			l_c10: REAL_64
			l_c11: REAL_64
			l_c12: REAL_64
			l_c13: REAL_64
			l_c14: REAL_64
			l_series: REAL_64
			l_result_erfc: PD_GSL_SF_RESULT_STRUCT
		do
			create l_result_erfc.make

			if a_x * a_x < 10.0 * 2.4607833005759251e-03 then
				l_y := a_x / 1.77245385090551602729816748334
				l_c3___3 := (4.0 - 3.14159265358979323846) / 3.0
				l_c4___2 := 2.0 * (1.0 - 3.14159265358979323846 / 3.0)
				l_c5___1 := -0.001829764677455021
				l_c6___2 := 0.02629651521057465
				l_c7___0 := -0.01621575378835404
				l_c8___0 := 0.00125993961762116
				l_c9 := 0.00556964649138
				l_c10 := -0.0045563339802
				l_c11 := 0.0009461589032
				l_c12 := 0.0013200243174
				l_c13 := -0.00142906
				l_c14 := 0.00048204
				l_series := l_c8___0 + l_y * (l_c9 + l_y * (l_c10 + l_y * (l_c11 + l_y * (l_c12 + l_y * (l_c13 + l_c14 * l_y)))))
				l_series := l_y * (1.0 + l_y * (1.0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * (l_c6___2 + l_y * (l_c7___0 + l_y * l_series)))))))
				a_result.item.val := -2.0 * l_series
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			elseif a_x > 8.0 then
				a_result.item.val := log_erfc8 (a_x)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				gsl_sf_erfc_e (a_x, l_result_erfc.address).do_nothing
				a_result.item.val := log (l_result_erfc.val)
				a_result.item.err := fabs (l_result_erfc.err / l_result_erfc.val)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
			end
		end

	gsl_sf_erf_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_result_erfc: PD_GSL_SF_RESULT_STRUCT
		do
			create l_result_erfc.make

			if (fabs (a_x)) < 1.0 then
				Result := (erfseries (a_x, a_result))
			else
				gsl_sf_erfc_e (a_x, l_result_erfc.address).do_nothing
				a_result.item.val := 1.0 - l_result_erfc.val
				a_result.item.err := l_result_erfc.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
			end
		end

	gsl_sf_erf_Z_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ex2: REAL_64
			js: JUMP_STATE
		do
			l_ex2 := (exp ((-a_x * a_x) / 2.0))
			a_result.item.val := l_ex2 / (1.41421356237309504880 * 1.77245385090551602729816748334)
			a_result.item.err := (fabs (a_x * a_result.item.val)) * 2.2204460492503131e-16
			a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			if (fabs (a_result.item.val)) < 2.2250738585072014e-308 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("underflow"), ce_string ("erfc.c"), 384, 15)
					Result := 15
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_erf_Q_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_result_erfc: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
		do
			create l_result_erfc.make

			l_stat := (gsl_sf_erfc_e (a_x / 1.41421356237309504880, l_result_erfc.address))
			a_result.item.val := 0.5 * l_result_erfc.val
			a_result.item.err := 0.5 * l_result_erfc.err
			a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			Result := l_stat
		end

	gsl_sf_hazard_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_result_ln_erfc: PD_GSL_SF_RESULT_STRUCT
			l_stat_l: INTEGER_32
			l_lnc: REAL_64
			l_arg: REAL_64
			l_stat_e: INTEGER_32
			l_tmp___1: REAL_64
			l_tmp___4: INTEGER_32
			l_tmp___5: INTEGER_32
			l_ix2: REAL_64
			l_corrB: REAL_64
			l_corrM: REAL_64
			l_corrT: REAL_64
		do
			create l_result_ln_erfc.make

			if a_x < 25.0 then
				l_stat_l := (gsl_sf_log_erfc_e (a_x / 1.41421356237309504880, l_result_ln_erfc.address))
				l_lnc := -0.22579135264472743236
				l_arg := (l_lnc - (0.5 * a_x) * a_x) - l_result_ln_erfc.val
				l_stat_e := (exp_inst.gsl_sf_exp_e (l_arg, a_result))
				l_tmp___1 := fabs (a_x)
				a_result.item.err := a_result.item.err + (((3.0 * (1.0 + l_tmp___1)) * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				a_result.item.err := a_result.item.err + (fabs (l_result_ln_erfc.err * a_result.item.val))
				if l_stat_l /= 0 then
					l_tmp___5 := l_stat_l
				else
					if l_stat_e /= 0 then
						l_tmp___4 := l_stat_e
					else
						l_tmp___4 := 0
					end
					l_tmp___5 := l_tmp___4
				end
				Result := l_tmp___5
			else
				l_ix2 := 1.0 / (a_x * a_x)
				l_corrB := 1.0 - (9.0 * l_ix2) * (1.0 - 11.0 * l_ix2)
				l_corrM := 1.0 - (5.0 * l_ix2) * (1.0 - (7.0 * l_ix2) * l_corrB)
				l_corrT := 1.0 - l_ix2 * (1.0 - (3.0 * l_ix2) * l_corrM)
				a_result.item.val := a_x / l_corrT
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			end
		end

	gsl_sf_erfc (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_erfc_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_erfc_e(x, &result)"), ce_string ("erfc.c"), 438, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_log_erfc (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_log_erfc_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_log_erfc_e(x, &result)"), ce_string ("erfc.c"), 443, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_erf (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_erf_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_erf_e(x, &result)"), ce_string ("erfc.c"), 448, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_erf_Z (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_erf_Z_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_erf_Z_e(x, &result)"), ce_string ("erfc.c"), 453, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_erf_Q (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_erf_Q_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_erf_Q_e(x, &result)"), ce_string ("erfc.c"), 458, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_hazard (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_hazard_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_hazard_e(x, &result)"), ce_string ("erfc.c"), 463, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
