class
	P_ERROR

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_error_handler.make
		end

feature {ANY} -- attributes

	gsl_error_handler: CE_ROUTINE

feature {ANY} -- routines

	gsl_error (a_reason: CE_POINTER [INTEGER_8]; a_file: CE_POINTER [INTEGER_8]; a_line: INTEGER_32; a_gsl_errno: INTEGER_32)
		local
			js: JUMP_STATE
		do
			if ((gsl_error_handler).c_pointer /= default_pointer) then
				gsl_error_handler.call ([a_reason, a_file, a_line, a_gsl_errno])
				js.return
			end
			if not js.return_called then
				stream_inst.gsl_stream_printf (ce_string ("ERROR"), a_file, a_line, a_reason)
				Io.standard_default.flush
				Io.error.put_string ("Default GSL error handler invoked.%N")
				Io.error.flush
				abort
			end
		end

	gsl_set_error_handler (a_new_handler: CE_ROUTINE): CE_ROUTINE
		local
			l_previous_handler: CE_ROUTINE
		do
			create l_previous_handler.make

			l_previous_handler.ce_gets (gsl_error_handler)
			gsl_error_handler.ce_gets (a_new_handler)
			Result := l_previous_handler
		end

	gsl_set_error_handler_off: CE_ROUTINE
		local
			l_previous_handler: CE_ROUTINE
		do
			create l_previous_handler.make

			l_previous_handler.ce_gets (gsl_error_handler)
			gsl_error_handler.ce_gets (no_error_handler_address)
			Result := l_previous_handler
		end

	no_error_handler (a_reason: CE_POINTER [INTEGER_8]; a_file: CE_POINTER [INTEGER_8]; a_line: INTEGER_32; a_gsl_errno: INTEGER_32)
		local
			l_reason: CE_POINTER [INTEGER_8]
			l_file: CE_POINTER [INTEGER_8]
			l_line: INTEGER_32
			l_gsl_errno: INTEGER_32
		do
			create l_reason.make
			create l_file.make

			l_reason.ce_gets (a_reason)
			l_file.ce_gets (a_file)
			l_line := a_line
			l_gsl_errno := a_gsl_errno
			l_reason.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			l_file.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			l_line := 0
			l_gsl_errno := 0
		end

	no_error_handler_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent no_error_handler)
		end

end
