class
	P_EVAL

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_cheb_eval (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2.0 * a_x - a_cs.item.a) - a_cs.item.b) / (a_cs.item.b - a_cs.item.a)
			l_y2 := 2.0 * l_y
			l_i := a_cs.item.order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + a_cs.item.c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			Result := ((l_y * l_d1 - l_d2) + 0.5 * a_cs.item.c[0])
		end

	gsl_cheb_eval_n (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_n: NATURAL_32; a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_eval_order: NATURAL_32
			l_tmp: NATURAL_32
			l_y: REAL_64
			l_y2: REAL_64
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			if a_n < a_cs.item.order then
				l_tmp := a_n
			else
				l_tmp := a_cs.item.order
			end
			l_eval_order := l_tmp
			l_y := ((2.0 * a_x - a_cs.item.a) - a_cs.item.b) / (a_cs.item.b - a_cs.item.a)
			l_y2 := 2.0 * l_y
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + a_cs.item.c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			Result := ((l_y * l_d1 - l_d2) + 0.5 * a_cs.item.c[0])
		end

	gsl_cheb_eval_err (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_x: REAL_64; a_result: CE_POINTER [REAL_64]; a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a_cs.item.a) - a_cs.item.b) / (a_cs.item.b - a_cs.item.a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			l_i := a_cs.item.order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + a_cs.item.c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * a_cs.item.c[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= a_cs.item.order)
			loop
				l_absc := l_absc + (fabs (a_cs.item.c[l_i.to_integer_32]))
				l_i := l_i + 1
			end
			a_abserr.item := (fabs (a_cs.item.c[(a_cs.item.order).to_integer_32])) + l_absc * 2.2204460492503131e-16
			Result := 0
		end

	gsl_cheb_eval_n_err (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_n: NATURAL_32; a_x: REAL_64; a_result: CE_POINTER [REAL_64]; a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_eval_order: NATURAL_32
			l_tmp: NATURAL_32
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a_cs.item.a) - a_cs.item.b) / (a_cs.item.b - a_cs.item.a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			if a_n < a_cs.item.order then
				l_tmp := a_n
			else
				l_tmp := a_cs.item.order
			end
			l_eval_order := l_tmp
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + a_cs.item.c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * a_cs.item.c[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= l_eval_order)
			loop
				l_absc := l_absc + (fabs (a_cs.item.c[l_i.to_integer_32]))
				l_i := l_i + 1
			end
			a_abserr.item := (fabs (a_cs.item.c[l_eval_order.to_integer_32])) + l_absc * 2.2204460492503131e-16
			Result := 0
		end

	gsl_cheb_eval_mode_e (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_x: REAL_64; a_mode: NATURAL_32; a_result: CE_POINTER [REAL_64]; a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_i: NATURAL_32
			l_d1: REAL_64
			l_d2: REAL_64
			l_y: REAL_64
			l_y2: REAL_64
			l_absc: REAL_64
			l_eval_order: NATURAL_32
			l_temp: REAL_64
		do
			l_d1 := 0.0
			l_d2 := 0.0
			l_y := ((2. * a_x - a_cs.item.a) - a_cs.item.b) / (a_cs.item.b - a_cs.item.a)
			l_y2 := 2.0 * l_y
			l_absc := 0.0
			if (gsl_gsl_mode_inst.GSL_MODE_PREC (a_mode)) = {NATURAL_32} 0 then
				l_eval_order := a_cs.item.order
			else
				l_eval_order := a_cs.item.order_sp
			end
			l_i := l_eval_order
			from
			until
				not (l_i >= {NATURAL_32} 1)
			loop
				l_temp := l_d1
				l_d1 := (l_y2 * l_d1 - l_d2) + a_cs.item.c[l_i.to_integer_32]
				l_d2 := l_temp
				l_i := l_i - 1
			end
			a_result.item := (l_y * l_d1 - l_d2) + 0.5 * a_cs.item.c[0]
			l_i := (0).to_natural_32
			from
			until
				not (l_i <= l_eval_order)
			loop
				l_absc := l_absc + (fabs (a_cs.item.c[l_i.to_integer_32]))
				l_i := l_i + 1
			end
			a_abserr.item := (fabs (a_cs.item.c[l_eval_order.to_integer_32])) + l_absc * 2.2204460492503131e-16
			Result := 0
		end

	gsl_cheb_eval_mode (a_cs: CE_POINTER [PD_GSL_CHEB_SERIES_STRUCT]; a_x: REAL_64; a_mode: NATURAL_32): REAL_64
		local
			l_result: REAL_64
			l_abserr: REAL_64
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			l_status := (gsl_cheb_eval_mode_e (a_cs, a_x, a_mode, create {CE_POINTER [REAL_64]}.make_shared ($l_result), create {CE_POINTER [REAL_64]}.make_shared ($l_abserr)))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_cheb_eval_mode"), ce_string ("eval.c"), 200, l_status)
					Result := l_result
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := l_result
			end
		end

end
