class
	P_FACTORIZE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	fft_complex_factorize (a_n: NATURAL_32; a_nf: CE_POINTER [NATURAL_32]; a_factors: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_complex_subtransforms: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
		do
			create l_complex_subtransforms.make (<<7>>)

			l_complex_subtransforms[0] := (7).to_natural_32
			l_complex_subtransforms[1] := (6).to_natural_32
			l_complex_subtransforms[2] := (5).to_natural_32
			l_complex_subtransforms[3] := (4).to_natural_32
			l_complex_subtransforms[4] := (3).to_natural_32
			l_complex_subtransforms[5] := (2).to_natural_32
			l_complex_subtransforms[6] := (0).to_natural_32
			l_status := (fft_factorize (a_n, create {CE_POINTER [NATURAL_32]}.make_cast (l_complex_subtransforms), a_nf, a_factors))
			Result := l_status
		end

	fft_halfcomplex_factorize (a_n: NATURAL_32; a_nf: CE_POINTER [NATURAL_32]; a_factors: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_halfcomplex_subtransforms: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
		do
			create l_halfcomplex_subtransforms.make (<<5>>)

			l_halfcomplex_subtransforms[0] := (5).to_natural_32
			l_halfcomplex_subtransforms[1] := (4).to_natural_32
			l_halfcomplex_subtransforms[2] := (3).to_natural_32
			l_halfcomplex_subtransforms[3] := (2).to_natural_32
			l_halfcomplex_subtransforms[4] := (0).to_natural_32
			l_status := (fft_factorize (a_n, create {CE_POINTER [NATURAL_32]}.make_cast (l_halfcomplex_subtransforms), a_nf, a_factors))
			Result := l_status
		end

	fft_real_factorize (a_n: NATURAL_32; a_nf: CE_POINTER [NATURAL_32]; a_factors: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_real_subtransforms: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
		do
			create l_real_subtransforms.make (<<5>>)

			l_real_subtransforms[0] := (5).to_natural_32
			l_real_subtransforms[1] := (4).to_natural_32
			l_real_subtransforms[2] := (3).to_natural_32
			l_real_subtransforms[3] := (2).to_natural_32
			l_real_subtransforms[4] := (0).to_natural_32
			l_status := (fft_factorize (a_n, create {CE_POINTER [NATURAL_32]}.make_cast (l_real_subtransforms), a_nf, a_factors))
			Result := l_status
		end

	fft_factorize (a_n: NATURAL_32; a_implemented_subtransforms: CE_POINTER [NATURAL_32]; a_n_factors: CE_POINTER [NATURAL_32]; a_factors: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_nf: NATURAL_32
			l_ntest: NATURAL_32
			l_factor: NATURAL_32
			l_i: NATURAL_32
			l_product: NATURAL_32
			js: JUMP_STATE
		do
			l_nf := (0).to_natural_32
			l_ntest := a_n
			l_i := (0).to_natural_32
			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("length n must be positive integer"), ce_string ("factorize.c"), 80, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called and then (a_n = {NATURAL_32} 1) then
				a_factors[0] := (1).to_natural_32
				a_n_factors.item := (1).to_natural_32
				Result := 0
				js.return
			end
			if not js.return_called then
				from
				until
					js.loop_left
				loop
					js.reset
					if (a_implemented_subtransforms[l_i.to_integer_32]).to_boolean then
						if l_ntest = {NATURAL_32} 1 then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_factor := a_implemented_subtransforms[l_i.to_integer_32]
						from
						until
							l_ntest \\ l_factor /= {NATURAL_32} 0
						loop
							l_ntest := l_ntest // l_factor
							a_factors[l_nf.to_integer_32] := l_factor
							l_nf := l_nf + 1
						end
						l_i := l_i + 1
					end
				end
				js.reset
				l_factor := (2).to_natural_32
				from
				until
					js.loop_left
				loop
					js.reset
					if l_ntest \\ l_factor = {NATURAL_32} 0 then
						if l_ntest = {NATURAL_32} 1 then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_ntest := l_ntest // l_factor
						a_factors[l_nf.to_integer_32] := l_factor
						l_nf := l_nf + 1
					end
				end
				js.reset
				l_factor := (3).to_natural_32
				from
				until
					not (not js.loop_left and then (l_ntest /= {NATURAL_32} 1))
				loop
					from
					until
						l_ntest \\ l_factor = {NATURAL_32} 0
					loop
						l_factor := l_factor + ({NATURAL_32} 2)
					end
					l_ntest := l_ntest // l_factor
					a_factors[l_nf.to_integer_32] := l_factor
					l_nf := l_nf + 1
				end
				l_product := (1).to_natural_32
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nf)
				loop
					l_product := l_product * (a_factors[l_i.to_integer_32])
					l_i := l_i + 1
				end
				if l_product /= a_n then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("factorization failed"), ce_string ("factorize.c"), 141, 7)
						Result := 7
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				a_n_factors.item := l_nf
				Result := 0
			end
		end

	fft_binary_logn (a_n: NATURAL_32): INTEGER_32
		local
			l_ntest: NATURAL_32
			l_binary_logn___0: NATURAL_32
			l_k: NATURAL_32
			js: JUMP_STATE
		do
			l_binary_logn___0 := (0).to_natural_32
			l_k := (1).to_natural_32
			from
			until
				not (l_k < a_n)
			loop
				l_k := l_k * ({NATURAL_32} 2)
				l_binary_logn___0 := l_binary_logn___0 + 1
			end
			l_ntest := ((1).bit_shift_left (l_binary_logn___0.to_integer_32)).to_natural_32
			if a_n /= l_ntest then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				Result := l_binary_logn___0.to_integer_32
			end
		end

end
