class
	P_GAUSS

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_ran_gaussian (a_r___3: CE_POINTER [PD_GSL_RNG]; a_sigma: REAL_64): REAL_64
		local
			l_x: REAL_64
			l_y: REAL_64
			l_r2___0: REAL_64
			js: JUMP_STATE
		do
			from
			until
				js.loop_left
			loop
				js.reset
				l_x := (-1).to_double + (2).to_double * (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
				l_y := (-1).to_double + (2).to_double * (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
				l_r2___0 := l_x * l_x + l_y * l_y
				if not (l_r2___0 > 1.0) then
					if l_r2___0 /= (0).to_double then
						js.break
					end
				end
			end
			js.reset
			Result := ((a_sigma * l_y) * (sqrt ((-2.0 * (log (l_r2___0))) / l_r2___0)))
		end

	gsl_ran_gaussian_ratio_method (a_r___3: CE_POINTER [PD_GSL_RNG]; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_v: REAL_64
			l_x: REAL_64
			l_y: REAL_64
			l_Q___1: REAL_64
			l_s: REAL_64
			l_t: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_r1___1: REAL_64
			l_r2___0: REAL_64
			js: JUMP_STATE
		do
			l_s := 0.449871
			l_t := -0.386595
			l_a___5 := 0.19600
			l_b___0 := 0.25472
			l_r1___1 := 0.27597
			l_r2___0 := 0.27846
			from
			until
				js.loop_left
			loop
				js.reset
				l_u := (1).to_double - (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3))
				l_v := (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3)) - 0.5
				l_v := l_v * (1.7156)
				l_x := l_u - l_s
				l_y := (fabs (l_v)) - l_t
				l_Q___1 := l_x * l_x + l_y * (l_a___5 * l_y - l_b___0 * l_x)
				if l_Q___1 >= l_r1___1 then
					if not (l_Q___1 > l_r2___0) then
						if not (l_v * l_v > (((-4).to_double * l_u) * l_u) * (log (l_u))) then
							js.break
						end
					end
				else
					js.break
				end
			end
			js.reset
			Result := (a_sigma * (l_v / l_u))
		end

	gsl_ran_gaussian_pdf (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_p: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
		do
			l_u := a_x / (fabs (a_sigma))
			l_tmp___0 := sqrt ((2).to_double * 3.14159265358979323846)
			l_tmp___1 := fabs (a_sigma)
			l_p := ((1).to_double / (l_tmp___0 * l_tmp___1)) * (exp ((-l_u * l_u) / (2).to_double))
			Result := l_p
		end

	gsl_ran_ugaussian (a_r___3: CE_POINTER [PD_GSL_RNG]): REAL_64
		do
			Result := (gsl_ran_gaussian (a_r___3, 1.0))
		end

	gsl_ran_ugaussian_ratio_method (a_r___3: CE_POINTER [PD_GSL_RNG]): REAL_64
		do
			Result := (gsl_ran_gaussian_ratio_method (a_r___3, 1.0))
		end

	gsl_ran_ugaussian_pdf (a_x: REAL_64): REAL_64
		do
			Result := (gsl_ran_gaussian_pdf (a_x, 1.0))
		end

	get_del (a_x: REAL_64; a_rational: REAL_64): REAL_64
		local
			l_xsq: REAL_64
			l_del: REAL_64
			l_result: REAL_64
			l_tmp___0: REAL_64
		do
			l_xsq := 0.0
			l_del := 0.0
			l_result := 0.0
			l_xsq := (floor (a_x * 16.0)) / 16.0
			l_del := (a_x - l_xsq) * (a_x + l_xsq)
			l_del := l_del * (0.5)
			l_tmp___0 := exp ((-0.5 * l_xsq) * l_xsq)
			l_result := (l_tmp___0 * (exp (-1.0 * l_del))) * a_rational
			Result := l_result
		end

	gauss_small (a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_result: REAL_64
			l_xsq: REAL_64
			l_xnum: REAL_64
			l_xden: REAL_64
			l_a___5: CE_ARRAY [REAL_64]
			l_b___0: CE_ARRAY [REAL_64]
		do
			create l_a___5.make (<<5>>)
			create l_b___0.make (<<4>>)

			l_result := 0.0
			l_a___5[0] := 2.2352520354606839287
			l_a___5[1] := 161.02823106855587881
			l_a___5[2] := 1067.6894854603709582
			l_a___5[3] := 18154.981253343561249
			l_a___5[4] := 0.065682337918207449113
			l_b___0[0] := 47.20258190468824187
			l_b___0[1] := 976.09855173777669322
			l_b___0[2] := 10260.932208618978205
			l_b___0[3] := 45507.789335026729956
			l_xsq := a_x * a_x
			l_xnum := l_a___5[4] * l_xsq
			l_xden := l_xsq
			l_i := {NATURAL_32} 0
			from
			until
				not (l_i < {NATURAL_32} 3)
			loop
				l_xnum := (l_xnum + l_a___5[l_i.to_integer_32]) * l_xsq
				l_xden := (l_xden + l_b___0[l_i.to_integer_32]) * l_xsq
				l_i := l_i + 1
			end
			l_result := (a_x * (l_xnum + l_a___5[3])) / (l_xden + l_b___0[3])
			Result := l_result
		end

	gauss_medium (a_x: REAL_64): REAL_64
		local
			l_i: NATURAL_32
			l_temp: REAL_64
			l_result: REAL_64
			l_xnum: REAL_64
			l_xden: REAL_64
			l_absx: REAL_64
			l_c___1: CE_ARRAY [REAL_64]
			l_d: CE_ARRAY [REAL_64]
		do
			create l_c___1.make (<<9>>)
			create l_d.make (<<8>>)

			l_temp := 0.0
			l_result := 0.0
			l_c___1[0] := 0.39894151208813466764
			l_c___1[1] := 8.8831497943883759412
			l_c___1[2] := 93.506656132177855979
			l_c___1[3] := 597.27027639480026226
			l_c___1[4] := 2494.5375852903726711
			l_c___1[5] := 6848.1904505362823326
			l_c___1[6] := 11602.651437647350124
			l_c___1[7] := 9842.7148383839780218
			l_c___1[8] := 1.0765576773720192317e-8
			l_d[0] := 22.266688044328115691
			l_d[1] := 235.38790178262499861
			l_d[2] := 1519.377599407554805
			l_d[3] := 6485.558298266760755
			l_d[4] := 18615.571640885098091
			l_d[5] := 34900.952721145977266
			l_d[6] := 38912.003286093271411
			l_d[7] := 19685.429676859990727
			l_absx := fabs (a_x)
			l_xnum := l_c___1[8] * l_absx
			l_xden := l_absx
			l_i := {NATURAL_32} 0
			from
			until
				not (l_i < {NATURAL_32} 7)
			loop
				l_xnum := (l_xnum + l_c___1[l_i.to_integer_32]) * l_absx
				l_xden := (l_xden + l_d[l_i.to_integer_32]) * l_absx
				l_i := l_i + 1
			end
			l_temp := (l_xnum + l_c___1[7]) / (l_xden + l_d[7])
			l_result := get_del (a_x, l_temp)
			Result := l_result
		end

	gauss_large (a_x: REAL_64): REAL_64
		local
			l_i: INTEGER_32
			l_result: REAL_64
			l_xsq: REAL_64
			l_temp: REAL_64
			l_xnum: REAL_64
			l_xden: REAL_64
			l_absx: REAL_64
			l_p: CE_ARRAY [REAL_64]
			l_q___3: CE_ARRAY [REAL_64]
		do
			create l_p.make (<<6>>)
			create l_q___3.make (<<5>>)

			l_p[0] := 0.21589853405795699
			l_p[1] := 0.1274011611602473639
			l_p[2] := 0.022235277870649807
			l_p[3] := 0.001421619193227893466
			l_p[4] := 2.9112874951168792e-5
			l_p[5] := 0.02307344176494017303
			l_q___3[0] := 1.28426009614491121
			l_q___3[1] := 0.468238212480865118
			l_q___3[2] := 0.0659881378689285515
			l_q___3[3] := 0.00378239633202758244
			l_q___3[4] := 7.29751555083966205e-5
			l_absx := fabs (a_x)
			l_xsq := 1.0 / (a_x * a_x)
			l_xnum := l_p[5] * l_xsq
			l_xden := l_xsq
			l_i := 0
			from
			until
				not (l_i < 4)
			loop
				l_xnum := (l_xnum + l_p[l_i]) * l_xsq
				l_xden := (l_xden + l_q___3[l_i]) * l_xsq
				l_i := l_i + 1
			end
			l_temp := (l_xsq * (l_xnum + l_p[4])) / (l_xden + l_q___3[4])
			l_temp := ((1.12837916709551257390 * 0.70710678118654752440) / 2.0 - l_temp) / l_absx
			l_result := get_del (a_x, l_temp)
			Result := l_result
		end

	gsl_cdf_ugaussian_P (a_x: REAL_64): REAL_64
		local
			l_result: REAL_64
			l_absx: REAL_64
			js: JUMP_STATE
		do
			l_absx := (fabs (a_x))
			if l_absx < 2.2204460492503131e-16 / (2).to_double then
				l_result := 0.5
				Result := l_result
				js.return
			elseif l_absx < 0.66291 then
				l_result := 0.5 + (gauss_small (a_x))
				Result := l_result
				js.return
			elseif l_absx < 4.0 * 1.41421356237309504880 then
				l_result := gauss_medium (a_x)
				if a_x > 0.0 then
					l_result := 1.0 - l_result
				end
				Result := l_result
				js.return
			elseif a_x > 8.572 then
				l_result := 1.0
				Result := l_result
				js.return
			elseif a_x < -37.519 then
				l_result := 0.0
				Result := l_result
				js.return
			else
				l_result := gauss_large (a_x)
				if a_x > 0.0 then
					l_result := 1.0 - l_result
				end
			end
			if not js.return_called then
				Result := l_result
			end
		end

	gsl_cdf_ugaussian_Q (a_x: REAL_64): REAL_64
		local
			l_result: REAL_64
			l_absx: REAL_64
			js: JUMP_STATE
		do
			l_absx := (fabs (a_x))
			if l_absx < 2.2204460492503131e-16 / (2).to_double then
				l_result := 0.5
				Result := l_result
				js.return
			elseif l_absx < 0.66291 then
				l_result := gauss_small (a_x)
				if a_x < 0.0 then
					l_result := (fabs (l_result)) + 0.5
				else
					l_result := 0.5 - l_result
				end
				Result := l_result
				js.return
			elseif l_absx < 4.0 * 1.41421356237309504880 then
				l_result := gauss_medium (a_x)
				if a_x < 0.0 then
					l_result := 1.0 - l_result
				end
				Result := l_result
				js.return
			elseif a_x > -(-37.519) then
				l_result := 0.0
				Result := l_result
				js.return
			elseif a_x < -8.572 then
				l_result := 1.0
				Result := l_result
				js.return
			else
				l_result := gauss_large (a_x)
				if a_x < 0.0 then
					l_result := 1.0 - l_result
				end
			end
			if not js.return_called then
				Result := l_result
			end
		end

	gsl_cdf_gaussian_P (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		do
			Result := (gsl_cdf_ugaussian_P (a_x / a_sigma))
		end

	gsl_cdf_gaussian_Q (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		do
			Result := (gsl_cdf_ugaussian_Q (a_x / a_sigma))
		end

end
