class
	P_HESSENBERG

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_linalg_hessenberg_decomp (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_c___1: PD_GSL_VECTOR_VIEW
			l_hv: PD_GSL_VECTOR_VIEW
			l_m___4: PD_GSL_MATRIX_VIEW
			l_tau_i: REAL_64
			js: JUMP_STATE
		do
			create l_c___1.make
			create l_hv.make
			create l_m___4.make

			l_N := a_A___0.item.size1
			if l_N /= a_A___0.item.size2 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 74, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N /= a_tau.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 78, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N < {NATURAL_32} 3 then
				Result := 0
				js.return
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 2)
				loop
					l_c___1.ce_gets (rowcol_source_inst.gsl_matrix_subcolumn (a_A___0, l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1))
					l_hv.ce_gets (subvector_source_inst.gsl_vector_subvector (a_tau, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1)))
					copy_source_inst.gsl_vector_memcpy (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))).do_nothing
					l_tau_i := householder_inst.gsl_linalg_householder_transform (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))
					l_m___4.ce_gets (submatrix_source_inst.gsl_matrix_submatrix (a_A___0, l_i + {NATURAL_32} 1, l_i, l_N - (l_i + {NATURAL_32} 1), l_N - l_i))
					householder_inst.gsl_linalg_householder_hm (l_tau_i, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.c_pointer + {PD_GSL_MATRIX_VIEW}.matrix_offset)).do_nothing
					l_m___4.ce_gets (submatrix_source_inst.gsl_matrix_submatrix (a_A___0, (0).to_natural_32, l_i + {NATURAL_32} 1, l_N, l_N - (l_i + {NATURAL_32} 1)))
					householder_inst.gsl_linalg_householder_mh (l_tau_i, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.c_pointer + {PD_GSL_MATRIX_VIEW}.matrix_offset)).do_nothing
					gsl_gsl_vector_double_inst.gsl_vector_set (a_tau, l_i, l_tau_i)
					l_c___1.ce_gets (subvector_source_inst.gsl_vector_subvector (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (1).to_natural_32, l_c___1.vector.size - {NATURAL_32} 1))
					l_hv.ce_gets (subvector_source_inst.gsl_vector_subvector (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (1).to_natural_32, l_hv.vector.size - {NATURAL_32} 1))
					copy_source_inst.gsl_vector_memcpy (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))).do_nothing
					l_i := l_i + 1
				end
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_hessenberg_unpack (a_H: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]; a_U: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_s: INTEGER_32
		do
			init_source_inst.gsl_matrix_set_identity (a_U)
			l_s := gsl_linalg_hessenberg_unpack_accum (a_H, a_tau, a_U)
			Result := l_s
		end

	gsl_linalg_hessenberg_unpack_accum (a_H: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]; a_V: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_N: NATURAL_32
			l_j: NATURAL_32
			l_tau_j: REAL_64
			l_c___1: PD_GSL_VECTOR_VIEW
			l_hv: PD_GSL_VECTOR_VIEW
			l_m___4: PD_GSL_MATRIX_VIEW
			js: JUMP_STATE
		do
			create l_c___1.make
			create l_hv.make
			create l_m___4.make

			l_N := a_H.item.size1
			if l_N /= a_H.item.size2 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 215, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N /= a_tau.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 219, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N /= a_V.item.size2 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("V matrix has wrong dimension"), ce_string ("hessenberg.c"), 223, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				if l_N < {NATURAL_32} 3 then
					Result := 0
					js.return
				end
				if not js.return_called then
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_N - {NATURAL_32} 2)
					loop
						l_c___1.ce_gets (rowcol_source_inst.gsl_matrix_column (a_H, l_j))
						l_tau_j := gsl_gsl_vector_double_inst.gsl_vector_get (a_tau, l_j)
						l_hv.ce_gets (subvector_source_inst.gsl_vector_subvector (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), l_j + {NATURAL_32} 1, l_N - (l_j + {NATURAL_32} 1)))
						l_m___4.ce_gets (submatrix_source_inst.gsl_matrix_submatrix (a_V, (0).to_natural_32, l_j + {NATURAL_32} 1, a_V.item.size1, l_N - (l_j + {NATURAL_32} 1)))
						householder_inst.gsl_linalg_householder_mh (l_tau_j, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.c_pointer + {PD_GSL_MATRIX_VIEW}.matrix_offset)).do_nothing
						l_j := l_j + 1
					end
					Result := 0
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_hessenberg_set_zero (a_H: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_N := a_H.item.size1
			if l_N < {NATURAL_32} 3 then
				Result := 0
				js.return
			end
			if not js.return_called then
				l_j := (0).to_natural_32
				from
				until
					not (not js.loop_left and then (l_j < l_N - {NATURAL_32} 2))
				loop
					l_i := l_j + {NATURAL_32} 2
					from
					until
						not (l_i < l_N)
					loop
						gsl_gsl_matrix_double_inst.gsl_matrix_set (a_H, l_i, l_j, 0.0)
						l_i := l_i + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_linalg_hessenberg_submatrix (a_M: CE_POINTER [PD_GSL_MATRIX]; a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_top: NATURAL_32; a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_N_M: NATURAL_32
			l_i: NATURAL_32
			l_c___1: PD_GSL_VECTOR_VIEW
			l_hv: PD_GSL_VECTOR_VIEW
			l_m___4: PD_GSL_MATRIX_VIEW
			l_tau_i: REAL_64
			js: JUMP_STATE
		do
			create l_c___1.make
			create l_hv.make
			create l_m___4.make

			l_N := a_A___0.item.size1
			l_N_M := a_M.item.size1
			if l_N /= a_A___0.item.size2 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("Hessenberg reduction requires square matrix"), ce_string ("hessenberg.c"), 360, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N /= a_tau.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("tau vector must match matrix size"), ce_string ("hessenberg.c"), 364, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif l_N < {NATURAL_32} 3 then
				Result := 0
				js.return
			else
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N - {NATURAL_32} 2)
				loop
					l_c___1.ce_gets (rowcol_source_inst.gsl_matrix_subcolumn (a_A___0, l_i, l_i + {NATURAL_32} 1, (l_N - l_i) - {NATURAL_32} 1))
					l_hv.ce_gets (subvector_source_inst.gsl_vector_subvector (a_tau, l_i + {NATURAL_32} 1, l_N - (l_i + {NATURAL_32} 1)))
					copy_source_inst.gsl_vector_memcpy (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))).do_nothing
					l_tau_i := householder_inst.gsl_linalg_householder_transform (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))
					l_m___4.ce_gets (submatrix_source_inst.gsl_matrix_submatrix (a_M, (a_top + l_i) + {NATURAL_32} 1, a_top + l_i, l_N - (l_i + {NATURAL_32} 1), (l_N_M - a_top) - l_i))
					householder_inst.gsl_linalg_householder_hm (l_tau_i, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.c_pointer + {PD_GSL_MATRIX_VIEW}.matrix_offset)).do_nothing
					l_m___4.ce_gets (submatrix_source_inst.gsl_matrix_submatrix (a_M, (0).to_natural_32, (a_top + l_i) + {NATURAL_32} 1, a_top + l_N, l_N - (l_i + {NATURAL_32} 1)))
					householder_inst.gsl_linalg_householder_mh (l_tau_i, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_m___4.c_pointer + {PD_GSL_MATRIX_VIEW}.matrix_offset)).do_nothing
					gsl_gsl_vector_double_inst.gsl_vector_set (a_tau, l_i, l_tau_i)
					l_c___1.ce_gets (subvector_source_inst.gsl_vector_subvector (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (1).to_natural_32, l_c___1.vector.size - {NATURAL_32} 1))
					l_hv.ce_gets (subvector_source_inst.gsl_vector_subvector (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (1).to_natural_32, l_hv.vector.size - {NATURAL_32} 1))
					copy_source_inst.gsl_vector_memcpy (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_c___1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_hv.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))).do_nothing
					l_i := l_i + 1
				end
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_hessenberg (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_tau: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		do
			Result := (gsl_linalg_hessenberg_decomp (a_A___0, a_tau))
		end

end
