class
	P_HOUSEHOLDER

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_linalg_householder_transform (a_v: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		local
			l_n: NATURAL_32
			l_alpha: REAL_64
			l_beta: REAL_64
			l_tau: REAL_64
			l_x: PD_GSL_VECTOR_VIEW
			l_xnorm: REAL_64
			l_tmp___1: REAL_64
			l_s: REAL_64
			js: JUMP_STATE
		do
			create l_x.make

			l_n := a_v.item.size
			if l_n = {NATURAL_32} 1 then
				Result := (0.0)
			else
				l_x.ce_gets ((subvector_source_inst.gsl_vector_subvector (a_v, (1).to_natural_32, l_n - {NATURAL_32} 1)))
				l_xnorm := (blas_inst.gsl_blas_dnrm2 ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))))
				if l_xnorm = (0).to_double then
					Result := (0.0)
					js.return
				end
				if not js.return_called then
					l_alpha := gsl_gsl_vector_double_inst.gsl_vector_get (a_v, (0).to_natural_32)
					if l_alpha >= 0.0 then
						l_tmp___1 := 1.0
					else
						l_tmp___1 := -1.0
					end
					l_beta := -l_tmp___1 * (hypot (l_alpha, l_xnorm))
					l_tau := (l_beta - l_alpha) / l_beta
					l_s := l_alpha - l_beta
					if (fabs (l_s)) > 2.2250738585072014e-308 then
						blas_inst.gsl_blas_dscal (1.0 / l_s, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))
						gsl_gsl_vector_double_inst.gsl_vector_set (a_v, (0).to_natural_32, l_beta)
					else
						blas_inst.gsl_blas_dscal (2.2204460492503131e-16 / l_s, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))
						blas_inst.gsl_blas_dscal (1.0 / 2.2204460492503131e-16, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))
						gsl_gsl_vector_double_inst.gsl_vector_set (a_v, (0).to_natural_32, l_beta)
					end
					Result := l_tau
				end
			end
		end

	gsl_linalg_householder_hm (a_tau: REAL_64; a_v: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wj: REAL_64
			l_tmp___0: REAL_64
			l_A0j: REAL_64
			l_Aij: REAL_64
			l_vi: REAL_64
			js: JUMP_STATE
		do
			if a_tau = 0.0 then
				Result := 0
				js.return
			end
			if not js.return_called then
				l_j := (0).to_natural_32
				from
				until
					not (not js.loop_left and then (l_j < a_A___0.item.size2))
				loop
					l_wj := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, (0).to_natural_32, l_j))
					l_i := (1).to_natural_32
					from
					until
						not (l_i < a_A___0.item.size1)
					loop
						l_tmp___0 := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_j)
						l_wj := l_wj + (l_tmp___0 * (gsl_gsl_vector_double_inst.gsl_vector_get (a_v, l_i)))
						l_i := l_i + 1
					end
					l_A0j := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, (0).to_natural_32, l_j))
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, (0).to_natural_32, l_j, l_A0j - a_tau * l_wj)
					l_i := (1).to_natural_32
					from
					until
						not (l_i < a_A___0.item.size1)
					loop
						l_Aij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_j))
						l_vi := (gsl_gsl_vector_double_inst.gsl_vector_get (a_v, l_i))
						gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, l_j, l_Aij - (a_tau * l_vi) * l_wj)
						l_i := l_i + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_linalg_householder_mh (a_tau: REAL_64; a_v: CE_POINTER [PD_GSL_VECTOR]; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wi: REAL_64
			l_tmp___0: REAL_64
			l_Ai0: REAL_64
			l_vj: REAL_64
			l_Aij: REAL_64
			js: JUMP_STATE
		do
			if a_tau = (0).to_double then
				Result := 0
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (not js.loop_left and then (l_i < a_A___0.item.size1))
				loop
					l_wi := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, (0).to_natural_32))
					l_j := (1).to_natural_32
					from
					until
						not (l_j < a_A___0.item.size2)
					loop
						l_tmp___0 := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_j)
						l_wi := l_wi + (l_tmp___0 * (gsl_gsl_vector_double_inst.gsl_vector_get (a_v, l_j)))
						l_j := l_j + 1
					end
					l_Ai0 := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, (0).to_natural_32))
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, (0).to_natural_32, l_Ai0 - a_tau * l_wi)
					l_j := (1).to_natural_32
					from
					until
						not (l_j < a_A___0.item.size2)
					loop
						l_vj := (gsl_gsl_vector_double_inst.gsl_vector_get (a_v, l_j))
						l_Aij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_j))
						gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, l_j, l_Aij - (a_tau * l_wi) * l_vj)
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_linalg_householder_hv (a_tau: REAL_64; a_v: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_N: NATURAL_32
			l_d0: REAL_64
			l_d1: REAL_64
			l_d: REAL_64
			l_v1: PD_GSL_VECTOR_CONST_VIEW
			l_w1: PD_GSL_VECTOR_VIEW
			l_w0: REAL_64
			js: JUMP_STATE
		do
			create l_v1.make
			create l_w1.make

			l_N := a_v.item.size
			if a_tau = (0).to_double then
				Result := 0
				js.return
			end
			if not js.return_called then
				l_d0 := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w, (0).to_natural_32))
				l_v1.ce_gets ((subvector_source_inst.gsl_vector_const_subvector (a_v, (1).to_natural_32, l_N - {NATURAL_32} 1)))
				l_w1.ce_gets ((subvector_source_inst.gsl_vector_subvector (a_w, (1).to_natural_32, l_N - {NATURAL_32} 1)))
				blas_inst.gsl_blas_ddot ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.c_pointer + {PD_GSL_VECTOR_CONST_VIEW}.vector_offset)), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_w1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)), create {CE_POINTER [REAL_64]}.make_shared ($l_d1)).do_nothing
				l_d := l_d0 + l_d1
				l_w0 := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w, (0).to_natural_32))
				gsl_gsl_vector_double_inst.gsl_vector_set (a_w, (0).to_natural_32, l_w0 - a_tau * l_d)
				blas_inst.gsl_blas_daxpy (-a_tau * l_d, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.c_pointer + {PD_GSL_VECTOR_CONST_VIEW}.vector_offset)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_w1.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)).do_nothing
				Result := 0
			end
		end

	gsl_linalg_householder_hm1 (a_tau: REAL_64; a_A___0: CE_POINTER [PD_GSL_MATRIX]): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_i___0: NATURAL_32
			l_j___0: NATURAL_32
			l_wj: REAL_64
			l_vi: REAL_64
			l_vi___0: REAL_64
			l_Aij: REAL_64
			l_vi___1: REAL_64
			js: JUMP_STATE
		do
			if a_tau = (0).to_double then
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, (0).to_natural_32, (0).to_natural_32, 1.0)
				l_j := (1).to_natural_32
				from
				until
					not (l_j < a_A___0.item.size2)
				loop
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, (0).to_natural_32, l_j, 0.0)
					l_j := l_j + 1
				end
				l_i := (1).to_natural_32
				from
				until
					not (l_i < a_A___0.item.size1)
				loop
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, (0).to_natural_32, 0.0)
					l_i := l_i + 1
				end
				Result := 0
				js.return
			end
			if not js.return_called then
				l_j___0 := (1).to_natural_32
				from
				until
					not (not js.loop_left and then (l_j___0 < a_A___0.item.size2))
				loop
					l_wj := 0.0
					l_i___0 := (1).to_natural_32
					from
					until
						not (l_i___0 < a_A___0.item.size1)
					loop
						l_vi := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i___0, (0).to_natural_32))
						l_wj := l_wj + ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i___0, l_j___0)) * l_vi)
						l_i___0 := l_i___0 + 1
					end
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, (0).to_natural_32, l_j___0, -a_tau * l_wj)
					l_i___0 := (1).to_natural_32
					from
					until
						not (l_i___0 < a_A___0.item.size1)
					loop
						l_vi___0 := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i___0, (0).to_natural_32))
						l_Aij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i___0, l_j___0))
						gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i___0, l_j___0, l_Aij - (a_tau * l_vi___0) * l_wj)
						l_i___0 := l_i___0 + 1
					end
					l_j___0 := l_j___0 + 1
				end
				l_i___0 := (1).to_natural_32
				from
				until
					not (l_i___0 < a_A___0.item.size1)
				loop
					l_vi___1 := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i___0, (0).to_natural_32))
					gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i___0, (0).to_natural_32, -a_tau * l_vi___1)
					l_i___0 := l_i___0 + 1
				end
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, (0).to_natural_32, (0).to_natural_32, 1.0 - a_tau)
				Result := 0
			end
		end

end
