class
	P_JACOBI

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	symschur2 (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_p: NATURAL_32; a_q___3: NATURAL_32; a_c___1: CE_POINTER [REAL_64]; a_s: CE_POINTER [REAL_64]): REAL_64
		local
			l_Apq: REAL_64
			l_App: REAL_64
			l_Aqq: REAL_64
			l_tau: REAL_64
			l_t: REAL_64
			l_c1___2: REAL_64
		do
			l_Apq := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, a_p, a_q___3))
			if l_Apq /= 0.0 then
				l_App := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, a_p, a_p))
				l_Aqq := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, a_q___3, a_q___3))
				l_tau := (l_Aqq - l_App) / (2.0 * l_Apq)
				if l_tau >= 0.0 then
					l_t := 1.0 / (l_tau + (hypot (1.0, l_tau)))
				else
					l_t := -1.0 / (-l_tau + (hypot (1.0, l_tau)))
				end
				l_c1___2 := 1.0 / (hypot (1.0, l_t))
				a_c___1.item := l_c1___2
				a_s.item := l_t * l_c1___2
			else
				a_c___1.item := 1.0
				a_s.item := 0.0
			end
			Result := (fabs (l_Apq))
		end

	apply_jacobi_L (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_p: NATURAL_32; a_q___3: NATURAL_32; a_c___1: REAL_64; a_s: REAL_64)
		local
			l_j: NATURAL_32
			l_N: NATURAL_32
			l_Apj: REAL_64
			l_Aqj: REAL_64
		do
			l_N := a_A___0.item.size2
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_N)
			loop
				l_Apj := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, a_p, l_j))
				l_Aqj := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, a_q___3, l_j))
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, a_p, l_j, l_Apj * a_c___1 - l_Aqj * a_s)
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, a_q___3, l_j, l_Apj * a_s + l_Aqj * a_c___1)
				l_j := l_j + 1
			end
		end

	apply_jacobi_R (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_p: NATURAL_32; a_q___3: NATURAL_32; a_c___1: REAL_64; a_s: REAL_64)
		local
			l_i: NATURAL_32
			l_M: NATURAL_32
			l_Aip: REAL_64
			l_Aiq: REAL_64
		do
			l_M := a_A___0.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_M)
			loop
				l_Aip := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, a_p))
				l_Aiq := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, a_q___3))
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, a_p, l_Aip * a_c___1 - l_Aiq * a_s)
				gsl_gsl_matrix_double_inst.gsl_matrix_set (a_A___0, l_i, a_q___3, l_Aip * a_s + l_Aiq * a_c___1)
				l_i := l_i + 1
			end
		end

	norm (a_A___0: CE_POINTER [PD_GSL_MATRIX]): REAL_64
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_sum: REAL_64
			l_scale: REAL_64
			l_ssq: REAL_64
			l_Aij: REAL_64
			l_ax: REAL_64
			js: JUMP_STATE
		do
			l_M := a_A___0.item.size1
			l_N := a_A___0.item.size2
			l_sum := 0.0
			l_scale := 0.0
			l_ssq := 1.0
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					l_Aij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, l_i, l_j))
					if l_Aij /= 0.0 then
						l_ax := (fabs (l_Aij))
						if l_scale < l_ax then
							l_ssq := 1.0 + (l_ssq * (l_scale / l_ax)) * (l_scale / l_ax)
							l_scale := l_ax
						else
							l_ssq := l_ssq + ((l_ax / l_scale) * (l_ax / l_scale))
						end
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			l_sum := l_scale * (sqrt (l_ssq))
			Result := l_sum
		end

	gsl_eigen_jacobi (a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX]; a_max_rot: NATURAL_32; a_nrot: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_i: NATURAL_32
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_red: REAL_64
			l_redsum: REAL_64
			l_nrm: REAL_64
			l_c___1: REAL_64
			l_s: REAL_64
			l_ep: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_a___5.item.size1
					l_N := a_a___5.item.size2
					l_redsum := 0.0
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("eigenproblem requires square matrix"), ce_string ("jacobi.c"), 146, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_M = a_evec.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_M = a_evec.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("eigenvector matrix must match input matrix"), ce_string ("jacobi.c"), 150, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if l_M /= a_eval.item.size then
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("eigenvalue vector must match input matrix"), ce_string ("jacobi.c"), 154, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
				when 10 then -- if done | if done
						if not js.return_called then
							init_source_inst.gsl_vector_set_zero (a_eval)
							init_source_inst.gsl_matrix_set_identity (a_evec)
							l_i := (0).to_natural_32
							from
							until
								not (not js.loop_left and then (l_i < a_max_rot))
							loop
								js.reset
								l_nrm := (norm (a_a___5))
								if l_nrm = 0.0 then
									js.break
								end
								if not js.loop_jumped then
									l_p := (0).to_natural_32
									from
									until
										not (not js.loop_left and then (l_p < l_N))
									loop
										l_q___3 := l_p + {NATURAL_32} 1
										from
										until
											not (l_q___3 < l_N)
										loop
											l_red := symschur2 (a_a___5, l_p, l_q___3, create {CE_POINTER [REAL_64]}.make_shared ($l_c___1), create {CE_POINTER [REAL_64]}.make_shared ($l_s))
											l_redsum := l_redsum + l_red
											apply_jacobi_L (a_a___5, l_p, l_q___3, l_c___1, l_s)
											apply_jacobi_R (a_a___5, l_p, l_q___3, l_c___1, l_s)
											apply_jacobi_R (a_evec, l_p, l_q___3, l_c___1, l_s)
											l_q___3 := l_q___3 + 1
										end
										l_p := l_p + 1
									end
									l_i := l_i + 1
								end
							end
							js.reset
							a_nrot.item := l_i
							l_p := (0).to_natural_32
							from
							until
								not (l_p < l_N)
							loop
								l_ep := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_a___5, l_p, l_p))
								gsl_gsl_vector_double_inst.gsl_vector_set (a_eval, l_p, l_ep)
								l_p := l_p + 1
							end
							if l_i = a_max_rot then
								Result := 11
								js.return
							end
						end
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_eigen_invert_jacobi (a_a___5: CE_POINTER [PD_GSL_MATRIX]; a_ainv: CE_POINTER [PD_GSL_MATRIX]; a_max_rot: NATURAL_32): INTEGER_32
		local
			l_n: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_nrot: NATURAL_32
			l_status: INTEGER_32
			l_eval: CE_POINTER [PD_GSL_VECTOR]
			l_evec: CE_POINTER [PD_GSL_MATRIX]
			l_tmp___1: CE_POINTER [PD_GSL_MATRIX]
			l_ainv_ij: REAL_64
			l_f: REAL_64
			l_vik: REAL_64
			l_vjk: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_eval.make
			create l_evec.make
			create l_tmp___1.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_a___5.item.size1 = a_a___5.item.size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_ainv.item.size1 = a_ainv.item.size2 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("jacobi method requires square matrix"), ce_string ("jacobi.c"), 208, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_a___5.item.size1 /= a_ainv.item.size2 then
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("inverse matrix must match input matrix"), ce_string ("jacobi.c"), 212, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
				when 7 then -- if done
						if not js.return_called then
							l_n := a_a___5.item.size2
							l_nrot := {NATURAL_32} 0
							l_eval.ce_gets ((init_source_inst.gsl_vector_alloc (l_n)))
							l_evec.ce_gets ((init_source_inst.gsl_matrix_alloc (l_n, l_n)))
							l_tmp___1.ce_gets ((init_source_inst.gsl_matrix_alloc (l_n, l_n)))
							copy_source_inst.gsl_matrix_memcpy (l_tmp___1, a_a___5).do_nothing
							l_status := gsl_eigen_jacobi (l_tmp___1, l_eval, l_evec, a_max_rot, create {CE_POINTER [NATURAL_32]}.make_shared ($l_nrot))
							l_i := (0).to_natural_32
							from
							until
								not (not js.loop_left and then (l_i < l_n))
							loop
								l_j := (0).to_natural_32
								from
								until
									not (not js.loop_left and then (l_j < l_n))
								loop
									l_ainv_ij := 0.0
									l_k := (0).to_natural_32
									from
									until
										not (l_k < l_n)
									loop
										l_f := 1.0 / (gsl_gsl_vector_double_inst.gsl_vector_get (l_eval, l_k))
										l_vik := (gsl_gsl_matrix_double_inst.gsl_matrix_get (l_evec, l_i, l_k))
										l_vjk := (gsl_gsl_matrix_double_inst.gsl_matrix_get (l_evec, l_j, l_k))
										l_ainv_ij := l_ainv_ij + ((l_vik * l_vjk) * l_f)
										l_k := l_k + 1
									end
									gsl_gsl_matrix_double_inst.gsl_matrix_set (a_ainv, l_i, l_j, l_ainv_ij)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
							init_source_inst.gsl_vector_free (l_eval)
							init_source_inst.gsl_matrix_free (l_evec)
							init_source_inst.gsl_matrix_free (l_tmp___1)
							if l_status.to_boolean then
								Result := l_status
							else
								Result := 0
							end
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
