class
	P_LAMBERT

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create c2___0.make (<<12>>)
		end

feature {ANY} -- attributes

	c2___0: CE_ARRAY [REAL_64]

	MAX_ITERS: NATURAL_32 assign set_MAX_ITERS

	set_MAX_ITERS (a_MAX_ITERS: NATURAL_32)
		do
			MAX_ITERS := a_MAX_ITERS
		end

	MAX_ITERS___0: NATURAL_32 assign set_MAX_ITERS___0

	set_MAX_ITERS___0 (a_MAX_ITERS___0: NATURAL_32)
		do
			MAX_ITERS___0 := a_MAX_ITERS___0
		end

feature {ANY} -- routines

	halley_iteration (a_x: REAL_64; a_w_initial: REAL_64; a_max_iters: NATURAL_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_w: REAL_64
			l_i: NATURAL_32
			l_tol: REAL_64
			l_e: REAL_64
			l_p: REAL_64
			l_t: REAL_64
			l_tmp___0: REAL_64
			js: JUMP_STATE
		do
			l_w := a_w_initial
			l_i := {NATURAL_32} 0
			from
			until
				not (not js.loop_left and then (l_i < a_max_iters))
			loop
				l_e := (exp (l_w))
				l_p := l_w + 1.0
				l_t := l_w * l_e - a_x
				if l_w > (0).to_double then
					l_t := (l_t / l_p) / l_e
				else
					l_t := l_t / (l_e * l_p - ((0.5 * (l_p + 1.0)) * l_t) / l_p)
				end
				l_w := l_w - l_t
				l_tmp___0 := fabs (l_p)
				l_tol := ((10).to_double * 2.2204460492503131e-16) * (gsl_gsl_minmax_inst.GSL_MAX_DBL ((fabs (l_w)), 1.0 / (l_tmp___0 * l_e)))
				if (fabs (l_t)) < l_tol then
					a_result.item.val := l_w
					a_result.item.err := 2.0 * l_tol
					Result := 0
					js.return
				end
				if not js.return_called then
					l_i := l_i + 1
				end
			end
			if not js.return_called then
				a_result.item.val := l_w
				a_result.item.err := fabs (l_w)
				Result := 11
			end
		end

	series_eval (a_r___3: REAL_64): REAL_64
		local
			l_t_8: REAL_64
			l_t_5: REAL_64
			l_t_1: REAL_64
		do
			l_t_8 := (c2___0[8] + (a_r___3 * (c2___0[9] + (a_r___3 * (c2___0[10] + (a_r___3 * c2___0[11]))))))
			l_t_5 := (c2___0[5] + (a_r___3 * (c2___0[6] + (a_r___3 * (c2___0[7] + (a_r___3 * l_t_8))))))
			l_t_1 := (c2___0[1] + (a_r___3 * (c2___0[2] + (a_r___3 * (c2___0[3] + (a_r___3 * (c2___0[4] + (a_r___3 * l_t_5))))))))
			Result := (c2___0[0] + (a_r___3 * l_t_1))
		end

	gsl_sf_lambert_W0_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_one_over_E: REAL_64
			l_q___3: REAL_64
			l_r___3: REAL_64
			l_w: REAL_64
			l_p: REAL_64
		do
			l_one_over_E := 1.0 / 2.7182818284590452354
			l_q___3 := a_x + l_one_over_E
			if a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
			elseif l_q___3 < 0.0 then
				a_result.item.val := -1.0
				a_result.item.err := sqrt (-l_q___3)
				Result := 1
			elseif l_q___3 = 0.0 then
				a_result.item.val := -1.0
				a_result.item.err := 2.2204460492503131e-16
				Result := 0
			elseif l_q___3 < 1.0e-03 then
				l_r___3 := (sqrt (l_q___3))
				a_result.item.val := series_eval (l_r___3)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				if a_x < 1.0 then
					l_p := (sqrt ((2.0 * 2.7182818284590452354) * l_q___3))
					l_w := -1.0 + l_p * (1.0 + l_p * (-1.0 / 3.0 + (l_p * 11.0) / 72.0))
				else
					l_w := log (a_x)
					if a_x > 3.0 then
						l_w := l_w - (log (l_w))
					end
				end
				Result := (halley_iteration (a_x, l_w, MAX_ITERS, a_result))
			end
		end

	gsl_sf_lambert_Wm1_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_one_over_E: REAL_64
			l_q___3: REAL_64
			l_w: REAL_64
			l_r___3: REAL_64
			l_L_1: REAL_64
			l_L_2: REAL_64
			js: JUMP_STATE
		do
			if a_x > 0.0 then
				Result := (gsl_sf_lambert_W0_e (a_x, a_result))
			elseif a_x = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
			else
				l_one_over_E := 1.0 / 2.7182818284590452354
				l_q___3 := a_x + l_one_over_E
				if l_q___3 < 0.0 then
					a_result.item.val := -1.0
					a_result.item.err := sqrt (-l_q___3)
					Result := 1
					js.return
				end
				if not js.return_called then
					if a_x < -1.0e-6 then
						l_r___3 := -(sqrt (l_q___3))
						l_w := series_eval (l_r___3)
						if l_q___3 < 3.0e-3 then
							a_result.item.val := l_w
							a_result.item.err := (5.0 * 2.2204460492503131e-16) * (fabs (l_w))
							Result := 0
							js.return
						end
					else
						l_L_1 := (log (-a_x))
						l_L_2 := (log (-l_L_1))
						l_w := (l_L_1 - l_L_2) + l_L_2 / l_L_1
					end
				end
				if not js.return_called then
					Result := (halley_iteration (a_x, l_w, MAX_ITERS___0, a_result))
				end
			end
		end

	gsl_sf_lambert_W0 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_lambert_W0_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_lambert_W0_e(x, &result)"), ce_string ("lambert.c"), 225, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_lambert_Wm1 (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_lambert_Wm1_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_lambert_Wm1_e(x, &result)"), ce_string ("lambert.c"), 230, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
