class
	P_LEGENDRE_H3D

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	legendre_H3d_lnnorm (a_ell: INTEGER_32; a_lambda: REAL_64; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_abs_lam: REAL_64
			l_rat: REAL_64
			l_ln_lam2ell2: REAL_64
			l_tmp___0: REAL_64
			l_lg_corrected: REAL_64
			l_angle_terms: REAL_64
			l_lg_r: PD_GSL_SF_RESULT_STRUCT
			l_lg_theta: PD_GSL_SF_RESULT_STRUCT
			l_ln_sinh: PD_GSL_SF_RESULT_STRUCT
			js: JUMP_STATE
		do
			create l_lg_r.make
			create l_lg_theta.make
			create l_ln_sinh.make

			l_abs_lam := (fabs (a_lambda))
			if l_abs_lam = 0.0 then
				a_result.item := 0.0
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("legendre_H3d.c"), 52, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_lambda > (a_ell.to_double + 1.0) / 6.0554544523933429e-06 then
				l_rat := (a_ell.to_double + 1.0) / a_lambda
				l_tmp___0 := log (a_lambda)
				l_ln_lam2ell2 := 2.0 * l_tmp___0 + (log (1.0 + l_rat * l_rat))
				l_lg_corrected := ((-2.0 * (a_ell.to_double + 1.0) + 1.14472988584940017414342735135) + (a_ell.to_double + 0.5) * l_ln_lam2ell2) + 1.0 / ((288.0 * a_lambda) * a_lambda)
				l_angle_terms := ((a_lambda * 2.0) * l_rat) * (1.0 - (l_rat * l_rat) / 3.0)
				a_result.item := (((log (l_abs_lam)) + l_lg_corrected) + l_angle_terms) - 1.14472988584940017414342735135
				Result := 0
				js.return
			else
				gamma_inst.gsl_sf_lngamma_complex_e (a_ell.to_double + 1.0, a_lambda, l_lg_r.address, l_lg_theta.address).do_nothing
				trig_inst.gsl_sf_lnsinh_e (3.14159265358979323846 * l_abs_lam, l_ln_sinh.address).do_nothing
				a_result.item := (((log (l_abs_lam)) + l_ln_sinh.val) + 2.0 * l_lg_r.val) - 1.14472988584940017414342735135
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	legendre_H3d_series (a_ell: INTEGER_32; a_lambda: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_nmax: INTEGER_32
			l_shheta: REAL_64
			l_ln_zp1: REAL_64
			l_ln_zm1: REAL_64
			l_zeta: REAL_64
			l_lg_lp32: PD_GSL_SF_RESULT_STRUCT
			l_term: REAL_64
			l_sum: REAL_64
			l_sum_err: REAL_64
			l_lnsheta: PD_GSL_SF_RESULT_STRUCT
			l_lnN: REAL_64
			l_lnpre_val: REAL_64
			l_lnpre_err: REAL_64
			l_lnprepow: REAL_64
			l_stat_e: INTEGER_32
			l_n: INTEGER_32
			l_tmp___6: REAL_64
			l_aR: REAL_64
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			l_tmp___14: INTEGER_32
			l_tmp___15: INTEGER_32
			js: JUMP_STATE
		do
			create l_lg_lp32.make
			create l_lnsheta.make

			l_nmax := 5000
			l_shheta := (sinh (0.5 * a_eta))
			l_ln_zp1 := 0.69314718055994530942 + (log (1.0 + l_shheta * l_shheta))
			l_ln_zm1 := 0.69314718055994530942 + 2.0 * (log (l_shheta))
			l_zeta := -l_shheta * l_shheta
			l_term := 1.0
			l_sum := 1.0
			l_sum_err := 0.0
			gamma_inst.gsl_sf_lngamma_e (a_ell.to_double + 3.0 / 2.0, l_lg_lp32.address).do_nothing
			trig_inst.gsl_sf_lnsinh_e (a_eta, l_lnsheta.address).do_nothing
			legendre_H3d_lnnorm (a_ell, a_lambda, create {CE_POINTER [REAL_64]}.make_shared ($l_lnN)).do_nothing
			l_lnprepow := (0.5 * (a_ell.to_double + 0.5)) * (l_ln_zm1 - l_ln_zp1)
			l_lnpre_val := ((l_lnprepow + 0.5 * (((l_lnN + 1.14472988584940017414342735135) - 0.69314718055994530942) - l_lnsheta.val)) - l_lg_lp32.val) - (log ((fabs (a_lambda))))
			l_lnpre_err := (l_lnsheta.err + l_lg_lp32.err) + 2.2204460492503131e-16 * (fabs (l_lnpre_val))
			l_lnpre_err := l_lnpre_err + ((2.0 * 2.2204460492503131e-16) * (((fabs (l_lnN)) + 1.14472988584940017414342735135) + 0.69314718055994530942))
			l_tmp___6 := fabs (l_ln_zm1)
			l_lnpre_err := l_lnpre_err + ((2.0 * 2.2204460492503131e-16) * ((0.5 * (a_ell.to_double + 0.5)) * (l_tmp___6 + (fabs (l_ln_zp1)))))
			l_n := 1
			from
			until
				not (not js.loop_left and then (l_n < l_nmax))
			loop
				js.reset
				l_aR := l_n.to_double - 0.5
				l_term := l_term * ((((l_aR * l_aR + a_lambda * a_lambda) * l_zeta) / ((a_ell + l_n).to_double + 0.5)) / l_n.to_double)
				l_sum := l_sum + l_term
				l_sum_err := l_sum_err + ((2.0 * 2.2204460492503131e-16) * (fabs (l_term)))
				if (fabs (l_term / l_sum)) < 2.0 * 2.2204460492503131e-16 then
					js.break
				end
				if not js.loop_jumped then
					l_n := l_n + 1
				end
			end
			js.reset
			l_stat_e := exp_inst.gsl_sf_exp_mult_err_e (l_lnpre_val, l_lnpre_err, l_sum, (fabs (l_term)) + l_sum_err, a_result)
			if l_stat_e /= 0 then
				l_tmp___15 := l_stat_e
			else
				if l_n = l_nmax then
					l_tmp___14 := 11
				else
					l_tmp___14 := 0
				end
				if l_tmp___14 /= 0 then
					if l_n = l_nmax then
						l_tmp___12 := 11
					else
						l_tmp___12 := 0
					end
					l_tmp___13 := l_tmp___12
				else
					l_tmp___13 := 0
				end
				l_tmp___15 := l_tmp___13
			end
			Result := l_tmp___15
		end

	legendre_H3d_CF1_ser (a_ell: INTEGER_32; a_lambda: REAL_64; a_coth_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_pre: REAL_64
			l_maxk: INTEGER_32
			l_tk: REAL_64
			l_sum: REAL_64
			l_rhok: REAL_64
			l_sum_err: REAL_64
			l_k: INTEGER_32
			l_tlk: REAL_64
			l_l1k: REAL_64
			l_ak: REAL_64
			js: JUMP_STATE
		do
			l_pre := (hypot (a_lambda, a_ell.to_double + 1.0)) / ((2.0 * a_ell.to_double + (3).to_double) * a_coth_eta)
			l_maxk := 20000
			l_tk := 1.0
			l_sum := 1.0
			l_rhok := 0.0
			l_sum_err := 0.0
			l_k := 1
			from
			until
				not (not js.loop_left and then (l_k < l_maxk))
			loop
				js.reset
				l_tlk := (2.0 * a_ell.to_double + 1.0) + 2.0 * l_k.to_double
				l_l1k := (a_ell.to_double + 1.0) + l_k.to_double
				l_ak := -(a_lambda * a_lambda + l_l1k * l_l1k) / (((l_tlk * (l_tlk + 2.0)) * a_coth_eta) * a_coth_eta)
				l_rhok := (-l_ak * (1.0 + l_rhok)) / (1.0 + l_ak * (1.0 + l_rhok))
				l_tk := l_tk * l_rhok
				l_sum := l_sum + l_tk
				l_sum_err := l_sum_err + (((2.0 * 2.2204460492503131e-16) * l_k.to_double) * (fabs (l_tk)))
				if (fabs (l_tk / l_sum)) < 2.2204460492503131e-16 then
					js.break
				end
				if not js.loop_jumped then
					l_k := l_k + 1
				end
			end
			js.reset
			a_result.item.val := l_pre * l_sum
			a_result.item.err := fabs (l_pre * l_tk)
			a_result.item.err := a_result.item.err + (fabs (l_pre * l_sum_err))
			a_result.item.err := a_result.item.err + ((4.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
			if l_k >= l_maxk then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("legendre_H3d.c"), 238, 11)
					Result := 11
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_legendre_H3d_0_e (a_lambda: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lam_eta: REAL_64
			l_s: PD_GSL_SF_RESULT_STRUCT
			l_f: REAL_64
			l_tmp___3: REAL_64
			l_f___0: REAL_64
			l_tmp___8: REAL_64
			js: JUMP_STATE
		do
			create l_s.make

			if a_eta < 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("legendre_H3d.c"), 253, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_eta = 0.0 then
				a_result.item.val := 1.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif a_lambda = 0.0 then
				a_result.item.val := 1.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			else
				l_lam_eta := a_lambda * a_eta
				trig_inst.gsl_sf_sin_err_e (l_lam_eta, (2.0 * 2.2204460492503131e-16) * (fabs (l_lam_eta)), l_s.address).do_nothing
				if a_eta > -0.5 * -3.6043653389117154e+01 then
					l_f := (2.0 / a_lambda) * (exp (-a_eta))
					a_result.item.val := l_f * l_s.val
					l_tmp___3 := fabs (l_f * l_s.val)
					a_result.item.err := (l_tmp___3 * ((fabs (a_eta)) + 1.0)) * 2.2204460492503131e-16
					a_result.item.err := a_result.item.err + ((fabs (l_f)) * l_s.err)
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				else
					l_f___0 := 1.0 / (a_lambda * (sinh (a_eta)))
					a_result.item.val := l_f___0 * l_s.val
					l_tmp___8 := fabs (l_f___0 * l_s.val)
					a_result.item.err := (l_tmp___8 * ((fabs (a_eta)) + 1.0)) * 2.2204460492503131e-16
					a_result.item.err := a_result.item.err + ((fabs (l_f___0)) * l_s.err)
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				end
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_legendre_H3d_1_e (a_lambda: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_xi: REAL_64
			l_lsq: REAL_64
			l_lsqp1: REAL_64
			l_etasq: REAL_64
			l_xisq: REAL_64
			l_term1: REAL_64
			l_term2: REAL_64
			l_sinh_term: REAL_64
			l_pre: REAL_64
			l_tmp___3: REAL_64
			l_sin_term: REAL_64
			l_cos_term: REAL_64
			l_coth_term: REAL_64
			l_sinh_term___0: REAL_64
			l_sin_term_err: REAL_64
			l_cos_term_err: REAL_64
			l_t1: REAL_64
			l_pre_val: REAL_64
			l_pre_err: REAL_64
			l_term1___0: REAL_64
			l_term2___0: REAL_64
			l_sin_xi_result: PD_GSL_SF_RESULT_STRUCT
			l_cos_xi_result: PD_GSL_SF_RESULT_STRUCT
			l_tmp___12: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_sin_xi_result.make
			create l_cos_xi_result.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_xi := (fabs (a_eta * a_lambda))
					l_lsq := a_lambda * a_lambda
					l_lsqp1 := l_lsq + 1.0
					if not (a_eta < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_left
						loop
							js.reset
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("legendre_H3d.c"), 293, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if a_eta /= 0.0 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						a_result.item.val := 0.0
						a_result.item.err := 0.0
						Result := 0
						js.return
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if a_lambda /= 0.0 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						a_result.item.val := 0.0
						a_result.item.err := 0.0
						Result := 0
						js.return
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (l_xi < 7.4009597974140505e-04) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if a_eta < 7.4009597974140505e-04 then
							l_etasq := a_eta * a_eta
							l_xisq := l_xi * l_xi
							l_term1 := (l_etasq + l_xisq) / 3.0
							l_term2 := -(((2.0 * l_etasq) * l_etasq + (5.0 * l_etasq) * l_xisq) + (3.0 * l_xisq) * l_xisq) / 90.0
							l_sinh_term := 1.0 - ((a_eta * a_eta) / 6.0) * (1.0 - ((7.0 / 60.0) * a_eta) * a_eta)
							l_pre := (l_sinh_term / (sqrt (l_lsqp1))) / a_eta
							a_result.item.val := l_pre * (l_term1 + l_term2)
							l_tmp___3 := fabs (l_term1)
							a_result.item.err := (l_pre * 2.2204460492503131e-16) * (l_tmp___3 + (fabs (l_term2)))
							a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
							Result := 0
							js.return
						else
							l_pos := -1 + 12 -- goto _L
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else: | _L:
						if l_xi < 7.4009597974140505e-04 then
							l_sin_term := 1.0 - ((l_xi * l_xi) / 6.0) * (1.0 - (l_xi * l_xi) / 20.0)
							l_cos_term := 1.0 - ((0.5 * l_xi) * l_xi) * (1.0 - (l_xi * l_xi) / 12.0)
							l_sin_term_err := 2.2204460492503131e-16
							l_cos_term_err := 2.2204460492503131e-16
						else
							trig_inst.gsl_sf_sin_e (l_xi, l_sin_xi_result.address).do_nothing
							trig_inst.gsl_sf_cos_e (l_xi, l_cos_xi_result.address).do_nothing
							l_sin_term := l_sin_xi_result.val / l_xi
							l_cos_term := l_cos_xi_result.val
							l_sin_term_err := l_sin_xi_result.err / (fabs (l_xi))
							l_cos_term_err := l_cos_xi_result.err
						end
						if a_eta < 7.4009597974140505e-04 then
							l_coth_term := 1.0 + ((a_eta * a_eta) / 3.0) * (1.0 - (a_eta * a_eta) / 15.0)
							l_sinh_term___0 := 1.0 - ((a_eta * a_eta) / 6.0) * (1.0 - ((7.0 / 60.0) * a_eta) * a_eta)
						else
							l_coth_term := a_eta / (tanh (a_eta))
							l_sinh_term___0 := a_eta / (sinh (a_eta))
						end
						l_t1 := (sqrt (l_lsqp1)) * a_eta
						l_pre_val := l_sinh_term___0 / l_t1
						l_pre_err := (2.0 * 2.2204460492503131e-16) * (fabs (l_pre_val))
						l_term1___0 := l_sin_term * l_coth_term
						l_term2___0 := l_cos_term
						a_result.item.val := l_pre_val * (l_term1___0 - l_term2___0)
						a_result.item.err := l_pre_err * (fabs (l_term1___0 - l_term2___0))
						a_result.item.err := a_result.item.err + (l_pre_val * (l_sin_term_err * l_coth_term + l_cos_term_err))
						l_tmp___12 := fabs (l_term1___0 - l_term2___0)
						a_result.item.err := a_result.item.err + (((l_pre_val * l_tmp___12) * ((fabs (a_eta)) + 1.0)) * 2.2204460492503131e-16)
						a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
						Result := 0
						js.return
				when 13 then -- if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 14 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_legendre_H3d_e (a_ell: INTEGER_32; a_lambda: REAL_64; a_eta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_abs_lam: REAL_64
			l_lsq: REAL_64
			l_xi: REAL_64
			l_cosh_eta: REAL_64
			l_P___2: PD_GSL_SF_RESULT_STRUCT
			l_lm: REAL_64
			l_stat_P: INTEGER_32
			l_lnN: REAL_64
			l_lnsh: PD_GSL_SF_RESULT_STRUCT
			l_ln_abslam: REAL_64
			l_lnpre_val: REAL_64
			l_lnpre_err: REAL_64
			l_stat_e: INTEGER_32
			l_tmp___9: REAL_64
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			l_P___3: PD_GSL_SF_RESULT_STRUCT
			l_lm___0: REAL_64
			l_stat_P___0: INTEGER_32
			l_lnN___0: REAL_64
			l_lnsh___0: PD_GSL_SF_RESULT_STRUCT
			l_ln_abslam___0: REAL_64
			l_lnpre_val___0: REAL_64
			l_lnpre_err___0: REAL_64
			l_stat_e___0: INTEGER_32
			l_tmp___15: REAL_64
			l_tmp___18: INTEGER_32
			l_tmp___19: INTEGER_32
			l_coth_eta: REAL_64
			l_coth_err_mult: REAL_64
			l_rH: PD_GSL_SF_RESULT_STRUCT
			l_stat_CF1: INTEGER_32
			l_Hlm1: REAL_64
			l_Hl: REAL_64
			l_Hlp1: REAL_64
			l_lp: INTEGER_32
			l_root_term_0: REAL_64
			l_root_term_1: REAL_64
			l_H0: PD_GSL_SF_RESULT_STRUCT
			l_stat_H0: INTEGER_32
			l_tmp___27: REAL_64
			l_tmp___30: INTEGER_32
			l_tmp___31: INTEGER_32
			l_H1: PD_GSL_SF_RESULT_STRUCT
			l_stat_H1: INTEGER_32
			l_tmp___34: REAL_64
			l_tmp___37: INTEGER_32
			l_tmp___38: INTEGER_32
			l_tmp___39: REAL_64
			js: JUMP_STATE
		do
			create l_P___2.make
			create l_lnsh.make
			create l_P___3.make
			create l_lnsh___0.make
			create l_rH.make
			create l_H0.make
			create l_H1.make

			l_abs_lam := (fabs (a_lambda))
			l_lsq := l_abs_lam * l_abs_lam
			l_xi := l_abs_lam * a_eta
			l_cosh_eta := (cosh (a_eta))
			if a_eta < 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("legendre_H3d.c"), 375, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_eta > 7.0978271289338397e+02 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("legendre_H3d.c"), 379, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_ell = 0 then
				Result := (gsl_sf_legendre_H3d_0_e (a_lambda, a_eta, a_result))
				js.return
			elseif a_ell = 1 then
				Result := (gsl_sf_legendre_H3d_1_e (a_lambda, a_eta, a_result))
				js.return
			elseif a_eta = 0.0 then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
				js.return
			elseif l_xi < 1.0 then
				Result := (legendre_H3d_series (a_ell, a_lambda, a_eta, a_result))
				js.return
			elseif (((a_ell * a_ell).to_double + l_lsq) / (sqrt (1.0 + l_lsq))) / (l_cosh_eta * l_cosh_eta) < 5.0 * 6.0554544523933429e-06 then
				l_stat_P := (legendre_con_inst.gsl_sf_conicalP_large_x_e ((-a_ell).to_double - 0.5, a_lambda, l_cosh_eta, l_P___2.address, create {CE_POINTER [REAL_64]}.make_shared ($l_lm)))
				if l_P___2.val = 0.0 then
					a_result.item.val := 0.0
					a_result.item.err := 0.0
					Result := l_stat_P
					js.return
				else
					trig_inst.gsl_sf_lnsinh_e (a_eta, l_lnsh.address).do_nothing
					legendre_H3d_lnnorm (a_ell, a_lambda, create {CE_POINTER [REAL_64]}.make_shared ($l_lnN)).do_nothing
					l_ln_abslam := log (l_abs_lam)
					l_lnpre_val := 0.5 * (((1.14472988584940017414342735135 + l_lnN) - 0.69314718055994530942) - l_lnsh.val) - l_ln_abslam
					l_lnpre_err := l_lnsh.err
					l_tmp___9 := fabs (l_lnN)
					l_lnpre_err := l_lnpre_err + ((2.0 * 2.2204460492503131e-16) * (0.5 * ((1.14472988584940017414342735135 + 0.69314718055994530942) + l_tmp___9) + (fabs (l_ln_abslam))))
					l_lnpre_err := l_lnpre_err + ((2.0 * 2.2204460492503131e-16) * (fabs (l_lnpre_val)))
					l_stat_e := exp_inst.gsl_sf_exp_mult_err_e (l_lnpre_val + l_lm, l_lnpre_err, l_P___2.val, l_P___2.err, a_result)
					if l_stat_e /= 0 then
						l_tmp___13 := l_stat_e
					else
						if l_stat_P /= 0 then
							l_tmp___12 := l_stat_P
						else
							l_tmp___12 := 0
						end
						l_tmp___13 := l_tmp___12
					end
					Result := l_tmp___13
					js.return
				end
			elseif l_abs_lam > (1000.0 * a_ell.to_double) * a_ell.to_double then
				l_stat_P___0 := (legendre_con_inst.gsl_sf_conicalP_xgt1_neg_mu_largetau_e (a_ell.to_double + 0.5, a_lambda, l_cosh_eta, a_eta, l_P___3.address, create {CE_POINTER [REAL_64]}.make_shared ($l_lm___0)))
				if l_P___3.val = 0.0 then
					a_result.item.val := 0.0
					a_result.item.err := 0.0
					Result := l_stat_P___0
					js.return
				else
					trig_inst.gsl_sf_lnsinh_e (a_eta, l_lnsh___0.address).do_nothing
					legendre_H3d_lnnorm (a_ell, a_lambda, create {CE_POINTER [REAL_64]}.make_shared ($l_lnN___0)).do_nothing
					l_ln_abslam___0 := log (l_abs_lam)
					l_lnpre_val___0 := 0.5 * (((1.14472988584940017414342735135 + l_lnN___0) - 0.69314718055994530942) - l_lnsh___0.val) - l_ln_abslam___0
					l_lnpre_err___0 := l_lnsh___0.err
					l_tmp___15 := fabs (l_lnN___0)
					l_lnpre_err___0 := l_lnpre_err___0 + (2.2204460492503131e-16 * (0.5 * ((1.14472988584940017414342735135 + 0.69314718055994530942) + l_tmp___15) + (fabs (l_ln_abslam___0))))
					l_lnpre_err___0 := l_lnpre_err___0 + ((2.0 * 2.2204460492503131e-16) * (fabs (l_lnpre_val___0)))
					l_stat_e___0 := exp_inst.gsl_sf_exp_mult_err_e (l_lnpre_val___0 + l_lm___0, l_lnpre_err___0, l_P___3.val, l_P___3.err, a_result)
					if l_stat_e___0 /= 0 then
						l_tmp___19 := l_stat_e___0
					else
						if l_stat_P___0 /= 0 then
							l_tmp___18 := l_stat_P___0
						else
							l_tmp___18 := 0
						end
						l_tmp___19 := l_tmp___18
					end
					Result := l_tmp___19
					js.return
				end
			else
				l_coth_eta := 1.0 / (tanh (a_eta))
				l_coth_err_mult := (fabs (a_eta)) + 1.0
				l_stat_CF1 := (legendre_H3d_CF1_ser (a_ell, a_lambda, l_coth_eta, l_rH.address))
				l_Hl := 1.4916681462400413e-154
				l_Hlp1 := l_rH.val * l_Hl
				l_lp := a_ell
				from
				until
					not (l_lp > 0)
				loop
					l_root_term_0 := (hypot (a_lambda, l_lp.to_double))
					l_root_term_1 := (hypot (a_lambda, l_lp.to_double + 1.0))
					l_Hlm1 := (((2.0 * l_lp.to_double + 1.0) * l_coth_eta) * l_Hl - l_root_term_1 * l_Hlp1) / l_root_term_0
					l_Hlp1 := l_Hl
					l_Hl := l_Hlm1
					l_lp := l_lp - 1
				end
				l_tmp___39 := fabs (l_Hl)
				if l_tmp___39 > (fabs (l_Hlp1)) then
					l_stat_H0 := (gsl_sf_legendre_H3d_0_e (a_lambda, a_eta, l_H0.address))
					a_result.item.val := (1.4916681462400413e-154 / l_Hl) * l_H0.val
					a_result.item.err := (1.4916681462400413e-154 / (fabs (l_Hl))) * l_H0.err
					l_tmp___27 := fabs (l_rH.err / l_rH.val)
					a_result.item.err := a_result.item.err + (((l_tmp___27 * (a_ell.to_double + 1.0)) * l_coth_err_mult) * (fabs (a_result.item.val)))
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
					if l_stat_H0 /= 0 then
						l_tmp___31 := l_stat_H0
					else
						if l_stat_CF1 /= 0 then
							l_tmp___30 := l_stat_CF1
						else
							l_tmp___30 := 0
						end
						l_tmp___31 := l_tmp___30
					end
					Result := l_tmp___31
					js.return
				else
					l_stat_H1 := (gsl_sf_legendre_H3d_1_e (a_lambda, a_eta, l_H1.address))
					a_result.item.val := (1.4916681462400413e-154 / l_Hlp1) * l_H1.val
					a_result.item.err := (1.4916681462400413e-154 / (fabs (l_Hlp1))) * l_H1.err
					l_tmp___34 := fabs (l_rH.err / l_rH.val)
					a_result.item.err := a_result.item.err + (((l_tmp___34 * (a_ell.to_double + 1.0)) * l_coth_err_mult) * (fabs (a_result.item.val)))
					a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
					if l_stat_H1 /= 0 then
						l_tmp___38 := l_stat_H1
					else
						if l_stat_CF1 /= 0 then
							l_tmp___37 := l_stat_CF1
						else
							l_tmp___37 := 0
						end
						l_tmp___38 := l_tmp___37
					end
					Result := l_tmp___38
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_legendre_H3d_array (a_lmax: INTEGER_32; a_lambda: REAL_64; a_eta: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_ell: INTEGER_32
			l_ell___0: INTEGER_32
			l_H0: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
			l_r_Hlp1: PD_GSL_SF_RESULT_STRUCT
			l_r_Hl: PD_GSL_SF_RESULT_STRUCT
			l_stat_lmax: INTEGER_32
			l_stat_lmaxm1: INTEGER_32
			l_stat_max: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_coth_eta: REAL_64
			l_stat_recursion: INTEGER_32
			l_Hlp1: REAL_64
			l_Hl: REAL_64
			l_Hlm1: REAL_64
			l_ell___1: INTEGER_32
			l_root_term_0: REAL_64
			l_root_term_1: REAL_64
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_H0.make
			create l_r_Hlp1.make
			create l_r_Hl.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_eta < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_lmax < 0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						l_ell := 0
						from
						until
							not (l_ell <= a_lmax)
						loop
							a_result_array[l_ell] := 0.0
							l_ell := l_ell + 1
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("legendre_H3d.c"), 503, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_eta > 7.0978271289338397e+02 then
							l_ell___0 := 0
							from
							until
								not (l_ell___0 <= a_lmax)
							loop
								a_result_array[l_ell___0] := 0.0
								l_ell___0 := l_ell___0 + 1
							end
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("overflow"), ce_string ("legendre_H3d.c"), 509, 16)
								Result := 16
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						elseif a_lmax = 0 then
							l_stat := (gsl_sf_legendre_H3d_e (0, a_lambda, a_eta, l_H0.address))
							a_result_array[0] := l_H0.val
							Result := l_stat
							js.return
						else
							l_stat_lmax := (gsl_sf_legendre_H3d_e (a_lmax, a_lambda, a_eta, l_r_Hlp1.address))
							l_stat_lmaxm1 := (gsl_sf_legendre_H3d_e (a_lmax - 1, a_lambda, a_eta, l_r_Hl.address))
							if l_stat_lmax /= 0 then
								l_tmp___3 := l_stat_lmax
							else
								if l_stat_lmaxm1 /= 0 then
									l_tmp___2 := l_stat_lmaxm1
								else
									l_tmp___2 := 0
								end
								l_tmp___3 := l_tmp___2
							end
							l_stat_max := l_tmp___3
							l_coth_eta := 1.0 / (tanh (a_eta))
							l_stat_recursion := 0
							l_Hlp1 := l_r_Hlp1.val
							l_Hl := l_r_Hl.val
							a_result_array[a_lmax] := l_Hlp1
							a_result_array[(a_lmax - 1)] := l_Hl
							l_ell___1 := a_lmax - 1
							from
							until
								not (l_ell___1 > 0)
							loop
								l_root_term_0 := (hypot (a_lambda, l_ell___1.to_double))
								l_root_term_1 := (hypot (a_lambda, l_ell___1.to_double + 1.0))
								l_Hlm1 := (((2.0 * l_ell___1.to_double + 1.0) * l_coth_eta) * l_Hl - l_root_term_1 * l_Hlp1) / l_root_term_0
								a_result_array[(l_ell___1 - 1)] := l_Hlm1
								if not (l_Hlm1 < 1.7976931348623157e+308) then
									l_stat_recursion := 16
								end
								l_Hlp1 := l_Hl
								l_Hl := l_Hlm1
								l_ell___1 := l_ell___1 - 1
							end
							if l_stat_recursion /= 0 then
								l_tmp___8 := l_stat_recursion
							else
								if l_stat_max /= 0 then
									l_tmp___7 := l_stat_max
								else
									l_tmp___7 := 0
								end
								l_tmp___8 := l_tmp___7
							end
							Result := l_tmp___8
							js.return
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_legendre_H3d_0 (a_lambda: REAL_64; a_eta: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_legendre_H3d_0_e (a_lambda, a_eta, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_legendre_H3d_0_e(lambda, eta, &result)"), ce_string ("legendre_H3d.c"), 557, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_legendre_H3d_1 (a_lambda: REAL_64; a_eta: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_legendre_H3d_1_e (a_lambda, a_eta, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_legendre_H3d_1_e(lambda, eta, &result)"), ce_string ("legendre_H3d.c"), 562, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_legendre_H3d (a_l: INTEGER_32; a_lambda: REAL_64; a_eta: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_legendre_H3d_e (a_l, a_lambda, a_eta, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_legendre_H3d_e(l, lambda, eta, &result)"), ce_string ("legendre_H3d.c"), 567, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
