class
	P_LINEAR_MINIMIZE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	interp_quad (a_f0: REAL_64; a_fp0: REAL_64; a_f1: REAL_64; a_zl: REAL_64; a_zh: REAL_64): REAL_64
		local
			l_fl: REAL_64
			l_fh: REAL_64
			l_c___1: REAL_64
			l_zmin: REAL_64
			l_fmin___0: REAL_64
			l_z: REAL_64
			l_f: REAL_64
		do
			l_fl := a_f0 + a_zl * (a_fp0 + a_zl * ((a_f1 - a_f0) - a_fp0))
			l_fh := a_f0 + a_zh * (a_fp0 + a_zh * ((a_f1 - a_f0) - a_fp0))
			l_c___1 := (2).to_double * ((a_f1 - a_f0) - a_fp0)
			l_zmin := a_zl
			l_fmin___0 := l_fl
			if l_fh < l_fmin___0 then
				l_zmin := a_zh
				l_fmin___0 := l_fh
			end
			if l_c___1 > (0).to_double then
				l_z := -a_fp0 / l_c___1
				if l_z > a_zl then
					if l_z < a_zh then
						l_f := a_f0 + l_z * (a_fp0 + l_z * ((a_f1 - a_f0) - a_fp0))
						if l_f < l_fmin___0 then
							l_zmin := l_z
							l_fmin___0 := l_f
						end
					end
				end
			end
			Result := l_zmin
		end

	cubic (a_c0___0: REAL_64; a_c1___2: REAL_64; a_c2___0: REAL_64; a_c3___3: REAL_64; a_z: REAL_64): REAL_64
		do
			Result := (a_c0___0 + a_z * (a_c1___2 + a_z * (a_c2___0 + a_z * a_c3___3)))
		end

	check_extremum (a_c0___0: REAL_64; a_c1___2: REAL_64; a_c2___0: REAL_64; a_c3___3: REAL_64; a_z: REAL_64; a_zmin: CE_POINTER [REAL_64]; a_fmin___0: CE_POINTER [REAL_64])
		local
			l_y: REAL_64
		do
			l_y := (cubic (a_c0___0, a_c1___2, a_c2___0, a_c3___3, a_z))
			if l_y < a_fmin___0.item then
				a_zmin.item := a_z
				a_fmin___0.item := l_y
			end
		end

	interp_cubic (a_f0: REAL_64; a_fp0: REAL_64; a_f1: REAL_64; a_fp1: REAL_64; a_zl: REAL_64; a_zh: REAL_64): REAL_64
		local
			l_eta: REAL_64
			l_xi: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_zmin: REAL_64
			l_fmin___0: REAL_64
			l_z0: REAL_64
			l_z1: REAL_64
			l_n: INTEGER_32
		do
			l_eta := ((3).to_double * (a_f1 - a_f0) - (2).to_double * a_fp0) - a_fp1
			l_xi := (a_fp0 + a_fp1) - (2).to_double * (a_f1 - a_f0)
			l_c0___0 := a_f0
			l_c1___2 := a_fp0
			l_c2___0 := l_eta
			l_c3___3 := l_xi
			l_zmin := a_zl
			l_fmin___0 := cubic (l_c0___0, l_c1___2, l_c2___0, l_c3___3, a_zl)
			check_extremum (l_c0___0, l_c1___2, l_c2___0, l_c3___3, a_zh, create {CE_POINTER [REAL_64]}.make_shared ($l_zmin), create {CE_POINTER [REAL_64]}.make_shared ($l_fmin___0))
			l_n := (solve_quadratic_inst.gsl_poly_solve_quadratic ((3).to_double * l_c3___3, (2).to_double * l_c2___0, l_c1___2, create {CE_POINTER [REAL_64]}.make_shared ($l_z0), create {CE_POINTER [REAL_64]}.make_shared ($l_z1)))
			if l_n = 2 then
				if l_z0 > a_zl then
					if l_z0 < a_zh then
						check_extremum (l_c0___0, l_c1___2, l_c2___0, l_c3___3, l_z0, create {CE_POINTER [REAL_64]}.make_shared ($l_zmin), create {CE_POINTER [REAL_64]}.make_shared ($l_fmin___0))
					end
				end
				if l_z1 > a_zl then
					if l_z1 < a_zh then
						check_extremum (l_c0___0, l_c1___2, l_c2___0, l_c3___3, l_z1, create {CE_POINTER [REAL_64]}.make_shared ($l_zmin), create {CE_POINTER [REAL_64]}.make_shared ($l_fmin___0))
					end
				end
			elseif l_n = 1 then
				if l_z0 > a_zl then
					if l_z0 < a_zh then
						check_extremum (l_c0___0, l_c1___2, l_c2___0, l_c3___3, l_z0, create {CE_POINTER [REAL_64]}.make_shared ($l_zmin), create {CE_POINTER [REAL_64]}.make_shared ($l_fmin___0))
					end
				end
			end
			Result := l_zmin
		end

	interpolate (a_a___5: REAL_64; a_fa: REAL_64; a_fpa: REAL_64; a_b___0: REAL_64; a_fb: REAL_64; a_fpb: REAL_64; a_xmin: REAL_64; a_xmax: REAL_64; a_order: INTEGER_32): REAL_64
		local
			l_z: REAL_64
			l_alpha: REAL_64
			l_zmin: REAL_64
			l_zmax: REAL_64
			l_tmp: REAL_64
		do
			l_zmin := (a_xmin - a_a___5) / (a_b___0 - a_a___5)
			l_zmax := (a_xmax - a_a___5) / (a_b___0 - a_a___5)
			if l_zmin > l_zmax then
				l_tmp := l_zmin
				l_zmin := l_zmax
				l_zmax := l_tmp
			end
			if a_order > 2 then
				if (infnan_inst.gsl_finite (a_fpb)).to_boolean then
					l_z := interp_cubic (a_fa, a_fpa * (a_b___0 - a_a___5), a_fb, a_fpb * (a_b___0 - a_a___5), l_zmin, l_zmax)
				else
					l_z := interp_quad (a_fa, a_fpa * (a_b___0 - a_a___5), a_fb, l_zmin, l_zmax)
				end
			else
				l_z := interp_quad (a_fa, a_fpa * (a_b___0 - a_a___5), a_fb, l_zmin, l_zmax)
			end
			l_alpha := a_a___5 + l_z * (a_b___0 - a_a___5)
			Result := l_alpha
		end

	minimize___2 (a_fn: CE_POINTER [PD_GSL_FUNCTION_FDF_STRUCT]; a_rho: REAL_64; a_sigma: REAL_64; a_tau1: REAL_64; a_tau2: REAL_64; a_tau3: REAL_64; a_order: INTEGER_32; a_alpha1: REAL_64; a_alpha_new: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_f0: REAL_64
			l_fp0: REAL_64
			l_falpha: REAL_64
			l_falpha_prev: REAL_64
			l_fpalpha: REAL_64
			l_fpalpha_prev: REAL_64
			l_delta___0: REAL_64
			l_alpha_next: REAL_64
			l_alpha: REAL_64
			l_alpha_prev: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_fa: REAL_64
			l_fb: REAL_64
			l_fpa: REAL_64
			l_fpb: REAL_64
			l_bracket_iters: NATURAL_32
			l_section_iters: NATURAL_32
			l_i: NATURAL_32
			l_lower: REAL_64
			l_upper: REAL_64
			l_tmp___1: NATURAL_32
			l_lower___0: REAL_64
			l_upper___0: REAL_64
			l_tmp___2: REAL_32
			l_tmp___4: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_alpha := a_alpha1
					l_alpha_prev := 0.0
					l_bracket_iters := (100).to_natural_32
					l_section_iters := (100).to_natural_32
					l_i := (0).to_natural_32;
					(a_fn.item.fdf).call ([0.0, a_fn.item.params, create {CE_POINTER [REAL_64]}.make_shared ($l_f0), create {CE_POINTER [REAL_64]}.make_shared ($l_fp0)])
					l_falpha_prev := l_f0
					l_fpalpha_prev := l_fp0
					l_a___5 := 0.0
					l_b___0 := l_alpha
					l_fa := l_f0
					l_fb := 0.0
					l_fpa := l_fp0
					l_fpb := 0.0
				when 1 then -- loop condition:
					if js.loop_left then
						l_pos := -1 + 9 -- skip loop
					end
				when 2 then -- loop body:
						js.reset
						l_tmp___1 := l_i
						l_i := l_i + 1
						if not (l_tmp___1 < l_bracket_iters) then
							js.break
						end
						if js.loop_jumped then
							l_pos := -1 + 8 -- skip if block
						end
				when 3 then -- if:
						l_falpha := (a_fn.item.f).real_64_item ([l_alpha, a_fn.item.params])
						if not (l_falpha > l_f0 + (l_alpha * a_rho) * l_fp0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if not (l_falpha >= l_falpha_prev) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						l_a___5 := l_alpha_prev
						l_fa := l_falpha_prev
						l_fpa := l_fpalpha_prev
						l_b___0 := l_alpha
						l_fb := l_falpha
						l_fpb := (builtin_nanf (ce_string (""))).to_double
						js.break
				when 8 then -- if done | if done
						if not js.loop_jumped then
							l_fpalpha := (a_fn.item.df).real_64_item ([l_alpha, a_fn.item.params])
							if (fabs (l_fpalpha)) <= -a_sigma * l_fp0 then
								a_alpha_new.item := l_alpha
								Result := 0
								js.return
							end
							if not js.return_called then
								if l_fpalpha >= (0).to_double then
									l_a___5 := l_alpha
									l_fa := l_falpha
									l_fpa := l_fpalpha
									l_b___0 := l_alpha_prev
									l_fb := l_falpha_prev
									l_fpb := l_fpalpha_prev
									js.break
								end
								if not js.loop_jumped then
									l_delta___0 := l_alpha - l_alpha_prev
									l_lower := l_alpha + l_delta___0
									l_upper := l_alpha + a_tau1 * l_delta___0
									l_alpha_next := interpolate (l_alpha_prev, l_falpha_prev, l_fpalpha_prev, l_alpha, l_falpha, l_fpalpha, l_lower, l_upper, a_order)
									l_alpha_prev := l_alpha
									l_falpha_prev := l_falpha
									l_fpalpha_prev := l_fpalpha
									l_alpha := l_alpha_next
								end
							end
						end
						l_pos := -1 + 1 -- back to loop condition
				when 9 then -- loop done
					js.reset
					if js.return_called then
						l_pos := -1 + 32 -- skip if block
					end
				when 10 then -- if: | loop condition:
						if js.loop_left then
							l_pos := -1 + 31 -- skip loop
						end
				when 11 then -- loop body:
						js.reset
						l_tmp___4 := l_i
						l_i := l_i + 1
						if not (l_tmp___4 < l_section_iters) then
							js.break
						end
						if js.loop_jumped then
							l_pos := -1 + 30 -- skip if block
						end
				when 12 then -- if:
						l_delta___0 := l_b___0 - l_a___5
						l_lower___0 := l_a___5 + a_tau2 * l_delta___0
						l_upper___0 := l_b___0 - a_tau3 * l_delta___0
						l_alpha := interpolate (l_a___5, l_fa, l_fpa, l_b___0, l_fb, l_fpb, l_lower___0, l_upper___0, a_order)
						l_falpha := (a_fn.item.f).real_64_item ([l_alpha, a_fn.item.params])
						if (l_a___5 - l_alpha) * l_fpa <= 2.2204460492503131e-16 then
							Result := 27
							js.return
						end
						if js.return_called then
							l_pos := -1 + 30 -- skip if block
						end
				when 13 then -- if:
						if not (l_falpha > l_f0 + (a_rho * l_alpha) * l_fp0) then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						l_b___0 := l_alpha
						l_fb := l_falpha
						l_tmp___2 := builtin_nanf (ce_string (""))
						l_fpb := l_tmp___2.to_double
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 30
				when 16 then -- else:
						if not (l_falpha >= l_fa) then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if:
						l_b___0 := l_alpha
						l_fb := l_falpha
						l_tmp___2 := builtin_nanf (ce_string (""))
						l_fpb := l_tmp___2.to_double
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 30
				when 19 then -- else:
						l_fpalpha := (a_fn.item.df).real_64_item ([l_alpha, a_fn.item.params])
						if (fabs (l_fpalpha)) <= -a_sigma * l_fp0 then
							a_alpha_new.item := l_alpha
							Result := 0
							js.return
						end
						if js.return_called then
							l_pos := -1 + 30 -- skip if block
						end
				when 20 then -- if:
						if not (l_b___0 - l_a___5 >= (0).to_double) then
							l_pos := -1 + 23 -- skip if block
						end
				when 21 then -- if:
						if l_fpalpha >= (0).to_double then
							l_pos := -1 + 25 -- goto _L___0
						else
							l_pos := -1 + 23 -- goto _L___1
						end
				when 22 then -- if block done, skip else block:
						l_pos := -1 + 30
				when 23 then -- else: | _L___1:
						if not (l_b___0 - l_a___5 <= (0).to_double) then
							l_pos := -1 + 29 -- skip if block
						end
				when 24 then -- if:
						if not (l_fpalpha <= (0).to_double) then
							l_pos := -1 + 27 -- skip if block
						end
				when 25 then -- if: | _L___0:
						l_b___0 := l_a___5
						l_fb := l_fa
						l_fpb := l_fpa
						l_a___5 := l_alpha
						l_fa := l_falpha
						l_fpa := l_fpalpha
				when 26 then -- if block done, skip else block:
						l_pos := -1 + 28
				when 27 then -- else:
						l_a___5 := l_alpha
						l_fa := l_falpha
						l_fpa := l_fpalpha
				when 28 then -- if done | if block done, skip else block:
						l_pos := -1 + 30
				when 29 then -- else:
						l_a___5 := l_alpha
						l_fa := l_falpha
						l_fpa := l_fpalpha
				when 30 then -- if done | if done | if done | if done | if done | if done
						l_pos := -1 + 10 -- back to loop condition
				when 31 then -- loop done
						js.reset
				when 32 then -- if done
					if not js.return_called then
						Result := 0
					end
				when 33 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

end
