class
	P_LMUTIL

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	enorm (a_f: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		do
			Result := (blas_inst.gsl_blas_dnrm2 (a_f))
		end

	scaled_enorm (a_d: CE_POINTER [PD_GSL_VECTOR]; a_f: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		local
			l_e2: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_fi: REAL_64
			l_di: REAL_64
			l_u: REAL_64
		do
			l_e2 := (0).to_double
			l_n := a_f.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_fi := (gsl_gsl_vector_double_inst.gsl_vector_get (a_f, l_i))
				l_di := (gsl_gsl_vector_double_inst.gsl_vector_get (a_d, l_i))
				l_u := l_di * l_fi
				l_e2 := l_e2 + (l_u * l_u)
				l_i := l_i + 1
			end
			Result := (sqrt (l_e2))
		end

	compute_delta (a_diag: CE_POINTER [PD_GSL_VECTOR]; a_x: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		local
			l_Dx: REAL_64
			l_factor: REAL_64
			l_tmp___0: REAL_64
		do
			l_Dx := (scaled_enorm (a_diag, a_x))
			l_factor := (100).to_double
			if l_Dx > (0).to_double then
				l_tmp___0 := l_factor * l_Dx
			else
				l_tmp___0 := l_factor
			end
			Result := l_tmp___0
		end

	compute_actual_reduction (a_fnorm: REAL_64; a_fnorm1: REAL_64): REAL_64
		local
			l_actred: REAL_64
			l_u: REAL_64
		do
			if 0.1 * a_fnorm1 < a_fnorm then
				l_u := a_fnorm1 / a_fnorm
				l_actred := (1).to_double - l_u * l_u
			else
				l_actred := (-1).to_double
			end
			Result := l_actred
		end

	compute_diag (a_J: CE_POINTER [PD_GSL_MATRIX]; a_diag: CE_POINTER [PD_GSL_VECTOR])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_n: NATURAL_32
			l_p: NATURAL_32
			l_sum: REAL_64
			l_Jij: REAL_64
			js: JUMP_STATE
		do
			l_n := a_J.item.size1
			l_p := a_J.item.size2
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_j < l_p))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_Jij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_J, l_i, l_j))
					l_sum := l_sum + (l_Jij * l_Jij)
					l_i := l_i + 1
				end
				if l_sum = (0).to_double then
					l_sum := 1.0
				end
				gsl_gsl_vector_double_inst.gsl_vector_set (a_diag, l_j, (sqrt (l_sum)))
				l_j := l_j + 1
			end
		end

	update_diag (a_J: CE_POINTER [PD_GSL_MATRIX]; a_diag: CE_POINTER [PD_GSL_VECTOR])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_n: NATURAL_32
			l_cnorm: REAL_64
			l_diagj: REAL_64
			l_sum: REAL_64
			l_Jij: REAL_64
			js: JUMP_STATE
		do
			l_n := a_diag.item.size
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_j < l_n))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_Jij := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_J, l_i, l_j))
					l_sum := l_sum + (l_Jij * l_Jij)
					l_i := l_i + 1
				end
				if l_sum = (0).to_double then
					l_sum := 1.0
				end
				l_cnorm := sqrt (l_sum)
				l_diagj := gsl_gsl_vector_double_inst.gsl_vector_get (a_diag, l_j)
				if l_cnorm > l_diagj then
					gsl_gsl_vector_double_inst.gsl_vector_set (a_diag, l_j, l_cnorm)
				end
				l_j := l_j + 1
			end
		end

	compute_rptdx (a_r___3: CE_POINTER [PD_GSL_MATRIX]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_dx: CE_POINTER [PD_GSL_VECTOR]; a_rptdx: CE_POINTER [PD_GSL_VECTOR])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_N: NATURAL_32
			l_sum: REAL_64
			l_pj: NATURAL_32
			l_tmp___0: REAL_64
			js: JUMP_STATE
		do
			l_N := a_dx.item.size
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < l_N))
			loop
				l_sum := (0).to_double
				l_j := l_i
				from
				until
					not (l_j < l_N)
				loop
					l_pj := (gsl_gsl_permutation_inst.gsl_permutation_get (a_p, l_j))
					l_tmp___0 := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_r___3, l_i, l_j)
					l_sum := l_sum + (l_tmp___0 * (gsl_gsl_vector_double_inst.gsl_vector_get (a_dx, l_pj)))
					l_j := l_j + 1
				end
				gsl_gsl_vector_double_inst.gsl_vector_set (a_rptdx, l_i, l_sum)
				l_i := l_i + 1
			end
		end

	compute_trial_step (a_x: CE_POINTER [PD_GSL_VECTOR]; a_dx: CE_POINTER [PD_GSL_VECTOR]; a_x_trial: CE_POINTER [PD_GSL_VECTOR])
		local
			l_i: NATURAL_32
			l_N: NATURAL_32
			l_pi: REAL_64
			l_xi: REAL_64
		do
			l_N := a_x.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_pi := (gsl_gsl_vector_double_inst.gsl_vector_get (a_dx, l_i))
				l_xi := (gsl_gsl_vector_double_inst.gsl_vector_get (a_x, l_i))
				gsl_gsl_vector_double_inst.gsl_vector_set (a_x_trial, l_i, l_xi + l_pi)
				l_i := l_i + 1
			end
		end

end
