class
	P_MATH

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_complex_polar (a_r___3: REAL_64; a_theta: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_r___3 * (cos (a_theta))
				l_z.dat[1] := a_r___3 * (sin (a_theta))
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_arg (a_z: PD_GSL_COMPLEX): REAL_64
		local
			l_x: REAL_64
			l_y: REAL_64
			js: JUMP_STATE
		do
			l_x := a_z.dat[0]
			l_y := a_z.dat[1]
			if l_x = 0.0 then
				if l_y = 0.0 then
					Result := (0).to_double
					js.return
				end
			end
			if not js.return_called then
				Result := (atan2 (l_y, l_x))
			end
		end

	gsl_complex_abs (a_z: PD_GSL_COMPLEX): REAL_64
		do
			Result := (hypot (a_z.dat[0], a_z.dat[1]))
		end

	gsl_complex_abs2 (a_z: PD_GSL_COMPLEX): REAL_64
		local
			l_x: REAL_64
			l_y: REAL_64
		do
			l_x := a_z.dat[0]
			l_y := a_z.dat[1]
			Result := (l_x * l_x + l_y * l_y)
		end

	gsl_complex_logabs (a_z: PD_GSL_COMPLEX): REAL_64
		local
			l_xabs: REAL_64
			l_yabs: REAL_64
			l_max: REAL_64
			l_u: REAL_64
			l_tmp___1: REAL_64
		do
			l_xabs := (fabs (a_z.dat[0]))
			l_yabs := (fabs (a_z.dat[1]))
			if l_xabs >= l_yabs then
				l_max := l_xabs
				l_u := l_yabs / l_xabs
			else
				l_max := l_yabs
				l_u := l_xabs / l_yabs
			end
			l_tmp___1 := log (l_max)
			Result := (l_tmp___1 + 0.5 * (log1p (l_u * l_u)))
		end

	gsl_complex_add (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_ar: REAL_64
			l_ai: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_ar := a_a___5.dat[0]
			l_ai := a_a___5.dat[1]
			l_br := a_b___0.dat[0]
			l_bi := a_b___0.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_ar + l_br
				l_z.dat[1] := l_ai + l_bi
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_add_real (a_a___5: PD_GSL_COMPLEX; a_x: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0] + a_x
				l_z.dat[1] := a_a___5.dat[1]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_add_imag (a_a___5: PD_GSL_COMPLEX; a_y: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0]
				l_z.dat[1] := a_a___5.dat[1] + a_y
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_sub (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_ar: REAL_64
			l_ai: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_ar := a_a___5.dat[0]
			l_ai := a_a___5.dat[1]
			l_br := a_b___0.dat[0]
			l_bi := a_b___0.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_ar - l_br
				l_z.dat[1] := l_ai - l_bi
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_sub_real (a_a___5: PD_GSL_COMPLEX; a_x: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0] - a_x
				l_z.dat[1] := a_a___5.dat[1]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_sub_imag (a_a___5: PD_GSL_COMPLEX; a_y: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0]
				l_z.dat[1] := a_a___5.dat[1] - a_y
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_mul (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_ar: REAL_64
			l_ai: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_ar := a_a___5.dat[0]
			l_ai := a_a___5.dat[1]
			l_br := a_b___0.dat[0]
			l_bi := a_b___0.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_ar * l_br - l_ai * l_bi
				l_z.dat[1] := l_ar * l_bi + l_ai * l_br
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_mul_real (a_a___5: PD_GSL_COMPLEX; a_x: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_x * a_a___5.dat[0]
				l_z.dat[1] := a_x * a_a___5.dat[1]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_mul_imag (a_a___5: PD_GSL_COMPLEX; a_y: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := -a_y * a_a___5.dat[1]
				l_z.dat[1] := a_y * a_a___5.dat[0]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_div (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_ar: REAL_64
			l_ai: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_s: REAL_64
			l_sbr: REAL_64
			l_sbi: REAL_64
			l_zr: REAL_64
			l_zi: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_ar := a_a___5.dat[0]
			l_ai := a_a___5.dat[1]
			l_br := a_b___0.dat[0]
			l_bi := a_b___0.dat[1]
			l_s := 1.0 / (gsl_complex_abs (a_b___0))
			l_sbr := l_s * l_br
			l_sbi := l_s * l_bi
			l_zr := (l_ar * l_sbr + l_ai * l_sbi) * l_s
			l_zi := (l_ai * l_sbr - l_ar * l_sbi) * l_s
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_zr
				l_z.dat[1] := l_zi
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_div_real (a_a___5: PD_GSL_COMPLEX; a_x: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0] / a_x
				l_z.dat[1] := a_a___5.dat[1] / a_x
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_div_imag (a_a___5: PD_GSL_COMPLEX; a_y: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[1] / a_y
				l_z.dat[1] := -a_a___5.dat[0] / a_y
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_conjugate (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := a_a___5.dat[0]
				l_z.dat[1] := -a_a___5.dat[1]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_negative (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := -a_a___5.dat[0]
				l_z.dat[1] := -a_a___5.dat[1]
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_inverse (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_s: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_s := 1.0 / (gsl_complex_abs (a_a___5))
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := (a_a___5.dat[0] * l_s) * l_s
				l_z.dat[1] := -(a_a___5.dat[1] * l_s) * l_s
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_sqrt (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			l_x: REAL_64
			l_y: REAL_64
			l_w: REAL_64
			l_t: REAL_64
			l_tmp___1: REAL_64
			l_t___0: REAL_64
			l_tmp___4: REAL_64
			l_ai: REAL_64
			l_ai___0: REAL_64
			l_vi: REAL_64
			l_tmp___7: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_a___5.dat[0] /= 0.0 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_a___5.dat[1] = 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := (0).to_double
								l_z.dat[1] := (0).to_double
								js.break
							end
							js.reset
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						l_x := (fabs (a_a___5.dat[0]))
						l_y := (fabs (a_a___5.dat[1]))
						if l_x >= l_y then
							l_t := l_y / l_x
							l_tmp___1 := sqrt (l_x)
							l_w := l_tmp___1 * (sqrt (0.5 * (1.0 + (sqrt (1.0 + l_t * l_t)))))
						else
							l_t___0 := l_x / l_y
							l_tmp___4 := sqrt (l_y)
							l_w := l_tmp___4 * (sqrt (0.5 * (l_t___0 + (sqrt (1.0 + l_t___0 * l_t___0)))))
						end
						if a_a___5.dat[0] >= 0.0 then
							l_ai := a_a___5.dat[1]
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := l_w
								l_z.dat[1] := l_ai / (2.0 * l_w)
								js.break
							end
							js.reset
						else
							l_ai___0 := a_a___5.dat[1]
							if l_ai___0 >= (0).to_double then
								l_tmp___7 := l_w
							else
								l_tmp___7 := -l_w
							end
							l_vi := l_tmp___7
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := l_ai___0 / (2.0 * l_vi)
								l_z.dat[1] := l_vi
								js.break
							end
							js.reset
						end
				when 4 then
					Result := l_z
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_complex_sqrt_real (a_x: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			if a_x >= (0).to_double then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := sqrt (a_x)
					l_z.dat[1] := 0.0
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := 0.0
					l_z.dat[1] := sqrt (-a_x)
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_exp (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_rho: REAL_64
			l_theta: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_rho := (exp (a_a___5.dat[0]))
			l_theta := a_a___5.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_rho * (cos (l_theta))
				l_z.dat[1] := l_rho * (sin (l_theta))
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_pow (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			l_logr: REAL_64
			l_theta: REAL_64
			l_br: REAL_64
			l_bi: REAL_64
			l_rho: REAL_64
			l_beta: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_a___5.dat[0] /= (0).to_double then
						l_pos := -1 + 9 -- skip if block
					end
				when 1 then -- if:
						if a_a___5.dat[1] /= 0.0 then
							l_pos := -1 + 7 -- skip if block
						end
				when 2 then -- if:
						if a_b___0.dat[0] /= (0).to_double then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						if a_b___0.dat[1] = 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := 1.0
								l_z.dat[1] := 0.0
								js.break
							end
							js.reset
						else
							l_pos := -1 + 5 -- goto _L
						end
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 5 then -- else: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := 0.0
							l_z.dat[1] := 0.0
							js.break
						end
						js.reset
				when 6 then -- if done | if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else:
						l_pos := -1 + 9 -- goto _L___2
				when 8 then -- if done | if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else: | _L___2:
						if a_b___0.dat[0] /= 1.0 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if a_b___0.dat[1] = 0.0 then
							Result := a_a___5
							js.return
						else
							l_pos := -1 + 12 -- goto _L___1
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else: | _L___1:
						if a_b___0.dat[0] /= -1.0 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if a_b___0.dat[1] = 0.0 then
							Result := (gsl_complex_inverse (a_a___5))
							js.return
						else
							l_pos := -1 + 15 -- goto _L___0
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else: | _L___0:
						l_logr := (gsl_complex_logabs (a_a___5))
						l_theta := (gsl_complex_arg (a_a___5))
						l_br := a_b___0.dat[0]
						l_bi := a_b___0.dat[1]
						l_rho := (exp (l_logr * l_br - l_bi * l_theta))
						l_beta := l_theta * l_br + l_bi * l_logr
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := l_rho * (cos (l_beta))
							l_z.dat[1] := l_rho * (sin (l_beta))
							js.break
						end
						js.reset
				when 16 then -- if done | if done
						if not js.return_called then
							Result := l_z
						end
				when 17 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_complex_pow_real (a_a___5: PD_GSL_COMPLEX; a_b___0: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			l_logr: REAL_64
			l_theta: REAL_64
			l_rho: REAL_64
			l_beta: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_a___5.dat[0] /= (0).to_double then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_a___5.dat[1] = (0).to_double then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := (0).to_double
								l_z.dat[1] := (0).to_double
								js.break
							end
							js.reset
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						l_logr := (gsl_complex_logabs (a_a___5))
						l_theta := (gsl_complex_arg (a_a___5))
						l_rho := (exp (l_logr * a_b___0))
						l_beta := l_theta * a_b___0
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := l_rho * (cos (l_beta))
							l_z.dat[1] := l_rho * (sin (l_beta))
							js.break
						end
						js.reset
				when 4 then
					Result := l_z
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_complex_log (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_logr: REAL_64
			l_theta: REAL_64
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			l_logr := (gsl_complex_logabs (a_a___5))
			l_theta := (gsl_complex_arg (a_a___5))
			from
			until
				js.loop_left
			loop
				js.reset
				l_z.dat[0] := l_logr
				l_z.dat[1] := l_theta
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_log10 (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_tmp: REAL_64
		do
			l_tmp := log (10.)
			Result := (gsl_complex_mul_real ((gsl_complex_log (a_a___5)), (1).to_double / l_tmp))
		end

	gsl_complex_log_b (a_a___5: PD_GSL_COMPLEX; a_b___0: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_tmp: PD_GSL_COMPLEX
		do
			create l_tmp.make

			l_tmp.ce_gets (gsl_complex_log (a_b___0))
			Result := (gsl_complex_div ((gsl_complex_log (a_a___5)), l_tmp))
		end

	gsl_complex_sin (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_tmp: REAL_64
			l_tmp___1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if l_I = 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := sin (l_R)
					l_z.dat[1] := 0.0
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_tmp := sin (l_R)
					l_z.dat[0] := l_tmp * (cosh (l_I))
					l_tmp___1 := cos (l_R)
					l_z.dat[1] := l_tmp___1 * (sinh (l_I))
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_cos (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_tmp: REAL_64
			l_tmp___1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if l_I = 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := cos (l_R)
					l_z.dat[1] := 0.0
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_tmp := cos (l_R)
					l_z.dat[0] := l_tmp * (cosh (l_I))
					l_tmp___1 := sin (l_R)
					l_z.dat[1] := l_tmp___1 * (sinh (-l_I))
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_tan (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_D: REAL_64
			l_tmp___0: REAL_64
			l_u: REAL_64
			l_C: REAL_64
			l_D___0: REAL_64
			l_tmp___8: REAL_64
			l_S: REAL_64
			l_T: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if (fabs (l_I)) < (1).to_double then
				l_tmp___0 := pow ((cos (l_R)), 2.0)
				l_D := l_tmp___0 + (pow ((sinh (l_I)), 2.0))
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := (0.5 * (sin ((2).to_double * l_R))) / l_D
					l_z.dat[1] := (0.5 * (sinh ((2).to_double * l_I))) / l_D
					js.break
				end
				js.reset
			else
				l_u := (exp (-l_I))
				l_C := ((2).to_double * l_u) / ((1).to_double - (pow (l_u, 2.0)))
				l_tmp___8 := pow ((cos (l_R)), 2.0)
				l_D___0 := (1).to_double + l_tmp___8 * (pow (l_C, 2.0))
				l_S := (pow (l_C, 2.0))
				l_T := 1.0 / (tanh (l_I))
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := ((0.5 * (sin ((2).to_double * l_R))) * l_S) / l_D___0
					l_z.dat[1] := l_T / l_D___0
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_sec (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_cos (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_csc (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_sin (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_cot (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_tan (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_arcsin (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_x: REAL_64
			l_y: REAL_64
			l_r___3: REAL_64
			l_s: REAL_64
			l_A___0: REAL_64
			l_B: REAL_64
			l_y2: REAL_64
			l_real: REAL_64
			l_imag: REAL_64
			l_A_crossover: REAL_64
			l_B_crossover: REAL_64
			l_D: REAL_64
			l_Apx: REAL_64
			l_D___0: REAL_64
			l_Am1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if l_I = (0).to_double then
				l_z.ce_gets (gsl_complex_arcsin_real (l_R))
			else
				l_x := (fabs (l_R))
				l_y := (fabs (l_I))
				l_r___3 := (hypot (l_x + (1).to_double, l_y))
				l_s := (hypot (l_x - (1).to_double, l_y))
				l_A___0 := 0.5 * (l_r___3 + l_s)
				l_B := l_x / l_A___0
				l_y2 := l_y * l_y
				l_A_crossover := 1.5
				l_B_crossover := 0.6417
				if l_B <= l_B_crossover then
					l_real := asin (l_B)
				elseif l_x <= (1).to_double then
					l_D := (0.5 * (l_A___0 + l_x)) * (l_y2 / ((l_r___3 + l_x) + (1).to_double) + (l_s + ((1).to_double - l_x)))
					l_real := atan (l_x / (sqrt (l_D)))
				else
					l_Apx := l_A___0 + l_x
					l_D___0 := 0.5 * (l_Apx / ((l_r___3 + l_x) + (1).to_double) + l_Apx / (l_s + (l_x - (1).to_double)))
					l_real := atan (l_x / (l_y * (sqrt (l_D___0))))
				end
				if l_A___0 <= l_A_crossover then
					if l_x < (1).to_double then
						l_Am1 := 0.5 * (l_y2 / (l_r___3 + (l_x + (1).to_double)) + l_y2 / (l_s + ((1).to_double - l_x)))
					else
						l_Am1 := 0.5 * (l_y2 / (l_r___3 + (l_x + (1).to_double)) + (l_s + (l_x - (1).to_double)))
					end
					l_imag := log1p (l_Am1 + (sqrt (l_Am1 * (l_A___0 + (1).to_double))))
				else
					l_imag := log (l_A___0 + (sqrt (l_A___0 * l_A___0 - (1).to_double)))
				end
				from
				until
					js.loop_left
				loop
					js.reset
					if l_R >= (0).to_double then
						l_z.dat[0] := l_real
					else
						l_z.dat[0] := -l_real
					end
					if l_I >= (0).to_double then
						l_z.dat[1] := l_imag
					else
						l_z.dat[1] := -l_imag
					end
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_arcsin_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			if (fabs (a_a___5)) <= 1.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := asin (a_a___5)
					l_z.dat[1] := 0.0
					js.break
				end
				js.reset
			elseif a_a___5 < 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := -1.57079632679489661923
					l_z.dat[1] := acosh (-a_a___5)
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := 1.57079632679489661923
					l_z.dat[1] := -(acosh (a_a___5))
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_arccos (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_x: REAL_64
			l_y: REAL_64
			l_r___3: REAL_64
			l_s: REAL_64
			l_A___0: REAL_64
			l_B: REAL_64
			l_y2: REAL_64
			l_real: REAL_64
			l_imag: REAL_64
			l_A_crossover: REAL_64
			l_B_crossover: REAL_64
			l_D: REAL_64
			l_Apx: REAL_64
			l_D___0: REAL_64
			l_Am1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if l_I = (0).to_double then
				l_z.ce_gets (gsl_complex_arccos_real (l_R))
			else
				l_x := (fabs (l_R))
				l_y := (fabs (l_I))
				l_r___3 := (hypot (l_x + (1).to_double, l_y))
				l_s := (hypot (l_x - (1).to_double, l_y))
				l_A___0 := 0.5 * (l_r___3 + l_s)
				l_B := l_x / l_A___0
				l_y2 := l_y * l_y
				l_A_crossover := 1.5
				l_B_crossover := 0.6417
				if l_B <= l_B_crossover then
					l_real := acos (l_B)
				elseif l_x <= (1).to_double then
					l_D := (0.5 * (l_A___0 + l_x)) * (l_y2 / ((l_r___3 + l_x) + (1).to_double) + (l_s + ((1).to_double - l_x)))
					l_real := atan ((sqrt (l_D)) / l_x)
				else
					l_Apx := l_A___0 + l_x
					l_D___0 := 0.5 * (l_Apx / ((l_r___3 + l_x) + (1).to_double) + l_Apx / (l_s + (l_x - (1).to_double)))
					l_real := atan ((l_y * (sqrt (l_D___0))) / l_x)
				end
				if l_A___0 <= l_A_crossover then
					if l_x < (1).to_double then
						l_Am1 := 0.5 * (l_y2 / (l_r___3 + (l_x + (1).to_double)) + l_y2 / (l_s + ((1).to_double - l_x)))
					else
						l_Am1 := 0.5 * (l_y2 / (l_r___3 + (l_x + (1).to_double)) + (l_s + (l_x - (1).to_double)))
					end
					l_imag := log1p (l_Am1 + (sqrt (l_Am1 * (l_A___0 + (1).to_double))))
				else
					l_imag := log (l_A___0 + (sqrt (l_A___0 * l_A___0 - (1).to_double)))
				end
				from
				until
					js.loop_left
				loop
					js.reset
					if l_R >= (0).to_double then
						l_z.dat[0] := l_real
					else
						l_z.dat[0] := 3.14159265358979323846 - l_real
					end
					if l_I >= (0).to_double then
						l_z.dat[1] := -l_imag
					else
						l_z.dat[1] := l_imag
					end
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_arccos_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			if (fabs (a_a___5)) <= 1.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := acos (a_a___5)
					l_z.dat[1] := (0).to_double
					js.break
				end
				js.reset
			elseif a_a___5 < 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := 3.14159265358979323846
					l_z.dat[1] := -(acosh (-a_a___5))
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := (0).to_double
					l_z.dat[1] := acosh (a_a___5)
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_arctan (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_r___3: REAL_64
			l_imag: REAL_64
			l_u: REAL_64
			l_tmp___0: REAL_64
			l_A___0: REAL_64
			l_B: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if l_I = (0).to_double then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := atan (l_R)
					l_z.dat[1] := (0).to_double
					js.break
				end
				js.reset
			else
				l_r___3 := (hypot (l_R, l_I))
				l_u := ((2).to_double * l_I) / ((1).to_double + l_r___3 * l_r___3)
				if (fabs (l_u)) < 0.1 then
					l_tmp___0 := log1p (l_u)
					l_imag := 0.25 * (l_tmp___0 - (log1p (-l_u)))
				else
					l_A___0 := (hypot (l_R, l_I + (1).to_double))
					l_B := (hypot (l_R, l_I - (1).to_double))
					l_imag := 0.5 * (log (l_A___0 / l_B))
				end
				if l_R = (0).to_double then
					if l_I > (1).to_double then
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := 1.57079632679489661923
							l_z.dat[1] := l_imag
							js.break
						end
						js.reset
					elseif l_I < (-1).to_double then
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := -1.57079632679489661923
							l_z.dat[1] := l_imag
							js.break
						end
						js.reset
					else
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := (0).to_double
							l_z.dat[1] := l_imag
							js.break
						end
						js.reset
					end
				else
					from
					until
						js.loop_left
					loop
						js.reset
						l_z.dat[0] := 0.5 * (atan2 ((2).to_double * l_R, ((1).to_double + l_r___3) * ((1).to_double - l_r___3)))
						l_z.dat[1] := l_imag
						js.break
					end
					js.reset
				end
			end
			Result := l_z
		end

	gsl_complex_arcsec (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_inverse (a_a___5)))
			Result := (gsl_complex_arccos (l_z))
		end

	gsl_complex_arcsec_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_a___5 <= -1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_a___5 >= 1.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := acos ((1).to_double / a_a___5)
							l_z.dat[1] := 0.0
							js.break
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_a___5 >= 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := (0).to_double
								l_z.dat[1] := acosh ((1).to_double / a_a___5)
								js.break
							end
							js.reset
						else
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := 3.14159265358979323846
								l_z.dat[1] := -(acosh ((-1).to_double / a_a___5))
								js.break
							end
							js.reset
						end
				when 7 then -- if done
						Result := l_z
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_complex_arccsc (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_inverse (a_a___5)))
			Result := (gsl_complex_arcsin (l_z))
		end

	gsl_complex_arccsc_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_a___5 <= -1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_a___5 >= 1.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := asin ((1).to_double / a_a___5)
							l_z.dat[1] := 0.0
							js.break
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_a___5 >= 0.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := 1.57079632679489661923
								l_z.dat[1] := -(acosh ((1).to_double / a_a___5))
								js.break
							end
							js.reset
						else
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := -1.57079632679489661923
								l_z.dat[1] := acosh ((-1).to_double / a_a___5)
								js.break
							end
							js.reset
						end
				when 7 then -- if done
						Result := l_z
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_complex_arccot (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			if a_a___5.dat[0] = 0.0 then
				if a_a___5.dat[1] = 0.0 then
					from
					until
						js.loop_left
					loop
						js.reset
						l_z.dat[0] := 1.57079632679489661923
						l_z.dat[1] := (0).to_double
						js.break
					end
					js.reset
				else
					l_z.ce_gets (gsl_complex_inverse (a_a___5))
					l_z.ce_gets (gsl_complex_arctan (l_z))
				end
			else
				l_z.ce_gets (gsl_complex_inverse (a_a___5))
				l_z.ce_gets (gsl_complex_arctan (l_z))
			end
			Result := l_z
		end

	gsl_complex_sinh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_tmp: REAL_64
			l_tmp___1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_tmp := sinh (l_R)
				l_z.dat[0] := l_tmp * (cos (l_I))
				l_tmp___1 := cosh (l_R)
				l_z.dat[1] := l_tmp___1 * (sin (l_I))
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_cosh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_tmp: REAL_64
			l_tmp___1: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			from
			until
				js.loop_left
			loop
				js.reset
				l_tmp := cosh (l_R)
				l_z.dat[0] := l_tmp * (cos (l_I))
				l_tmp___1 := sinh (l_R)
				l_z.dat[1] := l_tmp___1 * (sin (l_I))
				js.break
			end
			js.reset
			Result := l_z
		end

	gsl_complex_tanh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_R: REAL_64
			l_I: REAL_64
			l_z: PD_GSL_COMPLEX
			l_D: REAL_64
			l_tmp___0: REAL_64
			l_tmp___3: REAL_64
			l_D___0: REAL_64
			l_tmp___7: REAL_64
			l_F___0: REAL_64
			l_tmp___10: REAL_64
			js: JUMP_STATE
		do
			create l_z.make

			l_R := a_a___5.dat[0]
			l_I := a_a___5.dat[1]
			if (fabs (l_R)) < 1.0 then
				l_tmp___0 := pow ((cos (l_I)), 2.0)
				l_D := l_tmp___0 + (pow ((sinh (l_R)), 2.0))
				from
				until
					js.loop_left
				loop
					js.reset
					l_tmp___3 := sinh (l_R)
					l_z.dat[0] := (l_tmp___3 * (cosh (l_R))) / l_D
					l_z.dat[1] := (0.5 * (sin ((2).to_double * l_I))) / l_D
					js.break
				end
				js.reset
			else
				l_tmp___7 := pow ((cos (l_I)), 2.0)
				l_D___0 := l_tmp___7 + (pow ((sinh (l_R)), 2.0))
				l_tmp___10 := cos (l_I)
				l_F___0 := (1).to_double + (pow (l_tmp___10 / (sinh (l_R)), 2.0))
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := 1.0 / ((tanh (l_R)) * l_F___0)
					l_z.dat[1] := (0.5 * (sin ((2).to_double * l_I))) / l_D___0
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_sech (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_cosh (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_csch (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_sinh (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_coth (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_tanh (a_a___5)))
			Result := (gsl_complex_inverse (l_z))
		end

	gsl_complex_arcsinh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_mul_imag (a_a___5, 1.0)))
			l_z.ce_gets (gsl_complex_arcsin (l_z))
			l_z.ce_gets (gsl_complex_mul_imag (l_z, -1.0))
			Result := l_z
		end

	gsl_complex_arccosh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			l_tmp___0: REAL_64
		do
			create l_z.make

			l_z.ce_gets ((gsl_complex_arccos (a_a___5)))
			if l_z.dat[1] > (0).to_double then
				l_tmp___0 := -1.0
			else
				l_tmp___0 := 1.0
			end
			l_z.ce_gets (gsl_complex_mul_imag (l_z, l_tmp___0))
			Result := l_z
		end

	gsl_complex_arccosh_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
		do
			create l_z.make

			if a_a___5 >= (1).to_double then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := acosh (a_a___5)
					l_z.dat[1] := (0).to_double
					js.break
				end
				js.reset
			elseif a_a___5 >= -1.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := (0).to_double
					l_z.dat[1] := acos (a_a___5)
					js.break
				end
				js.reset
			else
				from
				until
					js.loop_left
				loop
					js.reset
					l_z.dat[0] := acosh (-a_a___5)
					l_z.dat[1] := 3.14159265358979323846
					js.break
				end
				js.reset
			end
			Result := l_z
		end

	gsl_complex_arctanh (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
		do
			create l_z.make

			if a_a___5.dat[1] = 0.0 then
				Result := (gsl_complex_arctanh_real (a_a___5.dat[0]))
			else
				l_z.ce_gets ((gsl_complex_mul_imag (a_a___5, 1.0)))
				l_z.ce_gets (gsl_complex_arctan (l_z))
				l_z.ce_gets (gsl_complex_mul_imag (l_z, -1.0))
				Result := l_z
			end
		end

	gsl_complex_arctanh_real (a_a___5: REAL_64): PD_GSL_COMPLEX
		local
			l_z: PD_GSL_COMPLEX
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_a___5 > -1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_a___5 < 1.0 then
							from
							until
								js.loop_left
							loop
								js.reset
								l_z.dat[0] := atanh (a_a___5)
								l_z.dat[1] := (0).to_double
								js.break
							end
							js.reset
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							l_z.dat[0] := atanh ((1).to_double / a_a___5)
							if a_a___5 < (0).to_double then
								l_z.dat[1] := 1.57079632679489661923
							else
								l_z.dat[1] := -1.57079632679489661923
							end
							js.break
						end
						js.reset
				when 4 then
					Result := l_z
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_complex_arcsech (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_t: PD_GSL_COMPLEX
		do
			create l_t.make

			l_t.ce_gets ((gsl_complex_inverse (a_a___5)))
			Result := (gsl_complex_arccosh (l_t))
		end

	gsl_complex_arccsch (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_t: PD_GSL_COMPLEX
		do
			create l_t.make

			l_t.ce_gets ((gsl_complex_inverse (a_a___5)))
			Result := (gsl_complex_arcsinh (l_t))
		end

	gsl_complex_arccoth (a_a___5: PD_GSL_COMPLEX): PD_GSL_COMPLEX
		local
			l_t: PD_GSL_COMPLEX
		do
			create l_t.make

			l_t.ce_gets ((gsl_complex_inverse (a_a___5)))
			Result := (gsl_complex_arctanh (l_t))
		end

end
