class
	P_PERMUTATION

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_permutation_size (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): NATURAL_32
		do
			Result := (a_p.item.size)
		end

	gsl_permutation_data (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): CE_POINTER [NATURAL_32]
		do
			Result := (a_p.item.data)
		end

	gsl_permutation_swap (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_i: NATURAL_32; a_j: NATURAL_32): INTEGER_32
		local
			l_size: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			if a_i >= l_size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("first index is out of range"), ce_string ("permutation.c"), 43, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called and then (a_j >= l_size) then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("second index is out of range"), ce_string ("permutation.c"), 48, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				if a_i /= a_j then
					l_tmp := a_p.item.data[a_i.to_integer_32]
					a_p.item.data[a_i.to_integer_32] := a_p.item.data[a_j.to_integer_32]
					a_p.item.data[a_j.to_integer_32] := l_tmp
				end
				Result := 0
			end
		end

	gsl_permutation_valid (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < l_size))
			loop
				if a_p.item.data[l_i.to_integer_32] >= l_size then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("permutation index outside range"), ce_string ("permutation.c"), 73, -1)
						Result := (-1)
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
				if not js.return_called then
					l_j := (0).to_natural_32
					from
					until
						not (not js.loop_left and then (l_j < l_i))
					loop
						if a_p.item.data[l_i.to_integer_32] = a_p.item.data[l_j.to_integer_32] then
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("duplicate permutation index"), ce_string ("permutation.c"), 80, -1)
								Result := (-1)
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
						if not js.return_called then
							l_j := l_j + 1
						end
					end
				end
				if not js.return_called then
					l_i := l_i + 1
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_permutation_reverse (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT])
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_size := a_p.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_size // {NATURAL_32} 2)
			loop
				l_j := (l_size - l_i) - {NATURAL_32} 1
				l_tmp := a_p.item.data[l_i.to_integer_32]
				a_p.item.data[l_i.to_integer_32] := a_p.item.data[l_j.to_integer_32]
				a_p.item.data[l_j.to_integer_32] := l_tmp
				l_i := l_i + 1
			end
		end

	gsl_permutation_inverse (a_inv: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			if a_inv.item.size /= l_size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation lengths are not equal"), ce_string ("permutation.c"), 114, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_size)
				loop
					a_inv.item.data[(a_p.item.data[l_i.to_integer_32]).to_integer_32] := l_i
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_permutation_next (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			if l_size < {NATURAL_32} 2 then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				l_i := l_size - {NATURAL_32} 2
				from
				until
					js.loop_left
				loop
					js.reset
					if a_p.item.data[l_i.to_integer_32] > a_p.item.data[(l_i + {NATURAL_32} 1).to_integer_32] then
						if l_i = {NATURAL_32} 0 then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_i := l_i - 1
					end
				end
				js.reset
				if l_i = {NATURAL_32} 0 then
					if a_p.item.data[0] > a_p.item.data[1] then
						Result := (-1)
						js.return
					end
				end
			end
			if not js.return_called then
				l_k := l_i + {NATURAL_32} 1
				l_j := l_i + {NATURAL_32} 2
				from
				until
					not (l_j < l_size)
				loop
					if a_p.item.data[l_j.to_integer_32] > a_p.item.data[l_i.to_integer_32] then
						if a_p.item.data[l_j.to_integer_32] < a_p.item.data[l_k.to_integer_32] then
							l_k := l_j
						end
					end
					l_j := l_j + 1
				end
				l_tmp := a_p.item.data[l_i.to_integer_32]
				a_p.item.data[l_i.to_integer_32] := a_p.item.data[l_k.to_integer_32]
				a_p.item.data[l_k.to_integer_32] := l_tmp
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j <= (l_size + l_i) // {NATURAL_32} 2)
				loop
					l_tmp___0 := a_p.item.data[l_j.to_integer_32]
					a_p.item.data[l_j.to_integer_32] := a_p.item.data[((l_size + l_i) - l_j).to_integer_32]
					a_p.item.data[((l_size + l_i) - l_j).to_integer_32] := l_tmp___0
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_permutation_prev (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_size: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			if l_size < {NATURAL_32} 2 then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				l_i := l_size - {NATURAL_32} 2
				from
				until
					js.loop_left
				loop
					js.reset
					if a_p.item.data[l_i.to_integer_32] < a_p.item.data[(l_i + {NATURAL_32} 1).to_integer_32] then
						if l_i = {NATURAL_32} 0 then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						l_i := l_i - 1
					end
				end
				js.reset
				if l_i = {NATURAL_32} 0 then
					if a_p.item.data[0] < a_p.item.data[1] then
						Result := (-1)
						js.return
					end
				end
			end
			if not js.return_called then
				l_k := l_i + {NATURAL_32} 1
				l_j := l_i + {NATURAL_32} 2
				from
				until
					not (l_j < l_size)
				loop
					if a_p.item.data[l_j.to_integer_32] < a_p.item.data[l_i.to_integer_32] then
						if a_p.item.data[l_j.to_integer_32] > a_p.item.data[l_k.to_integer_32] then
							l_k := l_j
						end
					end
					l_j := l_j + 1
				end
				l_tmp := a_p.item.data[l_i.to_integer_32]
				a_p.item.data[l_i.to_integer_32] := a_p.item.data[l_k.to_integer_32]
				a_p.item.data[l_k.to_integer_32] := l_tmp
				l_j := l_i + {NATURAL_32} 1
				from
				until
					not (l_j <= (l_size + l_i) // {NATURAL_32} 2)
				loop
					l_tmp___0 := a_p.item.data[l_j.to_integer_32]
					a_p.item.data[l_j.to_integer_32] := a_p.item.data[((l_size + l_i) - l_j).to_integer_32]
					a_p.item.data[((l_size + l_i) - l_j).to_integer_32] := l_tmp___0
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_permutation_mul (a_p: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_pa: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_pb: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_i: NATURAL_32
			l_size: NATURAL_32
			js: JUMP_STATE
		do
			l_size := a_p.item.size
			if a_pa.item.size /= l_size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("size of result does not match size of pa"), ce_string ("permutation.c"), 238, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called and then (a_pb.item.size /= l_size) then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("size of result does not match size of pb"), ce_string ("permutation.c"), 243, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_size)
				loop
					a_p.item.data[l_i.to_integer_32] := a_pb.item.data[(a_pa.item.data[l_i.to_integer_32]).to_integer_32]
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_permutation_memcpy (a_dest: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_src: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]): INTEGER_32
		local
			l_src_size: NATURAL_32
			l_dest_size: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_src_size := a_src.item.size
			l_dest_size := a_dest.item.size
			if l_src_size /= l_dest_size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation lengths are not equal"), ce_string ("permutation.c"), 262, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_src_size)
				loop
					a_dest.item.data[l_j.to_integer_32] := a_src.item.data[l_j.to_integer_32]
					l_j := l_j + 1
				end
				Result := 0
			end
		end

end
