class
	P_QMOMOF

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_integration_qawo_table_alloc (a_omega: REAL_64; a_L: REAL_64; a_sine: INTEGER_32; a_n: NATURAL_32): CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]
		local
			l_t: CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]
			l_chebmo: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_scale: REAL_64
			js: JUMP_STATE
		do
			create l_t.make
			create l_chebmo.make

			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("table length n must be positive integer"), ce_string ("qmomof.c"), 42, 1)
					Result := (create {CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_t.ce_gets (create {CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]}.make_cast ((malloc ({PD_GSL_INTEGRATION_QAWO_TABLE}.sizeof.to_natural_32))))
				if (l_t.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("failed to allocate space for qawo_table struct"), ce_string ("qmomof.c"), 51, 8)
						Result := (create {CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				l_chebmo.ce_gets (create {CE_POINTER [REAL_64]}.make_cast ((malloc (({NATURAL_32} 25 * a_n) * Real_64_bytes.to_natural_32))))
				if (l_chebmo.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
					free (create {CE_POINTER [ANY]}.make_cast (l_t))
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("failed to allocate space for chebmo block"), ce_string ("qmomof.c"), 60, 8)
						Result := (create {CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				l_t.item.n := a_n
				l_t.item.sine := a_sine
				l_t.item.omega := a_omega
				l_t.item.L := a_L
				l_t.item.par := (0.5 * a_omega) * a_L
				l_t.item.chebmo := l_chebmo
				l_scale := 1.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_t.item.n)
				loop
					compute_moments (l_t.item.par * l_scale, l_t.item.chebmo + ({NATURAL_32} 25 * l_i).to_integer_32)
					l_scale := l_scale * (0.5)
					l_i := l_i + 1
				end
				Result := l_t
			end
		end

	gsl_integration_qawo_table_set (a_t: CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]; a_omega: REAL_64; a_L: REAL_64; a_sine: INTEGER_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_scale: REAL_64
		do
			a_t.item.omega := a_omega
			a_t.item.sine := a_sine
			a_t.item.L := a_L
			a_t.item.par := (0.5 * a_omega) * a_L
			l_scale := 1.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_t.item.n)
			loop
				compute_moments (a_t.item.par * l_scale, a_t.item.chebmo + ({NATURAL_32} 25 * l_i).to_integer_32)
				l_scale := l_scale * (0.5)
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_integration_qawo_table_set_length (a_t: CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE]; a_L: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_scale: REAL_64
			js: JUMP_STATE
		do
			if a_L = a_t.item.L then
				Result := 0
				js.return
			end
			if not js.return_called then
				a_t.item.L := a_L
				a_t.item.par := (0.5 * a_t.item.omega) * a_L
				l_scale := 1.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_t.item.n)
				loop
					compute_moments (a_t.item.par * l_scale, a_t.item.chebmo + ({NATURAL_32} 25 * l_i).to_integer_32)
					l_scale := l_scale * (0.5)
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_integration_qawo_table_free (a_t: CE_POINTER [PD_GSL_INTEGRATION_QAWO_TABLE])
		local
			js: JUMP_STATE
		do
			if (a_t).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (a_t.item.chebmo))
				free (create {CE_POINTER [ANY]}.make_cast (a_t))
			end
		end

	compute_moments (a_par: REAL_64; a_chebmo: CE_POINTER [REAL_64])
		local
			l_v: CE_ARRAY [REAL_64]
			l_d: CE_ARRAY [REAL_64]
			l_d1: CE_ARRAY [REAL_64]
			l_d2: CE_ARRAY [REAL_64]
			l_noeq: NATURAL_32
			l_par2: REAL_64
			l_par4: REAL_64
			l_par22: REAL_64
			l_sinpar: REAL_64
			l_cospar: REAL_64
			l_i: NATURAL_32
			l_ac: REAL_64
			l_as: REAL_64
			l_an2: REAL_64
			l_ass: REAL_64
			l_asap: REAL_64
			l_an: REAL_64
			l_k: NATURAL_32
			l_k___0: NATURAL_32
			l_an___0: REAL_64
			l_an2___0: REAL_64
			l_k___1: NATURAL_32
			l_an2___1: REAL_64
			l_ass___0: REAL_64
			l_asap___0: REAL_64
			l_an___1: REAL_64
			l_k___2: NATURAL_32
			l_an___2: REAL_64
			l_an2___2: REAL_64
		do
			create l_v.make (<<28>>)
			create l_d.make (<<25>>)
			create l_d1.make (<<25>>)
			create l_d2.make (<<25>>)

			l_noeq := (25).to_natural_32
			l_par2 := a_par * a_par
			l_par4 := l_par2 * l_par2
			l_par22 := l_par2 + 2.0
			l_sinpar := (sin (a_par))
			l_cospar := (cos (a_par))
			l_ac := (8).to_double * l_cospar
			l_as := ((24).to_double * a_par) * l_sinpar
			l_v[0] := ((2).to_double * l_sinpar) / a_par
			l_v[1] := ((8).to_double * l_cospar + (((2).to_double * l_par2 - (8).to_double) * l_sinpar) / a_par) / l_par2
			l_v[2] := (((32).to_double * (l_par2 - (12).to_double)) * l_cospar + (((2).to_double * ((l_par2 - (80).to_double) * l_par2 + (192).to_double)) * l_sinpar) / a_par) / l_par4
			if (fabs (a_par)) <= (24).to_double then
				l_an := (6).to_double
				l_k := (0).to_natural_32
				from
				until
					not (l_k < l_noeq - {NATURAL_32} 1)
				loop
					l_an2 := l_an * l_an
					l_d[l_k.to_integer_32] := ((-2).to_double * (l_an2 - (4).to_double)) * (l_par22 - (2).to_double * l_an2)
					l_d2[l_k.to_integer_32] := ((l_an - (1).to_double) * (l_an - (2).to_double)) * l_par2
					l_d1[(l_k + {NATURAL_32} 1).to_integer_32] := ((l_an + (3).to_double) * (l_an + (4).to_double)) * l_par2
					l_v[(l_k + {NATURAL_32} 3).to_integer_32] := l_as - (l_an2 - (4).to_double) * l_ac
					l_an := l_an + (2.0)
					l_k := l_k + 1
				end
				l_an2 := l_an * l_an
				l_d[(l_noeq - {NATURAL_32} 1).to_integer_32] := ((-2).to_double * (l_an2 - (4).to_double)) * (l_par22 - (2).to_double * l_an2)
				l_v[(l_noeq + {NATURAL_32} 2).to_integer_32] := l_as - (l_an2 - (4).to_double) * l_ac
				l_v[3] := l_v[3] - (((56).to_double * l_par2) * l_v[2])
				l_ass := a_par * l_sinpar
				l_asap := ((((((((210).to_double * l_par2 - (1).to_double) * l_cospar - ((105).to_double * l_par2 - (63).to_double) * l_ass) / l_an2 - ((1).to_double - (15).to_double * l_par2) * l_cospar) + (15).to_double * l_ass) / l_an2 - l_cospar) + (3).to_double * l_ass) / l_an2 - l_cospar) / l_an2
				l_v[(l_noeq + {NATURAL_32} 2).to_integer_32] := l_v[(l_noeq + {NATURAL_32} 2).to_integer_32] - (((((2).to_double * l_asap) * l_par2) * (l_an - (1).to_double)) * (l_an - (2).to_double))
				dgtsl (l_noeq, l_d1, l_d, l_d2, l_v + 3).do_nothing
			else
				l_an___0 := (4).to_double
				l_k___0 := (3).to_natural_32
				from
				until
					not (l_k___0 < {NATURAL_32} 13)
				loop
					l_an2___0 := l_an___0 * l_an___0
					l_v[l_k___0.to_integer_32] := (((l_an2___0 - (4).to_double) * (((2).to_double * (l_par22 - (2).to_double * l_an2___0)) * l_v[(l_k___0 - {NATURAL_32} 1).to_integer_32] - l_ac) + l_as) - ((l_par2 * (l_an___0 + (1).to_double)) * (l_an___0 + (2).to_double)) * l_v[(l_k___0 - {NATURAL_32} 2).to_integer_32]) / ((l_par2 * (l_an___0 - (1).to_double)) * (l_an___0 - (2).to_double))
					l_an___0 := l_an___0 + (2.0)
					l_k___0 := l_k___0 + 1
				end
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < {NATURAL_32} 13)
			loop
				a_chebmo[({NATURAL_32} 2 * l_i).to_integer_32] := l_v[l_i.to_integer_32]
				l_i := l_i + 1
			end
			l_v[0] := ((2).to_double * (l_sinpar - a_par * l_cospar)) / l_par2
			l_v[1] := (((18).to_double - (48).to_double / l_par2) * l_sinpar) / l_par2 + (((-2).to_double + (48).to_double / l_par2) * l_cospar) / a_par
			l_ac := ((-24).to_double * a_par) * l_cospar
			l_as := (-8).to_double * l_sinpar
			if (fabs (a_par)) <= (24).to_double then
				l_an___1 := (5).to_double
				l_k___1 := (0).to_natural_32
				from
				until
					not (l_k___1 < l_noeq - {NATURAL_32} 1)
				loop
					l_an2___1 := l_an___1 * l_an___1
					l_d[l_k___1.to_integer_32] := ((-2).to_double * (l_an2___1 - (4).to_double)) * (l_par22 - (2).to_double * l_an2___1)
					l_d2[l_k___1.to_integer_32] := ((l_an___1 - (1).to_double) * (l_an___1 - (2).to_double)) * l_par2
					l_d1[(l_k___1 + {NATURAL_32} 1).to_integer_32] := ((l_an___1 + (3).to_double) * (l_an___1 + (4).to_double)) * l_par2
					l_v[(l_k___1 + {NATURAL_32} 2).to_integer_32] := l_ac + (l_an2___1 - (4).to_double) * l_as
					l_an___1 := l_an___1 + (2.0)
					l_k___1 := l_k___1 + 1
				end
				l_an2___1 := l_an___1 * l_an___1
				l_d[(l_noeq - {NATURAL_32} 1).to_integer_32] := ((-2).to_double * (l_an2___1 - (4).to_double)) * (l_par22 - (2).to_double * l_an2___1)
				l_v[(l_noeq + {NATURAL_32} 1).to_integer_32] := l_ac + (l_an2___1 - (4).to_double) * l_as
				l_v[2] := l_v[2] - (((42).to_double * l_par2) * l_v[1])
				l_ass___0 := a_par * l_cospar
				l_asap___0 := ((((((((105).to_double * l_par2 - (63).to_double) * l_ass___0 - ((210).to_double * l_par2 - (1).to_double) * l_sinpar) / l_an2___1 + ((15).to_double * l_par2 - (1).to_double) * l_sinpar) - (15).to_double * l_ass___0) / l_an2___1 - l_sinpar) - (3).to_double * l_ass___0) / l_an2___1 - l_sinpar) / l_an2___1
				l_v[(l_noeq + {NATURAL_32} 1).to_integer_32] := l_v[(l_noeq + {NATURAL_32} 1).to_integer_32] - (((((2).to_double * l_asap___0) * l_par2) * (l_an___1 - (1).to_double)) * (l_an___1 - (2).to_double))
				dgtsl (l_noeq, l_d1, l_d, l_d2, l_v + 2).do_nothing
			else
				l_an___2 := (3).to_double
				l_k___2 := (2).to_natural_32
				from
				until
					not (l_k___2 < {NATURAL_32} 12)
				loop
					l_an2___2 := l_an___2 * l_an___2
					l_v[l_k___2.to_integer_32] := (((l_an2___2 - (4).to_double) * (((2).to_double * (l_par22 - (2).to_double * l_an2___2)) * l_v[(l_k___2 - {NATURAL_32} 1).to_integer_32] + l_as) + l_ac) - ((l_par2 * (l_an___2 + (1).to_double)) * (l_an___2 + (2).to_double)) * l_v[(l_k___2 - {NATURAL_32} 2).to_integer_32]) / ((l_par2 * (l_an___2 - (1).to_double)) * (l_an___2 - (2).to_double))
					l_an___2 := l_an___2 + (2.0)
					l_k___2 := l_k___2 + 1
				end
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < {NATURAL_32} 12)
			loop
				a_chebmo[({NATURAL_32} 2 * l_i + {NATURAL_32} 1).to_integer_32] := l_v[l_i.to_integer_32]
				l_i := l_i + 1
			end
		end

	dgtsl (a_n: NATURAL_32; a_c___1: CE_POINTER [REAL_64]; a_d: CE_POINTER [REAL_64]; a_e: CE_POINTER [REAL_64]; a_b___0: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_t: REAL_64
			l_t___0: REAL_64
			l_t___1: REAL_64
			l_t___2: REAL_64
			l_tmp: REAL_64
			l_t___3: REAL_64
			l_kb: NATURAL_32
			js: JUMP_STATE
		do
			a_c___1[0] := a_d[0]
			if a_n = {NATURAL_32} 0 then
				Result := 0
				js.return
			end
			if not js.return_called and then (a_n = {NATURAL_32} 1) then
				a_b___0[0] := a_b___0[0] / (a_d[0])
				Result := 0
				js.return
			end
			if not js.return_called then
				a_d[0] := a_e[0]
				a_e[0] := (0).to_double
				a_e[(a_n - {NATURAL_32} 1).to_integer_32] := (0).to_double
				l_k := (0).to_natural_32
				from
				until
					not (not js.loop_left and then (l_k < a_n - {NATURAL_32} 1))
				loop
					l_k1 := l_k + {NATURAL_32} 1
					l_tmp := fabs (a_c___1[l_k1.to_integer_32])
					if l_tmp >= (fabs (a_c___1[l_k.to_integer_32])) then
						l_t := a_c___1[l_k1.to_integer_32]
						a_c___1[l_k1.to_integer_32] := a_c___1[l_k.to_integer_32]
						a_c___1[l_k.to_integer_32] := l_t
						l_t___0 := a_d[l_k1.to_integer_32]
						a_d[l_k1.to_integer_32] := a_d[l_k.to_integer_32]
						a_d[l_k.to_integer_32] := l_t___0
						l_t___1 := a_e[l_k1.to_integer_32]
						a_e[l_k1.to_integer_32] := a_e[l_k.to_integer_32]
						a_e[l_k.to_integer_32] := l_t___1
						l_t___2 := a_b___0[l_k1.to_integer_32]
						a_b___0[l_k1.to_integer_32] := a_b___0[l_k.to_integer_32]
						a_b___0[l_k.to_integer_32] := l_t___2
					end
					if a_c___1[l_k.to_integer_32] = (0).to_double then
						Result := (-1)
						js.return
					end
					if not js.return_called then
						l_t___3 := -a_c___1[l_k1.to_integer_32] / a_c___1[l_k.to_integer_32]
						a_c___1[l_k1.to_integer_32] := a_d[l_k1.to_integer_32] + l_t___3 * a_d[l_k.to_integer_32]
						a_d[l_k1.to_integer_32] := a_e[l_k1.to_integer_32] + l_t___3 * a_e[l_k.to_integer_32]
						a_e[l_k1.to_integer_32] := (0).to_double
						a_b___0[l_k1.to_integer_32] := a_b___0[l_k1.to_integer_32] + (l_t___3 * a_b___0[l_k.to_integer_32])
						l_k := l_k + 1
					end
				end
			end
			if not js.return_called and then (a_c___1[(a_n - {NATURAL_32} 1).to_integer_32] = (0).to_double) then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				a_b___0[(a_n - {NATURAL_32} 1).to_integer_32] := a_b___0[(a_n - {NATURAL_32} 1).to_integer_32] / (a_c___1[(a_n - {NATURAL_32} 1).to_integer_32])
				a_b___0[(a_n - {NATURAL_32} 2).to_integer_32] := (a_b___0[(a_n - {NATURAL_32} 2).to_integer_32] - a_d[(a_n - {NATURAL_32} 2).to_integer_32] * a_b___0[(a_n - {NATURAL_32} 1).to_integer_32]) / a_c___1[(a_n - {NATURAL_32} 2).to_integer_32]
				l_k := a_n
				from
				until
					not (l_k > {NATURAL_32} 2)
				loop
					l_kb := l_k - {NATURAL_32} 3
					a_b___0[l_kb.to_integer_32] := ((a_b___0[l_kb.to_integer_32] - a_d[l_kb.to_integer_32] * a_b___0[(l_kb + {NATURAL_32} 1).to_integer_32]) - a_e[l_kb.to_integer_32] * a_b___0[(l_kb + {NATURAL_32} 2).to_integer_32]) / a_c___1[l_kb.to_integer_32]
					l_k := l_k - 1
				end
				Result := 0
			end
		end

end
