class
	P_SOBOL

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create sobol_type.make
			create gsl_qrng_sobol.make
			create primitive_polynomials.make (<<40>>)
			create degree_table.make (<<40>>)
			create v_init.make (<<8, 40>>)
		end

feature {ANY} -- attributes

	sobol_type: PD_GSL_QRNG_TYPE

	gsl_qrng_sobol: CE_POINTER [PD_GSL_QRNG_TYPE]

	primitive_polynomials: CE_ARRAY [INTEGER_32]

	degree_table: CE_ARRAY [INTEGER_32]

	v_init: CE_ARRAY [CE_ARRAY [INTEGER_32]]

feature {ANY} -- routines

	sobol_state_size (a_dimension: NATURAL_32): NATURAL_32
		do
			Result := ({PD_SOBOL_STATE_T}.sizeof.to_natural_32)
		end

	sobol_state_size_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent sobol_state_size)
		end

	sobol_init (a_state: CE_POINTER [ANY]; a_dimension: NATURAL_32): INTEGER_32
		local
			l_s_state: CE_POINTER [PD_SOBOL_STATE_T]
			l_i_dim: NATURAL_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_ell: INTEGER_32
			l_poly_index: INTEGER_32
			l_degree_i: INTEGER_32
			l_includ: CE_ARRAY [INTEGER_32]
			l_p_i: INTEGER_32
			l_newv: INTEGER_32
			js: JUMP_STATE
		do
			create l_s_state.make
			create l_includ.make (<<8>>)

			l_s_state.ce_gets (create {CE_POINTER [PD_SOBOL_STATE_T]}.make_cast (a_state))
			if a_dimension < {NATURAL_32} 1 then
				Result := 4
				js.return
			elseif a_dimension > {NATURAL_32} 40 then
				Result := 4
				js.return
			end
			if not js.return_called then
				l_k := 0
				from
				until
					not (l_k < 30)
				loop
					(l_s_state.item.v_direction[l_k])[0] := 1
					l_k := l_k + 1
				end
				l_i_dim := {NATURAL_32} 1
				from
				until
					not (not js.loop_left and then (l_i_dim < a_dimension))
				loop
					l_poly_index := l_i_dim.to_integer_32
					l_degree_i := degree_table[l_poly_index]
					l_p_i := primitive_polynomials[l_poly_index]
					l_k := l_degree_i - 1
					from
					until
						not (l_k >= 0)
					loop
						l_includ[l_k] := (l_p_i \\ 2 = 1).to_integer
						l_p_i := l_p_i // (2)
						l_k := l_k - 1
					end
					l_j := 0
					from
					until
						not (l_j < l_degree_i)
					loop
						(l_s_state.item.v_direction[l_j])[l_i_dim.to_integer_32] := (v_init[l_j])[l_i_dim.to_integer_32]
						l_j := l_j + 1
					end
					l_j := l_degree_i
					from
					until
						not (not js.loop_left and then (l_j < 30))
					loop
						l_newv := (l_s_state.item.v_direction[l_j - l_degree_i])[l_i_dim.to_integer_32]
						l_ell := 1
						l_k := 0
						from
						until
							not (l_k < l_degree_i)
						loop
							l_ell := l_ell * (2)
							if (l_includ[l_k]).to_boolean then
								l_newv := (l_newv).bit_xor ((l_ell * (l_s_state.item.v_direction[(l_j - l_k) - 1])[l_i_dim.to_integer_32]))
							end
							l_k := l_k + 1
						end
						(l_s_state.item.v_direction[l_j])[l_i_dim.to_integer_32] := l_newv
						l_j := l_j + 1
					end
					l_i_dim := l_i_dim + 1
				end
				l_ell := 1
				l_j := 28
				from
				until
					not (not js.loop_left and then (l_j >= 0))
				loop
					l_ell := l_ell * (2)
					l_i_dim := {NATURAL_32} 0
					from
					until
						not (l_i_dim < a_dimension)
					loop
						(l_s_state.item.v_direction[l_j])[l_i_dim.to_integer_32] := (l_s_state.item.v_direction[l_j])[l_i_dim.to_integer_32] * l_ell
						l_i_dim := l_i_dim + 1
					end
					l_j := l_j - 1
				end
				l_s_state.item.last_denominator_inv := 1.0 / (2.0 * l_ell.to_double)
				l_s_state.item.sequence_count := {NATURAL_32} 0
				l_i_dim := {NATURAL_32} 0
				from
				until
					not (l_i_dim < a_dimension)
				loop
					l_s_state.item.last_numerator_vec[l_i_dim.to_integer_32] := 0
					l_i_dim := l_i_dim + 1
				end
				Result := 0
			end
		end

	sobol_init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent sobol_init)
		end

	sobol_get (a_state: CE_POINTER [ANY]; a_dimension: NATURAL_32; a_v: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_s_state: CE_POINTER [PD_SOBOL_STATE_T]
			l_i_dimension: NATURAL_32
			l_ell: INTEGER_32
			l_c___1: INTEGER_32
			l_direction_i: INTEGER_32
			l_old_numerator_i: INTEGER_32
			l_new_numerator_i: INTEGER_32
			js: JUMP_STATE
		do
			create l_s_state.make

			l_s_state.ce_gets (create {CE_POINTER [PD_SOBOL_STATE_T]}.make_cast (a_state))
			l_ell := 0
			l_c___1 := (l_s_state.item.sequence_count).to_integer_32
			from
			until
				js.loop_left
			loop
				js.reset
				l_ell := l_ell + 1
				if l_c___1 \\ 2 = 1 then
					l_c___1 := l_c___1 // (2)
				else
					js.break
				end
			end
			js.reset
			if l_ell > 30 then
				Result := 5
				js.return
			end
			if not js.return_called then
				l_i_dimension := {NATURAL_32} 0
				from
				until
					not (l_i_dimension < a_dimension)
				loop
					l_direction_i := (l_s_state.item.v_direction[l_ell - 1])[l_i_dimension.to_integer_32]
					l_old_numerator_i := l_s_state.item.last_numerator_vec[l_i_dimension.to_integer_32]
					l_new_numerator_i := (l_old_numerator_i).bit_xor (l_direction_i)
					l_s_state.item.last_numerator_vec[l_i_dimension.to_integer_32] := l_new_numerator_i
					a_v[l_i_dimension.to_integer_32] := l_new_numerator_i.to_double * l_s_state.item.last_denominator_inv
					l_i_dimension := l_i_dimension + 1
				end
				l_s_state.item.sequence_count := (l_s_state.item.sequence_count) + 1
				Result := 0
			end
		end

	sobol_get_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent sobol_get)
		end

end
