class
	P_SORTVECIND_SOURCE

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	index_long_double_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_long_double_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_96]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_long_double_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_long_double_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_long_double_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_LONG_DOUBLE]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_long_double_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_64]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_float_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_float_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [REAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_float_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_float_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_float_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_float_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_ulong_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_ulong_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_ulong_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_ulong_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_ulong_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_ULONG]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_ulong_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_long_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_long_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_long_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_long_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_long_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_LONG]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_long_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_uint_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_uint_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_32]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_uint_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_uint_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_uint_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_UINT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_uint_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_int_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32] < a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (a_data[(l_pki * a_stride).to_integer_32] < a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_int_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_32]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_int_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_int_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_int_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_INT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_int_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_ushort_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_ushort_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_16]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_ushort_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_ushort_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_ushort_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_USHORT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_ushort_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_short_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_short_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_16]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_short_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_short_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_short_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_SHORT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_short_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_uchar_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_uchar_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [NATURAL_8]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_uchar_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_uchar_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_uchar_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_UCHAR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_uchar_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

	index_char_downheap (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride: NATURAL_32; a_N: NATURAL_32; a_k: NATURAL_32)
		local
			l_k: NATURAL_32
			l_pki: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
		do
			l_k := a_k
			l_pki := a_p[l_k.to_integer_32]
			from
			until
				not (not js.loop_left and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				js.reset
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[(l_j + {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]).to_integer_32 then
						l_j := l_j + 1
					end
				end
				if not ((a_data[(l_pki * a_stride).to_integer_32]).to_integer_32 < (a_data[(a_p[l_j.to_integer_32] * a_stride).to_integer_32]).to_integer_32) then
					js.break
				end
				if not js.loop_jumped then
					a_p[l_k.to_integer_32] := a_p[l_j.to_integer_32]
					l_k := l_j
				end
			end
			js.reset
			a_p[l_k.to_integer_32] := l_pki
		end

	gsl_sort_char_index (a_p: CE_POINTER [NATURAL_32]; a_data: CE_POINTER [INTEGER_8]; a_stride: NATURAL_32; a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: NATURAL_32
			js: JUMP_STATE
		do
			if a_n = {NATURAL_32} 0 then
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_p[l_i.to_integer_32] := l_i
					l_i := l_i + 1
				end
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					js.loop_left
				loop
					js.reset
					l_k := l_k - 1
					index_char_downheap (a_p, a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						js.break
					end
				end
				js.reset
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_p[0]
					a_p[0] := a_p[l_N.to_integer_32]
					a_p[l_N.to_integer_32] := l_tmp
					l_N := l_N - 1
					index_char_downheap (a_p, a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	gsl_sort_vector_char_index (a_permutation: CE_POINTER [PD_GSL_PERMUTATION_STRUCT]; a_v: CE_POINTER [PD_GSL_VECTOR_CHAR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_permutation.item.size /= a_v.item.size then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("permutation and vector lengths are not equal"), ce_string ("sortvecind_source.c"), 100, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				gsl_sort_char_index (a_permutation.item.data, a_v.item.data, a_v.item.stride, a_v.item.size)
				Result := 0
			end
		end

end
