class
	P_STAT2D

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_histogram2d_sum (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_n: NATURAL_32
			l_sum: REAL_64
			l_i: NATURAL_32
			l_tmp: NATURAL_32
		do
			l_n := (a_h.item.nx * a_h.item.ny)
			l_sum := (0).to_double
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := l_i
				l_i := l_i + 1
				l_sum := l_sum + (a_h.item.bin[l_tmp.to_integer_32])
			end
			Result := l_sum
		end

	gsl_histogram2d_xmean (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wmean: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
			l_wij: REAL_64
			js: JUMP_STATE
		do
			l_nx := a_h.item.nx
			l_ny := a_h.item.ny
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < l_nx))
			loop
				l_xi := (a_h.item.xrange[(l_i + {NATURAL_32} 1).to_integer_32] + a_h.item.xrange[l_i.to_integer_32]) / 2.0
				l_wi := (0).to_double
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_wij := a_h.item.bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wi := l_wi + l_wij
					end
					l_j := l_j + 1
				end
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_xi) - l_wmean) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wmean.truncated_to_real_64
		end

	gsl_histogram2d_ymean (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wmean: REAL_96
			l_W: REAL_96
			l_yj: REAL_64
			l_wj: REAL_64
			l_wij: REAL_64
			js: JUMP_STATE
		do
			l_nx := a_h.item.nx
			l_ny := a_h.item.ny
			l_wmean := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_j < l_ny))
			loop
				l_yj := (a_h.item.yrange[(l_j + {NATURAL_32} 1).to_integer_32] + a_h.item.yrange[l_j.to_integer_32]) / 2.0
				l_wj := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_wij := a_h.item.bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wj := l_wj + l_wij
					end
					l_i := l_i + 1
				end
				if l_wj > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wj)
					l_wmean := l_wmean + ((real_96_from_real_64 (l_yj) - l_wmean) * (real_96_from_real_64 (l_wj) / l_W))
				end
				l_j := l_j + 1
			end
			Result := l_wmean.truncated_to_real_64
		end

	gsl_histogram2d_xsigma (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_xmean: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wvariance: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_wi: REAL_64
			l_wij: REAL_64
			l_xsigma: REAL_64
			js: JUMP_STATE
		do
			l_xmean := (gsl_histogram2d_xmean (a_h))
			l_nx := a_h.item.nx
			l_ny := a_h.item.ny
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_i < l_nx))
			loop
				l_xi := (a_h.item.xrange[(l_i + {NATURAL_32} 1).to_integer_32] + a_h.item.xrange[l_i.to_integer_32]) / (2).to_double - l_xmean
				l_wi := (0).to_double
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_ny)
				loop
					l_wij := a_h.item.bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wi := l_wi + l_wij
					end
					l_j := l_j + 1
				end
				if l_wi > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wvariance := l_wvariance + ((real_96_from_real_64 ((l_xi * l_xi)) - l_wvariance) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			l_xsigma := (sqrt (l_wvariance.truncated_to_real_64))
			Result := l_xsigma
		end

	gsl_histogram2d_ysigma (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_ymean: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wvariance: REAL_96
			l_W: REAL_96
			l_yj: REAL_64
			l_wj: REAL_64
			l_wij: REAL_64
			l_ysigma: REAL_64
			js: JUMP_STATE
		do
			l_ymean := (gsl_histogram2d_ymean (a_h))
			l_nx := a_h.item.nx
			l_ny := a_h.item.ny
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_j < l_ny))
			loop
				l_yj := (a_h.item.yrange[(l_j + {NATURAL_32} 1).to_integer_32] + a_h.item.yrange[l_j.to_integer_32]) / 2.0 - l_ymean
				l_wj := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_wij := a_h.item.bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_wj := l_wj + l_wij
					end
					l_i := l_i + 1
				end
				if l_wj > (0).to_double then
					l_W := l_W + real_96_from_real_64 (l_wj)
					l_wvariance := l_wvariance + ((real_96_from_real_64 ((l_yj * l_yj)) - l_wvariance) * (real_96_from_real_64 (l_wj) / l_W))
				end
				l_j := l_j + 1
			end
			l_ysigma := (sqrt (l_wvariance.truncated_to_real_64))
			Result := l_ysigma
		end

	gsl_histogram2d_cov (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): REAL_64
		local
			l_xmean: REAL_64
			l_ymean: REAL_64
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_wcovariance: REAL_96
			l_W: REAL_96
			l_xi: REAL_64
			l_yj: REAL_64
			l_wij: REAL_64
			js: JUMP_STATE
		do
			l_xmean := (gsl_histogram2d_xmean (a_h))
			l_ymean := (gsl_histogram2d_ymean (a_h))
			l_nx := a_h.item.nx
			l_ny := a_h.item.ny
			l_wcovariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_left and then (l_j < l_ny))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nx)
				loop
					l_xi := (a_h.item.xrange[(l_i + {NATURAL_32} 1).to_integer_32] + a_h.item.xrange[l_i.to_integer_32]) / 2.0 - l_xmean
					l_yj := (a_h.item.yrange[(l_j + {NATURAL_32} 1).to_integer_32] + a_h.item.yrange[l_j.to_integer_32]) / 2.0 - l_ymean
					l_wij := a_h.item.bin[(l_i * l_ny + l_j).to_integer_32]
					if l_wij > (0).to_double then
						l_W := l_W + real_96_from_real_64 (l_wij)
						l_wcovariance := l_wcovariance + ((real_96_from_real_64 ((l_xi * l_yj)) - l_wcovariance) * (real_96_from_real_64 (l_wij) / l_W))
					end
					l_i := l_i + 1
				end
				l_j := l_j + 1
			end
			Result := l_wcovariance.truncated_to_real_64
		end

end
