class
	P_SYMM

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_eigen_symm_alloc (a_n: NATURAL_32): CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]
		local
			l_w: CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]
			js: JUMP_STATE
		do
			create l_w.make

			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("matrix dimension must be positive integer"), ce_string ("symm.c"), 45, 4)
					Result := (create {CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				l_w.ce_gets (create {CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]}.make_cast ((malloc ({PD_GSL_EIGEN_SYMM_WORKSPACE}.sizeof.to_natural_32))))
				if (l_w.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("failed to allocate space for workspace"), ce_string ("symm.c"), 53, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				l_w.item.d := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_n * Real_64_bytes.to_natural_32)))
				if ((l_w.item.d).c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("failed to allocate space for diagonal"), ce_string ("symm.c"), 60, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				l_w.item.sd := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_n * Real_64_bytes.to_natural_32)))
				if ((l_w.item.sd).c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("failed to allocate space for subdiagonal"), ce_string ("symm.c"), 67, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				l_w.item.size := a_n
				Result := l_w
			end
		end

	gsl_eigen_symm_free (a_w: CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE])
		local
			js: JUMP_STATE
		do
			if (a_w).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (a_w.item.sd))
				free (create {CE_POINTER [ANY]}.make_cast (a_w.item.d))
				free (create {CE_POINTER [ANY]}.make_cast (a_w))
			end
		end

	gsl_eigen_symm (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_w: CE_POINTER [PD_GSL_EIGEN_SYMM_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_d: CE_POINTER [REAL_64]
			l_sd: CE_POINTER [REAL_64]
			l_a___5: NATURAL_32
			l_b___0: NATURAL_32
			l_A00: REAL_64
			l_d_vec: PD_GSL_VECTOR_VIEW
			l_sd_vec: PD_GSL_VECTOR_VIEW
			l_tau: PD_GSL_VECTOR_VIEW
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_n_block: NATURAL_32
			l_d_block: CE_POINTER [REAL_64]
			l_sd_block: CE_POINTER [REAL_64]
			l_d_vec___0: PD_GSL_VECTOR_VIEW
			js: JUMP_STATE
		do
			create l_d.make
			create l_sd.make
			create l_d_vec.make
			create l_sd_vec.make
			create l_tau.make
			create l_d_block.make
			create l_sd_block.make
			create l_d_vec___0.make

			if a_A___0.item.size1 /= a_A___0.item.size2 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("symm.c"), 91, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_eval.item.size /= a_A___0.item.size1 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("symm.c"), 95, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			else
				l_N := a_A___0.item.size1
				l_d.ce_gets (a_w.item.d)
				l_sd.ce_gets (a_w.item.sd)
				if l_N = {NATURAL_32} 1 then
					l_A00 := (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_A___0, (0).to_natural_32, (0).to_natural_32))
					gsl_gsl_vector_double_inst.gsl_vector_set (a_eval, (0).to_natural_32, l_A00)
					Result := 0
					js.return
				end
				if not js.return_called then
					l_d_vec.ce_gets ((view_source_inst.gsl_vector_view_array (l_d, l_N)))
					l_sd_vec.ce_gets ((view_source_inst.gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)))
					l_tau.ce_gets ((view_source_inst.gsl_vector_view_array (l_sd, l_N - {NATURAL_32} 1)))
					symmtd_inst.gsl_linalg_symmtd_decomp (a_A___0, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_tau.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)).do_nothing
					symmtd_inst.gsl_linalg_symmtd_unpack_T (a_A___0, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_sd_vec.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset)).do_nothing
					qrstep_inst.chop_small_elements___2 (l_N, l_d, l_sd)
					l_b___0 := l_N - {NATURAL_32} 1
					from
					until
						not (not js.loop_left and then (l_b___0 > {NATURAL_32} 0))
					loop
						js.reset
						if l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32] = 0.0 then
							l_b___0 := l_b___0 - 1
							js.continue
						else
							if Real_64_bytes.to_natural_32 = Real_32_bytes.to_natural_32 then
								l_tmp___7 := (isnanf ((l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]).truncated_to_real))
							else
								if Real_64_bytes.to_natural_32 = Real_64_bytes.to_natural_32 then
									l_tmp___6 := (isnan (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32]))
								else
									l_tmp___6 := (isnanl (real_96_from_real_64 (l_sd[(l_b___0 - {NATURAL_32} 1).to_integer_32])))
								end
								l_tmp___7 := l_tmp___6
							end
							if l_tmp___7.to_boolean then
								l_b___0 := l_b___0 - 1
								js.continue
							end
						end
						if not js.loop_jumped then
							l_a___5 := l_b___0 - {NATURAL_32} 1
							from
							until
								not (not js.loop_left and then (l_a___5 > {NATURAL_32} 0))
							loop
								js.reset
								if l_sd[(l_a___5 - {NATURAL_32} 1).to_integer_32] = 0.0 then
									js.break
								end
								if not js.loop_jumped then
									l_a___5 := l_a___5 - 1
								end
							end
							js.reset
							l_n_block := (l_b___0 - l_a___5) + {NATURAL_32} 1
							l_d_block.ce_gets (l_d + l_a___5.to_integer_32)
							l_sd_block.ce_gets (l_sd + l_a___5.to_integer_32)
							qrstep_inst.qrstep___2 (l_n_block, l_d_block, l_sd_block, create {CE_POINTER [REAL_64]}.make_shared (default_pointer), create {CE_POINTER [REAL_64]}.make_shared (default_pointer))
							qrstep_inst.chop_small_elements___2 (l_n_block, l_d_block, l_sd_block)
						end
					end
					l_d_vec___0.ce_gets ((view_source_inst.gsl_vector_view_array (l_d, l_N)))
					copy_source_inst.gsl_vector_memcpy (a_eval, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_d_vec___0.c_pointer + {PD_GSL_VECTOR_VIEW}.vector_offset))).do_nothing
					Result := 0
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
