class
	P_TRIG

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create sinc_data.make (<<17>>)
			create sinc_cs.make
			create sin_data.make (<<12>>)
			create sin_cs.make
			create cos_data.make (<<11>>)
			create cos_cs.make
		end

feature {ANY} -- attributes

	sinc_data: CE_ARRAY [REAL_64]

	sinc_cs: PD_CHEB_SERIES_STRUCT

	sin_data: CE_ARRAY [REAL_64]

	sin_cs: PD_CHEB_SERIES_STRUCT

	cos_data: CE_ARRAY [REAL_64]

	cos_cs: PD_CHEB_SERIES_STRUCT

feature {ANY} -- routines

	sinh_series (a_x: REAL_64; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_y: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
		do
			l_y := a_x * a_x
			l_c0___0 := 1.0 / 6.0
			l_c1___2 := 1.0 / 120.0
			l_c2___0 := 1.0 / 5040.0
			l_c3___3 := 1.0 / 362880.0
			l_c4___2 := 1.0 / 39916800.0
			l_c5___1 := 1.0 / 6227020800.0
			l_c6___2 := 1.0 / 1307674368000.0
			l_c7___0 := 1.0 / 355687428096000.0
			a_result.item := a_x * (1.0 + l_y * (l_c0___0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * (l_c6___2 + l_y * l_c7___0))))))))
			Result := 0
		end

	cosh_m1_series (a_x: REAL_64; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_y: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_c5___1: REAL_64
			l_c6___2: REAL_64
			l_c7___0: REAL_64
			l_c8___0: REAL_64
		do
			l_y := a_x * a_x
			l_c0___0 := 0.5
			l_c1___2 := 1.0 / 24.0
			l_c2___0 := 1.0 / 720.0
			l_c3___3 := 1.0 / 40320.0
			l_c4___2 := 1.0 / 3628800.0
			l_c5___1 := 1.0 / 479001600.0
			l_c6___2 := 1.0 / 87178291200.0
			l_c7___0 := 1.0 / 20922789888000.0
			l_c8___0 := 1.0 / 6402373705728000.0
			a_result.item := l_y * (l_c0___0 + l_y * (l_c1___2 + l_y * (l_c2___0 + l_y * (l_c3___3 + l_y * (l_c4___2 + l_y * (l_c5___1 + l_y * (l_c6___2 + l_y * (l_c7___0 + l_y * l_c8___0))))))))
			Result := 0
		end

	gsl_sf_sin_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_P1___0: REAL_64
			l_P2___0: REAL_64
			l_P3___0: REAL_64
			l_sgn_x: REAL_64
			l_tmp: INTEGER_32
			l_abs_x: REAL_64
			l_x2___1: REAL_64
			l_sgn_result: REAL_64
			l_y: REAL_64
			l_octant: INTEGER_32
			l_stat_cs: INTEGER_32
			l_z: REAL_64
			l_sin_cs_result: PD_GSL_SF_RESULT_STRUCT
			l_t: REAL_64
			l_cos_cs_result: PD_GSL_SF_RESULT_STRUCT
			l_t___0: REAL_64
		do
			create l_sin_cs_result.make
			create l_cos_cs_result.make

			l_P1___0 := 7.85398125648498535156e-1
			l_P2___0 := 3.77489470793079817668e-8
			l_P3___0 := 2.69515142907905952645e-15
			if a_x >= 0.0 then
				l_tmp := 1
			else
				l_tmp := -1
			end
			l_sgn_x := l_tmp.to_double
			l_abs_x := (fabs (a_x))
			if l_abs_x < 1.2207031250000000e-04 then
				l_x2___1 := a_x * a_x
				a_result.item.val := a_x * (1.0 - l_x2___1 / 6.0)
				a_result.item.err := fabs (((a_x * l_x2___1) * l_x2___1) / 100.0)
				Result := 0
			else
				l_sgn_result := l_sgn_x
				l_y := (floor (l_abs_x / (0.25 * 3.14159265358979323846)))
				l_octant := (l_y - (ldexp ((floor ((ldexp (l_y, -3)))), 3))).truncated_to_integer
				if ((l_octant).bit_and (1)).to_boolean then
					l_octant := l_octant + 1
					l_octant := (l_octant).bit_and ((7))
					l_y := l_y + (1.0)
				end
				if l_octant > 3 then
					l_octant := l_octant - (4)
					l_sgn_result := -l_sgn_result
				end
				l_z := ((l_abs_x - l_y * l_P1___0) - l_y * l_P2___0) - l_y * l_P3___0
				if l_octant = 0 then
					l_t := (8.0 * (fabs (l_z))) / 3.14159265358979323846 - 1.0
					l_stat_cs := cheb_eval_inst.cheb_eval_e ((sin_cs.address), l_t, l_sin_cs_result.address)
					a_result.item.val := l_z * (1.0 + (l_z * l_z) * l_sin_cs_result.val)
				else
					l_t___0 := (8.0 * (fabs (l_z))) / 3.14159265358979323846 - 1.0
					l_stat_cs := cheb_eval_inst.cheb_eval_e ((cos_cs.address), l_t___0, l_cos_cs_result.address)
					a_result.item.val := 1.0 - ((0.5 * l_z) * l_z) * (1.0 - (l_z * l_z) * l_cos_cs_result.val)
				end
				a_result.item.val := a_result.item.val * l_sgn_result
				if l_abs_x > 1.0 / 2.2204460492503131e-16 then
					a_result.item.err := fabs (a_result.item.val)
				elseif l_abs_x > 100.0 / 1.4901161193847656e-08 then
					a_result.item.err := ((2.0 * l_abs_x) * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				elseif l_abs_x > 0.1 / 1.4901161193847656e-08 then
					a_result.item.err := (2.0 * 1.4901161193847656e-08) * (fabs (a_result.item.val))
				else
					a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				end
				Result := l_stat_cs
			end
		end

	gsl_sf_cos_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_P1___0: REAL_64
			l_P2___0: REAL_64
			l_P3___0: REAL_64
			l_abs_x: REAL_64
			l_x2___1: REAL_64
			l_sgn_result: REAL_64
			l_y: REAL_64
			l_octant: INTEGER_32
			l_stat_cs: INTEGER_32
			l_z: REAL_64
			l_cos_cs_result: PD_GSL_SF_RESULT_STRUCT
			l_t: REAL_64
			l_sin_cs_result: PD_GSL_SF_RESULT_STRUCT
			l_t___0: REAL_64
		do
			create l_cos_cs_result.make
			create l_sin_cs_result.make

			l_P1___0 := 7.85398125648498535156e-1
			l_P2___0 := 3.77489470793079817668e-8
			l_P3___0 := 2.69515142907905952645e-15
			l_abs_x := (fabs (a_x))
			if l_abs_x < 1.2207031250000000e-04 then
				l_x2___1 := a_x * a_x
				a_result.item.val := 1.0 - 0.5 * l_x2___1
				a_result.item.err := fabs ((l_x2___1 * l_x2___1) / 12.0)
				Result := 0
			else
				l_sgn_result := 1.0
				l_y := (floor (l_abs_x / (0.25 * 3.14159265358979323846)))
				l_octant := (l_y - (ldexp ((floor ((ldexp (l_y, -3)))), 3))).truncated_to_integer
				if ((l_octant).bit_and (1)).to_boolean then
					l_octant := l_octant + 1
					l_octant := (l_octant).bit_and ((7))
					l_y := l_y + (1.0)
				end
				if l_octant > 3 then
					l_octant := l_octant - (4)
					l_sgn_result := -l_sgn_result
				end
				if l_octant > 1 then
					l_sgn_result := -l_sgn_result
				end
				l_z := ((l_abs_x - l_y * l_P1___0) - l_y * l_P2___0) - l_y * l_P3___0
				if l_octant = 0 then
					l_t := (8.0 * (fabs (l_z))) / 3.14159265358979323846 - 1.0
					l_stat_cs := cheb_eval_inst.cheb_eval_e ((cos_cs.address), l_t, l_cos_cs_result.address)
					a_result.item.val := 1.0 - ((0.5 * l_z) * l_z) * (1.0 - (l_z * l_z) * l_cos_cs_result.val)
				else
					l_t___0 := (8.0 * (fabs (l_z))) / 3.14159265358979323846 - 1.0
					l_stat_cs := cheb_eval_inst.cheb_eval_e ((sin_cs.address), l_t___0, l_sin_cs_result.address)
					a_result.item.val := l_z * (1.0 + (l_z * l_z) * l_sin_cs_result.val)
				end
				a_result.item.val := a_result.item.val * l_sgn_result
				if l_abs_x > 1.0 / 2.2204460492503131e-16 then
					a_result.item.err := fabs (a_result.item.val)
				elseif l_abs_x > 100.0 / 1.4901161193847656e-08 then
					a_result.item.err := ((2.0 * l_abs_x) * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				elseif l_abs_x > 0.1 / 1.4901161193847656e-08 then
					a_result.item.err := (2.0 * 1.4901161193847656e-08) * (fabs (a_result.item.val))
				else
					a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				end
				Result := l_stat_cs
			end
		end

	gsl_sf_hypot_e (a_x: REAL_64; a_y: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_min: REAL_64
			l_max: REAL_64
			l_rat: REAL_64
			l_root_term: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x /= 0.0 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_y = 0.0 then
							a_result.item.val := 0.0
							a_result.item.err := 0.0
							Result := 0
							js.return
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						l_a___5 := (fabs (a_x))
						l_b___0 := (fabs (a_y))
						l_min := (gsl_gsl_minmax_inst.GSL_MIN_DBL (l_a___5, l_b___0))
						l_max := (gsl_gsl_minmax_inst.GSL_MAX_DBL (l_a___5, l_b___0))
						l_rat := l_min / l_max
						l_root_term := (sqrt (1.0 + l_rat * l_rat))
						if l_max < 1.7976931348623157e+308 / l_root_term then
							a_result.item.val := l_max * l_root_term
							a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
							Result := 0
							js.return
						else
							from
							until
								js.loop_left
							loop
								js.reset
								a_result.item.val := (builtin_inff).to_double
								a_result.item.err := (builtin_inff).to_double
								from
								until
									js.loop_left
								loop
									js.reset
									error_inst.gsl_error (ce_string ("overflow"), ce_string ("trig.c"), 335, 16)
									Result := 16
									js.return
									if not js.return_called then
										js.break
									end
								end
								js.reset
								if not js.return_called then
									js.break
								end
							end
							js.reset
						end
				when 4 then
					if not js.return_called then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_complex_sin_e (a_zr: REAL_64; a_zi: REAL_64; a_szr: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_szi: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ch_m1: REAL_64
			l_sh: REAL_64
			l_ex: REAL_64
			l_ch: REAL_64
			l_sh___0: REAL_64
			js: JUMP_STATE
		do
			if (fabs (a_zi)) < 1.0 then
				sinh_series (a_zi, create {CE_POINTER [REAL_64]}.make_shared ($l_sh)).do_nothing
				cosh_m1_series (a_zi, create {CE_POINTER [REAL_64]}.make_shared ($l_ch_m1)).do_nothing
				a_szr.item.val := (sin (a_zr)) * (l_ch_m1 + 1.0)
				a_szi.item.val := (cos (a_zr)) * l_sh
				a_szr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_szr.item.val))
				a_szi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_szi.item.val))
				Result := 0
				js.return
			elseif (fabs (a_zi)) < 7.0978271289338397e+02 then
				l_ex := (exp (a_zi))
				l_ch := 0.5 * (l_ex + 1.0 / l_ex)
				l_sh___0 := 0.5 * (l_ex - 1.0 / l_ex)
				a_szr.item.val := (sin (a_zr)) * l_ch
				a_szi.item.val := (cos (a_zr)) * l_sh___0
				a_szr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_szr.item.val))
				a_szi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_szi.item.val))
				Result := 0
				js.return
			else
				from
				until
					js.loop_left
				loop
					js.reset
					a_szr.item.val := (builtin_inff).to_double
					a_szr.item.err := (builtin_inff).to_double
					a_szi.item.val := (builtin_inff).to_double
					a_szi.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("trig.c"), 369, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_complex_cos_e (a_zr: REAL_64; a_zi: REAL_64; a_czr: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_czi: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ch_m1: REAL_64
			l_sh: REAL_64
			l_ex: REAL_64
			l_ch: REAL_64
			l_sh___0: REAL_64
			js: JUMP_STATE
		do
			if (fabs (a_zi)) < 1.0 then
				sinh_series (a_zi, create {CE_POINTER [REAL_64]}.make_shared ($l_sh)).do_nothing
				cosh_m1_series (a_zi, create {CE_POINTER [REAL_64]}.make_shared ($l_ch_m1)).do_nothing
				a_czr.item.val := (cos (a_zr)) * (l_ch_m1 + 1.0)
				a_czi.item.val := -(sin (a_zr)) * l_sh
				a_czr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_czr.item.val))
				a_czi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_czi.item.val))
				Result := 0
				js.return
			elseif (fabs (a_zi)) < 7.0978271289338397e+02 then
				l_ex := (exp (a_zi))
				l_ch := 0.5 * (l_ex + 1.0 / l_ex)
				l_sh___0 := 0.5 * (l_ex - 1.0 / l_ex)
				a_czr.item.val := (cos (a_zr)) * l_ch
				a_czi.item.val := -(sin (a_zr)) * l_sh___0
				a_czr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_czr.item.val))
				a_czi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_czi.item.val))
				Result := 0
				js.return
			else
				from
				until
					js.loop_left
				loop
					js.reset
					a_czr.item.val := (builtin_inff).to_double
					a_czr.item.err := (builtin_inff).to_double
					a_czi.item.val := (builtin_inff).to_double
					a_czi.item.err := (builtin_inff).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("trig.c"), 402, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_complex_logsin_e (a_zr: REAL_64; a_zi: REAL_64; a_lszr: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_lszi: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_sin_r: PD_GSL_SF_RESULT_STRUCT
			l_sin_i: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_sin_r.make
			create l_sin_i.make

			if a_zi > 60.0 then
				a_lszr.item.val := -0.69314718055994530942 + a_zi
				a_lszi.item.val := 0.5 * 3.14159265358979323846 - a_zr
				a_lszr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_lszr.item.val))
				a_lszi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_lszi.item.val))
			elseif a_zi < -60.0 then
				a_lszr.item.val := -0.69314718055994530942 - a_zi
				a_lszi.item.val := -0.5 * 3.14159265358979323846 + a_zr
				a_lszr.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_lszr.item.val))
				a_lszi.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_lszi.item.val))
			else
				gsl_sf_complex_sin_e (a_zr, a_zi, l_sin_r.address, l_sin_i.address).do_nothing
				l_status := log_inst.gsl_sf_complex_log_e (l_sin_r.val, l_sin_i.val, a_lszr, a_lszi)
				if l_status = 1 then
					from
					until
						js.loop_left
					loop
						js.reset
						a_lszr.item.val := (builtin_nanf (ce_string (""))).to_double
						a_lszr.item.err := (builtin_nanf (ce_string (""))).to_double
						a_lszi.item.val := (builtin_nanf (ce_string (""))).to_double
						a_lszi.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("trig.c"), 432, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				Result := (gsl_sf_angle_restrict_symm_e (create {CE_POINTER [REAL_64]}.make_shared (a_lszi.c_pointer + {PD_GSL_SF_RESULT_STRUCT}.val_offset)))
			end
		end

	gsl_sf_lnsinh_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_eps: REAL_64
			js: JUMP_STATE
		do
			if a_x <= 0.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("trig.c"), 445, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif (fabs (a_x)) < 1.0 then
				sinh_series (a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_eps)).do_nothing
				a_result.item.val := log (l_eps)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			elseif a_x < -0.5 * -3.6043653389117154e+01 then
				a_result.item.val := a_x + (log (0.5 * (1.0 - (exp (-2.0 * a_x)))))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				a_result.item.val := -0.69314718055994530942 + a_x
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_lncosh_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_eps: REAL_64
		do
			if (fabs (a_x)) < 1.0 then
				cosh_m1_series (a_x, create {CE_POINTER [REAL_64]}.make_shared ($l_eps)).do_nothing
				Result := (log_inst.gsl_sf_log_1plusx_e (l_eps, a_result))
			elseif a_x < -0.5 * -3.6043653389117154e+01 then
				a_result.item.val := a_x + (log (0.5 * (1.0 + (exp (-2.0 * a_x)))))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				a_result.item.val := -0.69314718055994530942 + a_x
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			end
		end

	gsl_sf_polar_to_rect (a_r___3: REAL_64; a_theta: REAL_64; a_x: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_y: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_t: REAL_64
			l_status: INTEGER_32
			l_c___1: REAL_64
			l_s: REAL_64
		do
			l_t := a_theta
			l_status := (gsl_sf_angle_restrict_symm_e (create {CE_POINTER [REAL_64]}.make_shared ($l_t)))
			l_c___1 := (cos (l_t))
			l_s := (sin (l_t))
			a_x.item.val := a_r___3 * (cos (l_t))
			a_y.item.val := a_r___3 * (sin (l_t))
			a_x.item.err := a_r___3 * (fabs ((l_s * 2.2204460492503131e-16) * l_t))
			a_x.item.err := a_x.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_x.item.val)))
			a_y.item.err := a_r___3 * (fabs ((l_c___1 * 2.2204460492503131e-16) * l_t))
			a_y.item.err := a_y.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_y.item.val)))
			Result := l_status
		end

	gsl_sf_rect_to_polar (a_x: REAL_64; a_y: REAL_64; a_r___3: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]; a_theta: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_stat_h: INTEGER_32
			js: JUMP_STATE
		do
			l_stat_h := (gsl_sf_hypot_e (a_x, a_y, a_r___3))
			if a_r___3.item.val > 0.0 then
				a_theta.item.val := atan2 (a_y, a_x)
				a_theta.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_theta.item.val))
				Result := l_stat_h
				js.return
			else
				from
				until
					js.loop_left
				loop
					js.reset
					a_theta.item.val := (builtin_nanf (ce_string (""))).to_double
					a_theta.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("trig.c"), 528, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_angle_restrict_symm_err_e (a_theta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_P1___0: REAL_64
			l_P2___0: REAL_64
			l_P3___0: REAL_64
			l_TwoPi: REAL_64
			l_y: REAL_64
			l_tmp: INTEGER_32
			l_r___3: REAL_64
			l_delta___0: REAL_64
			l_tmp___6: REAL_64
			js: JUMP_STATE
		do
			l_P1___0 := (4).to_double * 7.8539812564849853515625e-01
			l_P2___0 := (4).to_double * 3.7748947079307981766760e-08
			l_P3___0 := (4).to_double * 2.6951514290790594840552e-15
			l_TwoPi := (2).to_double * ((l_P1___0 + l_P2___0) + l_P3___0)
			if a_theta >= 0.0 then
				l_tmp := 1
			else
				l_tmp := -1
			end
			l_y := (l_tmp * 2).to_double * (floor ((fabs (a_theta)) / l_TwoPi))
			l_r___3 := ((a_theta - l_y * l_P1___0) - l_y * l_P2___0) - l_y * l_P3___0
			if l_r___3 > 3.14159265358979323846 then
				l_r___3 := ((l_r___3 - (2).to_double * l_P1___0) - (2).to_double * l_P2___0) - (2).to_double * l_P3___0
			elseif l_r___3 < -3.14159265358979323846 then
				l_r___3 := ((l_r___3 + (2).to_double * l_P1___0) + (2).to_double * l_P2___0) + (2).to_double * l_P3___0
			end
			a_result.item.val := l_r___3
			if (fabs (a_theta)) > 0.0625 / 2.2204460492503131e-16 then
				a_result.item.val := (builtin_nanf (ce_string (""))).to_double
				a_result.item.err := (builtin_nanf (ce_string (""))).to_double
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("trig.c"), 552, 17)
					Result := 17
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif (fabs (a_theta)) > 0.0625 / 1.4901161193847656e-08 then
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val - a_theta))
				Result := 0
				js.return
			else
				l_delta___0 := (fabs (a_result.item.val - a_theta))
				if l_delta___0 < 3.14159265358979323846 then
					l_tmp___6 := l_delta___0
				else
					l_tmp___6 := 3.14159265358979323846
				end
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * l_tmp___6
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_angle_restrict_pos_err_e (a_theta: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_P1___0: REAL_64
			l_P2___0: REAL_64
			l_P3___0: REAL_64
			l_TwoPi: REAL_64
			l_y: REAL_64
			l_r___3: REAL_64
			l_delta___0: REAL_64
			l_tmp___3: REAL_64
			js: JUMP_STATE
		do
			l_P1___0 := (4).to_double * 7.85398125648498535156e-01
			l_P2___0 := (4).to_double * 3.77489470793079817668e-08
			l_P3___0 := (4).to_double * 2.69515142907905952645e-15
			l_TwoPi := (2).to_double * ((l_P1___0 + l_P2___0) + l_P3___0)
			l_y := (2).to_double * (floor (a_theta / l_TwoPi))
			l_r___3 := ((a_theta - l_y * l_P1___0) - l_y * l_P2___0) - l_y * l_P3___0
			if l_r___3 > l_TwoPi then
				l_r___3 := ((l_r___3 - (2).to_double * l_P1___0) - (2).to_double * l_P2___0) - (2).to_double * l_P3___0
			elseif l_r___3 < (0).to_double then
				l_r___3 := ((l_r___3 + (2).to_double * l_P1___0) + (2).to_double * l_P2___0) + (2).to_double * l_P3___0
			end
			a_result.item.val := l_r___3
			if (fabs (a_theta)) > 0.0625 / 2.2204460492503131e-16 then
				a_result.item.val := (builtin_nanf (ce_string (""))).to_double
				a_result.item.err := fabs (a_result.item.val)
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("error"), ce_string ("trig.c"), 588, 17)
					Result := 17
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif (fabs (a_theta)) > 0.0625 / 1.4901161193847656e-08 then
				a_result.item.err := 2.2204460492503131e-16 * (fabs (a_result.item.val - a_theta))
				Result := 0
				js.return
			else
				l_delta___0 := (fabs (a_result.item.val - a_theta))
				if l_delta___0 < 3.14159265358979323846 then
					l_tmp___3 := l_delta___0
				else
					l_tmp___3 := 3.14159265358979323846
				end
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * l_tmp___3
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_angle_restrict_symm_e (a_theta: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_r___3: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
		do
			create l_r___3.make

			l_stat := (gsl_sf_angle_restrict_symm_err_e (a_theta.item, l_r___3.address))
			a_theta.item := l_r___3.val
			Result := l_stat
		end

	gsl_sf_angle_restrict_pos_e (a_theta: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_r___3: PD_GSL_SF_RESULT_STRUCT
			l_stat: INTEGER_32
		do
			create l_r___3.make

			l_stat := (gsl_sf_angle_restrict_pos_err_e (a_theta.item, l_r___3.address))
			a_theta.item := l_r___3.val
			Result := l_stat
		end

	gsl_sf_sin_err_e (a_x: REAL_64; a_dx: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_stat_s: INTEGER_32
		do
			l_stat_s := (gsl_sf_sin_e (a_x, a_result))
			a_result.item.err := a_result.item.err + (fabs ((cos (a_x)) * a_dx))
			a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
			Result := l_stat_s
		end

	gsl_sf_cos_err_e (a_x: REAL_64; a_dx: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_stat_c: INTEGER_32
		do
			l_stat_c := (gsl_sf_cos_e (a_x, a_result))
			a_result.item.err := a_result.item.err + (fabs ((sin (a_x)) * a_dx))
			a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
			Result := l_stat_c
		end

	gsl_sf_sinc_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_ax: REAL_64
			l_r___3: REAL_64
			l_s: PD_GSL_SF_RESULT_STRUCT
			l_stat_s: INTEGER_32
		do
			create l_s.make

			l_ax := (fabs (a_x))
			if l_ax < 0.8 then
				Result := (cheb_eval_inst.cheb_eval_e ((sinc_cs.address), 2.0 * l_ax - 1.0, a_result))
			elseif l_ax < 100.0 then
				a_result.item.val := (sin (3.14159265358979323846 * l_ax)) / (3.14159265358979323846 * l_ax)
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				l_r___3 := 3.14159265358979323846 * l_ax
				l_stat_s := (gsl_sf_sin_e (l_r___3, l_s.address))
				a_result.item.val := l_s.val / l_r___3
				a_result.item.err := l_s.err / l_r___3 + (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := l_stat_s
			end
		end

	gsl_sf_sin (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_sin_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_sin_e(x, &result)"), ce_string ("trig.c"), 726, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_cos (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_cos_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_cos_e(x, &result)"), ce_string ("trig.c"), 731, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_hypot (a_x: REAL_64; a_y: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_hypot_e (a_x, a_y, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_hypot_e(x, y, &result)"), ce_string ("trig.c"), 736, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_lnsinh (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_lnsinh_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_lnsinh_e(x, &result)"), ce_string ("trig.c"), 741, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_lncosh (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_lncosh_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_lncosh_e(x, &result)"), ce_string ("trig.c"), 746, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_angle_restrict_symm (a_theta: REAL_64): REAL_64
		local
			l_result: REAL_64
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			l_result := a_theta
			l_status := (gsl_sf_angle_restrict_symm_e (create {CE_POINTER [REAL_64]}.make_shared ($l_result)))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_angle_restrict_symm_e(&result)"), ce_string ("trig.c"), 752, l_status)
					Result := l_result
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := l_result
			end
		end

	gsl_sf_angle_restrict_pos (a_theta: REAL_64): REAL_64
		local
			l_result: REAL_64
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			l_result := a_theta
			l_status := (gsl_sf_angle_restrict_pos_e (create {CE_POINTER [REAL_64]}.make_shared ($l_result)))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_angle_restrict_pos_e(&result)"), ce_string ("trig.c"), 758, l_status)
					Result := l_result
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := l_result
			end
		end

	gsl_sf_sinc (a_x: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_sinc_e (a_x, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_sinc_e(x, &result)"), ce_string ("trig.c"), 770, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
