class
	P_ZETA

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create zeta_xlt1_data.make (<<14>>)
			create zeta_xlt1_cs.make
			create zeta_xgt1_data.make (<<30>>)
			create zeta_xgt1_cs.make
			create zetam1_inter_data.make (<<24>>)
			create zetam1_inter_cs.make
			create zetam1_pos_int_table.make (<<101>>)
			create zeta_neg_int_table.make (<<50>>)
			create hzeta_c.make (<<15>>)
			create eta_pos_int_table.make (<<101>>)
			create eta_neg_int_table.make (<<50>>)
		end

feature {ANY} -- attributes

	zeta_xlt1_data: CE_ARRAY [REAL_64]

	zeta_xlt1_cs: PD_CHEB_SERIES_STRUCT

	zeta_xgt1_data: CE_ARRAY [REAL_64]

	zeta_xgt1_cs: PD_CHEB_SERIES_STRUCT

	zetam1_inter_data: CE_ARRAY [REAL_64]

	zetam1_inter_cs: PD_CHEB_SERIES_STRUCT

	zetam1_pos_int_table: CE_ARRAY [REAL_64]

	zeta_neg_int_table: CE_ARRAY [REAL_64]

	hzeta_c: CE_ARRAY [REAL_64]

	eta_pos_int_table: CE_ARRAY [REAL_64]

	eta_neg_int_table: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	riemann_zeta_sgt0 (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_tmp: REAL_64
			l_x: REAL_64
			l_c___2: PD_GSL_SF_RESULT_STRUCT
			l_f2: REAL_64
			l_f3: REAL_64
			l_f5: REAL_64
			l_f7: REAL_64
		do
			create l_c___1.make
			create l_c___2.make

			if a_s < 1.0 then
				cheb_eval_inst.cheb_eval_e ((zeta_xlt1_cs.address), 2.0 * a_s - 1.0, l_c___1.address).do_nothing
				a_result.item.val := l_c___1.val / (a_s - 1.0)
				l_tmp := fabs (a_s - 1.0)
				a_result.item.err := l_c___1.err / l_tmp + 2.2204460492503131e-16 * (fabs (a_result.item.val))
				Result := 0
			elseif a_s <= 20.0 then
				l_x := (2.0 * a_s - 21.0) / 19.0
				cheb_eval_inst.cheb_eval_e ((zeta_xgt1_cs.address), l_x, l_c___2.address).do_nothing
				a_result.item.val := l_c___2.val / (a_s - 1.0)
				a_result.item.err := l_c___2.err / (a_s - 1.0) + 2.2204460492503131e-16 * (fabs (a_result.item.val))
				Result := 0
			else
				l_f2 := 1.0 - (pow (2.0, -a_s))
				l_f3 := 1.0 - (pow (3.0, -a_s))
				l_f5 := 1.0 - (pow (5.0, -a_s))
				l_f7 := 1.0 - (pow (7.0, -a_s))
				a_result.item.val := 1.0 / (((l_f2 * l_f3) * l_f5) * l_f7)
				a_result.item.err := (3.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			end
		end

	riemann_zeta1ms_slt0 (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_x: REAL_64
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_f2: REAL_64
			l_f3: REAL_64
			l_f5: REAL_64
			l_f7: REAL_64
		do
			create l_c___1.make

			if a_s > -19.0 then
				l_x := ((-19).to_double - 2.0 * a_s) / 19.0
				cheb_eval_inst.cheb_eval_e ((zeta_xgt1_cs.address), l_x, l_c___1.address).do_nothing
				a_result.item.val := l_c___1.val / -a_s
				a_result.item.err := l_c___1.err / -a_s + 2.2204460492503131e-16 * (fabs (a_result.item.val))
				Result := 0
			else
				l_f2 := 1.0 - (pow (2.0, -(1.0 - a_s)))
				l_f3 := 1.0 - (pow (3.0, -(1.0 - a_s)))
				l_f5 := 1.0 - (pow (5.0, -(1.0 - a_s)))
				l_f7 := 1.0 - (pow (7.0, -(1.0 - a_s)))
				a_result.item.val := 1.0 / (((l_f2 * l_f3) * l_f5) * l_f7)
				a_result.item.err := (3.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			end
		end

	riemann_zeta_minus_1_intermediate_s (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_t: REAL_64
			l_c___1: PD_GSL_SF_RESULT_STRUCT
			l_tmp: REAL_64
		do
			create l_c___1.make

			l_t := (a_s - 10.0) / 5.0
			cheb_eval_inst.cheb_eval_e ((zetam1_inter_cs.address), l_t, l_c___1.address).do_nothing
			l_tmp := exp (l_c___1.val)
			a_result.item.val := l_tmp + (pow (2.0, -a_s))
			a_result.item.err := (l_c___1.err + 2.0 * 2.2204460492503131e-16) * a_result.item.val
			Result := 0
		end

	riemann_zeta_minus1_large_s (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_c___1: REAL_64
			l_d: REAL_64
			l_e: REAL_64
			l_f: REAL_64
			l_t1: REAL_64
			l_t2: REAL_64
			l_numt: REAL_64
			l_zeta: REAL_64
		do
			l_a___5 := (pow (2.0, -a_s))
			l_b___0 := (pow (3.0, -a_s))
			l_c___1 := (pow (5.0, -a_s))
			l_d := (pow (7.0, -a_s))
			l_e := (pow (11.0, -a_s))
			l_f := (pow (13.0, -a_s))
			l_t1 := ((((l_a___5 + l_b___0) + l_c___1) + l_d) + l_e) + l_f
			l_t2 := (((l_a___5 * ((((l_b___0 + l_c___1) + l_d) + l_e) + l_f) + l_b___0 * (((l_c___1 + l_d) + l_e) + l_f)) + l_c___1 * ((l_d + l_e) + l_f)) + l_d * (l_e + l_f)) + l_e * l_f
			l_numt := l_t1 - l_t2
			l_zeta := 1.0 / ((((((1.0 - l_a___5) * (1.0 - l_b___0)) * (1.0 - l_c___1)) * (1.0 - l_d)) * (1.0 - l_e)) * (1.0 - l_f))
			a_result.item.val := l_numt * l_zeta
			a_result.item.err := (((15.0 / a_s + 1.0) * 6.0) * 2.2204460492503131e-16) * a_result.item.val
			Result := 0
		end

	gsl_sf_hzeta_e (a_s: REAL_64; a_q___3: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_max_bits: REAL_64
			l_ln_term0: REAL_64
			l_tmp___4: REAL_64
			l_p1: REAL_64
			l_p2: REAL_64
			l_p3: REAL_64
			l_jmax: INTEGER_32
			l_kmax___0: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_pmax: REAL_64
			l_scp: REAL_64
			l_pcp: REAL_64
			l_ans: REAL_64
			l_delta___0: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_s <= 1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___2
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else:
						if not (a_q___3 <= 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___2:
						from
						until
							js.loop_left
						loop
							js.reset
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("zeta.c"), 719, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else:
						l_max_bits := 54.0
						l_ln_term0 := -a_s * (log (a_q___3))
						if not (l_ln_term0 < -7.0839641853226408e+02 + 1.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							js.loop_left
						loop
							js.reset
							a_result.item.val := 0.0
							a_result.item.err := 2.2250738585072014e-308
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("underflow"), ce_string ("zeta.c"), 726, 15)
								Result := 15
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else:
						if not (l_ln_term0 > 7.0978271289338397e+02 - 1.0) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						from
						until
							js.loop_left
						loop
							js.reset
							a_result.item.val := (builtin_inff).to_double
							a_result.item.err := (builtin_inff).to_double
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.gsl_error (ce_string ("overflow"), ce_string ("zeta.c"), 729, 16)
								Result := 16
								js.return
								if not js.return_called then
									js.break
								end
							end
							js.reset
							if not js.return_called then
								js.break
							end
						end
						js.reset
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else:
						if not (a_s > l_max_bits) then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if a_q___3 < 1.0 then
							a_result.item.val := pow (a_q___3, -a_s)
							l_tmp___4 := fabs (a_result.item.val)
							a_result.item.err := (2.0 * 2.2204460492503131e-16) * l_tmp___4
							Result := 0
							js.return
						else
							l_pos := -1 + 15 -- goto _L___1
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else: | _L___1:
						if not (a_s > 0.5 * l_max_bits) then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if a_q___3 < 0.25 then
							a_result.item.val := pow (a_q___3, -a_s)
							l_tmp___4 := fabs (a_result.item.val)
							a_result.item.err := (2.0 * 2.2204460492503131e-16) * l_tmp___4
							Result := 0
							js.return
						else
							l_pos := -1 + 18 -- goto _L___0
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else: | _L___0:
						if not (a_s > 0.5 * l_max_bits) then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						if a_q___3 < 1.0 then
							l_p1 := (pow (a_q___3, -a_s))
							l_p2 := (pow (a_q___3 / (1.0 + a_q___3), a_s))
							l_p3 := (pow (a_q___3 / (2.0 + a_q___3), a_s))
							a_result.item.val := l_p1 * ((1.0 + l_p2) + l_p3)
							a_result.item.err := (2.2204460492503131e-16 * (0.5 * a_s + 2.0)) * (fabs (a_result.item.val))
							Result := 0
							js.return
						else
							l_pos := -1 + 21 -- goto _L
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L:
						l_jmax := 12
						l_kmax___0 := 10
						l_pmax := (pow (l_kmax___0.to_double + a_q___3, -a_s))
						l_scp := a_s
						l_pcp := l_pmax / (l_kmax___0.to_double + a_q___3)
						l_ans := l_pmax * ((l_kmax___0.to_double + a_q___3) / (a_s - 1.0) + 0.5)
						l_k := 0
						from
						until
							not (l_k < l_kmax___0)
						loop
							l_ans := l_ans + (pow (l_k.to_double + a_q___3, -a_s))
							l_k := l_k + 1
						end
						l_j := 0
						from
						until
							not (not js.loop_left and then (l_j <= l_jmax))
						loop
							js.reset
							l_delta___0 := (hzeta_c[l_j + 1] * l_scp) * l_pcp
							l_ans := l_ans + l_delta___0
							if (fabs (l_delta___0 / l_ans)) < 0.5 * 2.2204460492503131e-16 then
								js.break
							end
							if not js.loop_jumped then
								l_scp := l_scp * (((a_s + (2 * l_j).to_double) + (1).to_double) * ((a_s + (2 * l_j).to_double) + (2).to_double))
								l_pcp := l_pcp / ((l_kmax___0.to_double + a_q___3) * (l_kmax___0.to_double + a_q___3))
								l_j := l_j + 1
							end
						end
						js.reset
						a_result.item.val := l_ans
						a_result.item.err := ((2.0 * (l_jmax.to_double + 1.0)) * 2.2204460492503131e-16) * (fabs (l_ans))
						Result := 0
						js.return
				when 22 then -- if done | if done | if done | if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 23 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_zeta_e (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_zeta_one_minus_s: PD_GSL_SF_RESULT_STRUCT
			l_stat_zoms: INTEGER_32
			l_sin_term: REAL_64
			l_tmp___6: REAL_64
			l_twopi_pow: CE_ARRAY [REAL_64]
			l_n: INTEGER_32
			l_fs: REAL_64
			l_p: REAL_64
			l_g: PD_GSL_SF_RESULT_STRUCT
			l_stat_g: INTEGER_32
			l_tmp___13: REAL_64
			l_tmp___15: INTEGER_32
			l_tmp___16: INTEGER_32
			js: JUMP_STATE
		do
			create l_zeta_one_minus_s.make
			create l_twopi_pow.make (<<18>>)
			create l_g.make

			if a_s = 1.0 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("zeta.c"), 781, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_s >= 0.0 then
				Result := (riemann_zeta_sgt0 (a_s, a_result))
				js.return
			else
				l_stat_zoms := (riemann_zeta1ms_slt0 (a_s, l_zeta_one_minus_s.address))
				if (fmod (a_s, 2.0)) = 0.0 then
					l_tmp___6 := 0.0
				else
					l_tmp___6 := (sin ((0.5 * 3.14159265358979323846) * (fmod (a_s, 4.0)))) / 3.14159265358979323846
				end
				l_sin_term := l_tmp___6
				if l_sin_term = 0.0 then
					a_result.item.val := 0.0
					a_result.item.err := 0.0
					Result := 0
					js.return
				elseif a_s > (-170).to_double then
					l_twopi_pow[0] := 1.0
					l_twopi_pow[1] := 9.589560061550901348e+007
					l_twopi_pow[2] := 9.195966217409212684e+015
					l_twopi_pow[3] := 8.818527036583869903e+023
					l_twopi_pow[4] := 8.456579467173150313e+031
					l_twopi_pow[5] := 8.109487671573504384e+039
					l_twopi_pow[6] := 7.776641909496069036e+047
					l_twopi_pow[7] := 7.457457466828644277e+055
					l_twopi_pow[8] := 7.151373628461452286e+063
					l_twopi_pow[9] := 6.857852693272229709e+071
					l_twopi_pow[10] := 6.576379029540265771e+079
					l_twopi_pow[11] := 6.306458169130020789e+087
					l_twopi_pow[12] := 6.047615938853066678e+095
					l_twopi_pow[13] := 5.799397627482402614e+103
					l_twopi_pow[14] := 5.561367186955830005e+111
					l_twopi_pow[15] := 5.333106466365131227e+119
					l_twopi_pow[16] := 5.114214477385391780e+127
					l_twopi_pow[17] := 4.904306689854036836e+135
					l_n := (floor (-a_s / 10.0)).truncated_to_integer
					l_fs := a_s + 10.0 * l_n.to_double
					l_p := (pow (2.0 * 3.14159265358979323846, l_fs)) / l_twopi_pow[l_n]
					l_stat_g := (gamma_inst.gsl_sf_gamma_e (1.0 - a_s, l_g.address))
					a_result.item.val := ((l_p * l_g.val) * l_sin_term) * l_zeta_one_minus_s.val
					a_result.item.err := (fabs ((l_p * l_g.val) * l_sin_term)) * l_zeta_one_minus_s.err
					a_result.item.err := a_result.item.err + ((fabs ((l_p * l_sin_term) * l_zeta_one_minus_s.val)) * l_g.err)
					l_tmp___13 := fabs (a_s)
					a_result.item.err := a_result.item.err + ((2.2204460492503131e-16 * (l_tmp___13 + 2.0)) * (fabs (a_result.item.val)))
					if l_stat_g /= 0 then
						l_tmp___16 := l_stat_g
					else
						if l_stat_zoms /= 0 then
							l_tmp___15 := l_stat_zoms
						else
							l_tmp___15 := 0
						end
						l_tmp___16 := l_tmp___15
					end
					Result := l_tmp___16
					js.return
				else
					from
					until
						js.loop_left
					loop
						js.reset
						a_result.item.val := (builtin_inff).to_double
						a_result.item.err := (builtin_inff).to_double
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.gsl_error (ce_string ("overflow"), ce_string ("zeta.c"), 847, 16)
							Result := 16
							js.return
							if not js.return_called then
								js.break
							end
						end
						js.reset
						if not js.return_called then
							js.break
						end
					end
					js.reset
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_zeta_int_e (a_n: INTEGER_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_n < 0 then
				if not ((a_n).bit_and (1)).to_boolean then
					a_result.item.val := 0.0
					a_result.item.err := 0.0
					Result := 0
					js.return
				elseif a_n > -99 then
					a_result.item.val := zeta_neg_int_table[-(a_n + 1) // 2]
					a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
					Result := 0
					js.return
				else
					Result := (gsl_sf_zeta_e (a_n.to_double, a_result))
					js.return
				end
			elseif a_n = 1 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("zeta.c"), 873, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_n <= 100 then
				a_result.item.val := 1.0 + zetam1_pos_int_table[a_n]
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				a_result.item.val := 1.0
				a_result.item.err := 2.2204460492503131e-16
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_zetam1_e (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_stat: INTEGER_32
		do
			if a_s <= 5.0 then
				l_stat := (gsl_sf_zeta_e (a_s, a_result))
				a_result.item.val := a_result.item.val - (1.0)
				Result := l_stat
			elseif a_s < 15.0 then
				Result := (riemann_zeta_minus_1_intermediate_s (a_s, a_result))
			else
				Result := (riemann_zeta_minus1_large_s (a_s, a_result))
			end
		end

	gsl_sf_zetam1_int_e (a_n: INTEGER_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_n < 0 then
				if not ((a_n).bit_and (1)).to_boolean then
					a_result.item.val := -1.0
					a_result.item.err := 0.0
					Result := 0
					js.return
				elseif a_n > -99 then
					a_result.item.val := zeta_neg_int_table[-(a_n + 1) // 2] - 1.0
					a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
					Result := 0
					js.return
				else
					Result := (gsl_sf_zeta_e (a_n.to_double, a_result))
					js.return
				end
			elseif a_n = 1 then
				from
				until
					js.loop_left
				loop
					js.reset
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_left
					loop
						js.reset
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("zeta.c"), 927, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					js.reset
					if not js.return_called then
						js.break
					end
				end
				js.reset
			elseif a_n <= 100 then
				a_result.item.val := zetam1_pos_int_table[a_n]
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
				js.return
			else
				Result := (gsl_sf_zetam1_e (a_n.to_double, a_result))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_eta_int_e (a_n: INTEGER_32; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_z: PD_GSL_SF_RESULT_STRUCT
			l_p: PD_GSL_SF_RESULT_STRUCT
			l_stat_z: INTEGER_32
			l_stat_p: INTEGER_32
			l_stat_m: INTEGER_32
			l_tmp___4: REAL_64
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
		do
			create l_z.make
			create l_p.make

			if a_n > 100 then
				a_result.item.val := 1.0
				a_result.item.err := 2.2204460492503131e-16
				Result := 0
			elseif a_n >= 0 then
				a_result.item.val := eta_pos_int_table[a_n]
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			elseif not ((a_n).bit_and (1)).to_boolean then
				a_result.item.val := 0.0
				a_result.item.err := 0.0
				Result := 0
			elseif a_n > -99 then
				a_result.item.val := eta_neg_int_table[-(a_n + 1) // 2]
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				l_stat_z := (gsl_sf_zeta_int_e (a_n, l_z.address))
				l_stat_p := (exp_inst.gsl_sf_exp_e ((1.0 - a_n.to_double) * 0.69314718055994530942, l_p.address))
				l_stat_m := (elementary_inst.gsl_sf_multiply_e (-l_p.val, l_z.val, a_result))
				l_tmp___4 := fabs ((l_p.err * (0.69314718055994530942 * (1.0 - a_n.to_double))) * l_z.val)
				a_result.item.err := l_tmp___4 + l_z.err * (fabs (l_p.val))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_m /= 0 then
					l_tmp___9 := l_stat_m
				else
					if l_stat_p /= 0 then
						l_tmp___8 := l_stat_p
					else
						if l_stat_z /= 0 then
							l_tmp___7 := l_stat_z
						else
							l_tmp___7 := 0
						end
						l_tmp___8 := l_tmp___7
					end
					l_tmp___9 := l_tmp___8
				end
				Result := l_tmp___9
			end
		end

	gsl_sf_eta_e (a_s: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_del: REAL_64
			l_c0___0: REAL_64
			l_c1___2: REAL_64
			l_c2___0: REAL_64
			l_c3___3: REAL_64
			l_c4___2: REAL_64
			l_z: PD_GSL_SF_RESULT_STRUCT
			l_p: PD_GSL_SF_RESULT_STRUCT
			l_stat_z: INTEGER_32
			l_stat_p: INTEGER_32
			l_stat_m: INTEGER_32
			l_tmp___3: REAL_64
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
		do
			create l_z.make
			create l_p.make

			if a_s > 100.0 then
				a_result.item.val := 1.0
				a_result.item.err := 2.2204460492503131e-16
				Result := 0
			elseif (fabs (a_s - 1.0)) < 10.0 * 7.4009597974140505e-04 then
				l_del := a_s - 1.0
				l_c0___0 := 0.69314718055994530942
				l_c1___2 := 0.69314718055994530942 * (0.57721566490153286060651209008 - 0.5 * 0.69314718055994530942)
				l_c2___0 := -0.0326862962794492996
				l_c3___3 := 0.0015689917054155150
				l_c4___2 := 0.00074987242112047532
				a_result.item.val := l_c0___0 + l_del * (l_c1___2 + l_del * (l_c2___0 + l_del * (l_c3___3 + l_del * l_c4___2)))
				a_result.item.err := (2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val))
				Result := 0
			else
				l_stat_z := (gsl_sf_zeta_e (a_s, l_z.address))
				l_stat_p := (exp_inst.gsl_sf_exp_e ((1.0 - a_s) * 0.69314718055994530942, l_p.address))
				l_stat_m := (elementary_inst.gsl_sf_multiply_e (1.0 - l_p.val, l_z.val, a_result))
				l_tmp___3 := fabs ((l_p.err * (0.69314718055994530942 * (1.0 - a_s))) * l_z.val)
				a_result.item.err := l_tmp___3 + l_z.err * (fabs (l_p.val))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				if l_stat_m /= 0 then
					l_tmp___8 := l_stat_m
				else
					if l_stat_p /= 0 then
						l_tmp___7 := l_stat_p
					else
						if l_stat_z /= 0 then
							l_tmp___6 := l_stat_z
						else
							l_tmp___6 := 0
						end
						l_tmp___7 := l_tmp___6
					end
					l_tmp___8 := l_tmp___7
				end
				Result := l_tmp___8
			end
		end

	gsl_sf_zeta (a_s: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_zeta_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_zeta_e(s, &result)"), ce_string ("zeta.c"), 1019, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_hzeta (a_s: REAL_64; a_a___5: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_hzeta_e (a_s, a_a___5, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_hzeta_e(s, a, &result)"), ce_string ("zeta.c"), 1024, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_zeta_int (a_s: INTEGER_32): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_zeta_int_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_zeta_int_e(s, &result)"), ce_string ("zeta.c"), 1029, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_zetam1 (a_s: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_zetam1_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_zetam1_e(s, &result)"), ce_string ("zeta.c"), 1034, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_zetam1_int (a_s: INTEGER_32): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_zetam1_int_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_zetam1_int_e(s, &result)"), ce_string ("zeta.c"), 1039, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_eta_int (a_s: INTEGER_32): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_eta_int_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_eta_int_e(s, &result)"), ce_string ("zeta.c"), 1044, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

	gsl_sf_eta (a_s: REAL_64): REAL_64
		local
			l_result: PD_GSL_SF_RESULT_STRUCT
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make

			l_status := (gsl_sf_eta_e (a_s, l_result.address))
			if l_status /= 0 then
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.gsl_error (ce_string ("gsl_sf_eta_e(s, &result)"), ce_string ("zeta.c"), 1049, l_status)
					Result := (l_result.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
				js.reset
			end
			if not js.return_called then
				Result := (l_result.val)
			end
		end

end
