class
	P_HTML_R

inherit
	LINKS_DATA
		redefine
			default_create
		end
	S_SYS_SELECT
	S_STDLIB
	S_UNISTD
	S_STRING
	S_BITS_ERRNO
	S_SYS_SOCKET
	S_STDIO
	S_FCNTL
	S_NETINET_IN
	S_SYS_STAT
	S_SYS_UTSNAME
	S_NETDB
	S_SIGNAL
	S_PWD
	S_GRP
	S_TIME
	S_DIRENT
	S_TERMIOS
	S_SYS_IOCTL
	S_SYS_TIME
	S_SYS_WAIT
	S_UTIME

feature {NONE} -- Initialization

	default_create
		do
			create palette.make (<<17>>)
			create rgb_cache.make (<<4096>>)
			create last_tag_to_move.make
			create last_tag_for_newline.make
			create last_link.make
			create last_target.make
			create last_image.make
			create last_form.make
			create convert_table.make
			create buffer.make (<<256>>)
			create ta_cache.make
			create table_cache.make
			create table_cache_hash.make (<<4096>>)
			create format_cache.make
		end

feature {ANY} -- attributes

	palette: CE_ARRAY [PD_RGB]

	rgb_cache: CE_ARRAY [PD_RGB_CACHE_ENTRY]

	cache_init: INTEGER_32 assign set_cache_init

	set_cache_init (a_cache_init: INTEGER_32)
		do
			cache_init := a_cache_init
		end

	nowrap: INTEGER_32 assign set_nowrap

	set_nowrap (a_nowrap: INTEGER_32)
		do
			nowrap := a_nowrap
		end

	last_tag_to_move: CE_POINTER [PD_TAG]

	last_tag_for_newline: CE_POINTER [PD_TAG]

	last_link: CE_POINTER [NATURAL_8]

	last_target: CE_POINTER [NATURAL_8]

	last_image: CE_POINTER [NATURAL_8]

	last_form: CE_POINTER [PD_FORM_CONTROL]

	nobreak: INTEGER_32 assign set_nobreak

	set_nobreak (a_nobreak: INTEGER_32)
		do
			nobreak := a_nobreak
		end

	convert_table: CE_POINTER [PD_CONV_TABLE]

	buffer: CE_ARRAY [INTEGER_8]

	ta_cache: PD_TEXT_ATTRIB_BEGINNING

	bg_cache: INTEGER_32 assign set_bg_cache

	set_bg_cache (a_bg_cache: INTEGER_32)
		do
			bg_cache := a_bg_cache
		end

	fg_cache: INTEGER_32 assign set_fg_cache

	set_fg_cache (a_fg_cache: INTEGER_32)
		do
			fg_cache := a_fg_cache
		end

	g_ctrl_num: INTEGER_32 assign set_g_ctrl_num

	set_g_ctrl_num (a_g_ctrl_num: INTEGER_32)
		do
			g_ctrl_num := a_g_ctrl_num
		end

	table_cache: PD_LIST_HEAD

	table_cache_hash: CE_ARRAY [CE_POINTER [PD_TABLE_CACHE_ENTRY]]

	format_cache: PD_LIST_HEAD

	format_cache_entries: INTEGER_32 assign set_format_cache_entries

	set_format_cache_entries (a_format_cache_entries: INTEGER_32)
		do
			format_cache_entries := a_format_cache_entries
		end

feature {ANY} -- routines

	color_distance (a_c1: CE_POINTER [PD_RGB]; a_c2: CE_POINTER [PD_RGB]): INTEGER_32
		do
			Result := (((3 * ((a_c1.item.r).to_integer_32 - (a_c2.item.r).to_integer_32)) * ((a_c1.item.r).to_integer_32 - (a_c2.item.r).to_integer_32) + (4 * ((a_c1.item.g).to_integer_32 - (a_c2.item.g).to_integer_32)) * ((a_c1.item.g).to_integer_32 - (a_c2.item.g).to_integer_32)) + (2 * ((a_c1.item.b).to_integer_32 - (a_c2.item.b).to_integer_32)) * ((a_c1.item.b).to_integer_32 - (a_c2.item.b).to_integer_32))
		end

	find_nearest_color (a_r: CE_POINTER [PD_RGB]; a_l: INTEGER_32): INTEGER_32
		local
			l_dist: INTEGER_32
			l_dst: INTEGER_32
			l_min: INTEGER_32
			l_i: INTEGER_32
			l_h: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not cache_init.to_boolean then
						l_pos := -1 + 2 -- goto initialize
					end
				when 1 then -- back:
					l_h := (((((((a_r.item.r).to_integer_32).bit_shift_left (3)) + (((a_r.item.g).to_integer_32).bit_shift_left (2))) + (a_r.item.b).to_integer_32) + a_l)).bit_and (4095)
					if rgb_cache[l_h].color /= -1 then
						if rgb_cache[l_h].l = a_l then
							if (rgb_cache[l_h].rgb.r).to_integer_32 = (a_r.item.r).to_integer_32 then
								if (rgb_cache[l_h].rgb.g).to_integer_32 = (a_r.item.g).to_integer_32 then
									if (rgb_cache[l_h].rgb.b).to_integer_32 = (a_r.item.b).to_integer_32 then
										Result := (rgb_cache[l_h].color)
										js.return
									end
								end
							end
						end
					end
					if not js.return_called then
						l_dist := 16777215
						l_min := 0
						l_i := 0
						from
						until
							not (l_i < a_l)
						loop
							l_dst := color_distance (a_r, palette + l_i)
							if l_dst < l_dist then
								l_dist := l_dst
								l_min := l_i
							end
							l_i := l_i + 1
						end
						rgb_cache[l_h].color := l_min
						rgb_cache[l_h].l := a_l
						rgb_cache[l_h].rgb.r := a_r.item.r
						rgb_cache[l_h].rgb.g := a_r.item.g
						rgb_cache[l_h].rgb.b := a_r.item.b
						Result := l_min
						js.return
					end
					if js.return_called then
						l_pos := -1 + 3 -- skip if block
					end
				when 2 then -- if: | initialize:
						l_h := 0
						from
						until
							not (l_h < 4096)
						loop
							rgb_cache[l_h].color := -1
							l_h := l_h + 1
						end
						cache_init := 1
						l_pos := -1 + 1 -- goto back
				when 3 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	fg_color (a_fg: INTEGER_32; a_bg: INTEGER_32): INTEGER_32
		local
			l_l: INTEGER_32
			l_tmp: INTEGER_32
			l_h: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_bg < a_fg then
						l_tmp := a_bg
					else
						l_tmp := a_fg
					end
					l_l := l_tmp
					if a_bg < a_fg then
						l_tmp___0 := a_fg
					else
						l_tmp___0 := a_bg
					end
					l_h := l_tmp___0
					if l_l /= l_h then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 23 -- goto _L___2
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 3 then -- else:
						if l_l.to_boolean then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_h = 4 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 8 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 12 then
							l_pos := -1 + 23 -- goto _L___2
						else
							l_pos := -1 + 6 -- goto _L___7
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 6 then -- else: | _L___7:
						if l_l /= 1 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_h = 3 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 5 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 8 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 12 then
							l_pos := -1 + 23 -- goto _L___2
						else
							l_pos := -1 + 9 -- goto _L___6
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 9 then -- else: | _L___6:
						if l_l /= 2 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_h = 6 then
							l_pos := -1 + 23 -- goto _L___2
						else
							l_pos := -1 + 12 -- goto _L___5
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 12 then -- else: | _L___5:
						if l_l /= 3 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_h = 5 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 12 then
							l_pos := -1 + 23 -- goto _L___2
						else
							l_pos := -1 + 15 -- goto _L___4
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 15 then -- else: | _L___4:
						if l_l /= 4 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if l_h = 8 then
							l_pos := -1 + 23 -- goto _L___2
						elseif l_h = 12 then
							l_pos := -1 + 23 -- goto _L___2
						else
							l_pos := -1 + 18 -- goto _L___3
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 18 then -- else: | _L___3:
						if l_l /= 5 then
							l_pos := -1 + 31 -- skip if block
						end
				when 19 then -- if:
						if l_h /= 8 then
							l_pos := -1 + 22 -- skip if block
						end
				when 20 then -- if:
						l_pos := -1 + 23 -- goto _L___2
				when 21 then -- if block done, skip else block:
						l_pos := -1 + 31
				when 22 then -- else:
						if l_h /= 12 then
							l_pos := -1 + 31 -- skip if block
						end
				when 23 then -- if: | _L___2:
						if a_fg /= 4 then
							l_pos := -1 + 26 -- skip if block
						end
				when 24 then -- if:
						l_pos := -1 + 27 -- goto _L___0
				when 25 then -- if block done, skip else block:
						l_pos := -1 + 30
				when 26 then -- else:
						if a_fg /= 12 then
							l_pos := -1 + 29 -- skip if block
						end
				when 27 then -- if: | _L___0:
						if a_bg = 0 then
							l_tmp___2 := 6
						elseif a_bg = 8 then
							l_tmp___2 := 6
						else
							l_pos := -1 + 29 -- goto _L
						end
				when 28 then -- if block done, skip else block:
						l_pos := -1 + 30
				when 29 then -- else: | _L:
						if a_bg = 2 then
							l_tmp___1 := 1
						elseif a_bg = 6 then
							l_tmp___1 := 1
						elseif a_bg = 7 then
							l_tmp___1 := 1
						else
							l_tmp___1 := 0
						end
						l_tmp___2 := 7 - 7 * l_tmp___1
				when 30 then -- if done
						Result := l_tmp___2
						js.return
				when 31 then -- if done | if done | if done | if done | if done | if done | if done | if done
						if not js.return_called then
							Result := a_fg
						end
				when 32 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	xpand_lines (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32)
		local
			l_y: INTEGER_32
			l_i: INTEGER_32
			js: JUMP_STATE
		do
			l_y := a_y
			if (a_p.item.data).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				if ((((l_y.to_natural_32 + (a_p.item.yp).to_natural_32) + {NATURAL_32} 127)).bit_and ({NATURAL_32} 4294967168)) > {NATURAL_32} 2147483647 then
					from
					until
						False
					loop
						error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 134])
						kbd_inst.fatal_tty_exit
						exit (4)
					end
				end
				l_y := l_y + (a_p.item.yp)
				if l_y >= (a_p.item.data).item.y then
					if ((((l_y + 1) + 127)).bit_and (-128)) > ((((a_p.item.data).item.y + 127)).bit_and (-128)) then
						if ((((l_y.to_natural_32 + {NATURAL_32} 1) + {NATURAL_32} 127)).bit_and ({NATURAL_32} 4294967168)) > {NATURAL_32} 2147483647 // {PD_LINE}.sizeof.to_natural_32 then
							from
							until
								False
							loop
								error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 139])
								kbd_inst.fatal_tty_exit
								exit (4)
							end
						end
						(a_p.item.data).item.data := create {CE_POINTER [PD_LINE]}.make_cast ((error_inst.debug_mem_realloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 140, create {CE_POINTER [ANY]}.make_cast ((a_p.item.data).item.data), ((((l_y + 1) + 127)).bit_and (-128)).to_natural_32 * {PD_LINE}.sizeof.to_natural_32)))
					end
					l_i := (a_p.item.data).item.y
					from
					until
						not (l_i <= l_y)
					loop
						((a_p.item.data).item.data + l_i).item.l := 0;
						((a_p.item.data).item.data + l_i).item.c := (a_p.item.bgcolor).to_natural_16;
						((a_p.item.data).item.data + l_i).item.d := create {CE_POINTER [NATURAL_16]}.make_shared (default_pointer + -1)
						l_i := l_i + 1
					end
					(a_p.item.data).item.y := l_i
				end
			end
		end

	xpand_line (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32; a_x: INTEGER_32)
		local
			l_y: INTEGER_32
			l_x: INTEGER_32
			l_i: INTEGER_32
			js: JUMP_STATE
		do
			l_y := a_y
			l_x := a_x
			if (a_p.item.data).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				if ((((l_x.to_natural_32 + (a_p.item.xp).to_natural_32) + {NATURAL_32} 127)).bit_and ({NATURAL_32} 4294967168)) > {NATURAL_32} 2147483647 then
					from
					until
						False
					loop
						error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 154])
						kbd_inst.fatal_tty_exit
						exit (4)
					end
				end
				l_x := l_x + (a_p.item.xp)
				l_y := l_y + (a_p.item.yp)
				if l_y >= (a_p.item.data).item.y then
					from
					until
						False
					loop
						links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
						links_inst.errline := 159
						error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("line does not exist"))])
					end
					js.return
				end
			end
			if not js.return_called and then (l_x >= ((a_p.item.data).item.data + l_y).item.l) then
				if ((((l_x + 1) + 127)).bit_and (-128)) > (((((a_p.item.data).item.data + l_y).item.l + 127)).bit_and (-128)) then
					if ((((l_x.to_natural_32 + {NATURAL_32} 1) + {NATURAL_32} 127)).bit_and ({NATURAL_32} 4294967168)) > {NATURAL_32} 2147483647 // Natural_16_bytes.to_natural_32 then
						from
						until
							False
						loop
							error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 166])
							kbd_inst.fatal_tty_exit
							exit (4)
						end
					end
					((a_p.item.data).item.data + l_y).item.d := create {CE_POINTER [NATURAL_16]}.make_cast ((error_inst.debug_mem_realloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 167, create {CE_POINTER [ANY]}.make_cast (((a_p.item.data).item.data + l_y).item.d), ((((l_x + 1) + 127)).bit_and (-128)).to_natural_32 * Natural_16_bytes.to_natural_32)))
				end
				l_i := ((a_p.item.data).item.data + l_y).item.l
				from
				until
					not (l_i <= l_x)
				loop
					((a_p.item.data).item.data + l_y).item.d[l_i] := (((((((a_p.item.data).item.data + l_y).item.c).to_integer_32).bit_shift_left (11))).bit_or (32)).to_natural_16
					l_i := l_i + 1
				end
				((a_p.item.data).item.data + l_y).item.c := (a_p.item.bgcolor).to_natural_16;
				((a_p.item.data).item.data + l_y).item.l := l_i
			end
		end

	r_xpand_spaces (a_p: CE_POINTER [PD_PART]; a_l: INTEGER_32)
		local
			l_c: CE_POINTER [NATURAL_8]
		do
			create l_c.make

			if a_l.to_natural_32 >= {NATURAL_32} 2147483647 then
				from
				until
					False
				loop
					error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 179])
					kbd_inst.fatal_tty_exit
					exit (4)
				end
			end
			l_c.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast ((error_inst.debug_mem_realloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 180, create {CE_POINTER [ANY]}.make_cast (a_p.item.spaces), (a_l + 1).to_natural_32))))
			memset (create {CE_POINTER [ANY]}.make_cast ((l_c + a_p.item.spl)), 0, ((a_l - a_p.item.spl) + 1).to_natural_32).do_nothing
			a_p.item.spl := a_l + 1
			a_p.item.spaces := l_c
		end

	xpand_spaces (a_p: CE_POINTER [PD_PART]; a_l: INTEGER_32)
		do
			if a_l.to_natural_32 >= (a_p.item.spl).to_natural_32 then
				r_xpand_spaces (a_p, a_l)
			end
		end

	set_hchar (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_c: NATURAL_32)
		do
			xpand_lines (a_p, a_y)
			xpand_line (a_p, a_y, a_x);
			((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d[(a_p.item.xp + a_x)] := a_c.to_natural_16
		end

	set_hchars (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_xl: INTEGER_32; a_c: NATURAL_32)
		local
			l_x: INTEGER_32
			l_xl: INTEGER_32
		do
			l_x := a_x
			l_xl := a_xl
			xpand_lines (a_p, a_y)
			xpand_line (a_p, a_y, (l_x + l_xl) - 1)
			from
			until
				not (l_xl.to_boolean)
			loop
				((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d[(a_p.item.xp + l_x)] := a_c.to_natural_16
				l_xl := l_xl - 1
				l_x := l_x + 1
			end
		end

	xset_hchar (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_c: NATURAL_32)
		do
			set_hchar (a_p, a_x, a_y, a_c)
		end

	xset_hchars (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_xl: INTEGER_32; a_c: NATURAL_32)
		do
			set_hchars (a_p, a_x, a_y, a_xl, a_c)
		end

	xxpand_lines (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32)
		do
			xpand_lines (a_p, a_y)
		end

	xxpand_line (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32; a_x: INTEGER_32)
		do
			xpand_line (a_p, a_y, a_x)
		end

	set_hline (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_xl: INTEGER_32; a_d: CE_POINTER [NATURAL_8]; a_c: NATURAL_32; a_spc: INTEGER_32)
		local
			l_x: INTEGER_32
			l_xl: INTEGER_32
			l_d: CE_POINTER [NATURAL_8]
		do
			create l_d.make

			l_x := a_x
			l_xl := a_xl
			l_d.ce_gets (a_d)
			xpand_lines (a_p, a_y)
			xpand_line (a_p, a_y, (l_x + l_xl) - 1)
			if a_spc.to_boolean then
				xpand_spaces (a_p, (l_x + l_xl) - 1)
			end
			from
			until
				not (l_xl.to_boolean)
			loop
				if a_spc.to_boolean then
					a_p.item.spaces[l_x] := ((l_d.item).to_integer_32 = 32).to_integer.to_natural_8
				end
				if ((a_p.item.data).c_pointer /= default_pointer) then
					((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d[(a_p.item.xp + l_x)] := (((l_d.item).to_natural_32).bit_or (a_c)).to_natural_16
				end
				l_xl := l_xl - 1
				l_x := l_x + 1
				l_d.ce_gets (l_d + 1)
			end
		end

	move_links (a_p: CE_POINTER [PD_PART]; a_xf: INTEGER_32; a_yf: INTEGER_32; a_xt___1: INTEGER_32; a_yt: INTEGER_32)
		local
			l_n: INTEGER_32
			l_t: CE_POINTER [PD_TAG]
			l_w: INTEGER_32
			l_i: INTEGER_32
			l_link___0: CE_POINTER [PD_LINK]
			js: JUMP_STATE
		do
			create l_t.make
			create l_link___0.make

			l_w := 0
			if (a_p.item.data).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				xpand_lines (a_p, a_yt)
				l_n := links_inst.last_link_to_move
				from
				until
					not (not js.loop_left and then (l_n < (a_p.item.data).item.nlinks))
				loop
					l_link___0.ce_gets ((a_p.item.data).item.links + l_n)
					l_i := 0
					from
					until
						not (l_i < l_link___0.item.n)
					loop
						if (l_link___0.item.pos + l_i).item.y >= a_p.item.yp + a_yf then
							l_w := 1
							if (l_link___0.item.pos + l_i).item.y = a_p.item.yp + a_yf then
								if (l_link___0.item.pos + l_i).item.x >= a_p.item.xp + a_xf then
									if a_yt >= 0 then
										(l_link___0.item.pos + l_i).item.y := a_p.item.yp + a_yt;
										(l_link___0.item.pos + l_i).item.x := (l_link___0.item.pos + l_i).item.x + (-a_xf + a_xt___1)
									else
										memmove (create {CE_POINTER [ANY]}.make_cast ((l_link___0.item.pos + l_i)), create {CE_POINTER [ANY]}.make_cast ((l_link___0.item.pos + (l_i + 1))), ((l_link___0.item.n - l_i) - 1).to_natural_32 * {PD_POINT}.sizeof.to_natural_32).do_nothing
										l_link___0.item.n := (l_link___0.item.n) - 1
										l_i := l_i - 1
									end
								end
							end
						end
						l_i := l_i + 1
					end
					if not l_w.to_boolean then
						links_inst.last_link_to_move := l_n
					end
					l_n := l_n + 1
				end
				l_w := 0
				if a_yt >= 0 then
					l_t.ce_gets (last_tag_to_move.item.next)
					from
					until
						(l_t.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.tags_offset)).c_pointer.to_integer_32).to_natural_32
					loop
						if l_t.item.y = a_p.item.yp + a_yf then
							l_w := 1
							if l_t.item.x >= a_p.item.xp + a_xf then
								l_t.item.y := a_p.item.yp + a_yt
								l_t.item.x := l_t.item.x + (-a_xf + a_xt___1)
							end
						end
						if not l_w.to_boolean then
							last_tag_to_move.ce_gets (l_t)
						end
						l_t.ce_gets (l_t.item.next)
					end
				end
			end
		end

	copy_chars (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_xl: INTEGER_32; a_d: CE_POINTER [NATURAL_16])
		local
			l_x: INTEGER_32
			l_xl: INTEGER_32
			l_d: CE_POINTER [NATURAL_16]
			js: JUMP_STATE
		do
			create l_d.make

			l_x := a_x
			l_xl := a_xl
			l_d.ce_gets (a_d)
			if l_xl <= 0 then
				js.return
			end
			if not js.return_called then
				xpand_lines (a_p, a_y)
				xpand_line (a_p, a_y, (l_x + l_xl) - 1)
				from
				until
					not (l_xl.to_boolean)
				loop
					((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d[(a_p.item.xp + l_x)] := l_d.item
					l_xl := l_xl - 1
					l_x := l_x + 1
					l_d.ce_gets (l_d + 1)
				end
			end
		end

	move_chars (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32; a_nx: INTEGER_32; a_ny: INTEGER_32)
		local
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			js: JUMP_STATE
		do
			if ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp < 0 then
				l_tmp := 0
			else
				l_tmp := ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp
			end
			if l_tmp - a_x <= 0 then
				js.return
			end
			if not js.return_called then
				if ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp < 0 then
					l_tmp___0 := 0
				else
					l_tmp___0 := ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp
				end
				copy_chars (a_p, a_nx, a_ny, l_tmp___0 - a_x, ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d + (a_p.item.xp + a_x));
				((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l := a_p.item.xp + a_x
				move_links (a_p, a_x, a_y, a_nx, a_ny)
			end
		end

	shift_chars (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32; a_s: INTEGER_32)
		local
			l_a: CE_POINTER [NATURAL_16]
			l_l: INTEGER_32
			l_tmp: INTEGER_32
		do
			create l_a.make

			if ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp < 0 then
				l_tmp := 0
			else
				l_tmp := ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp
			end
			l_l := l_tmp
			if l_l.to_natural_32 > {NATURAL_32} 2147483647 // Natural_16_bytes.to_natural_32 then
				from
				until
					False
				loop
					error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 308])
					kbd_inst.fatal_tty_exit
					exit (4)
				end
			end
			l_a.ce_gets (create {CE_POINTER [NATURAL_16]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 309, l_l.to_natural_32 * Natural_16_bytes.to_natural_32))))
			memcpy (create {CE_POINTER [ANY]}.make_cast (l_a), create {CE_POINTER [ANY]}.make_cast ((((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.d + a_p.item.xp)), l_l.to_natural_32 * Natural_16_bytes.to_natural_32).do_nothing
			set_hchars (a_p, 0, a_y, a_s, (((((((a_p.item.data).item.data + a_y).item.c).to_integer_32).bit_shift_left (11))).bit_or (32)).to_natural_32)
			copy_chars (a_p, a_s, a_y, l_l, l_a)
			error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 313, create {CE_POINTER [ANY]}.make_cast (l_a))
			move_links (a_p, 0, a_y, a_s, a_y)
		end

	del_chars (a_p: CE_POINTER [PD_PART]; a_x: INTEGER_32; a_y: INTEGER_32)
		do
			((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l := a_p.item.xp + a_x
			move_links (a_p, a_x, a_y, -1, -1)
		end

	split_line (a_p: CE_POINTER [PD_PART]): INTEGER_32
		local
			l_i: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > 0 then
						l_i := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
					else
						l_i := 0
					end
				when 1 then -- loop condition:
					if not (l_i >= (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin) then
						l_pos := -1 + 4 -- skip loop
					end
				when 2 then -- loop body:
						if l_i < a_p.item.spl then
							if (a_p.item.spaces[l_i]).to_boolean then
								l_pos := -1 + 9 -- goto split
							end
						end
				when 3 then
						l_i := l_i - 1
						l_pos := -1 + 1 -- back to loop condition
				when 4 then -- loop done
					l_i := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin
				when 5 then -- loop condition:
					if not (l_i < a_p.item.cx) then
						l_pos := -1 + 8 -- skip loop
					end
				when 6 then -- loop body:
						if l_i < a_p.item.spl then
							if (a_p.item.spaces[l_i]).to_boolean then
								l_pos := -1 + 9 -- goto split
							end
						end
				when 7 then
						l_i := l_i + 1
						l_pos := -1 + 5 -- back to loop condition
				when 8 then -- loop done
					if a_p.item.cx + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > a_p.item.x then
						a_p.item.x := a_p.item.cx + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
					end
					Result := 0
					js.return
					if js.return_called then
						l_pos := -1 + 10 -- skip if block
					end
				when 9 then -- if: | split:
						if l_i + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > a_p.item.x then
							a_p.item.x := l_i + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
						end
						if ((a_p.item.data).c_pointer /= default_pointer) then
							if (((((a_p.item.data).item.data + (a_p.item.yp + a_p.item.cy)).item.d[(a_p.item.xp + l_i)]).to_integer_32).bit_and (255)) /= 32 then
								from
								until
									False
								loop
									links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
									links_inst.errline := 352
									error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("bad split: %%c")), ((((a_p.item.data).item.data + (a_p.item.yp + a_p.item.cy)).item.d[(a_p.item.xp + l_i)]).to_integer_8).to_integer_32])
								end
							end
							move_chars (a_p, l_i + 1, a_p.item.cy, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin, a_p.item.cy + 1)
							del_chars (a_p, l_i, a_p.item.cy)
						end
						memmove (create {CE_POINTER [ANY]}.make_cast (a_p.item.spaces), create {CE_POINTER [ANY]}.make_cast (((a_p.item.spaces + l_i) + 1)), ((a_p.item.spl - l_i) - 1).to_natural_32).do_nothing
						memset (create {CE_POINTER [ANY]}.make_cast ((((a_p.item.spaces + a_p.item.spl) - l_i) - 1)), 0, (l_i + 1).to_natural_32).do_nothing
						memmove (create {CE_POINTER [ANY]}.make_cast ((a_p.item.spaces + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin)), create {CE_POINTER [ANY]}.make_cast (a_p.item.spaces), (a_p.item.spl - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin).to_natural_32).do_nothing
						a_p.item.cy := (a_p.item.cy) + 1
						a_p.item.cx := a_p.item.cx - ((l_i - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin) + 1)
						if a_p.item.cx = (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin then
							a_p.item.cx := -1
						end
						if a_p.item.y < a_p.item.cy + (a_p.item.cx /= -1).to_integer then
							a_p.item.y := a_p.item.cy + (a_p.item.cx /= -1).to_integer
						end
						Result := (1 + (a_p.item.cx = -1).to_integer)
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	align_line (a_p: CE_POINTER [PD_PART]; a_y: INTEGER_32)
		local
			l_na: INTEGER_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			js: JUMP_STATE
		do
			if (a_p.item.data).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				if ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp < 0 then
					l_tmp := 0
				else
					l_tmp := ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp
				end
				if l_tmp.to_boolean then
					if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align = 0 then
						js.return
					elseif (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align = 4 then
						js.return
					elseif (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align = 3 then
						js.return
					end
				else
					js.return
				end
			end
			if not js.return_called then
				if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > 0 then
					l_tmp___0 := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
				else
					l_tmp___0 := 0
				end
				if ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp < 0 then
					l_tmp___1 := 0
				else
					l_tmp___1 := ((a_p.item.data).item.data + (a_p.item.yp + a_y)).item.l - a_p.item.xp
				end
				l_na := l_tmp___0 - l_tmp___1
				if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align = 1 then
					l_na := l_na // (2)
				end
				if l_na > 0 then
					shift_chars (a_p, a_y, l_na)
				end
			end
		end

	new_link (a_f: CE_POINTER [PD_F_DATA]): CE_POINTER [PD_LINK]
		local
			l_tmp___0: INTEGER_32
			js: JUMP_STATE
		do
			if (a_f).c_pointer = default_pointer then
				Result := (create {CE_POINTER [PD_LINK]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				if not ((a_f.item.nlinks).bit_and (63)).to_boolean then
					if (a_f.item.nlinks).to_natural_32 > {NATURAL_32} 2147483647 // {PD_LINK}.sizeof.to_natural_32 - {NATURAL_32} 64 then
						from
						until
							False
						loop
							error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 381])
							kbd_inst.fatal_tty_exit
							exit (4)
						end
					end
					a_f.item.links := create {CE_POINTER [PD_LINK]}.make_cast ((error_inst.debug_mem_realloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 382, create {CE_POINTER [ANY]}.make_cast (a_f.item.links), (a_f.item.nlinks + 64).to_natural_32 * {PD_LINK}.sizeof.to_natural_32)))
				end
				memset (create {CE_POINTER [ANY]}.make_cast ((a_f.item.links + a_f.item.nlinks)), 0, {PD_LINK}.sizeof.to_natural_32).do_nothing
				l_tmp___0 := a_f.item.nlinks
				a_f.item.nlinks := (a_f.item.nlinks) + 1
				Result := (a_f.item.links + l_tmp___0)
			end
		end

	html_tag (a_f: CE_POINTER [PD_F_DATA]; a_t: CE_POINTER [NATURAL_8]; a_x: INTEGER_32; a_y: INTEGER_32)
		local
			l_tag: CE_POINTER [PD_TAG]
			l_tt: CE_POINTER [NATURAL_8]
			l_ll: INTEGER_32
			js: JUMP_STATE
		do
			create l_tag.make
			create l_tt.make

			if (a_f).c_pointer = default_pointer then
				js.return
			end
			if not js.return_called then
				l_tt.ce_gets (links_inst.init_str_x (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 394))
				l_ll := 0
				url_inst.add_conv_str (l_tt.address, create {CE_POINTER [INTEGER_32]}.make_shared ($l_ll), a_t, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (a_t))).to_integer_32, -2)
				l_tag.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 397, ({PD_TAG}.sizeof.to_natural_32 + (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_tt)))) + {NATURAL_32} 1))))
				l_tag.item.x := a_x
				l_tag.item.y := a_y
				strcpy (create {CE_POINTER [INTEGER_8]}.make_cast ((l_tag.item.name)), create {CE_POINTER [INTEGER_8]}.make_cast (l_tt)).do_nothing
				from
				until
					js.loop_left
				loop
					js.reset
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset))))
					l_tag.item.next := (create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset)))).item.next
					l_tag.item.prev := create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset)));
					(create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset)))).item.next := l_tag;
					(l_tag.item.next).item.prev := l_tag
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset))))
					js.break
				end
				js.reset
				if (last_tag_for_newline.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.tags_offset)).c_pointer.to_integer_32).to_natural_32 then
					last_tag_for_newline.ce_gets (l_tag)
				end
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 403, create {CE_POINTER [ANY]}.make_cast (l_tt))
			end
		end

	put_chars_conv (a_p: CE_POINTER [PD_PART]; a_c: CE_POINTER [NATURAL_8]; a_l: INTEGER_32): INTEGER_32
		local
			l_bp: INTEGER_32
			l_pp: INTEGER_32
			l_total: INTEGER_32
			l_e: CE_POINTER [NATURAL_8]
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_t: CE_POINTER [PD_CONV_TABLE]
			l_i: INTEGER_32
			l_tmp___1: INTEGER_32
			l_i___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_tmp___5: CE_POINTER [NATURAL_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_e.make
			create l_t.make
			create l_tmp___5.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_bp := 0
					l_pp := 0
					l_total := 0
					if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr).bit_and (16)).to_boolean then
						put_chars (a_p, a_c, a_l)
						Result := a_l
						js.return
					end
					if js.return_called then
						l_pos := -1 + 31 -- skip if block
					end
				when 1 then -- if:
						if not a_l.to_boolean then
							put_chars (a_p, create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer), 0)
						end
				when 2 then -- loop condition:
						if not (not js.loop_left and then (l_pp < a_l)) then
							l_pos := -1 + 30 -- skip loop
						end
				when 3 then -- loop body:
						js.reset
						if not ((a_c[l_pp]).to_integer_32 < 128) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if (a_c[l_pp]).to_integer_32 = 38 then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | putc:
						l_tmp := l_bp
						l_bp := l_bp + 1
						l_tmp___0 := l_pp
						l_pp := l_pp + 1
						buffer[l_tmp] := (a_c[l_tmp___0]).to_integer_8
						if l_bp < 256 then
							js.continue
						end
						if not js.loop_jumped then
							l_pos := -1 + 23 -- goto flush
						end
				when 6 then -- if done
						if js.loop_jumped then
							l_pos := -1 + 21 -- skip if block
						end
				when 7 then -- if:
						if (a_c[l_pp]).to_integer_32 = 38 then
							l_pos := -1 + 17 -- skip if block
						end
				when 8 then -- if:
						if (convert_table).c_pointer = default_pointer then
							l_pos := -1 + 5 -- goto putc
						end
				when 9 then
						l_t.ce_gets (convert_table)
						l_i := l_pp
				when 10 then -- decode:
						if ((l_t + (a_c[l_i]).to_integer_32).item.t).to_boolean then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_e.ce_gets ((l_t + (a_c[l_i]).to_integer_32).item.u.str)
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 13 then -- else:
						l_tmp___1 := l_i
						l_i := l_i + 1
						l_t.ce_gets ((l_t + (a_c[l_tmp___1]).to_integer_32).item.u.tbl)
						if l_i >= a_l then
							l_pos := -1 + 5 -- goto putc
						end
				when 14 then
						l_pos := -1 + 10 -- goto decode
				when 15 then
						l_pp := l_i + 1
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 17 then -- else:
						l_i___0 := l_pp + 1
						if (links_inst.d_opt.item.plain).to_boolean then
							l_pos := -1 + 5 -- goto putc
						end
				when 18 then
						from
						until
							js.loop_left
						loop
							js.reset
							if l_i___0 < a_l then
								if (a_c[l_i___0]).to_integer_32 /= 59 then
									if (a_c[l_i___0]).to_integer_32 /= 38 then
										if not ((a_c[l_i___0]).to_integer_32 > 32) then
											js.break
										end
									else
										js.break
									end
								else
									js.break
								end
							else
								js.break
							end
							if not js.loop_jumped then
								l_i___0 := l_i___0 + 1
							end
						end
						js.reset
						l_e.ce_gets (charsets_inst.get_entity_string (a_c + (l_pp + 1), (l_i___0 - l_pp) - 1, links_inst.d_opt.item.cp))
						if (l_e).c_pointer = default_pointer then
							l_pos := -1 + 5 -- goto putc
						end
				when 19 then
						if l_i___0 < a_l then
							if (a_c[l_i___0]).to_integer_32 = 59 then
								l_tmp___2 := 1
							else
								l_tmp___2 := 0
							end
						else
							l_tmp___2 := 0
						end
						l_pp := l_i___0 + l_tmp___2
				when 20 then -- if done
						if not (l_e[0]).to_boolean then
							js.continue
						end
				when 21 then -- if done
						if not (not js.loop_jumped and then (not (l_e[1]).to_boolean)) then
							l_pos := -1 + 24 -- skip if block
						end
				when 22 then -- if:
						l_tmp___3 := l_bp
						l_bp := l_bp + 1
						buffer[l_tmp___3] := (l_e[0]).to_integer_8
						if l_bp < 256 then
							js.continue
						end
						if js.loop_jumped then
							l_pos := -1 + 24 -- skip if block
						end
				when 23 then -- if: | flush:
						l_e.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("")))
						l_pos := -1 + 27 -- goto flush1
				when 24 then -- if done
						if js.loop_jumped then
							l_pos := -1 + 29 -- skip if block
						end
				when 25 then -- if: | loop condition:
						if not ((l_e.item).to_boolean) then
							l_pos := -1 + 29 -- skip loop
						end
				when 26 then -- loop body:
						js.reset
						l_tmp___4 := l_bp
						l_bp := l_bp + 1
						l_tmp___5.ce_gets (l_e)
						l_e.ce_gets (l_e + 1)
						buffer[l_tmp___4] := (l_tmp___5.item).to_integer_8
						if l_bp < 256 then
							js.continue
						end
						if js.loop_jumped then
							l_pos := -1 + 28 -- skip if block
						end
				when 27 then -- if: | flush1:
						put_chars (a_p, create {CE_POINTER [NATURAL_8]}.make_cast (buffer), l_bp)
						l_total := l_total + l_bp
						l_bp := 0
				when 28 then -- if done
						l_pos := -1 + 25 -- back to loop condition
				when 29 then -- loop done | if done
						l_pos := -1 + 2 -- back to loop condition
				when 30 then -- loop done
						if l_bp.to_boolean then
							put_chars (a_p, create {CE_POINTER [NATURAL_8]}.make_cast (buffer), l_bp)
						end
						l_total := l_total + l_bp
						Result := l_total
				when 31 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	put_chars_conv_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent put_chars_conv)
		end

	put_chars (a_p: CE_POINTER [PD_PART]; a_c: CE_POINTER [NATURAL_8]; a_l: INTEGER_32)
		local
			l_c: CE_POINTER [NATURAL_8]
			l_l: INTEGER_32
			l_bg: INTEGER_32
			l_fg: INTEGER_32
			l_i: INTEGER_32
			l_link___0: CE_POINTER [PD_LINK]
			l_pt: CE_POINTER [PD_POINT]
			l_tmp___0: INTEGER_32
			l_x: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_s: CE_ARRAY [NATURAL_8]
			l_fl: CE_POINTER [NATURAL_8]
			l_ft: CE_POINTER [NATURAL_8]
			l_fi: CE_POINTER [NATURAL_8]
			l_ff___0: CE_POINTER [PD_FORM_CONTROL]
			l_tmp___5: CE_POINTER [NATURAL_8]
			l_tmp___6: CE_POINTER [NATURAL_8]
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_c.make
			create l_link___0.make
			create l_pt.make
			create l_s.make (<<64>>)
			create l_fl.make
			create l_ft.make
			create l_fi.make
			create l_ff___0.make
			create l_tmp___5.make
			create l_tmp___6.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_c.ce_gets (a_c)
					l_l := a_l
					if l_l < 0 then
						from
						until
							False
						loop
							error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 492])
							kbd_inst.fatal_tty_exit
							exit (4)
						end
					end
					from
					until
						js.loop_left
					loop
						js.reset
						if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align /= 4 then
							if a_p.item.cx = -1 then
								if l_l.to_boolean then
									if (l_c.item).to_integer_32 /= 32 then
										js.break
									end
								else
									js.break
								end
							else
								js.break
							end
						else
							js.break
						end
						if not js.loop_jumped then
							l_c.ce_gets (l_c + 1)
							l_l := l_l - 1
						end
					end
					js.reset
					if not l_l.to_boolean then
						js.return
					end
					if js.return_called then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if (l_c[0]).to_integer_32 /= 32 then
							last_tag_for_newline.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.tags_offset))))
						elseif (l_c[1]).to_boolean then
							if (l_c[1]).to_integer_32 /= 32 then
								last_tag_for_newline.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.tags_offset))))
							end
						end
						if a_p.item.cx < (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin then
							a_p.item.cx := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin
						end
						if ((last_link).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						elseif ((last_image).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						elseif ((last_form).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						elseif (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						elseif (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						elseif (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- goto process_link
						end
				when 2 then -- no_l:
						if (memcmp (create {CE_POINTER [ANY]}.make_cast ((ta_cache.address)), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_TEXT_ATTRIB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset))), {PD_TEXT_ATTRIB_BEGINNING}.sizeof.to_natural_32)).to_boolean then
							l_pos := -1 + 36 -- goto format_change
						end
				when 3 then
						l_bg := bg_cache
						l_fg := fg_cache
				when 4 then -- end_format_change:
						if a_p.item.cx = (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin then
							if (l_c.item).to_integer_32 = 32 then
								if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align /= 4 then
									l_c.ce_gets (l_c + 1)
									l_l := l_l - 1
								end
							end
						end
						if a_p.item.y < a_p.item.cy + 1 then
							a_p.item.y := a_p.item.cy + 1
						end
						if nowrap.to_boolean then
							if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > 0 then
								l_tmp___0 := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
							else
								l_tmp___0 := 0
							end
							if a_p.item.cx + l_l > l_tmp___0 then
								js.return
							end
						end
				when 5 then -- if done
					if not js.return_called then
						set_hline (a_p, a_p.item.cx, a_p.item.cy, l_l, l_c, ((((((((((l_fg).bit_and (8))).bit_shift_left (3))).bit_or (((l_bg).bit_shift_left (3))))).bit_or (((l_fg).bit_and (7))))).bit_shift_left (8)).to_natural_32, 1)
						a_p.item.cx := a_p.item.cx + l_l
						nobreak := 0
						if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align /= 4 then
							from
							until
								js.loop_left
							loop
								js.reset
								if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > 0 then
									l_tmp___1 := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width - (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
								else
									l_tmp___1 := 0
								end
								if a_p.item.cx > l_tmp___1 then
									if not (a_p.item.cx > (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin) then
										js.break
									end
								else
									js.break
								end
								if not js.loop_jumped then
									l_x := split_line (a_p)
									if not l_x.to_boolean then
										js.break
									end
								end
								if not js.loop_jumped then
									align_line (a_p, a_p.item.cy - 1)
									nobreak := l_x - 1
								end
							end
							js.reset
						end
						l_tmp___3 := a_p.item.xa + l_l
						a_p.item.xa := l_tmp___3
						if (l_c[(l_l - 1)]).to_integer_32 = 32 then
							if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align /= 4 then
								l_tmp___4 := 1
							else
								l_tmp___4 := 0
							end
						else
							l_tmp___4 := 0
						end
						if ((l_tmp___3 - l_tmp___4) + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin) + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > a_p.item.xmax then
							if (l_c[(l_l - 1)]).to_integer_32 = 32 then
								if (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align /= 4 then
									l_tmp___2 := 1
								else
									l_tmp___2 := 0
								end
							else
								l_tmp___2 := 0
							end
							a_p.item.xmax := ((a_p.item.xa - l_tmp___2) + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin) + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
						end
						js.return
					end
					if js.return_called then
						l_pos := -1 + 37 -- skip if block
					end
				when 6 then -- if: | process_link:
						if (last_link).c_pointer = default_pointer then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 13 -- goto _L___3
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 35
				when 9 then -- else:
						if (last_image).c_pointer = default_pointer then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L___3
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 35
				when 12 then -- else:
						if (last_form).c_pointer = default_pointer then
							l_pos := -1 + 30 -- skip if block
						end
				when 13 then -- if: | _L___3:
						if not ((links_inst.xstrcmp ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link, last_link)).to_boolean) then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						l_pos := -1 + 30 -- goto _L___2
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 29
				when 16 then -- else:
						if not ((links_inst.xstrcmp ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target, last_target)).to_boolean) then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if:
						l_pos := -1 + 30 -- goto _L___2
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 29
				when 19 then -- else:
						if not ((links_inst.xstrcmp ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image, last_image)).to_boolean) then
							l_pos := -1 + 22 -- skip if block
						end
				when 20 then -- if:
						l_pos := -1 + 30 -- goto _L___2
				when 21 then -- if block done, skip else block:
						l_pos := -1 + 29
				when 22 then -- else:
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form).c_pointer.to_integer_32).to_natural_32 /= (last_form.c_pointer.to_integer_32).to_natural_32 then
							l_pos := -1 + 28 -- skip if block
						end
				when 23 then -- if:
						if (a_p.item.data).c_pointer = default_pointer then
							l_pos := -1 + 26 -- goto x
						end
				when 24 then
						l_link___0.ce_gets ((a_p.item.data).item.links + ((a_p.item.data).item.nlinks - 1))
						if not ((a_p.item.data).item.nlinks).to_boolean then
							from
							until
								False
							loop
								links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
								links_inst.errline := 533
								error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("no link"))])
							end
							l_pos := -1 + 2 -- goto no_l
						end
				when 25 then
						l_pos := -1 + 34 -- goto set_link
				when 26 then -- x:
						
				when 27 then -- if block done, skip else block:
						l_pos := -1 + 29
				when 28 then -- else:
						l_pos := -1 + 30 -- goto _L___2
				when 29 then -- if done | if done | if done | if done | if block done, skip else block:
						l_pos := -1 + 35
				when 30 then -- else: | _L___2:
						if ((last_link).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 539, create {CE_POINTER [ANY]}.make_cast (last_link))
						end
						if ((last_target).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 540, create {CE_POINTER [ANY]}.make_cast (last_target))
						end
						if ((last_image).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 541, create {CE_POINTER [ANY]}.make_cast (last_image))
						end
						last_image.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
						last_target.ce_gets (last_image)
						last_link.ce_gets (last_target)
						last_form.ce_gets (create {CE_POINTER [PD_FORM_CONTROL]}.make_shared (default_pointer))
						if ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link).c_pointer = default_pointer then
							if ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image).c_pointer = default_pointer then
								if ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form).c_pointer = default_pointer then
									l_pos := -1 + 2 -- goto no_l
								end
							end
						end
				when 31 then
						if (links_inst.d_opt.item.num_links).to_boolean then
							l_fl.ce_gets ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link)
							l_ft.ce_gets ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target)
							l_fi.ce_gets ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image)
							l_ff___0.ce_gets ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form)
							l_tmp___6.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer));
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image := l_tmp___6
							l_tmp___5.ce_gets (l_tmp___6);
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target := l_tmp___5;
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link := l_tmp___5;
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form := create {CE_POINTER [PD_FORM_CONTROL]}.make_shared (default_pointer)
							l_s[0] := (('[').code).to_natural_8
							links_inst.snzprint (l_s + 1, 62, (a_p.item.link_num).to_integer_32).do_nothing
							strcat (create {CE_POINTER [INTEGER_8]}.make_cast (l_s), ce_string ("]")).do_nothing
							put_chars (a_p, l_s, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_s))).to_integer_32)
							if ((l_ff___0).c_pointer /= default_pointer) then
								if l_ff___0.item.type = 4 then
									line_break (a_p)
								end
							end
							if a_p.item.cx = -1 then
								a_p.item.cx := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin
							end
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link := l_fl;
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target := l_ft;
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image := l_fi;
							(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form := l_ff___0
						end
						a_p.item.link_num := (a_p.item.link_num) + 1
						last_link.ce_gets (links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 561, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link))
						last_target.ce_gets (links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 562, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target))
						last_image.ce_gets (links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 563, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image))
						last_form.ce_gets ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form)
						if (a_p.item.data).c_pointer = default_pointer then
							l_pos := -1 + 2 -- goto no_l
						end
				when 32 then
						l_link___0.ce_gets (new_link (a_p.item.data))
						if (l_link___0).c_pointer = default_pointer then
							l_pos := -1 + 2 -- goto no_l
						end
				when 33 then
						l_link___0.item.num := a_p.item.link_num - 1
						l_link___0.item.pos := create {CE_POINTER [PD_POINT]}.make_shared (default_pointer + -1)
						if (last_form).c_pointer = default_pointer then
							l_link___0.item.type := 0
							l_link___0.item.where := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 571, last_link)
							l_link___0.item.target := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 572, last_target)
						else
							if last_form.item.type = 1 then
								l_link___0.item.type := 4
							elseif last_form.item.type = 2 then
								l_link___0.item.type := 4
							elseif last_form.item.type = 3 then
								l_link___0.item.type := 4
							else
								if last_form.item.type = 4 then
									l_tmp___10 := 5
								else
									if last_form.item.type = 5 then
										l_tmp___9 := 2
									elseif last_form.item.type = 6 then
										l_tmp___9 := 2
									else
										if last_form.item.type = 7 then
											l_tmp___8 := 3
										else
											l_tmp___8 := 1
										end
										l_tmp___9 := l_tmp___8
									end
									l_tmp___10 := l_tmp___9
								end
								l_link___0.item.type := l_tmp___10
							end
							l_link___0.item.form := last_form
							l_link___0.item.target := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 576, last_form.item.target)
						end
						l_link___0.item.where_img := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 578, last_image)
						if l_link___0.item.type /= 4 then
							if l_link___0.item.type /= 5 then
								l_bg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.clink_offset), 8)
								l_fg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.bg_offset), 8)
								l_fg := fg_color (l_fg, l_bg)
							else
								l_fg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.fg_offset), 8)
								l_bg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.bg_offset), 8)
								l_fg := fg_color (l_fg, l_bg)
							end
						else
							l_fg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.fg_offset), 8)
							l_bg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.bg_offset), 8)
							l_fg := fg_color (l_fg, l_bg)
						end
						l_link___0.item.sel_color := ((((((((l_fg).bit_and (8))).bit_shift_left (3))).bit_or (((l_fg).bit_and (7))))).bit_or (((l_bg).bit_shift_left (3)))).to_natural_32
						l_link___0.item.n := 0
				when 34 then -- set_link:
						if (l_link___0.item.n).to_natural_32 + l_l.to_natural_32 > {NATURAL_32} 2147483647 // {PD_POINT}.sizeof.to_natural_32 then
							from
							until
								False
							loop
								error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 591])
								kbd_inst.fatal_tty_exit
								exit (4)
							end
						end
						l_pt.ce_gets (create {CE_POINTER [PD_POINT]}.make_cast ((error_inst.debug_mem_realloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 592, create {CE_POINTER [ANY]}.make_cast (l_link___0.item.pos), (l_link___0.item.n + l_l).to_natural_32 * {PD_POINT}.sizeof.to_natural_32))))
						l_link___0.item.pos := l_pt
						l_i := 0
						from
						until
							not (l_i < l_l)
						loop
							(l_pt + (l_link___0.item.n + l_i)).item.x := (a_p.item.xp + a_p.item.cx) + l_i;
							(l_pt + (l_link___0.item.n + l_i)).item.y := a_p.item.yp + a_p.item.cy
							l_i := l_i + 1
						end
						l_link___0.item.n := l_link___0.item.n + l_l
				when 35 then -- if done | if done
						l_pos := -1 + 2 -- goto no_l
				when 36 then -- format_change:
						l_bg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.bg_offset), 8)
						l_fg := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.fg_offset), 16)
						l_fg := fg_color (l_fg, l_bg)
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr).bit_and (2)).to_boolean then
							l_fg := (l_fg).bit_xor ((1))
						end
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr).bit_and (4)).to_boolean then
							l_fg := (((l_fg).bit_xor (4))).bit_or (8)
						end
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr).bit_and (1)).to_boolean then
							l_fg := (l_fg).bit_or ((8))
						end
						l_fg := fg_color (l_fg, l_bg)
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr).bit_and (16)).to_boolean then
							l_bg := (l_bg).bit_or ((16))
						end
						memcpy (create {CE_POINTER [ANY]}.make_cast ((ta_cache.address)), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_TEXT_ATTRIB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset))), {PD_TEXT_ATTRIB_BEGINNING}.sizeof.to_natural_32).do_nothing
						fg_cache := l_fg
						bg_cache := l_bg
						l_pos := -1 + 4 -- goto end_format_change
				when 37 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	line_break (a_p: CE_POINTER [PD_PART])
		local
			l_t: CE_POINTER [PD_TAG]
			l_tmp: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_t.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_p.item.cx + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin > a_p.item.x then
						a_p.item.x := a_p.item.cx + (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin
					end
					if nobreak.to_boolean then
						nobreak := 0
						a_p.item.cx := -1
						a_p.item.xa := 0
						js.return
					end
					if js.return_called then
						l_pos := -1 + 4 -- skip if block
					end
				when 1 then -- if:
						if (a_p.item.data).c_pointer = default_pointer then
							l_pos := -1 + 3 -- goto e
						end
				when 2 then
						xpand_lines (a_p, a_p.item.cy + 1)
						if a_p.item.cx > (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin then
							if ((a_p.item.data).item.data + (a_p.item.yp + a_p.item.cy)).item.l - a_p.item.xp < 0 then
								l_tmp := 0
							else
								l_tmp := ((a_p.item.data).item.data + (a_p.item.yp + a_p.item.cy)).item.l - a_p.item.xp
							end
							if l_tmp > a_p.item.cx - 1 then
								if (((((a_p.item.data).item.data + (a_p.item.yp + a_p.item.cy)).item.d[(a_p.item.xp + (a_p.item.cx - 1))]).to_integer_32).bit_and (255)) = 32 then
									del_chars (a_p, a_p.item.cx - 1, a_p.item.cy)
									a_p.item.cx := (a_p.item.cx) - 1
								end
							end
						end
						if a_p.item.cx > 0 then
							align_line (a_p, a_p.item.cy)
						end
						if ((a_p.item.data).c_pointer /= default_pointer) then
							l_t.ce_gets (last_tag_for_newline)
							from
							until
								js.loop_left
							loop
								js.reset
								if ((l_t).c_pointer /= default_pointer) then
									if (l_t.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.tags_offset)).c_pointer.to_integer_32).to_natural_32 then
										js.break
									end
								else
									js.break
								end
								if not js.loop_jumped then
									l_t.item.x := a_p.item.xp
									l_t.item.y := a_p.item.yp + (a_p.item.cy + 1)
									l_t.ce_gets (l_t.item.prev)
								end
							end
							js.reset
						end
				when 3 then -- e:
						a_p.item.cy := (a_p.item.cy) + 1
						a_p.item.cx := -1
						a_p.item.xa := 0
						memset (create {CE_POINTER [ANY]}.make_cast (a_p.item.spaces), 0, (a_p.item.spl).to_natural_32).do_nothing
				when 4 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	line_break_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent line_break)
		end

	html_form_control (a_p: CE_POINTER [PD_PART]; a_fc: CE_POINTER [PD_FORM_CONTROL])
		local
			l_tmp: INTEGER_32
			l_dv: CE_POINTER [NATURAL_8]
			l_p___0: CE_POINTER [NATURAL_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_dv.make
			create l_p___0.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if (a_p.item.data).c_pointer = default_pointer then
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_p.c_pointer + {PD_PART}.uf_offset))))
							a_fc.item.next := (create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_p.c_pointer + {PD_PART}.uf_offset)))).item.next
							a_fc.item.prev := create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_p.c_pointer + {PD_PART}.uf_offset)));
							(create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_p.c_pointer + {PD_PART}.uf_offset)))).item.next := a_fc;
							(a_fc.item.next).item.prev := a_fc
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_p.c_pointer + {PD_PART}.uf_offset))))
							js.break
						end
						js.reset
						js.return
					end
					if js.return_called then
						l_pos := -1 + 10 -- skip if block
					end
				when 1 then -- if:
						l_tmp := g_ctrl_num
						g_ctrl_num := g_ctrl_num + 1
						a_fc.item.g_ctrl_num := l_tmp
						if a_fc.item.type /= 1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 4 then -- else:
						if a_fc.item.type /= 2 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 7 then -- else:
						if a_fc.item.type /= 4 then
							l_pos := -1 + 9 -- skip if block
						end
				when 8 then -- if: | _L:
						l_dv.ce_gets ((charsets_inst.convert_string (convert_table, a_fc.item.default_value, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (a_fc.item.default_value))).to_integer_32)))
						if ((l_dv).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 655, create {CE_POINTER [ANY]}.make_cast (a_fc.item.default_value))
							a_fc.item.default_value := l_dv
						end
				when 9 then -- if done | if done
						if a_fc.item.type = 4 then
							l_p___0.ce_gets (a_fc.item.default_value)
							from
							until
								not ((l_p___0[0]).to_boolean)
							loop
								if (l_p___0[0]).to_integer_32 = 13 then
									if (l_p___0[1]).to_integer_32 = 10 then
										memmove (create {CE_POINTER [ANY]}.make_cast (l_p___0), create {CE_POINTER [ANY]}.make_cast ((l_p___0 + 1)), (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_p___0)))).do_nothing
										l_p___0.ce_gets (l_p___0 - 1)
									else
										l_p___0[0] := (('%N').code).to_natural_8
									end
								end
								l_p___0.ce_gets (l_p___0 + 1)
							end
						end
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.forms_offset))))
							a_fc.item.next := (create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.forms_offset)))).item.next
							a_fc.item.prev := create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.forms_offset)));
							(create {CE_POINTER [PD_FORM_CONTROL]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.forms_offset)))).item.next := a_fc;
							(a_fc.item.next).item.prev := a_fc
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared ((a_p.item.data).c_pointer + {PD_F_DATA}.forms_offset))))
							js.break
						end
						js.reset
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	add_frameset_entry (a_fsd: CE_POINTER [PD_FRAMESET_DESC]; a_subframe: CE_POINTER [PD_FRAMESET_DESC]; a_name: CE_POINTER [NATURAL_8]; a_url: CE_POINTER [NATURAL_8])
		local
			js: JUMP_STATE
		do
			if a_fsd.item.yp >= a_fsd.item.y then
				js.return
			end
			if not js.return_called then
				a_fsd.item.f[a_fsd.item.xp + a_fsd.item.yp * a_fsd.item.x].subframe := a_subframe
				a_fsd.item.f[a_fsd.item.xp + a_fsd.item.yp * a_fsd.item.x].name := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 679, a_name)
				a_fsd.item.f[a_fsd.item.xp + a_fsd.item.yp * a_fsd.item.x].url := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 680, a_url)
				a_fsd.item.xp := (a_fsd.item.xp) + 1
				if a_fsd.item.xp >= a_fsd.item.x then
					a_fsd.item.xp := 0
					a_fsd.item.yp := (a_fsd.item.yp) + 1
				end
			end
		end

	create_frameset (a_fda: CE_POINTER [PD_F_DATA]; a_fp: CE_POINTER [PD_FRAMESET_PARAM]): CE_POINTER [PD_FRAMESET_DESC]
		local
			l_i: INTEGER_32
			l_fd: CE_POINTER [PD_FRAMESET_DESC]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_fd.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if (a_fp.item.x).to_boolean then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if (a_fp.item.y).to_boolean then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 689
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("zero size of frameset"))])
						end
						Result := (create {CE_POINTER [PD_FRAMESET_DESC]}.make_shared (default_pointer))
						js.return
				when 5 then -- if done
						if not js.return_called then
							if (a_fp.item.x).to_boolean then
								if ((a_fp.item.x).to_natural_32 * (a_fp.item.y).to_natural_32) // (a_fp.item.x).to_natural_32 /= (a_fp.item.y).to_natural_32 then
									from
									until
										False
									loop
										error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 692])
										kbd_inst.fatal_tty_exit
										exit (4)
									end
								end
							end
							if (a_fp.item.x).to_natural_32 * (a_fp.item.y).to_natural_32 > ({NATURAL_32} 2147483647 - {PD_FRAMESET_DESC}.sizeof.to_natural_32) // {PD_FRAME_DESC}.sizeof.to_natural_32 then
								from
								until
									False
								loop
									error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 693])
									kbd_inst.fatal_tty_exit
									exit (4)
								end
							end
							l_fd.ce_gets (create {CE_POINTER [PD_FRAMESET_DESC]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 694, {PD_FRAMESET_DESC}.sizeof.to_natural_32 + (a_fp.item.x * a_fp.item.y).to_natural_32 * {PD_FRAME_DESC}.sizeof.to_natural_32))))
							memset (create {CE_POINTER [ANY]}.make_cast (l_fd), 0, {PD_FRAMESET_DESC}.sizeof.to_natural_32 + (a_fp.item.x * a_fp.item.y).to_natural_32 * {PD_FRAME_DESC}.sizeof.to_natural_32).do_nothing
							l_fd.item.n := a_fp.item.x * a_fp.item.y
							l_fd.item.x := a_fp.item.x
							l_fd.item.y := a_fp.item.y
							l_i := 0
							from
							until
								not (l_i < l_fd.item.n)
							loop
								l_fd.item.f[l_i].xw := a_fp.item.xw[l_i \\ a_fp.item.x]
								l_fd.item.f[l_i].yw := a_fp.item.yw[l_i // a_fp.item.x]
								l_i := l_i + 1
							end
							if ((a_fp.item.parent).c_pointer /= default_pointer) then
								add_frameset_entry (a_fp.item.parent, l_fd, create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer), create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
							elseif (a_fda.item.frame_desc).c_pointer = default_pointer then
								a_fda.item.frame_desc := l_fd
							else
								error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 705, create {CE_POINTER [ANY]}.make_cast (l_fd))
								l_fd.ce_gets (create {CE_POINTER [PD_FRAMESET_DESC]}.make_shared (default_pointer))
							end
							Result := l_fd
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	create_frame (a_fp: CE_POINTER [PD_FRAME_PARAM])
		do
			add_frameset_entry (a_fp.item.parent, create {CE_POINTER [PD_FRAMESET_DESC]}.make_shared (default_pointer), a_fp.item.name, a_fp.item.url)
		end

	html_special (a_args: TUPLE [a_p: CE_POINTER [PD_PART]; a_c: INTEGER_32]): CE_POINTER [ANY]
		local
			l_l: CE_VA_LIST
			l_t: CE_POINTER [NATURAL_8]
			l_fc: CE_POINTER [PD_FORM_CONTROL]
			l_fsp: CE_POINTER [PD_FRAMESET_PARAM]
			l_fp: CE_POINTER [PD_FRAME_PARAM]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_t.make
			create l_fc.make
			create l_fsp.make
			create l_fp.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					create l_l.make (a_args, 3)
					inspect a_args.a_c -- switch 1 head
					when 0 then l_pos := -1 + 1
					when 1 then l_pos := -1 + 2
					when 2 then l_pos := -1 + 3
					when 3 then l_pos := -1 + 4
					when 4 then l_pos := -1 + 5
					when 5 then l_pos := -1 + 6
					when 6 then l_pos := -1 + 7
					else l_pos := -1 + 8
					end
				when 1 then -- switch 1, case 0
					l_t.ce_gets ((create {CE_POINTER [NATURAL_8]}.make_cast (l_l.pointer_item)))
					html_tag (a_args.a_p.item.data, l_t, a_args.a_p.item.xp + a_args.a_p.item.cx, a_args.a_p.item.yp + a_args.a_p.item.cy)
					l_pos := -1 + 9 -- switch 1; break
				when 2 then -- switch 1, case 1
					l_fc.ce_gets ((create {CE_POINTER [PD_FORM_CONTROL]}.make_cast (l_l.pointer_item)))
					html_form_control (a_args.a_p, l_fc)
					l_pos := -1 + 9 -- switch 1; break
				when 3 then -- switch 1, case 2
					Result := (create {CE_POINTER [ANY]}.make_cast (convert_table))
					js.return
				when 4 then -- switch 1, case 3
					if not js.return_called then
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer + (((a_args.a_p.item.data).c_pointer /= default_pointer)).to_integer))
						js.return
					end
				when 5 then -- switch 1, case 4
					if not js.return_called then
						l_fsp.ce_gets ((create {CE_POINTER [PD_FRAMESET_PARAM]}.make_cast (l_l.pointer_item)))
						Result := (create {CE_POINTER [ANY]}.make_cast ((create_frameset (a_args.a_p.item.data, l_fsp))))
						js.return
					end
				when 6 then -- switch 1, case 5
					if not js.return_called then
						l_fp.ce_gets ((create {CE_POINTER [PD_FRAME_PARAM]}.make_cast (l_l.pointer_item)))
						create_frame (l_fp)
					end
					l_pos := -1 + 9 -- switch 1; break
				when 7 then -- switch 1, case 6
					nowrap := l_l.integer_32_item
					l_pos := -1 + 9 -- switch 1; break
				when 8 then -- switch 1, default:
					from
					until
						False
					loop
						links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
						links_inst.errline := 753
						error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_special: unknown code %%d")), a_args.a_c])
					end
				when 9 then -- switch 1, done:
					if not js.return_called then
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
					end
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	html_special_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent html_special)
		end

	do_format (a_start: CE_POINTER [INTEGER_8]; a_end: CE_POINTER [INTEGER_8]; a_part: CE_POINTER [PD_PART]; a_head___0: CE_POINTER [NATURAL_8])
		do
			html_inst.parse_html (create {CE_POINTER [NATURAL_8]}.make_cast (a_start), create {CE_POINTER [NATURAL_8]}.make_cast (a_end), (put_chars_conv_address), (line_break_address), (html_special_address), create {CE_POINTER [ANY]}.make_cast (a_part), a_head___0)
		end

	free_table_cache
		local
			l_tce: CE_POINTER [PD_TABLE_CACHE_ENTRY]
			l_hash___0: INTEGER_32
			l_a__: CE_POINTER [PD_XLIST_HEAD]
		do
			create l_tce.make
			create l_a__.make

			l_tce.ce_gets (create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_cast (table_cache.next))
			from
			until
				(l_tce.c_pointer.to_integer_32).to_natural_32 = ((table_cache.address).c_pointer.to_integer_32).to_natural_32
			loop
				l_hash___0 := ((((l_tce.item.start).c_pointer.to_integer_32).to_integer_32 + l_tce.item.xs)).bit_and (4095)
				table_cache_hash[l_hash___0] := create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_shared (default_pointer)
				l_tce.ce_gets (l_tce.item.next)
			end
			error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((table_cache.address)))
			l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (table_cache.next))
			from
			until
				(l_a__.c_pointer.to_integer_32).to_natural_32 = ((table_cache.address).c_pointer.to_integer_32).to_natural_32
			loop
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
				l_a__.ce_gets (l_a__.item.next)
			end
			l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (table_cache.prev))
			from
			until
				(l_a__.c_pointer.to_integer_32).to_natural_32 = ((table_cache.address).c_pointer.to_integer_32).to_natural_32
			loop
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
				l_a__.ce_gets (l_a__.item.prev)
			end
			from
			until
				((table_cache.next).c_pointer.to_integer_32).to_natural_32 = ((table_cache.address).c_pointer.to_integer_32).to_natural_32
			loop
				l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (table_cache.next))
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__));
				(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_a__.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_a__.item.prev);
				(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_a__.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_a__.item.next)
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 794, create {CE_POINTER [ANY]}.make_cast (l_a__))
			end
			error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((table_cache.address)))
		end

	format_html_part (a_start: CE_POINTER [NATURAL_8]; a_end: CE_POINTER [NATURAL_8]; a_align: INTEGER_32; a_m: INTEGER_32; a_width: INTEGER_32; a_data: CE_POINTER [PD_F_DATA]; a_xs: INTEGER_32; a_ys: INTEGER_32; a_head___0: CE_POINTER [NATURAL_8]; a_link_num: INTEGER_32): CE_POINTER [PD_PART]
		local
			l_p: CE_POINTER [PD_PART]
			l_e: CE_POINTER [PD_HTML_ELEMENT]
			l_llm: INTEGER_32
			l_ltm: CE_POINTER [PD_TAG]
			l_lm: INTEGER_32
			l_ef: INTEGER_32
			l_fc: CE_POINTER [PD_FORM_CONTROL]
			l_tce: CE_POINTER [PD_TABLE_CACHE_ENTRY]
			l_n: CE_POINTER [PD_NODE]
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_n___0: CE_POINTER [PD_NODE]
			l_a__: CE_POINTER [PD_XLIST_HEAD]
			l_hash___0: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_p.make
			create l_e.make
			create l_ltm.make
			create l_fc.make
			create l_tce.make
			create l_n.make
			create l_n___0.make
			create l_a__.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_llm := links_inst.last_link_to_move
					l_ltm.ce_gets (last_tag_to_move)
					l_lm := links_inst.margin
					l_ef := links_inst.empty_format
					if (a_data).c_pointer = default_pointer then
						l_tce.ce_gets (table_cache_hash[(((a_start.c_pointer.to_integer_32).to_integer_32 + a_xs)).bit_and (4095)])
						from
						until
							not (not js.loop_left and then ((l_tce).c_pointer /= default_pointer))
						loop
							if ((l_tce.item.start).c_pointer.to_integer_32).to_natural_32 = (a_start.c_pointer.to_integer_32).to_natural_32 then
								if ((l_tce.item.end2).c_pointer.to_integer_32).to_natural_32 = (a_end.c_pointer.to_integer_32).to_natural_32 then
									if l_tce.item.align = a_align then
										if l_tce.item.m = a_m then
											if l_tce.item.width = a_width then
												if l_tce.item.xs = a_xs then
													if l_tce.item.link_num = a_link_num then
														l_p.ce_gets (create {CE_POINTER [PD_PART]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 812, {PD_PART}.sizeof.to_natural_32))))
														memcpy (create {CE_POINTER [ANY]}.make_cast (l_p), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_PART]}.make_shared (l_tce.c_pointer + {PD_TABLE_CACHE_ENTRY}.p_offset))), {PD_PART}.sizeof.to_natural_32).do_nothing
														Result := l_p
														js.return
													end
												end
											end
										end
									end
								end
							end
							if not js.return_called then
								l_tce.ce_gets (l_tce.item.hash_next)
							end
						end
					end
					if not js.return_called and then (a_ys < 0) then
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 820
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("format_html_part: ys == %%d")), a_ys])
						end
						Result := (create {CE_POINTER [PD_PART]}.make_shared (default_pointer))
						js.return
					end
					if js.return_called then
						l_pos := -1 + 10 -- skip if block
					end
				when 1 then -- if:
						if ((a_data).c_pointer /= default_pointer) then
							l_n.ce_gets (create {CE_POINTER [PD_NODE]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 825, {PD_NODE}.sizeof.to_natural_32))))
							l_n.item.x := a_xs
							l_n.item.y := a_ys
							if not links_inst.table_level.to_boolean then
								l_n.item.xw := 2147483647
							else
								l_n.item.xw := a_width
							end
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.nodes_offset))))
								l_n.item.next := (create {CE_POINTER [PD_NODE]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.nodes_offset)))).item.next
								l_n.item.prev := create {CE_POINTER [PD_NODE]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.nodes_offset)));
								(create {CE_POINTER [PD_NODE]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.nodes_offset)))).item.next := l_n;
								(l_n.item.next).item.prev := l_n
								error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.nodes_offset))))
								js.break
							end
							js.reset
						end
						if ((a_data).c_pointer /= default_pointer) then
							links_inst.last_link_to_move := a_data.item.nlinks
						else
							links_inst.last_link_to_move := 0
						end
						if ((a_data).c_pointer /= default_pointer) then
							last_tag_to_move.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.tags_offset))))
						else
							last_tag_to_move.ce_gets (create {CE_POINTER [PD_TAG]}.make_shared (default_pointer))
						end
						if ((a_data).c_pointer /= default_pointer) then
							last_tag_for_newline.ce_gets (create {CE_POINTER [PD_TAG]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_data.c_pointer + {PD_F_DATA}.tags_offset))))
						else
							last_tag_for_newline.ce_gets (create {CE_POINTER [PD_TAG]}.make_shared (default_pointer))
						end
						links_inst.margin := a_m
						links_inst.empty_format := ((a_data).c_pointer = default_pointer).to_integer
						if ((last_link).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 837, create {CE_POINTER [ANY]}.make_cast (last_link))
						end
						if ((last_image).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 838, create {CE_POINTER [ANY]}.make_cast (last_image))
						end
						if ((last_target).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 839, create {CE_POINTER [ANY]}.make_cast (last_target))
						end
						last_target.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
						last_image.ce_gets (last_target)
						last_link.ce_gets (last_image)
						last_form.ce_gets (create {CE_POINTER [PD_FORM_CONTROL]}.make_shared (default_pointer))
						nobreak := 1
						l_p.ce_gets (create {CE_POINTER [PD_PART]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 843, {PD_PART}.sizeof.to_natural_32))))
						l_tmp___2 := 0
						l_p.item.y := l_tmp___2
						l_p.item.x := l_tmp___2
						l_p.item.data := a_data
						l_p.item.xp := a_xs
						l_p.item.yp := a_ys
						l_tmp___3 := 0
						l_p.item.xa := l_tmp___3
						l_p.item.xmax := l_tmp___3
						l_p.item.bgcolor := find_nearest_color (create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.parattr_offset + {PD_PAR_ATTRIB}.bgcolor_offset), 8)
						l_p.item.spaces := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer + -1)
						l_p.item.spl := 0
						l_p.item.link_num := a_link_num
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset))))
						l_p.item.uf.next := create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)))
						l_p.item.uf.prev := create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)))
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset))))
						html_inst.html_stack_dup
						l_e.ce_gets (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next));
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.dontkill := 2;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.namelen := 0;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align := a_align;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin := a_m;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin := a_m;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width := a_width;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.list_level := 0;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.list_number := {NATURAL_32} 0;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.dd_margin := 0
						l_p.item.cx := -1
						l_p.item.cy := 0
						do_format (create {CE_POINTER [INTEGER_8]}.make_cast (a_start), create {CE_POINTER [INTEGER_8]}.make_cast (a_end), l_p, a_head___0)
						if l_p.item.xmax < l_p.item.x then
							l_p.item.xmax := l_p.item.x
						end
						nobreak := 0
						links_inst.line_breax := 1
						if ((last_link).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 870, create {CE_POINTER [ANY]}.make_cast (last_link))
						end
						if ((last_image).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 871, create {CE_POINTER [ANY]}.make_cast (last_image))
						end
						if ((last_target).c_pointer /= default_pointer) then
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 872, create {CE_POINTER [ANY]}.make_cast (last_target))
						end
				when 2 then -- loop condition:
						if not (not js.loop_left and then (((html_inst.html_stack.next).c_pointer.to_integer_32).to_natural_32 /= (l_e.c_pointer.to_integer_32).to_natural_32)) then
							l_pos := -1 + 9 -- skip loop
						end
				when 3 then -- loop body:
						js.reset
						html_inst.kill_html_stack_item (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next))
						if (((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next))).c_pointer /= default_pointer) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if ((html_inst.html_stack.next).c_pointer.to_integer_32).to_natural_32 /= ((html_inst.html_stack.address).c_pointer.to_integer_32).to_natural_32 then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 876
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html stack trashed"))])
						end
						js.break
				when 8 then -- if done
						l_pos := -1 + 2 -- back to loop condition
				when 9 then -- loop done
						js.reset;
						(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.dontkill := 0
						html_inst.kill_html_stack_item (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next))
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 882, create {CE_POINTER [ANY]}.make_cast (l_p.item.spaces))
						if ((a_data).c_pointer /= default_pointer) then
							l_n___0.ce_gets (create {CE_POINTER [PD_NODE]}.make_cast (a_data.item.nodes.next))
							l_n___0.item.yw := (a_ys - l_n___0.item.y) + l_p.item.y
						end
						l_fc.ce_gets (create {CE_POINTER [PD_FORM_CONTROL]}.make_cast (l_p.item.uf.next))
						from
						until
							(l_fc.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)).c_pointer.to_integer_32).to_natural_32
						loop
							view_inst.destroy_fc (l_fc)
							l_fc.ce_gets (l_fc.item.next)
						end
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset))))
						l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (l_p.item.uf.next))
						from
						until
							(l_a__.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)).c_pointer.to_integer_32).to_natural_32
						loop
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
							l_a__.ce_gets (l_a__.item.next)
						end
						l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (l_p.item.uf.prev))
						from
						until
							(l_a__.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)).c_pointer.to_integer_32).to_natural_32
						loop
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
							l_a__.ce_gets (l_a__.item.prev)
						end
						from
						until
							((l_p.item.uf.next).c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset)).c_pointer.to_integer_32).to_natural_32
						loop
							l_a__.ce_gets (create {CE_POINTER [PD_XLIST_HEAD]}.make_cast (l_p.item.uf.next))
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__));
							(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_a__.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_a__.item.prev);
							(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_a__.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_a__.item.next)
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_a__))
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 888, create {CE_POINTER [ANY]}.make_cast (l_a__))
						end
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (l_p.c_pointer + {PD_PART}.uf_offset))))
						links_inst.last_link_to_move := l_llm
						last_tag_to_move.ce_gets (l_ltm)
						links_inst.margin := l_lm
						links_inst.empty_format := l_ef
						last_target.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
						last_image.ce_gets (last_target)
						last_link.ce_gets (last_image)
						last_form.ce_gets (create {CE_POINTER [PD_FORM_CONTROL]}.make_shared (default_pointer))
						if links_inst.table_level > 1 then
							if (a_data).c_pointer = default_pointer then
								l_tce.ce_gets (create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 898, {PD_TABLE_CACHE_ENTRY}.sizeof.to_natural_32))))
								l_tce.item.start := a_start
								l_tce.item.end2 := a_end
								l_tce.item.align := a_align
								l_tce.item.m := a_m
								l_tce.item.width := a_width
								l_tce.item.xs := a_xs
								l_tce.item.link_num := a_link_num
								memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_PART]}.make_shared (l_tce.c_pointer + {PD_TABLE_CACHE_ENTRY}.p_offset))), create {CE_POINTER [ANY]}.make_cast (l_p), {PD_PART}.sizeof.to_natural_32).do_nothing
								from
								until
									js.loop_left
								loop
									js.reset
									error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((table_cache.address)))
									l_tce.item.next := (create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_cast ((table_cache.address))).item.next
									l_tce.item.prev := create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_cast ((table_cache.address));
									(create {CE_POINTER [PD_TABLE_CACHE_ENTRY]}.make_cast ((table_cache.address))).item.next := l_tce;
									(l_tce.item.next).item.prev := l_tce
									error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((table_cache.address)))
									js.break
								end
								js.reset
								l_hash___0 := (((a_start.c_pointer.to_integer_32).to_integer_32 + a_xs)).bit_and (4095)
								l_tce.item.hash_next := table_cache_hash[l_hash___0]
								table_cache_hash[l_hash___0] := l_tce
							end
						end
						Result := l_p
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	push_base_format (a_url: CE_POINTER [NATURAL_8]; a_opt: CE_POINTER [PD_DOCUMENT_OPTIONS])
		local
			l_e: CE_POINTER [PD_HTML_ELEMENT]
			l_tmp___0: CE_POINTER [NATURAL_8]
			l_tmp___1: CE_POINTER [NATURAL_8]
			l_tmp___2: CE_POINTER [NATURAL_8]
			l_tmp___3: NATURAL_32
			js: JUMP_STATE
		do
			create l_e.make
			create l_tmp___0.make
			create l_tmp___1.make
			create l_tmp___2.make

			if ((html_inst.html_stack.next).c_pointer.to_integer_32).to_natural_32 /= ((html_inst.html_stack.address).c_pointer.to_integer_32).to_natural_32 then
				from
				until
					False
				loop
					links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
					links_inst.errline := 919
					error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("something on html stack"))])
				end
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
				html_inst.html_stack.next := create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address))
				html_inst.html_stack.prev := create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address))
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
			end
			l_e.ce_gets (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 922, {PD_HTML_ELEMENT}.sizeof.to_natural_32))))
			memset (create {CE_POINTER [ANY]}.make_cast (l_e), 0, {PD_HTML_ELEMENT}.sizeof.to_natural_32).do_nothing
			from
			until
				js.loop_left
			loop
				js.reset
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
				l_e.item.next := (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast ((html_inst.html_stack.address))).item.next
				l_e.item.prev := create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast ((html_inst.html_stack.address));
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast ((html_inst.html_stack.address))).item.next := l_e;
				(l_e.item.next).item.prev := l_e
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
				js.break
			end
			js.reset;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.attr := 0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.fontsize := 3
			l_tmp___2.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer));
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.select2 := l_tmp___2
			l_tmp___1.ce_gets (l_tmp___2);
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.image := l_tmp___1
			l_tmp___0.ce_gets (l_tmp___1);
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target := l_tmp___0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.link := l_tmp___0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.form := create {CE_POINTER [PD_FORM_CONTROL]}.make_shared (default_pointer)
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.fg_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_opt.c_pointer + {PD_DOCUMENT_OPTIONS}.default_fg_offset))), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.bg_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_opt.c_pointer + {PD_DOCUMENT_OPTIONS}.default_bg_offset))), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.clink_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_opt.c_pointer + {PD_DOCUMENT_OPTIONS}.default_link_offset))), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.attr_offset + {PD_TEXT_ATTRIB}.vlink_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_opt.c_pointer + {PD_DOCUMENT_OPTIONS}.default_vlink_offset))), {PD_RGB}.sizeof.to_natural_32).do_nothing;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.href_base := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 933, a_url);
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.attr.target_base := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 934, a_opt.item.framename)
			if (a_opt.item.plain).to_boolean then
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align := 4
			else
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align := 0
			end
			if (a_opt.item.plain).to_boolean then
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin := 0
			else
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin := a_opt.item.margin
			end
			if (a_opt.item.plain).to_boolean then
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin := 0
			else
				(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.rightmargin := a_opt.item.margin
			end
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.width := a_opt.item.xw
			l_tmp___3 := {NATURAL_32} 0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.list_number := l_tmp___3;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.list_level := l_tmp___3.to_integer_32;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.dd_margin := a_opt.item.margin;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.flags := 0
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared ((create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).c_pointer + {PD_HTML_ELEMENT}.parattr_offset + {PD_PAR_ATTRIB}.bgcolor_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_opt.c_pointer + {PD_DOCUMENT_OPTIONS}.default_bg_offset))), {PD_RGB}.sizeof.to_natural_32).do_nothing;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.invisible := 0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.name := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer);
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.namelen := 0;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.options := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer);
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.linebreak := 1;
			(create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.dontkill := 1
		end

	get_convert_table (a_head___0: CE_POINTER [NATURAL_8]; a_to: INTEGER_32; a_def: INTEGER_32; a_frm: CE_POINTER [INTEGER_32]; a_aa: CE_POINTER [INTEGER_32]; a_hard: INTEGER_32): CE_POINTER [PD_CONV_TABLE]
		local
			l_from: INTEGER_32
			l_a: CE_POINTER [NATURAL_8]
			l_b: CE_POINTER [NATURAL_8]
			l_p: CE_POINTER [NATURAL_8]
			js: JUMP_STATE
		do
			create l_a.make
			create l_b.make
			create l_p.make

			l_from := -1
			l_p.ce_gets (a_head___0)
			from
			until
				js.loop_left
			loop
				js.reset
				if l_from = -1 then
					if ((l_p).c_pointer /= default_pointer) then
						l_a.ce_gets (http_inst.parse_http_header (l_p, create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("Content-Type")), l_p.address))
						if (l_a).c_pointer = default_pointer then
							js.break
						end
					else
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					l_b.ce_gets (http_inst.parse_header_param (l_a, create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("charset"))))
					if ((l_b).c_pointer /= default_pointer) then
						l_from := charsets_inst.get_cp_index (l_b)
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 957, create {CE_POINTER [ANY]}.make_cast (l_b))
					end
					error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 959, create {CE_POINTER [ANY]}.make_cast (l_a))
				end
			end
			js.reset
			if l_from = -1 then
				if ((a_head___0).c_pointer /= default_pointer) then
					l_a.ce_gets (http_inst.parse_http_header (a_head___0, create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("Content-Charset")), create {CE_POINTER [CE_POINTER [NATURAL_8]]}.make_shared (default_pointer)))
					if ((l_a).c_pointer /= default_pointer) then
						l_from := charsets_inst.get_cp_index (l_a)
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 963, create {CE_POINTER [ANY]}.make_cast (l_a))
					end
				end
			end
			if l_from = -1 then
				if ((a_head___0).c_pointer /= default_pointer) then
					l_a.ce_gets (http_inst.parse_http_header (a_head___0, create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("Charset")), create {CE_POINTER [CE_POINTER [NATURAL_8]]}.make_shared (default_pointer)))
					if ((l_a).c_pointer /= default_pointer) then
						l_from := charsets_inst.get_cp_index (l_a)
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 967, create {CE_POINTER [ANY]}.make_cast (l_a))
					end
				end
			end
			if ((a_aa).c_pointer /= default_pointer) then
				a_aa.item := (l_from = -1).to_integer
				if a_hard.to_boolean then
					if not (a_aa.item).to_boolean then
						a_aa.item := 2
					end
				end
			end
			if a_hard.to_boolean then
				l_from := a_def
			elseif l_from = -1 then
				l_from := a_def
			end
			if ((a_frm).c_pointer /= default_pointer) then
				a_frm.item := l_from
			end
			Result := (charsets_inst.get_translation_table (l_from, a_to))
		end

	format_html (a_ce: CE_POINTER [PD_CACHE_ENTRY]; a_screen: CE_POINTER [PD_F_DATA])
		local
			l_url: CE_POINTER [NATURAL_8]
			l_fr: CE_POINTER [PD_FRAGMENT]
			l_rp: CE_POINTER [PD_PART]
			l_start: CE_POINTER [NATURAL_8]
			l_end: CE_POINTER [NATURAL_8]
			l_head___0: CE_POINTER [NATURAL_8]
			l_t: CE_POINTER [NATURAL_8]
			l_hdl: INTEGER_32
			l_i: INTEGER_32
			js: JUMP_STATE
		do
			create l_url.make
			create l_fr.make
			create l_rp.make
			create l_start.make
			create l_end.make
			create l_head___0.make
			create l_t.make

			l_url.ce_gets (a_ce.item.url)
			memset (create {CE_POINTER [ANY]}.make_cast (table_cache_hash), 0, (Pointer_bytes.to_natural_32 * (4096).to_natural_32)).do_nothing
			links_inst.d_opt.ce_gets (create {CE_POINTER [PD_DOCUMENT_OPTIONS]}.make_shared (a_screen.c_pointer + {PD_F_DATA}.opt_offset))
			a_screen.item.use_tag := a_ce.item.count
			cache_inst.defrag_entry (a_ce)
			l_fr.ce_gets (create {CE_POINTER [PD_FRAGMENT]}.make_cast (a_ce.item.frag.next))
			if (l_fr.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ce.c_pointer + {PD_CACHE_ENTRY}.frag_offset)).c_pointer.to_integer_32).to_natural_32 then
				l_start.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
				l_end.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			elseif (l_fr.item.offset).to_boolean then
				l_start.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
				l_end.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			elseif not (l_fr.item.length).to_boolean then
				l_start.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
				l_end.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			else
				l_start.ce_gets (l_fr.item.data)
				l_end.ce_gets (l_fr.item.data + l_fr.item.length)
			end
			links_inst.startf.ce_gets (l_start)
			links_inst.eofff.ce_gets (l_end)
			l_head___0.ce_gets (links_inst.init_str_x (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 998))
			l_hdl := 0
			if ((a_ce.item.head).c_pointer /= default_pointer) then
				links_inst.add_to_str (l_head___0.address, create {CE_POINTER [INTEGER_32]}.make_shared ($l_hdl), a_ce.item.head)
			end
			html_inst.scan_http_equiv (l_start, l_end, l_head___0.address, create {CE_POINTER [INTEGER_32]}.make_shared ($l_hdl), l_t.address)
			convert_table.ce_gets (get_convert_table (l_head___0, a_screen.item.opt.cp, a_screen.item.opt.assume_cp, create {CE_POINTER [INTEGER_32]}.make_shared (a_screen.c_pointer + {PD_F_DATA}.cp_offset), create {CE_POINTER [INTEGER_32]}.make_shared (a_screen.c_pointer + {PD_F_DATA}.ass_offset), a_screen.item.opt.hard_assume))
			a_screen.item.opt.real_cp := a_screen.item.cp
			l_i := links_inst.d_opt.item.plain
			links_inst.d_opt.item.plain := 0
			a_screen.item.title := charsets_inst.convert_string (convert_table, l_t, (strlen (create {CE_POINTER [INTEGER_8]}.make_cast (l_t))).to_integer_32)
			links_inst.d_opt.item.plain := l_i
			error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1006, create {CE_POINTER [ANY]}.make_cast (l_t))
			push_base_format (l_url, create {CE_POINTER [PD_DOCUMENT_OPTIONS]}.make_shared (a_screen.c_pointer + {PD_F_DATA}.opt_offset))
			links_inst.table_level := 0
			g_ctrl_num := 0
			links_inst.last_form_tag.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			links_inst.last_form_attr.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			links_inst.last_input_tag.ce_gets (create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer))
			l_rp.ce_gets (format_html_part (l_start, l_end, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.align, (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next)).item.parattr.leftmargin, a_screen.item.opt.xw, a_screen, 0, 0, l_head___0, 1))
			if ((l_rp).c_pointer /= default_pointer) then
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1013, create {CE_POINTER [ANY]}.make_cast (l_rp))
			end
			error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1014, create {CE_POINTER [ANY]}.make_cast (l_head___0))
			a_screen.item.x := 0
			l_i := a_screen.item.y - 1
			from
			until
				not (not js.loop_left and then (l_i >= 0))
			loop
				js.reset
				if not ((a_screen.item.data + l_i).item.l).to_boolean then
					error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1017, create {CE_POINTER [ANY]}.make_cast ((a_screen.item.data + l_i).item.d))
					a_screen.item.y := (a_screen.item.y) - 1
				else
					js.break
				end
				if not js.loop_jumped then
					l_i := l_i - 1
				end
			end
			js.reset
			l_i := 0
			from
			until
				not (l_i < a_screen.item.y)
			loop
				if (a_screen.item.data + l_i).item.l > a_screen.item.x then
					a_screen.item.x := (a_screen.item.data + l_i).item.l
				end
				l_i := l_i + 1
			end
			if ((html_inst.form.action).c_pointer /= default_pointer) then
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1021, create {CE_POINTER [ANY]}.make_cast (html_inst.form.action))
				html_inst.form.action := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer)
			end
			if ((html_inst.form.target).c_pointer /= default_pointer) then
				error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1022, create {CE_POINTER [ANY]}.make_cast (html_inst.form.target))
				html_inst.form.target := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer)
			end
			html_inst.kill_html_stack_item (create {CE_POINTER [PD_HTML_ELEMENT]}.make_cast (html_inst.html_stack.next))
			if ((html_inst.html_stack.next).c_pointer.to_integer_32).to_natural_32 /= ((html_inst.html_stack.address).c_pointer.to_integer_32).to_natural_32 then
				from
				until
					False
				loop
					links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
					links_inst.errline := 1025
					error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html stack not empty after operation"))])
				end
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
				html_inst.html_stack.next := create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address))
				html_inst.html_stack.prev := create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address))
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((html_inst.html_stack.address)))
			end
			a_screen.item.bg := (7).bit_shift_left (8)
			view_inst.sort_links (a_screen)
			if ((a_screen.item.frame_desc).c_pointer /= default_pointer) then
				a_screen.item.frame := 1
			end
		end

	compare_opt (a_o1: CE_POINTER [PD_DOCUMENT_OPTIONS]; a_o2: CE_POINTER [PD_DOCUMENT_OPTIONS]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_o1.item.xw = a_o2.item.xw then
				if a_o1.item.yw = a_o2.item.yw then
					if a_o1.item.xp = a_o2.item.xp then
						if a_o1.item.yp = a_o2.item.yp then
							if a_o1.item.col = a_o2.item.col then
								if a_o1.item.cp = a_o2.item.cp then
									if a_o1.item.assume_cp = a_o2.item.assume_cp then
										if a_o1.item.hard_assume = a_o2.item.hard_assume then
											if a_o1.item.tables = a_o2.item.tables then
												if a_o1.item.frames = a_o2.item.frames then
													if a_o1.item.images = a_o2.item.images then
														if a_o1.item.margin = a_o2.item.margin then
															if a_o1.item.plain = a_o2.item.plain then
																if not (memcmp (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o1.c_pointer + {PD_DOCUMENT_OPTIONS}.default_fg_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o2.c_pointer + {PD_DOCUMENT_OPTIONS}.default_fg_offset))), {PD_RGB}.sizeof.to_natural_32)).to_boolean then
																	if not (memcmp (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o1.c_pointer + {PD_DOCUMENT_OPTIONS}.default_bg_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o2.c_pointer + {PD_DOCUMENT_OPTIONS}.default_bg_offset))), {PD_RGB}.sizeof.to_natural_32)).to_boolean then
																		if not (memcmp (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o1.c_pointer + {PD_DOCUMENT_OPTIONS}.default_link_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o2.c_pointer + {PD_DOCUMENT_OPTIONS}.default_link_offset))), {PD_RGB}.sizeof.to_natural_32)).to_boolean then
																			if not (memcmp (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o1.c_pointer + {PD_DOCUMENT_OPTIONS}.default_vlink_offset))), create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (a_o2.c_pointer + {PD_DOCUMENT_OPTIONS}.default_vlink_offset))), {PD_RGB}.sizeof.to_natural_32)).to_boolean then
																				if a_o1.item.num_links = a_o2.item.num_links then
																					if a_o1.item.table_order = a_o2.item.table_order then
																						if not (strcasecmp (create {CE_POINTER [INTEGER_8]}.make_cast (a_o1.item.framename), create {CE_POINTER [INTEGER_8]}.make_cast (a_o2.item.framename))).to_boolean then
																							Result := 0
																							js.return
																						end
																					end
																				end
																			end
																		end
																	end
																end
															end
														end
													end
												end
											end
										end
									end
								end
							end
						end
					end
				end
			end
			if not js.return_called then
				Result := 1
			end
		end

	copy_opt (a_o1: CE_POINTER [PD_DOCUMENT_OPTIONS]; a_o2: CE_POINTER [PD_DOCUMENT_OPTIONS])
		do
			memcpy (create {CE_POINTER [ANY]}.make_cast (a_o1), create {CE_POINTER [ANY]}.make_cast (a_o2), {PD_DOCUMENT_OPTIONS}.sizeof.to_natural_32).do_nothing
			a_o1.item.framename := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1072, a_o2.item.framename)
		end

	shrink_format_cache (a_u: INTEGER_32)
		local
			l_ce: CE_POINTER [PD_F_DATA]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ce.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					delete_unused_format_cache_entries
					if format_cache_entries < 0 then
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 1083
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("format_cache_entries underflow"))])
						end
						format_cache_entries := 0
					end
					l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.prev))
				when 1 then -- loop condition:
					if js.loop_left then
						l_pos := -1 + 10 -- skip loop
					end
				when 2 then -- loop body:
						js.reset
						if not (a_u.to_boolean) then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						l_pos := -1 + 6 -- goto _L
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 5 then -- else:
						if not (format_cache_entries > default_inst.max_format_cache_entries) then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L:
						if (l_ce.c_pointer.to_integer_32).to_natural_32 = ((format_cache.address).c_pointer.to_integer_32).to_natural_32 then
							js.break
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						js.break
				when 9 then -- if done
						if not js.loop_jumped and then ((l_ce.item.refcount).to_boolean) then
							l_ce.ce_gets (l_ce.item.prev)
							js.continue
						end
						if not js.loop_jumped then
							l_ce.ce_gets (l_ce.item.prev)
							view_inst.destroy_formatted (l_ce.item.next)
							format_cache_entries := format_cache_entries - 1
						end
						l_pos := -1 + 1 -- back to loop condition
				when 10 then -- loop done
					js.reset
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	count_format_cache
		local
			l_ce: CE_POINTER [PD_F_DATA]
		do
			create l_ce.make

			format_cache_entries := 0
			l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.next))
			from
			until
				(l_ce.c_pointer.to_integer_32).to_natural_32 = ((format_cache.address).c_pointer.to_integer_32).to_natural_32
			loop
				if not (l_ce.item.refcount).to_boolean then
					format_cache_entries := format_cache_entries + 1
				end
				l_ce.ce_gets (l_ce.item.next)
			end
		end

	delete_unused_format_cache_entries
		local
			l_ce: CE_POINTER [PD_F_DATA]
			l_cee: CE_POINTER [PD_CACHE_ENTRY]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ce.make
			create l_cee.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.next))
				when 1 then -- loop condition:
					if not (not js.loop_left and then ((l_ce.c_pointer.to_integer_32).to_natural_32 /= ((format_cache.address).c_pointer.to_integer_32).to_natural_32)) then
						l_pos := -1 + 12 -- skip loop
					end
				when 2 then -- loop body:
						l_cee.ce_gets (create {CE_POINTER [PD_CACHE_ENTRY]}.make_shared (default_pointer))
						if (l_ce.item.refcount).to_boolean then
							l_pos := -1 + 11 -- skip if block
						end
				when 3 then -- if:
						if not ((cache_inst.find_in_cache (l_ce.item.url, l_cee.address)).to_boolean) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 6 then -- else:
						if ((l_cee).c_pointer /= default_pointer) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 9 then -- else:
						if l_cee.item.count = l_ce.item.use_tag then
							l_pos := -1 + 11 -- skip if block
						end
				when 10 then -- if: | _L:
						if (l_cee).c_pointer = default_pointer then
							from
							until
								False
							loop
								links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
								links_inst.errline := 1111
								error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("file %%s disappeared from cache")), l_ce.item.url])
							end
						end
						l_ce.ce_gets (l_ce.item.prev)
						view_inst.destroy_formatted (l_ce.item.next)
						format_cache_entries := format_cache_entries - 1
				when 11 then -- if done | if done | if done
						l_ce.ce_gets (l_ce.item.next)
						l_pos := -1 + 1 -- back to loop condition
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	format_cache_reactivate (a_ce: CE_POINTER [PD_F_DATA])
		local
			js: JUMP_STATE
		do
			error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (a_ce));
			(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (a_ce.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (a_ce.item.prev);
			(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (a_ce.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (a_ce.item.next)
			error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (a_ce))
			from
			until
				js.loop_left
			loop
				js.reset
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((format_cache.address)))
				a_ce.item.next := (create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address))).item.next
				a_ce.item.prev := create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address));
				(create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address))).item.next := a_ce;
				(a_ce.item.next).item.prev := a_ce
				error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((format_cache.address)))
				js.break
			end
			js.reset
		end

	cached_format_html (a_vs: CE_POINTER [PD_VIEW_STATE]; a_screen: CE_POINTER [PD_F_DATA_C]; a_opt: CE_POINTER [PD_DOCUMENT_OPTIONS])
		local
			l_n: CE_POINTER [NATURAL_8]
			l_ce: CE_POINTER [PD_F_DATA]
			l_cee: CE_POINTER [PD_CACHE_ENTRY]
			l_tmp: INTEGER_32
			l_tmp___1: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_n.make
			create l_ce.make
			create l_cee.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if (a_vs).c_pointer = default_pointer then
						js.return
					end
					if js.return_called then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						l_n.ce_gets (a_screen.item.name)
						a_screen.item.name := create {CE_POINTER [NATURAL_8]}.make_shared (default_pointer)
						view_inst.detach_formatted (a_screen)
						a_screen.item.name := l_n
						a_screen.item.link_bg := create {CE_POINTER [PD_LINK_BG]}.make_shared (default_pointer)
						a_screen.item.link_bg_n := 0
						a_screen.item.vs := a_vs
						l_tmp := -1
						a_screen.item.yl := l_tmp
						a_screen.item.xl := l_tmp
						a_screen.item.f_data := create {CE_POINTER [PD_F_DATA]}.make_shared (default_pointer)
						l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.next))
				when 2 then -- loop condition:
						if not (not js.loop_left and then ((l_ce.c_pointer.to_integer_32).to_natural_32 /= ((format_cache.address).c_pointer.to_integer_32).to_natural_32)) then
							l_pos := -1 + 15 -- skip loop
						end
				when 3 then -- loop body:
						js.reset
						if (strcmp (create {CE_POINTER [INTEGER_8]}.make_cast (l_ce.item.url), create {CE_POINTER [INTEGER_8]}.make_cast ((a_vs.item.url)))).to_boolean then
							l_pos := -1 + 14 -- skip if block
						end
				when 4 then -- if:
						if (compare_opt (create {CE_POINTER [PD_DOCUMENT_OPTIONS]}.make_shared (l_ce.c_pointer + {PD_F_DATA}.opt_offset), a_opt)).to_boolean then
							l_pos := -1 + 14 -- skip if block
						end
				when 5 then -- if:
						l_cee.ce_gets (create {CE_POINTER [PD_CACHE_ENTRY]}.make_shared (default_pointer))
						if not ((cache_inst.find_in_cache (a_vs.item.url, l_cee.address)).to_boolean) then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if:
						l_pos := -1 + 12 -- goto _L
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 8 then -- else:
						if ((l_cee).c_pointer /= default_pointer) then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						l_pos := -1 + 12 -- goto _L
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 11 then -- else:
						if l_cee.item.count = l_ce.item.use_tag then
							l_pos := -1 + 13 -- skip if block
						end
				when 12 then -- if: | _L:
						if (l_cee).c_pointer = default_pointer then
							from
							until
								False
							loop
								links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
								links_inst.errline := 1142
								error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("file %%s disappeared from cache")), l_ce.item.url])
							end
						end
						if not (l_ce.item.refcount).to_boolean then
							l_ce.ce_gets (l_ce.item.prev)
							view_inst.destroy_formatted (l_ce.item.next)
							format_cache_entries := format_cache_entries - 1
						end
						js.continue
				when 13 then -- if done | if done
						if not js.loop_jumped then
							format_cache_reactivate (l_ce)
							l_tmp___1 := l_ce.item.refcount
							l_ce.item.refcount := (l_ce.item.refcount) + 1
							if not l_tmp___1.to_boolean then
								format_cache_entries := format_cache_entries - 1
							end
							a_screen.item.f_data := l_ce
							l_pos := -1 + 22 -- goto sx
						end
				when 14 then -- if done
						if not js.loop_left then
							l_ce.ce_gets (l_ce.item.next)
						end
						l_pos := -1 + 2 -- back to loop condition
				when 15 then -- loop done
						if not ((cache_inst.find_in_cache (a_vs.item.url, l_cee.address)).to_boolean) then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						l_pos := -1 + 19 -- goto _L___0
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 18 then -- else:
						if ((l_cee).c_pointer /= default_pointer) then
							l_pos := -1 + 20 -- skip if block
						end
				when 19 then -- if: | _L___0:
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 1156
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("document to format not found"))])
						end
						js.return
				when 20 then -- if done | if done
						if js.return_called then
							l_pos := -1 + 23 -- skip if block
						end
				when 21 then -- if:
							l_cee.item.refcount := (l_cee.item.refcount) + 1
							main_inst.shrink_memory (0)
							l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1161, {PD_F_DATA}.sizeof.to_natural_32))))
							view_inst.init_formatted (l_ce)
							l_ce.item.refcount := 1
							l_ce.item.url := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1164, a_vs.item.url)
							copy_opt (create {CE_POINTER [PD_DOCUMENT_OPTIONS]}.make_shared (l_ce.c_pointer + {PD_F_DATA}.opt_offset), a_opt)
							from
							until
								js.loop_left
							loop
								js.reset
								error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((format_cache.address)))
								l_ce.item.next := (create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address))).item.next
								l_ce.item.prev := create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address));
								(create {CE_POINTER [PD_F_DATA]}.make_cast ((format_cache.address))).item.next := l_ce;
								(l_ce.item.next).item.prev := l_ce
								error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast ((format_cache.address)))
								js.break
							end
							js.reset
							a_screen.item.f_data := l_ce
							l_ce.item.time_to_get := -(select_inst.get_time)
							format_html (l_cee, l_ce)
							l_ce.item.time_to_get := l_ce.item.time_to_get + (select_inst.get_time)
				when 22 then -- sx:
							a_screen.item.xw := l_ce.item.opt.xw
							a_screen.item.yw := l_ce.item.opt.yw
							a_screen.item.xp := l_ce.item.opt.xp
							a_screen.item.yp := l_ce.item.opt.yp
				when 23 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	formatted_info (a_type: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_ce: CE_POINTER [PD_F_DATA]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ce.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_i := 0
					inspect a_type -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 3 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2
					l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.next))
					from
					until
						(l_ce.c_pointer.to_integer_32).to_natural_32 = ((format_cache.address).c_pointer.to_integer_32).to_natural_32
					loop
						l_i := l_i + 1
						l_ce.ce_gets (l_ce.item.next)
					end
					Result := l_i.to_integer_32
					js.return
				when 2 then -- switch 1, case 3
					if not js.return_called then
						l_ce.ce_gets (create {CE_POINTER [PD_F_DATA]}.make_cast (format_cache.next))
						from
						until
							(l_ce.c_pointer.to_integer_32).to_natural_32 = ((format_cache.address).c_pointer.to_integer_32).to_natural_32
						loop
							l_i := l_i + ((l_ce.item.refcount).to_boolean).to_integer
							l_ce.ce_gets (l_ce.item.next)
						end
						Result := l_i.to_integer_32
						js.return
					end
				when 3 then -- switch 1, default:
					if not js.return_called then
						from
						until
							False
						loop
							links_inst.errfile.ce_gets (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")))
							links_inst.errline := 1190
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("formatted_info: bad request"))])
						end
					end
				when 4 then -- switch 1, done:
					if not js.return_called then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	add_frame_to_list (a_ses: CE_POINTER [PD_SESSION]; a_fd: CE_POINTER [PD_F_DATA_C])
		local
			l_f: CE_POINTER [PD_F_DATA_C]
			l_fp: CE_POINTER [PD_F_DATA_C]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_f.make
			create l_fp.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_f.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.next))
				when 1 then -- loop condition:
					if not (not js.loop_left and then ((l_f.c_pointer.to_integer_32).to_natural_32 /= ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.scrn_frames_offset)).c_pointer.to_integer_32).to_natural_32)) then
						l_pos := -1 + 9 -- skip loop
					end
				when 2 then -- loop body:
						if not (l_f.item.yp > a_fd.item.yp) then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 5 then -- else:
						if l_f.item.yp /= a_fd.item.yp then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if:
						if not (l_f.item.xp > a_fd.item.xp) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_f.item.prev))
							a_fd.item.next := (l_f.item.prev).item.next
							a_fd.item.prev := l_f.item.prev;
							(l_f.item.prev).item.next := a_fd;
							(a_fd.item.next).item.prev := a_fd
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_f.item.prev))
							js.break
						end
						js.reset
						js.return
				when 8 then -- if done | if done
						if not js.return_called then
							l_f.ce_gets (l_f.item.next)
						end
						l_pos := -1 + 1 -- back to loop condition
				when 9 then -- loop done
					if not js.return_called then
						l_fp.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.prev))
						from
						until
							js.loop_left
						loop
							js.reset
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_fp))
							a_fd.item.next := l_fp.item.next
							a_fd.item.prev := l_fp
							l_fp.item.next := a_fd;
							(a_fd.item.next).item.prev := a_fd
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_fp))
							js.break
						end
						js.reset
					end
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	find_fd (a_ses: CE_POINTER [PD_SESSION]; a_name: CE_POINTER [NATURAL_8]; a_depth___0: INTEGER_32; a_x: INTEGER_32; a_y: INTEGER_32): CE_POINTER [PD_F_DATA_C]
		local
			l_fd: CE_POINTER [PD_F_DATA_C]
			js: JUMP_STATE
		do
			create l_fd.make

			l_fd.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.prev))
			from
			until
				not (not js.loop_left and then ((l_fd.c_pointer.to_integer_32).to_natural_32 /= ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.scrn_frames_offset)).c_pointer.to_integer_32).to_natural_32))
			loop
				if not (strcasecmp (create {CE_POINTER [INTEGER_8]}.make_cast (l_fd.item.name), create {CE_POINTER [INTEGER_8]}.make_cast (a_name))).to_boolean then
					if not (l_fd.item.used).to_boolean then
						l_fd.item.used := 1
						l_fd.item.depth := a_depth___0
						Result := l_fd
						js.return
					end
				end
				if not js.return_called then
					l_fd.ce_gets (l_fd.item.prev)
				end
			end
			if not js.return_called then
				l_fd.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1216, {PD_F_DATA_C}.sizeof.to_natural_32))))
				memset (create {CE_POINTER [ANY]}.make_cast (l_fd), 0, {PD_F_DATA_C}.sizeof.to_natural_32).do_nothing
				l_fd.item.used := 1
				l_fd.item.name := links_inst.debug_stracpy (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1219, a_name)
				l_fd.item.depth := a_depth___0
				l_fd.item.xp := a_x
				l_fd.item.yp := a_y
				l_fd.item.search_word := create {CE_POINTER [CE_POINTER [NATURAL_8]]}.make_shared (a_ses.c_pointer + {PD_SESSION}.search_word_offset)
				add_frame_to_list (a_ses, l_fd)
				Result := l_fd
			end
		end

	format_frame (a_ses: CE_POINTER [PD_SESSION]; a_name: CE_POINTER [NATURAL_8]; a_url: CE_POINTER [NATURAL_8]; a_o: CE_POINTER [PD_DOCUMENT_OPTIONS]; a_depth___0: INTEGER_32): CE_POINTER [PD_F_DATA_C]
		local
			l_ce: CE_POINTER [PD_CACHE_ENTRY]
			l_vs: CE_POINTER [PD_VIEW_STATE]
			l_fd: CE_POINTER [PD_F_DATA_C]
			l_fr: CE_POINTER [PD_FRAME]
			l_u: CE_POINTER [NATURAL_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ce.make
			create l_vs.make
			create l_fd.make
			create l_fr.make
			create l_u.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start: | repeat:
					l_fr.ce_gets (session_inst.ses_find_frame (a_ses, a_name))
					if (l_fr).c_pointer = default_pointer then
						Result := (create {CE_POINTER [PD_F_DATA_C]}.make_shared (default_pointer))
						js.return
					end
					if not js.return_called then
						l_vs.ce_gets (create {CE_POINTER [PD_VIEW_STATE]}.make_shared (l_fr.c_pointer + {PD_FRAME}.vs_offset))
						if (cache_inst.find_in_cache (l_vs.item.url, l_ce.address)).to_boolean then
							Result := (create {CE_POINTER [PD_F_DATA_C]}.make_shared (default_pointer))
							js.return
						elseif (l_ce).c_pointer = default_pointer then
							Result := (create {CE_POINTER [PD_F_DATA_C]}.make_shared (default_pointer))
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if ((l_ce.item.redirect).c_pointer /= default_pointer) then
							if l_fr.item.redirect_cnt < 10 then
								l_u.ce_gets (url_inst.join_urls (l_vs.item.url, l_ce.item.redirect))
								if ((l_u).c_pointer /= default_pointer) then
									l_fr.item.redirect_cnt := (l_fr.item.redirect_cnt) + 1
									session_inst.ses_change_frame_url (a_ses, a_name, l_u).do_nothing
									error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1243, create {CE_POINTER [ANY]}.make_cast (l_u))
									l_pos := -1 + 0 -- goto repeat
								end
							end
						end
				when 2 then
						l_fd.ce_gets (find_fd (a_ses, a_name, a_depth___0, a_o.item.xp, a_o.item.yp))
						if (l_fd).c_pointer = default_pointer then
							Result := (create {CE_POINTER [PD_F_DATA_C]}.make_shared (default_pointer))
							js.return
						end
				when 3 then -- if done
					if not js.return_called then
						cached_format_html (l_vs, l_fd, a_o)
						Result := l_fd
					end
				when 4 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	format_frames (a_ses: CE_POINTER [PD_SESSION]; a_fsd: CE_POINTER [PD_FRAMESET_DESC]; a_op: CE_POINTER [PD_DOCUMENT_OPTIONS]; a_depth___0: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_n: INTEGER_32
			l_o: PD_DOCUMENT_OPTIONS
			l_fdc: CE_POINTER [PD_F_DATA_C]
			js: JUMP_STATE
		do
			create l_o.make
			create l_fdc.make

			if a_depth___0 > 5 then
				js.return
			end
			if not js.return_called then
				memcpy (create {CE_POINTER [ANY]}.make_cast ((l_o.address)), create {CE_POINTER [ANY]}.make_cast (a_op), {PD_DOCUMENT_OPTIONS}.sizeof.to_natural_32).do_nothing
				if (l_o.margin).to_boolean then
					l_o.margin := 1
				end
				l_n := 0
				l_j := 0
				from
				until
					not (not js.loop_left and then (l_j < a_fsd.item.y))
				loop
					l_o.xp := a_op.item.xp
					l_i := 0
					from
					until
						not (l_i < a_fsd.item.x)
					loop
						l_o.xw := a_fsd.item.f[l_n].xw
						l_o.yw := a_fsd.item.f[l_n].yw
						l_o.framename := a_fsd.item.f[l_n].name
						if ((a_fsd.item.f[l_n].subframe).c_pointer /= default_pointer) then
							format_frames (a_ses, a_fsd.item.f[l_n].subframe, l_o.address, a_depth___0 + 1)
						elseif ((a_fsd.item.f[l_n].name).c_pointer /= default_pointer) then
							l_fdc.ce_gets (format_frame (a_ses, a_fsd.item.f[l_n].name, a_fsd.item.f[l_n].url, l_o.address, a_depth___0))
							if ((l_fdc).c_pointer /= default_pointer) then
								if ((l_fdc.item.f_data).c_pointer /= default_pointer) then
									if ((l_fdc.item.f_data).item.frame).to_boolean then
										format_frames (a_ses, (l_fdc.item.f_data).item.frame_desc, l_o.address, a_depth___0 + 1)
									end
								end
							end
						end
						l_o.xp := l_o.xp + (l_o.xw + 1)
						l_n := l_n + 1
						l_i := l_i + 1
					end
					l_o.yp := l_o.yp + (l_o.yw + 1)
					l_j := l_j + 1
				end
			end
		end

	html_interpret (a_ses: CE_POINTER [PD_SESSION])
		local
			l_l: CE_POINTER [PD_VIEW_STATE]
			l_o: PD_DOCUMENT_OPTIONS
			l_fd: CE_POINTER [PD_F_DATA_C]
			l_cf: CE_POINTER [PD_F_DATA_C]
			l_fdp: CE_POINTER [PD_F_DATA_C]
			l_n: INTEGER_32
			js: JUMP_STATE
		do
			create l_l.make
			create l_o.make
			create l_fd.make
			create l_cf.make
			create l_fdp.make

			l_cf.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_shared (default_pointer))
			if (a_ses.item.screen).c_pointer = default_pointer then
				a_ses.item.screen := create {CE_POINTER [PD_F_DATA_C]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1296, {PD_F_DATA_C}.sizeof.to_natural_32)))
				memset (create {CE_POINTER [ANY]}.make_cast (a_ses.item.screen), 0, {PD_F_DATA_C}.sizeof.to_natural_32).do_nothing;
				(a_ses.item.screen).item.search_word := create {CE_POINTER [CE_POINTER [NATURAL_8]]}.make_shared (a_ses.c_pointer + {PD_SESSION}.search_word_offset)
			end
			if ((a_ses.item.history.next).c_pointer.to_integer_32).to_natural_32 /= ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.history_offset)).c_pointer.to_integer_32).to_natural_32 then
				l_l.ce_gets (create {CE_POINTER [PD_VIEW_STATE]}.make_shared ((create {CE_POINTER [PD_LOCATION]}.make_cast (a_ses.item.history.next)).c_pointer + {PD_LOCATION}.vs_offset))
			else
				l_l.ce_gets (create {CE_POINTER [PD_VIEW_STATE]}.make_shared (default_pointer))
			end
			l_o.xp := 0
			l_o.yp := 1
			l_o.xw := (a_ses.item.term).item.x
			if (a_ses.item.term).item.y < 2 then
				l_o.yw := 0
			else
				l_o.yw := (a_ses.item.term).item.y - 2
			end
			l_o.col := ((a_ses.item.term).item.spec).item.col
			l_o.cp := ((a_ses.item.term).item.spec).item.charset
			links_inst.ds2do (create {CE_POINTER [PD_DOCUMENT_SETUP]}.make_shared (a_ses.c_pointer + {PD_SESSION}.ds_offset), l_o.address)
			if ((l_l).c_pointer /= default_pointer) then
				l_o.plain := l_l.item.plain
			else
				l_o.plain := 1
			end
			if l_o.plain = -1 then
				l_o.plain := 0
			end
			if ((l_l).c_pointer /= default_pointer) then
				l_l.item.plain := l_o.plain
			end
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (l_o.c_pointer + {PD_DOCUMENT_OPTIONS}.default_fg_offset))), create {CE_POINTER [ANY]}.make_cast ((default_inst.default_fg.address)), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (l_o.c_pointer + {PD_DOCUMENT_OPTIONS}.default_bg_offset))), create {CE_POINTER [ANY]}.make_cast ((default_inst.default_bg.address)), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (l_o.c_pointer + {PD_DOCUMENT_OPTIONS}.default_link_offset))), create {CE_POINTER [ANY]}.make_cast ((default_inst.default_link.address)), {PD_RGB}.sizeof.to_natural_32).do_nothing
			memcpy (create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [PD_RGB]}.make_shared (l_o.c_pointer + {PD_DOCUMENT_OPTIONS}.default_vlink_offset))), create {CE_POINTER [ANY]}.make_cast ((default_inst.default_vlink.address)), {PD_RGB}.sizeof.to_natural_32).do_nothing
			l_o.framename := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string (""))
			l_fd.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.next))
			from
			until
				(l_fd.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.scrn_frames_offset)).c_pointer.to_integer_32).to_natural_32
			loop
				l_fd.item.used := 0
				l_fd.ce_gets (l_fd.item.next)
			end
			cached_format_html (l_l, a_ses.item.screen, l_o.address)
			if (((a_ses.item.screen).item.f_data).c_pointer /= default_pointer) then
				if (((a_ses.item.screen).item.f_data).item.frame).to_boolean then
					l_cf.ce_gets (view_inst.current_frame (a_ses))
					format_frames (a_ses, ((a_ses.item.screen).item.f_data).item.frame_desc, l_o.address, 0)
				end
			end
			l_fd.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.next))
			from
			until
				(l_fd.c_pointer.to_integer_32).to_natural_32 = ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.scrn_frames_offset)).c_pointer.to_integer_32).to_natural_32
			loop
				if not (l_fd.item.used).to_boolean then
					l_fdp.ce_gets (l_fd.item.prev)
					view_inst.detach_formatted (l_fd)
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_fd));
					(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_fd.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_fd.item.prev);
					(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_fd.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_fd.item.next)
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_fd))
					error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1327, create {CE_POINTER [ANY]}.make_cast (l_fd))
					l_fd.ce_gets (l_fdp)
				end
				l_fd.ce_gets (l_fd.item.next)
			end
			if ((l_cf).c_pointer /= default_pointer) then
				l_n := 0
				l_fd.ce_gets (create {CE_POINTER [PD_F_DATA_C]}.make_cast (a_ses.item.scrn_frames.next))
				from
				until
					not (not js.loop_left and then ((l_fd.c_pointer.to_integer_32).to_natural_32 /= ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_ses.c_pointer + {PD_SESSION}.scrn_frames_offset)).c_pointer.to_integer_32).to_natural_32))
				loop
					js.reset
					if ((l_fd.item.f_data).c_pointer /= default_pointer) then
						if ((l_fd.item.f_data).item.frame).to_boolean then
							js.continue
						end
					end
					if not js.loop_jumped and then ((l_fd.c_pointer.to_integer_32).to_natural_32 = (l_cf.c_pointer.to_integer_32).to_natural_32) then
						(create {CE_POINTER [PD_LOCATION]}.make_cast (a_ses.item.history.next)).item.vs.current_link := l_n
						js.break
					end
					if not js.loop_jumped then
						l_n := l_n + 1
					end
					if not js.loop_left then
						l_fd.ce_gets (l_fd.item.next)
					end
				end
				js.reset
			end
		end

	add_srch_chr (a_f: CE_POINTER [PD_F_DATA]; a_c: NATURAL_8; a_x: INTEGER_32; a_y: INTEGER_32; a_nn: INTEGER_32)
		local
			l_n: INTEGER_32
			js: JUMP_STATE
		do
			l_n := a_f.item.nsearch
			if a_c.to_integer_32 = 32 then
				if not l_n.to_boolean then
					js.return
				elseif ((a_f.item.search + (l_n - 1)).item.c).to_integer_32 = 32 then
					js.return
				end
			end
			if not js.return_called then
				(a_f.item.search + l_n).item.c := a_c;
				(a_f.item.search + l_n).item.x := a_x;
				(a_f.item.search + l_n).item.y := a_y;
				(a_f.item.search + l_n).item.n := a_nn
				a_f.item.nsearch := (a_f.item.nsearch) + 1
			end
		end

	sort_srch (a_f: CE_POINTER [PD_F_DATA])
		local
			l_i: INTEGER_32
			l_min: CE_POINTER [INTEGER_32]
			l_max: CE_POINTER [INTEGER_32]
			l_s: CE_POINTER [PD_SEARCH]
		do
			create l_min.make
			create l_max.make
			create l_s.make

			if (a_f.item.y).to_natural_32 > {NATURAL_32} 2147483647 // Pointer_bytes.to_natural_32 then
				from
				until
					False
				loop
					error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 1369])
					kbd_inst.fatal_tty_exit
					exit (4)
				end
			end
			if (a_f.item.y).to_natural_32 > {NATURAL_32} 2147483647 // Integer_32_bytes.to_natural_32 then
				from
				until
					False
				loop
					error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 1370])
					kbd_inst.fatal_tty_exit
					exit (4)
				end
			end
			a_f.item.slines1 := create {CE_POINTER [CE_POINTER [PD_SEARCH]]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1371, (a_f.item.y).to_natural_32 * Pointer_bytes.to_natural_32)))
			a_f.item.slines2 := create {CE_POINTER [CE_POINTER [PD_SEARCH]]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1372, (a_f.item.y).to_natural_32 * Pointer_bytes.to_natural_32)))
			l_min.ce_gets (create {CE_POINTER [INTEGER_32]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1373, (a_f.item.y).to_natural_32 * Integer_32_bytes.to_natural_32))))
			l_max.ce_gets (create {CE_POINTER [INTEGER_32]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1374, (a_f.item.y).to_natural_32 * Integer_32_bytes.to_natural_32))))
			memset (create {CE_POINTER [ANY]}.make_cast (a_f.item.slines1), 0, (a_f.item.y).to_natural_32 * Pointer_bytes.to_natural_32).do_nothing
			memset (create {CE_POINTER [ANY]}.make_cast (a_f.item.slines2), 0, (a_f.item.y).to_natural_32 * Pointer_bytes.to_natural_32).do_nothing
			l_i := 0
			from
			until
				not (l_i < a_f.item.y)
			loop
				l_min[l_i] := 2147483647
				l_max[l_i] := 0
				l_i := l_i + 1
			end
			l_i := 0
			from
			until
				not (l_i < a_f.item.nsearch)
			loop
				l_s.ce_gets (a_f.item.search + l_i)
				if l_s.item.x < l_min[l_s.item.y] then
					l_min[l_s.item.y] := l_s.item.x
					a_f.item.slines1[l_s.item.y] := l_s
				end
				if l_s.item.x + l_s.item.n > l_max[l_s.item.y] then
					l_max[l_s.item.y] := l_s.item.x + l_s.item.n
					a_f.item.slines2[l_s.item.y] := l_s
				end
				l_i := l_i + 1
			end
			error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1383, create {CE_POINTER [ANY]}.make_cast (l_min))
			error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1384, create {CE_POINTER [ANY]}.make_cast (l_max))
		end

	is_spc (a_c: NATURAL_16): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			if (a_c.to_natural_8).to_integer_32 <= 32 then
				l_tmp := 1
			elseif ((a_c.to_integer_32).bit_and (32768)).to_boolean then
				l_tmp := 1
			else
				l_tmp := 0
			end
			Result := l_tmp
		end

	get_srch (a_f: CE_POINTER [PD_F_DATA]): INTEGER_32
		local
			l_n: CE_POINTER [PD_NODE]
			l_cnt: INTEGER_32
			l_cc: INTEGER_32
			l_x: INTEGER_32
			l_y: INTEGER_32
			l_xm: INTEGER_32
			l_ym: INTEGER_32
			l_ns: INTEGER_32
			l_c: NATURAL_8
			l_xx: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_n.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_cnt := 0
					l_cc := ((a_f.item.search).c_pointer = default_pointer).to_integer
					l_n.ce_gets (create {CE_POINTER [PD_NODE]}.make_cast (a_f.item.nodes.prev))
				when 1 then -- loop condition:
					if not (not js.loop_left and then ((l_n.c_pointer.to_integer_32).to_natural_32 /= ((create {CE_POINTER [PD_LIST_HEAD]}.make_shared (a_f.c_pointer + {PD_F_DATA}.nodes_offset)).c_pointer.to_integer_32).to_natural_32)) then
						l_pos := -1 + 23 -- skip loop
					end
				when 2 then -- loop body:
						l_xm := l_n.item.x + l_n.item.xw
						l_ym := l_n.item.y + l_n.item.yw
						l_y := l_n.item.y
				when 3 then -- loop condition:
						if js.loop_left then
							l_pos := -1 + 22 -- skip loop
						end
				when 4 then -- loop body:
						js.reset
						if l_y < l_ym then
							if not (l_y < a_f.item.y) then
								js.break
							end
						else
							js.break
						end
						if js.loop_jumped then
							l_pos := -1 + 21 -- skip if block
						end
				when 5 then -- if:
						l_ns := 1
						l_x := l_n.item.x
				when 6 then -- loop condition:
						if js.loop_left then
							l_pos := -1 + 20 -- skip loop
						end
				when 7 then -- loop body:
						js.reset
						if l_x < l_xm then
							if not (l_x < (a_f.item.data + l_y).item.l) then
								js.break
							end
						else
							js.break
						end
						if not js.loop_jumped then
							l_c := ((a_f.item.data + l_y).item.d[l_x]).to_natural_8
							if (is_spc ((a_f.item.data + l_y).item.d[l_x])).to_boolean then
								l_c := ((' ').code).to_natural_8
							end
							if l_c.to_integer_32 = 32 then
								if l_ns.to_boolean then
									js.continue
								end
							end
						end
						if not js.loop_jumped then
							l_c := charsets_inst.charset_upcase (l_c, a_f.item.opt.cp)
							if l_ns.to_boolean then
								if not l_cc.to_boolean then
									add_srch_chr (a_f, l_c, l_x, l_y, 1)
								else
									l_cnt := l_cnt + 1
								end
								l_ns := 0
								js.continue
							end
						end
						if js.loop_jumped then
							l_pos := -1 + 19 -- skip if block
						end
				when 8 then -- if:
						if l_c.to_integer_32 = 32 then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						if not l_cc.to_boolean then
							add_srch_chr (a_f, l_c, l_x, l_y, 1)
						else
							l_cnt := l_cnt + 1
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 11 then -- else:
						l_xx := l_x + 1
				when 12 then -- loop condition:
						if js.loop_left then
							l_pos := -1 + 17 -- skip loop
						end
				when 13 then -- loop body:
						js.reset
						if l_xx < l_xm then
							if not (l_xx < (a_f.item.data + l_y).item.l) then
								js.break
							end
						else
							js.break
						end
						if js.loop_jumped then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if not (is_spc ((a_f.item.data + l_y).item.d[l_xx])).to_boolean then
							l_pos := -1 + 18 -- goto ja_uz_z_toho_programovani_asi_zcvoknu
						end
				when 15 then
						l_xx := l_xx + 1
				when 16 then -- if done
						l_pos := -1 + 12 -- back to loop condition
				when 17 then -- loop done
						js.reset
						l_xx := l_x
				when 18 then -- ja_uz_z_toho_programovani_asi_zcvoknu:
						if not l_cc.to_boolean then
							add_srch_chr (a_f, ((' ').code).to_natural_8, l_x, l_y, l_xx - l_x)
						else
							l_cnt := l_cnt + 1
						end
						if l_xx = l_x then
							js.break
						end
						if not js.loop_jumped then
							l_x := l_xx - 1
						end
				when 19 then -- if done | if done
						if not js.loop_left then
							l_x := l_x + 1
						end
						l_pos := -1 + 6 -- back to loop condition
				when 20 then -- loop done
						js.reset
						if not l_cc.to_boolean then
							add_srch_chr (a_f, ((' ').code).to_natural_8, l_x, l_y, 0)
						else
							l_cnt := l_cnt + 1
						end
						l_y := l_y + 1
				when 21 then -- if done
						l_pos := -1 + 3 -- back to loop condition
				when 22 then -- loop done
						js.reset
						l_n.ce_gets (l_n.item.prev)
						l_pos := -1 + 1 -- back to loop condition
				when 23 then -- loop done
					Result := l_cnt
				when 24 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	get_search_data (a_f: CE_POINTER [PD_F_DATA])
		local
			l_n: INTEGER_32
			js: JUMP_STATE
		do
			if ((a_f.item.search).c_pointer /= default_pointer) then
				js.return
			end
			if not js.return_called then
				l_n := get_srch (a_f)
				a_f.item.nsearch := 0
				if l_n.to_natural_32 > {NATURAL_32} 2147483647 // {PD_SEARCH}.sizeof.to_natural_32 then
					from
					until
						False
					loop
						error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: attempting to allocate too large block at %%s:%%d")), ce_string ("html_r.c"), 1443])
						kbd_inst.fatal_tty_exit
						exit (4)
					end
				end
				a_f.item.search := create {CE_POINTER [PD_SEARCH]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("html_r.c")), 1444, l_n.to_natural_32 * {PD_SEARCH}.sizeof.to_natural_32)))
				get_srch (a_f).do_nothing
				from
				until
					js.loop_left
				loop
					js.reset
					if (a_f.item.nsearch).to_boolean then
						if ((a_f.item.search + (a_f.item.nsearch - 1)).item.c).to_integer_32 /= 32 then
							js.break
						end
					else
						js.break
					end
					if not js.loop_jumped then
						a_f.item.nsearch := (a_f.item.nsearch) - 1
					end
				end
				js.reset
				sort_srch (a_f)
			end
		end

end
