class
	P_FTP

inherit
	LIBCURL_DATA
		redefine
			default_create
		end
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {NONE} -- Initialization

	default_create
		do
			create Curl_handler_ftp.make
			create Curl_handler_ftp_proxy.make
			create mode.make (<<2, 5>>)
			create mode___0.make (<<2, 5>>)
			create ftpauth.make (<<2, 4>>)
		end

feature {ANY} -- attributes

	Curl_handler_ftp: PD_CURL_HANDLER

	Curl_handler_ftp_proxy: PD_CURL_HANDLER

	mode: CE_ARRAY [CE_ARRAY [INTEGER_8]]

	mode___0: CE_ARRAY [CE_ARRAY [INTEGER_8]]

	ftpauth: CE_ARRAY [CE_ARRAY [INTEGER_8]]

feature {ANY} -- routines

	freedirs (a_ftpc: CE_POINTER [PD_FTP_CONN])
		local
			l_i: INTEGER_32
		do
			if ((a_ftpc.item.dirs).c_pointer /= default_pointer) then
				l_i := 0
				from
				until
					not (l_i < a_ftpc.item.dirdepth)
				loop
					if ((a_ftpc.item.dirs[l_i]).c_pointer /= default_pointer) then
						easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (a_ftpc.item.dirs[l_i])])
						a_ftpc.item.dirs[l_i] := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					end
					l_i := l_i + 1
				end
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (a_ftpc.item.dirs)])
				a_ftpc.item.dirs := create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer)
				a_ftpc.item.dirdepth := 0
			end
			if ((a_ftpc.item.file).c_pointer /= default_pointer) then
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (a_ftpc.item.file)])
				a_ftpc.item.file := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			end
		end

	isBadFtpString (a_string: CE_POINTER [INTEGER_8]): BOOLEAN
		local
			l_tmp___1: INTEGER_32
		do
			if (0).to_natural_32 /= ((strchr (a_string, ('%R').code)).c_pointer.to_integer_32).to_natural_32 then
				l_tmp___1 := 1
			elseif (0).to_natural_32 /= ((strchr (a_string, ('%N').code)).c_pointer.to_integer_32).to_natural_32 then
				l_tmp___1 := 1
			else
				l_tmp___1 := 0
			end
			Result := l_tmp___1.to_boolean
		end

	AllowServerConnect (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sock: INTEGER_32
			l_timeout_ms: INTEGER_32
			l_interval_ms: INTEGER_32
			l_s: INTEGER_32
			l_add: PD_CURL_SOCKADDR_STORAGE
			l_size: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_add.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					l_sock := a_conn.item.sock[1]
					l_s := -1
					l_size := {PD_CURL_SOCKADDR_STORAGE}.sizeof.to_natural_32
				when 1 then -- loop condition:
					if js.loop_left then
						l_pos := -1 + 8 -- skip loop
					end
				when 2 then -- loop body:
						l_timeout_ms := connect_inst.Curl_timeleft (a_conn, create {CE_POINTER [SD_TIMEVAL]}.make_shared (default_pointer), (1).to_boolean)
						if l_timeout_ms < 0 then
							sendf_inst.Curl_failf ([l_data, ce_string ("Timeout while waiting for server connect")])
							Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 7 -- skip if block
						end
				when 3 then -- if:
						l_interval_ms := 1000
						if l_timeout_ms < l_interval_ms then
							l_interval_ms := l_timeout_ms
						end
						inspect (select_inst.Curl_socket_ready (l_sock, -1, l_interval_ms.to_integer_32)) -- switch 1 head
						when -1 then l_pos := -1 + 4
						when 0 then l_pos := -1 + 5
						else l_pos := -1 + 6
						end
				when 4 then -- switch 1, case -1
						sendf_inst.Curl_failf ([l_data, ce_string ("Error while waiting for server connect")])
						Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
						js.return
				when 5 then -- switch 1, case 0
						l_pos := -1 + 7 -- switch 1; break
				when 6 then -- switch 1, default:
						if 0 = (getsockname (l_sock, create {CE_POINTER [SD_SOCKADDR]}.make_cast ((l_add.address)), (create {CE_POINTER [NATURAL_32]}.make_shared ($l_size)))) then
							l_size := {PD_CURL_SOCKADDR_STORAGE}.sizeof.to_natural_32
							l_s := accept (l_sock, create {CE_POINTER [SD_SOCKADDR]}.make_cast ((l_add.address)), (create {CE_POINTER [NATURAL_32]}.make_shared ($l_size)))
						end
						close (l_sock).do_nothing
						if -1 = l_s then
							sendf_inst.Curl_failf ([l_data, ce_string ("Error accept()ing server connect")])
							Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
							js.return
						end
						if not js.return_called then
							sendf_inst.Curl_infof ([l_data, ce_string ("Connection accepted from server%N")])
							a_conn.item.sock[1] := l_s
							nonblock_inst.curlx_nonblock (l_s, 1).do_nothing
							Result := ({PC_CURLCODE}.CURLE_OK)
							js.return
						end
				when 7 then -- switch 1, done: | if done
						l_pos := -1 + 1 -- back to loop condition
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_endofresp (a_pp: CE_POINTER [PD_PINGPONG]; a_code: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_line: CE_POINTER [INTEGER_8]
			l_len: NATURAL_32
			js: JUMP_STATE
		do
			create l_line.make

			l_line.ce_gets (a_pp.item.linestart_resp)
			l_len := a_pp.item.nread_resp
			if l_len > {NATURAL_32} 3 then
				if ((((ctype_b_loc).item[((l_line[0]).to_natural_8).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
					if ((((ctype_b_loc).item[((l_line[1]).to_natural_8).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
						if ((((ctype_b_loc).item[((l_line[2]).to_natural_8).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
							if 32 = (l_line[3]).to_integer_32 then
								a_code.item := atoi (l_line)
								Result := 1
								js.return
							end
						end
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	ftp_endofresp_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_endofresp)
		end

	ftp_readresp (a_sockfd: INTEGER_32; a_pp: CE_POINTER [PD_PINGPONG]; a_ftpcode: CE_POINTER [INTEGER_32]; a_size: CE_POINTER [NATURAL_32]): INTEGER_32
		local
			l_conn: CE_POINTER [PD_CONNECTDATA]
			l_result: INTEGER_32
			l_code: INTEGER_32
		do
			create l_conn.make

			l_conn.ce_gets (a_pp.item.conn)
			l_result := {PC_CURLCODE}.CURLE_OK
			l_result := pingpong_inst.Curl_pp_readresp (a_sockfd, a_pp, create {CE_POINTER [INTEGER_32]}.make_shared ($l_code), a_size);
			(l_conn.item.data).item.info.httpcode := l_code
			if ((a_ftpcode).c_pointer /= default_pointer) then
				a_ftpcode.item := l_code
			end
			Result := l_result
		end

	Curl_GetFTPResponse (a_nreadp: CE_POINTER [INTEGER_32]; a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_ftpcode: CE_POINTER [INTEGER_32]
			l_sockfd: INTEGER_32
			l_timeout: INTEGER_32
			l_interval_ms: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_nread: NATURAL_32
			l_cache_skip: INTEGER_32
			l_value_to_be_ignored: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpcode.make
			create l_data.make
			create l_ftpc.make
			create l_pp.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_ftpcode.ce_gets (a_ftpcode)
					l_sockfd := a_conn.item.sock[0]
					l_data.ce_gets (a_conn.item.data)
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
					l_cache_skip := 0
					l_value_to_be_ignored := 0
					if ((l_ftpcode).c_pointer /= default_pointer) then
						l_ftpcode.item := 0
					else
						l_ftpcode.ce_gets (create {CE_POINTER [INTEGER_32]}.make_shared ($l_value_to_be_ignored))
					end
					a_nreadp.item := 0
				when 1 then -- loop condition:
					if js.loop_left then
						l_pos := -1 + 13 -- skip loop
					end
				when 2 then -- loop body:
						js.reset
						if not (l_ftpcode.item).to_boolean then
							if l_result.to_boolean then
								js.break
							end
						else
							js.break
						end
						if js.loop_jumped then
							l_pos := -1 + 12 -- skip if block
						end
				when 3 then -- if:
						l_timeout := pingpong_inst.Curl_pp_state_timeout (l_pp)
						if l_timeout <= 0 then
							sendf_inst.Curl_failf ([l_data, ce_string ("FTP response timeout")])
							Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 12 -- skip if block
						end
				when 4 then -- if:
						l_interval_ms := 1000
						if l_timeout < l_interval_ms then
							l_interval_ms := l_timeout
						end
						if (l_pp.item.cache).c_pointer = default_pointer then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if not (l_cache_skip < 2) then
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L:
						inspect (select_inst.Curl_socket_ready (l_sockfd, -1, l_interval_ms.to_integer_32)) -- switch 1 head
						when -1 then l_pos := -1 + 8
						when 0 then l_pos := -1 + 9
						else l_pos := -1 + 10
						end
				when 8 then -- switch 1, case -1
						sendf_inst.Curl_failf ([l_data, ce_string ("FTP response aborted due to select/poll error: %%d"), (errno_location).item])
						Result := ({PC_CURLCODE}.CURLE_RECV_ERROR)
						js.return
				when 9 then -- switch 1, case 0
						if not js.return_called then
							if (progress_inst.Curl_pgrsUpdate (a_conn)).to_boolean then
								Result := ({PC_CURLCODE}.CURLE_ABORTED_BY_CALLBACK)
								js.return
							end
						end
						if not js.return_called then
							js.continue
						end
				when 10 then -- switch 1, default:
						l_pos := -1 + 11 -- switch 1; break
				when 11 then -- switch 1, done:
						if not js.loop_jumped then
							l_result := ftp_readresp (l_sockfd, l_pp, l_ftpcode, create {CE_POINTER [NATURAL_32]}.make_shared ($l_nread))
							if l_result.to_boolean then
								js.break
							end
						end
						if not js.loop_jumped then
							if not l_nread.to_boolean then
								if ((l_pp.item.cache).c_pointer /= default_pointer) then
									l_cache_skip := l_cache_skip + 1
								else
									l_cache_skip := 0
								end
							else
								l_cache_skip := 0
							end
							a_nreadp.item := ((a_nreadp.item).to_natural_32 + l_nread).to_integer_32
						end
				when 12 then -- if done | if done
						l_pos := -1 + 1 -- back to loop condition
				when 13 then -- loop done
					js.reset
					if not js.return_called then
						l_pp.item.pending_resp := (0).to_boolean
						Result := l_result
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	state (a_conn: CE_POINTER [PD_CONNECTDATA]; a_newstate: INTEGER_32)
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
		do
			create l_ftpc.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_ftpc.item.state := a_newstate
		end

	ftp_state_user (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_tmp: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_tmp.make

			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			if ((l_ftp.item.user).c_pointer /= default_pointer) then
				l_tmp.ce_gets (l_ftp.item.user)
			else
				l_tmp.ce_gets (ce_string (""))
			end
			l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("USER %%s"), l_tmp])
			if l_result.to_natural_32 /= {NATURAL_32} 0 then
				Result := l_result
				js.return
			end
			if not js.return_called then
				state (a_conn, {PC_FTPSTATE}.FTP_USER);
				(a_conn.item.data).item.state.ftp_trying_alternative := (0).to_boolean
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	ftp_state_pwd (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			js: JUMP_STATE
		do
			l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("PWD"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
			if l_result.to_natural_32 /= {NATURAL_32} 0 then
				Result := l_result
				js.return
			end
			if not js.return_called then
				state (a_conn, {PC_FTPSTATE}.FTP_PWD)
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	ftp_getsock (a_conn: CE_POINTER [PD_CONNECTDATA]; a_socks: CE_POINTER [INTEGER_32]; a_numsocks: INTEGER_32): INTEGER_32
		do
			Result := (pingpong_inst.Curl_pp_getsock (create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), a_socks, a_numsocks))
		end

	ftp_getsock_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_getsock)
		end

	ftp_state_cwd (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpc.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					if not (l_ftpc.item.cwddone) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_result := ftp_state_post_cwd (a_conn)
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						l_ftpc.item.count2 := 0
						if (a_conn.item.data).item.set.ftp_create_missing_dirs = 2 then
							l_ftpc.item.count3 := 1
						else
							l_ftpc.item.count3 := 0
						end
						if not (a_conn.item.bits.reuse) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if ((l_ftpc.item.entrypath).c_pointer /= default_pointer) then
							l_ftpc.item.count1 := 0
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("CWD %%s"), l_ftpc.item.entrypath])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_CWD)
							end
						else
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:
						if (l_ftpc.item.dirdepth).to_boolean then
							l_ftpc.item.count1 := 1
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("CWD %%s"), l_ftpc.item.dirs[(l_ftpc.item.count1 - 1)]])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_CWD)
							end
						else
							l_result := ftp_state_post_cwd (a_conn)
						end
				when 7 then -- if done
						if not js.return_called then
							Result := l_result
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_use_port (a_conn: CE_POINTER [PD_CONNECTDATA]; a_fcmd: INTEGER_32): INTEGER_32
		local
			l_fcmd: INTEGER_32
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_portsock: INTEGER_32
			l_myhost: CE_ARRAY [INTEGER_8]
			l_ss: PD_CURL_SOCKADDR_STORAGE
			l_res: CE_POINTER [PD_CURL_ADDRINFO]
			l_ai: CE_POINTER [PD_CURL_ADDRINFO]
			l_sslen: NATURAL_32
			l_hbuf: CE_ARRAY [INTEGER_8]
			l_sa: CE_POINTER [SD_SOCKADDR]
			l_sa4: CE_POINTER [SD_SOCKADDR_IN]
			l_sa6: CE_POINTER [SD_SOCKADDR_IN6]
			l_tmp: CE_ARRAY [INTEGER_8]
			l_rc: INTEGER_32
			l_error: INTEGER_32
			l_host: CE_POINTER [INTEGER_8]
			l_string_ftpport: CE_POINTER [INTEGER_8]
			l_h: CE_POINTER [PD_CURL_DNS_ENTRY]
			l_port_min: NATURAL_16
			l_port_max: NATURAL_16
			l_port: NATURAL_16
			l_addr: CE_POINTER [INTEGER_8]
			l_addrlen: NATURAL_32
			l_tmp___2: NATURAL_32
			l_ip_start: CE_POINTER [INTEGER_8]
			l_ip_end: CE_POINTER [INTEGER_8]
			l_port_start: CE_POINTER [INTEGER_8]
			l_port_sep: CE_POINTER [INTEGER_8]
			l_tmp___28: INTEGER_32
			l_source: CE_POINTER [INTEGER_8]
			l_dest: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpc.make
			create l_data.make
			create l_myhost.make (<<256>>)
			create l_ss.make
			create l_res.make
			create l_ai.make
			create l_hbuf.make (<<1025>>)
			create l_sa.make
			create l_sa4.make
			create l_sa6.make
			create l_tmp.make (<<1024>>)
			create l_host.make
			create l_string_ftpport.make
			create l_h.make
			create l_addr.make
			create l_ip_start.make
			create l_ip_end.make
			create l_port_start.make
			create l_port_sep.make
			create l_source.make
			create l_dest.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_fcmd := a_fcmd
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_data.ce_gets (a_conn.item.data)
					l_portsock := -1
					l_myhost[0] := (('%/0/').code).to_integer_8
					l_sa.ce_gets (create {CE_POINTER [SD_SOCKADDR]}.make_cast ((l_ss.address)))
					l_sa4.ce_gets (create {CE_POINTER [SD_SOCKADDR_IN]}.make_cast (l_sa))
					l_sa6.ce_gets (create {CE_POINTER [SD_SOCKADDR_IN6]}.make_cast (l_sa))
					l_host.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					l_string_ftpport.ce_gets (l_data.item.set.str[9])
					l_h.ce_gets (create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer))
					l_port_min := (0).to_natural_16
					l_port_max := (0).to_natural_16
					l_addr.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					if ((l_data.item.set.str[9]).c_pointer /= default_pointer) then
						if (strlen (l_data.item.set.str[9])) > {NATURAL_32} 1 then
							if {NATURAL_32} 46 > (strlen (l_string_ftpport)) then
								l_tmp___2 := (46).to_natural_32
							else
								l_tmp___2 := (strlen (l_string_ftpport))
							end
							l_addrlen := l_tmp___2
							l_ip_start.ce_gets (l_string_ftpport)
							l_ip_end.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							l_port_start.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							l_port_sep.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							l_addr.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_ccalloc.pointer_item ([l_addrlen + {NATURAL_32} 1, (1).to_natural_32])))))
							if (l_addr).c_pointer = default_pointer then
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								js.return
							end
							if not js.return_called then
								if (l_string_ftpport[0]).to_integer_32 = 91 then
									l_ip_start.ce_gets (l_string_ftpport + 1)
									l_ip_end.ce_gets (strchr (l_string_ftpport, (']').code))
									if (l_ip_end.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32 then
										strncpy (l_addr, l_ip_start, (l_ip_end.distance (l_ip_start)).to_natural_32).do_nothing
									end
								elseif (l_string_ftpport[0]).to_integer_32 = 58 then
									l_ip_end.ce_gets (l_string_ftpport)
								else
									l_ip_end.ce_gets (strchr (l_string_ftpport, (':').code))
									if (l_ip_end.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32 then
										if (inet_pton (10, l_string_ftpport, create {CE_POINTER [ANY]}.make_cast (l_sa6))) = 1 then
											l_port_max := (0).to_natural_16
											l_port_min := l_port_max
											strcpy (l_addr, l_string_ftpport).do_nothing
											l_ip_end.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
										else
											strncpy (l_addr, l_string_ftpport, (l_ip_end.distance (l_ip_start)).to_natural_32).do_nothing
										end
									else
										strcpy (l_addr, l_string_ftpport).do_nothing
									end
								end
								if (l_ip_end.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32 then
									l_port_start.ce_gets (strchr (l_ip_end, (':').code))
									if (l_port_start.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32 then
										l_port_min := warnless_inst.curlx_ultous ((strtoul ((l_port_start + 1), create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer), 10)))
										l_port_sep.ce_gets (strchr (l_port_start, ('-').code))
										if (l_port_sep.c_pointer.to_integer_32).to_natural_32 /= (0).to_natural_32 then
											l_port_max := warnless_inst.curlx_ultous ((strtoul ((l_port_sep + 1), create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer), 10)))
										else
											l_port_max := l_port_min
										end
									end
								end
								if l_port_min.to_integer_32 > l_port_max.to_integer_32 then
									l_port_max := (0).to_natural_16
									l_port_min := l_port_max
								end
								if (l_addr[0]).to_integer_32 /= 0 then
									if (((if2ip_inst.Curl_if2ip ((a_conn.item.ip_addr).item.ai_family, l_addr, l_hbuf, (Integer_8_bytes.to_natural_32 * (1025).to_natural_32).to_integer_32))).c_pointer /= default_pointer) then
										l_host.ce_gets (l_hbuf)
									else
										l_host.ce_gets (l_addr)
									end
								else
									l_host.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
								end
							end
						end
					end
					if not (not js.return_called and then ((l_host).c_pointer = default_pointer)) then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_sslen := {PD_CURL_SOCKADDR_STORAGE}.sizeof.to_natural_32
						if (getsockname (a_conn.item.sock[0], l_sa, (create {CE_POINTER [NATURAL_32]}.make_shared ($l_sslen)))).to_boolean then
							sendf_inst.Curl_failf ([l_data, ce_string ("getsockname() failed: %%s"), (strerror_inst.Curl_strerror (a_conn, (errno_location).item))])
							if ((l_addr).c_pointer /= default_pointer) then
								easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_addr)])
							end
							Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 6 -- skip if block
						end
				when 2 then -- if:
						inspect (l_sa.item.sa_family).to_integer_32 -- switch 1 head
						when 10 then l_pos := -1 + 3
						else l_pos := -1 + 4
						end
				when 3 then -- switch 1, case 10
						inet_ntop ((l_sa.item.sa_family).to_integer_32, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN6_ADDR]}.make_shared (l_sa6.c_pointer + {SD_SOCKADDR_IN6}.sin6_addr_offset))), l_hbuf, (Integer_8_bytes.to_natural_32 * (1025).to_natural_32)).do_nothing
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, default:
						inet_ntop ((l_sa.item.sa_family).to_integer_32, create {CE_POINTER [ANY]}.make_cast ((create {CE_POINTER [SD_IN_ADDR]}.make_shared (l_sa4.c_pointer + {SD_SOCKADDR_IN}.sin_addr_offset))), l_hbuf, (Integer_8_bytes.to_natural_32 * (1025).to_natural_32)).do_nothing
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
						l_host.ce_gets (l_hbuf)
				when 6 then -- if done | if done
						if not js.return_called then
							l_rc := hostip_inst.Curl_resolv (a_conn, l_host, 0, l_h.address)
							if l_rc = 1 then
								hostsyn_inst.Curl_wait_for_resolv (a_conn, l_h.address).do_nothing
							end
							if ((l_h).c_pointer /= default_pointer) then
								l_res.ce_gets (l_h.item.addr)
								hostip_inst.Curl_resolv_unlock (l_data, l_h)
							else
								l_res.ce_gets (create {CE_POINTER [PD_CURL_ADDRINFO]}.make_shared (default_pointer))
							end
							if ((l_addr).c_pointer /= default_pointer) then
								easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_addr)])
							end
							if (l_res.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
								sendf_inst.Curl_failf ([l_data, ce_string ("Curl_resolv failed, we can not recover!")])
								Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
								js.return
							end
						end
						if not js.return_called then
							l_portsock := -1
							l_error := 0
							l_ai.ce_gets (l_res)
							from
							until
								not (not js.loop_left and then ((l_ai).c_pointer /= default_pointer))
							loop
								js.reset
								if l_ai.item.ai_socktype = 0 then
									l_ai.item.ai_socktype := a_conn.item.socktype
								end
								l_portsock := socket (l_ai.item.ai_family, l_ai.item.ai_socktype, l_ai.item.ai_protocol)
								if l_portsock = -1 then
									l_error := (errno_location).item
									js.continue
								end
								if not js.loop_jumped then
									js.break
								end
								if not js.loop_left then
									l_ai.ce_gets (l_ai.item.ai_next)
								end
							end
							js.reset
							if (l_ai).c_pointer = default_pointer then
								sendf_inst.Curl_failf ([l_data, ce_string ("socket failure: %%s"), (strerror_inst.Curl_strerror (a_conn, l_error))])
								Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
								js.return
							end
						end
						if not js.return_called then
							memcpy (create {CE_POINTER [ANY]}.make_cast (l_sa), create {CE_POINTER [ANY]}.make_cast (l_ai.item.ai_addr), l_ai.item.ai_addrlen).do_nothing
							l_sslen := l_ai.item.ai_addrlen
							l_port := l_port_min
							from
							until
								not (not js.loop_left and then (l_port.to_integer_32 <= l_port_max.to_integer_32))
							loop
								js.reset
								if (l_sa.item.sa_family).to_integer_32 = 2 then
									l_sa4.item.sin_port := htons (l_port)
								else
									l_sa6.item.sin6_port := htons (l_port)
								end
								if (bind (l_portsock, l_sa, l_sslen)).to_boolean then
									l_error := (errno_location).item
									if l_error = 99 then
										sendf_inst.Curl_failf ([l_data, ce_string ("bind(port=%%hu) failed: %%s"), l_port.to_integer_32, (strerror_inst.Curl_strerror (a_conn, l_error))])
										l_sslen := {PD_CURL_SOCKADDR_STORAGE}.sizeof.to_natural_32
										if (getsockname (a_conn.item.sock[0], l_sa, (create {CE_POINTER [NATURAL_32]}.make_shared ($l_sslen)))).to_boolean then
											sendf_inst.Curl_failf ([l_data, ce_string ("getsockname() failed: %%s"), (strerror_inst.Curl_strerror (a_conn, (errno_location).item))])
											close (l_portsock).do_nothing
											Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
											js.return
										end
										if not js.return_called then
											l_port := l_port_min
											js.continue
										end
									elseif l_error /= 98 then
										if l_error /= 13 then
											sendf_inst.Curl_failf ([l_data, ce_string ("bind(port=%%hu) failed: %%s"), l_port.to_integer_32, (strerror_inst.Curl_strerror (a_conn, l_error))])
											close (l_portsock).do_nothing
											Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
											js.return
										end
									end
								else
									js.break
								end
								if not js.return_called then
									if not js.loop_jumped then
										l_port := (l_port.to_integer_32 + 1).to_natural_16
									end
									if not js.loop_left then
										
									end
								end
							end
							js.reset
						end
						if not js.return_called and then (l_port.to_integer_32 > l_port_max.to_integer_32) then
							sendf_inst.Curl_failf ([l_data, ce_string ("bind() failed, we ran out of ports!")])
							close (l_portsock).do_nothing
							Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
							js.return
						end
						if not js.return_called then
							l_sslen := {PD_CURL_SOCKADDR_STORAGE}.sizeof.to_natural_32
							if (getsockname (l_portsock, l_sa, (create {CE_POINTER [NATURAL_32]}.make_shared ($l_sslen)))).to_boolean then
								sendf_inst.Curl_failf ([l_data, ce_string ("getsockname() failed: %%s"), (strerror_inst.Curl_strerror (a_conn, (errno_location).item))])
								close (l_portsock).do_nothing
								Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
								js.return
							end
						end
						if not js.return_called then
							if (listen (l_portsock, 1)).to_boolean then
								sendf_inst.Curl_failf ([l_data, ce_string ("socket failure: %%s"), (strerror_inst.Curl_strerror (a_conn, (errno_location).item))])
								close (l_portsock).do_nothing
								Result := ({PC_CURLCODE}.CURLE_FTP_PORT_FAILED)
								js.return
							end
						end
						if js.return_called then
							l_pos := -1 + 16 -- skip if block
						end
				when 7 then -- if:
							hostip_inst.Curl_printable_address (l_ai, l_myhost, (Integer_8_bytes.to_natural_32 * (256).to_natural_32)).do_nothing
							if not a_conn.item.bits.ftp_use_eprt then
								if a_conn.item.bits.ipv6 then
									a_conn.item.bits.ftp_use_eprt := (1).to_boolean
								end
							end
				when 8 then -- loop condition:
							if not (not js.loop_left and then (l_fcmd.to_natural_32 /= {NATURAL_32} 2)) then
								l_pos := -1 + 15 -- skip loop
							end
				when 9 then -- loop body:
							js.reset
							if not a_conn.item.bits.ftp_use_eprt then
								if {NATURAL_32} 0 = l_fcmd.to_natural_32 then
									js.continue
								end
							end
							if not js.loop_jumped and then ({NATURAL_32} 1 = l_fcmd.to_natural_32) then
								if (l_sa.item.sa_family).to_integer_32 /= 2 then
									js.continue
								end
							end
							if js.loop_jumped then
								l_pos := -1 + 14 -- skip if block
							end
				when 10 then -- if:
							inspect (l_sa.item.sa_family).to_integer_32 -- switch 2 head
							when 2 then l_pos := -1 + 11
							when 10 then l_pos := -1 + 12
							else l_pos := -1 + 13
							end
				when 11 then -- switch 2, case 2
							l_port := ntohs (l_sa4.item.sin_port)
							l_pos := -1 + 14 -- switch 2; break
				when 12 then -- switch 2, case 10
							l_port := ntohs (l_sa6.item.sin6_port)
							l_pos := -1 + 14 -- switch 2; break
				when 13 then -- switch 2, default:
							js.continue
				when 14 then -- switch 2, done: | if done
							if not js.loop_jumped then
								if {NATURAL_32} 0 = l_fcmd.to_natural_32 then
									if (l_sa.item.sa_family).to_integer_32 = 2 then
										l_tmp___28 := 1
									else
										l_tmp___28 := 2
									end
									l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("%%s |%%d|%%s|%%hu|"), mode[l_fcmd], l_tmp___28, l_myhost, l_port.to_integer_32])
									if l_result.to_boolean then
										Result := l_result
										js.return
									end
									if not js.return_called then
										js.break
									end
								elseif {NATURAL_32} 1 = l_fcmd.to_natural_32 then
									l_source.ce_gets (l_myhost)
									l_dest.ce_gets (l_tmp)
									from
									until
										js.loop_left
									loop
										js.reset
										if ((l_source).c_pointer /= default_pointer) then
											if not (l_source[0]).to_boolean then
												js.break
											end
										else
											js.break
										end
										if not js.loop_jumped then
											if (l_source[0]).to_integer_32 = 46 then
												l_dest[0] := ((',').code).to_integer_8
											else
												l_dest[0] := l_source[0]
											end
											l_dest.ce_gets (l_dest + 1)
											l_source.ce_gets (l_source + 1)
										end
									end
									js.reset
									l_dest[0] := (0).to_integer_8
									mprintf_inst.curl_msnprintf ([l_dest, (20).to_natural_32, ce_string (",%%d,%%d"), (l_port.to_integer_32).bit_shift_right (8), (l_port.to_integer_32).bit_and (255)]).do_nothing
									l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("%%s %%s"), mode[l_fcmd], l_tmp])
									if l_result.to_boolean then
										Result := l_result
										js.return
									end
									if not js.return_called then
										js.break
									end
								end
							end
							if not js.loop_left then
								l_fcmd := (l_fcmd.to_natural_32 + {NATURAL_32} 1).to_integer_32
							end
							l_pos := -1 + 8 -- back to loop condition
				when 15 then -- loop done
							js.reset
				when 16 then -- if done
						if not js.return_called then
							l_ftpc.item.count1 := l_fcmd
							if -1 /= a_conn.item.sock[1] then
								close (a_conn.item.sock[1]).do_nothing
							end
							a_conn.item.sock[1] := l_portsock
							a_conn.item.bits.tcpconnect := (1).to_boolean
							state (a_conn, {PC_FTPSTATE}.FTP_PORT)
							Result := l_result
						end
				when 17 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_use_pasv (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_result: INTEGER_32
			l_modeoff: INTEGER_32
			js: JUMP_STATE
		do
			create l_ftpc.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_result := {PC_CURLCODE}.CURLE_OK
			if not a_conn.item.bits.ftp_use_epsv then
				if a_conn.item.bits.ipv6 then
					a_conn.item.bits.ftp_use_epsv := (1).to_boolean
				end
			end
			if a_conn.item.bits.ftp_use_epsv then
				l_modeoff := 0
			else
				l_modeoff := 1
			end
			l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("%%s"), mode___0[l_modeoff]])
			if l_result.to_natural_32 /= {NATURAL_32} 0 then
				Result := l_result
				js.return
			end
			if not js.return_called then
				l_ftpc.item.count1 := l_modeoff
				state (a_conn, {PC_FTPSTATE}.FTP_PASV)
				sendf_inst.Curl_infof ([a_conn.item.data, ce_string ("Connect data stream passively%N")])
				Result := l_result
			end
		end

	ftp_state_post_rest (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_tmp: CE_POINTER [INTEGER_8]
			l_tmp___0: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_data.make
			create l_tmp.make
			create l_tmp___0.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			l_data.ce_gets (a_conn.item.data)
			if (l_ftp.item.transfer).to_natural_32 /= {NATURAL_32} 0 then
				state (a_conn, {PC_FTPSTATE}.FTP_RETR_PREQUOTE)
				l_result := ftp_state_quote (a_conn, (1).to_boolean, {PC_FTPSTATE}.FTP_RETR_PREQUOTE)
			elseif l_data.item.set.ftp_use_port then
				l_result := ftp_state_use_port (a_conn, {PC_FTPPORT}.EPRT)
			elseif l_data.item.set.ftp_use_pret then
				if (a_conn.item.proto.ftpc.file).c_pointer = default_pointer then
					if ((l_data.item.set.str[4]).c_pointer /= default_pointer) then
						l_tmp___0.ce_gets (l_data.item.set.str[4])
					else
						if l_data.item.set.ftp_list_only then
							l_tmp.ce_gets (ce_string ("NLST"))
						else
							l_tmp.ce_gets (ce_string ("LIST"))
						end
						l_tmp___0.ce_gets (l_tmp)
					end
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("PRET %%s"), l_tmp___0])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
				elseif l_data.item.set.upload then
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("PRET STOR %%s"), a_conn.item.proto.ftpc.file])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
				else
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("PRET RETR %%s"), a_conn.item.proto.ftpc.file])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
				end
				if not js.return_called then
					state (a_conn, {PC_FTPSTATE}.FTP_PRET)
				end
			else
				l_result := ftp_state_use_pasv (a_conn)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_post_size (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if (l_ftp.item.transfer).to_natural_32 /= {NATURAL_32} 0 then
				if ((l_ftpc.item.file).c_pointer /= default_pointer) then
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("REST %%d"), 0])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
					if not js.return_called then
						state (a_conn, {PC_FTPSTATE}.FTP_REST)
					end
				else
					l_result := ftp_state_post_rest (a_conn)
				end
			else
				l_result := ftp_state_post_rest (a_conn)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_post_type (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if (l_ftp.item.transfer).to_natural_32 = {NATURAL_32} 1 then
				if ((l_ftpc.item.file).c_pointer /= default_pointer) then
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("SIZE %%s"), l_ftpc.item.file])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
					if not js.return_called then
						state (a_conn, {PC_FTPSTATE}.FTP_SIZE)
					end
				else
					l_result := ftp_state_post_size (a_conn)
				end
			else
				l_result := ftp_state_post_size (a_conn)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_post_listtype (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_cmd: CE_POINTER [INTEGER_8]
			l_lstArg: CE_POINTER [INTEGER_8]
			l_slashPos: CE_POINTER [INTEGER_8]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: CE_POINTER [INTEGER_8]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_data.make
			create l_cmd.make
			create l_lstArg.make
			create l_slashPos.make
			create l_tmp___1.make
			create l_tmp___2.make
			create l_tmp___3.make
			create l_tmp___4.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			l_lstArg.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			if (l_data.item.set.ftp_filemethod).to_natural_32 = {NATURAL_32} 2 then
				if ((l_data.item.state.path).c_pointer /= default_pointer) then
					if (l_data.item.state.path[0]).to_boolean then
						if (((strchr (l_data.item.state.path, ('/').code))).c_pointer /= default_pointer) then
							l_lstArg.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([l_data.item.state.path])))
							if (l_lstArg).c_pointer = default_pointer then
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								js.return
							end
							if not js.return_called then
								if (l_lstArg[((strlen (l_lstArg)) - {NATURAL_32} 1).to_integer_32]).to_integer_32 /= 47 then
									l_slashPos.ce_gets (strrchr (l_lstArg, ('/').code))
									if ((l_slashPos).c_pointer /= default_pointer) then
										l_slashPos[1] := (('%/0/').code).to_integer_8
									end
								end
							end
						end
					end
				end
			end
			if not js.return_called then
				if ((l_lstArg).c_pointer /= default_pointer) then
					l_tmp___1.ce_gets (l_lstArg)
				else
					l_tmp___1.ce_gets (ce_string (""))
				end
				if ((l_lstArg).c_pointer /= default_pointer) then
					l_tmp___2.ce_gets (ce_string (" "))
				else
					l_tmp___2.ce_gets (ce_string (""))
				end
				if ((l_data.item.set.str[4]).c_pointer /= default_pointer) then
					l_tmp___4.ce_gets (l_data.item.set.str[4])
				else
					if l_data.item.set.ftp_list_only then
						l_tmp___3.ce_gets (ce_string ("NLST"))
					else
						l_tmp___3.ce_gets (ce_string ("LIST"))
					end
					l_tmp___4.ce_gets (l_tmp___3)
				end
				l_cmd.ce_gets (mprintf_inst.curl_maprintf ([ce_string ("%%s%%s%%s"), l_tmp___4, l_tmp___2, l_tmp___1]))
				if (l_cmd).c_pointer = default_pointer then
					if ((l_lstArg).c_pointer /= default_pointer) then
						easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_lstArg)])
					end
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
			end
			if not js.return_called then
				l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("%%s"), l_cmd])
				if l_result.to_natural_32 /= {NATURAL_32} 0 then
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				if ((l_lstArg).c_pointer /= default_pointer) then
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_lstArg)])
				end
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_cmd)])
				state (a_conn, {PC_FTPSTATE}.FTP_LIST)
				Result := l_result
			end
		end

	ftp_state_post_retrtype (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			l_result := ftp_state_quote (a_conn, (1).to_boolean, {PC_FTPSTATE}.FTP_RETR_PREQUOTE)
			Result := l_result
		end

	ftp_state_post_stortype (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			l_result := ftp_state_quote (a_conn, (1).to_boolean, {PC_FTPSTATE}.FTP_STOR_PREQUOTE)
			Result := l_result
		end

	ftp_state_post_mdtm (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_data.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			l_data.ce_gets (a_conn.item.data)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if l_data.item.set.opt_no_body then
				if ((l_ftpc.item.file).c_pointer /= default_pointer) then
					if (ftp_need_type (a_conn, l_data.item.set.prefer_ascii)).to_boolean then
						l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_INFO
						l_result := ftp_nb_type (a_conn, l_data.item.set.prefer_ascii, {PC_FTPSTATE}.FTP_TYPE)
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
					else
						l_result := ftp_state_post_type (a_conn)
					end
				else
					l_result := ftp_state_post_type (a_conn)
				end
			else
				l_result := ftp_state_post_type (a_conn)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_post_cwd (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftpc.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_data.ce_gets (a_conn.item.data)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					if not (l_data.item.set.get_filetime) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not ((l_data.item.set.timecondition).to_boolean) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						if ((l_ftpc.item.file).c_pointer /= default_pointer) then
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("MDTM %%s"), l_ftpc.item.file])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_MDTM)
							end
						else
							l_result := ftp_state_post_mdtm (a_conn)
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_result := ftp_state_post_mdtm (a_conn)
				when 7 then -- if done
						if not js.return_called then
							Result := l_result
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_ul_setup (a_conn: CE_POINTER [PD_CONNECTDATA]; a_sizechecked: BOOLEAN): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_seekerr: INTEGER_32
			l_passed: INTEGER_64
			l_readthisamountnow: INTEGER_64
			l_actuallyread: INTEGER_64
			l_tmp___0: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftp.make
			create l_data.make
			create l_ftpc.make
			create l_tmp___0.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
					l_data.ce_gets (a_conn.item.data)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_seekerr := 0
					if not ((l_data.item.state.resume_from).to_boolean) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if not a_sizechecked then
							l_pos := -1 + 5 -- goto _L
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 3 then -- else: | _L___0:
						if not (l_data.item.state.resume_from > {INTEGER_64} 0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if not a_sizechecked then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						if l_data.item.state.resume_from < {INTEGER_64} 0 then
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("SIZE %%s"), l_ftpc.item.file])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_STOR_SIZE)
								Result := l_result
								js.return
							end
						end
						if not js.return_called then
							l_data.item.set.ftp_append := (1).to_boolean
							if ((a_conn.item.seek_func).c_pointer /= default_pointer) then
								l_seekerr := (a_conn.item.seek_func).integer_32_item ([a_conn.item.seek_client, l_data.item.state.resume_from, 0])
							end
							if l_seekerr /= 0 then
								if l_seekerr /= 2 then
									sendf_inst.Curl_failf ([l_data, ce_string ("Could not seek stream")])
									Result := ({PC_CURLCODE}.CURLE_FTP_COULDNT_USE_REST)
									js.return
								else
									l_passed := (0).to_integer_64
									from
									until
										js.loop_left
									loop
										js.reset
										l_readthisamountnow := l_data.item.state.resume_from - l_passed
										if l_readthisamountnow > {INTEGER_64} 16384 then
											l_readthisamountnow := (16384).to_integer_64
										end
										l_actuallyread := ((a_conn.item.fread_func).natural_32_item ([l_data.item.state.buffer, (1).to_natural_32, l_readthisamountnow.to_natural_32, a_conn.item.fread_in])).to_integer_64
										l_passed := l_passed + l_actuallyread
										if l_actuallyread <= {INTEGER_64} 0 then
											sendf_inst.Curl_failf ([l_data, ce_string ("Failed to read data")])
											Result := ({PC_CURLCODE}.CURLE_FTP_COULDNT_USE_REST)
											js.return
										elseif l_actuallyread > l_readthisamountnow then
											sendf_inst.Curl_failf ([l_data, ce_string ("Failed to read data")])
											Result := ({PC_CURLCODE}.CURLE_FTP_COULDNT_USE_REST)
											js.return
										end
										if not js.return_called and then (not (l_passed < l_data.item.state.resume_from)) then
											js.break
										end
									end
									js.reset
								end
							end
						end
						if not js.return_called and then (l_data.item.set.infilesize > {INTEGER_64} 0) then
							l_data.item.set.infilesize := l_data.item.set.infilesize - (l_data.item.state.resume_from)
							if l_data.item.set.infilesize <= {INTEGER_64} 0 then
								sendf_inst.Curl_infof ([l_data, ce_string ("File already completely uploaded%N")])
								transfer_inst.Curl_setup_transfer (a_conn, -1, (-1).to_integer_64, (0).to_boolean, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer), -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
								l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_NONE
								state (a_conn, {PC_FTPSTATE}.FTP_STOP)
								Result := ({PC_CURLCODE}.CURLE_OK)
								js.return
							end
						end
				when 6 then -- if done | if done
						if not js.return_called then
							if l_data.item.set.ftp_append then
								l_tmp___0.ce_gets (ce_string ("APPE %%s"))
							else
								l_tmp___0.ce_gets (ce_string ("STOR %%s"))
							end
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), l_tmp___0, l_ftpc.item.file])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
						end
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_STOR)
							Result := l_result
						end
				when 7 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_quote (a_conn: CE_POINTER [PD_CONNECTDATA]; a_init: BOOLEAN; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_quote: BOOLEAN
			l_item: CE_POINTER [PD_CURL_SLIST]
			l_i: INTEGER_32
			l_cmd: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make
			create l_item.make
			create l_cmd.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_quote := (0).to_boolean
					inspect a_instate -- switch 1 head
					when 13 then l_pos := -1 + 2
					when 14 then l_pos := -1 + 2
					when 15 then l_pos := -1 + 3
					else l_pos := -1 + 1
					end
				when 1 then -- switch 1, default:
					l_item.ce_gets (l_data.item.set.quote)
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, case 13 | switch 1, case 14
					l_item.ce_gets (l_data.item.set.prequote)
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, case 15
					l_item.ce_gets (l_data.item.set.postquote)
					l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
					if a_init then
						l_ftpc.item.count1 := 0
					else
						l_ftpc.item.count1 := (l_ftpc.item.count1) + 1
					end
					if ((l_item).c_pointer /= default_pointer) then
						l_i := 0
						from
						until
							js.loop_left
						loop
							js.reset
							if l_i < l_ftpc.item.count1 then
								if (l_item).c_pointer = default_pointer then
									js.break
								end
							else
								js.break
							end
							if not js.loop_jumped then
								l_item.ce_gets (l_item.item.next)
								l_i := l_i + 1
							end
						end
						js.reset
						if ((l_item).c_pointer /= default_pointer) then
							l_cmd.ce_gets (l_item.item.data)
							if (l_cmd[0]).to_integer_32 = 42 then
								l_cmd.ce_gets (l_cmd + 1)
								l_ftpc.item.count2 := 1
							else
								l_ftpc.item.count2 := 0
							end
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("%%s"), l_cmd])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, a_instate)
								l_quote := (1).to_boolean
							end
						end
					end
					if not (not js.return_called and then (not l_quote)) then
						l_pos := -1 + 10 -- skip if block
					end
				when 5 then -- if:
						inspect a_instate -- switch 2 head
						when 13 then l_pos := -1 + 7
						when 14 then l_pos := -1 + 8
						when 15 then l_pos := -1 + 9
						else l_pos := -1 + 6
						end
				when 6 then -- switch 2, default:
						l_result := ftp_state_cwd (a_conn)
						l_pos := -1 + 10 -- switch 2; break
				when 7 then -- switch 2, case 13
						if (l_ftp.item.transfer).to_natural_32 /= {NATURAL_32} 0 then
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						elseif l_ftpc.item.known_filesize /= -{INTEGER_64} 1 then
							progress_inst.Curl_pgrsSetDownloadSize (l_data, l_ftpc.item.known_filesize)
							l_result := ftp_state_post_retr_size (a_conn, l_ftpc.item.known_filesize)
						else
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("SIZE %%s"), l_ftpc.item.file])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_RETR_SIZE)
							end
						end
						l_pos := -1 + 10 -- switch 2; break
				when 8 then -- switch 2, case 14
						l_result := ftp_state_ul_setup (a_conn, (0).to_boolean)
						l_pos := -1 + 10 -- switch 2; break
				when 9 then -- switch 2, case 15
						l_pos := -1 + 10 -- switch 2; break
				when 10 then -- switch 2, done: | if done
						if not js.return_called then
							Result := l_result
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_pasv_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_conninfo: CE_POINTER [PD_CURL_ADDRINFO]
			l_addr: CE_POINTER [PD_CURL_DNS_ENTRY]
			l_rc: INTEGER_32
			l_connectport: NATURAL_16
			l_newport: NATURAL_16
			l_connected: BOOLEAN
			l_newhost: CE_ARRAY [INTEGER_8]
			l_str: CE_POINTER [INTEGER_8]
			l_ptr: CE_POINTER [INTEGER_8]
			l_num: NATURAL_32
			l_separator: CE_ARRAY [INTEGER_8]
			l_sep1: INTEGER_8
			l_i: INTEGER_32
			l_ip: CE_ARRAY [INTEGER_32]
			l_port: CE_ARRAY [INTEGER_32]
			l_http_proxy: PD_HTTP
			l_ftp_save: CE_POINTER [PD_FTP]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpc.make
			create l_data.make
			create l_conninfo.make
			create l_addr.make
			create l_newhost.make (<<48>>)
			create l_str.make
			create l_ptr.make
			create l_separator.make (<<4>>)
			create l_ip.make (<<4>>)
			create l_port.make (<<2>>)
			create l_http_proxy.make
			create l_ftp_save.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_data.ce_gets (a_conn.item.data)
					l_addr.ce_gets (create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer))
					l_newport := (0).to_natural_16
					l_str.ce_gets (l_data.item.state.buffer + 4)
					if l_ftpc.item.count1 /= 0 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_ftpcode = 229 then
							l_ptr.ce_gets ((strchr (l_str, ('(').code)))
							if ((l_ptr).c_pointer /= default_pointer) then
								l_ptr.ce_gets (l_ptr + 1)
								if 5 = (sscanf ([l_ptr, ce_string ("%%c%%c%%c%%u%%c"), l_separator + 0, l_separator + 1, l_separator + 2, create {CE_POINTER [NATURAL_32]}.make_shared ($l_num), l_separator + 3])) then
									l_sep1 := l_separator[0]
									l_i := 1
									from
									until
										not (not js.loop_left and then (l_i < 4))
									loop
										js.reset
										if (l_separator[l_i]).to_integer_32 /= l_sep1.to_integer_32 then
											l_ptr.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
											js.break
										end
										if not js.loop_jumped then
											l_i := l_i + 1
										end
									end
									js.reset
									if ((l_ptr).c_pointer /= default_pointer) then
										l_newport := ((l_num).bit_and ({NATURAL_32} 65535)).to_natural_16
										if a_conn.item.bits.tunnel_proxy then
											mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
										elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 5 then
											mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
										elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 7 then
											mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
										elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 4 then
											mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
										elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 6 then
											mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
										else
											mprintf_inst.curl_msnprintf ([l_newhost, (48).to_natural_32, ce_string ("%%s"), a_conn.item.ip_addr_str]).do_nothing
										end
									end
								else
									l_ptr.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
								end
							end
							if (l_ptr).c_pointer = default_pointer then
								sendf_inst.Curl_failf ([l_data, ce_string ("Weirdly formatted EPSV reply")])
								Result := ({PC_CURLCODE}.CURLE_FTP_WEIRD_PASV_REPLY)
								js.return
							end
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else: | _L___0:
						if l_ftpc.item.count1 /= 1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if a_ftpcode = 227 then
							from
							until
								not (not js.loop_left and then ((l_str[0]).to_boolean))
							loop
								js.reset
								if 6 = (sscanf ([l_str, ce_string ("%%d,%%d,%%d,%%d,%%d,%%d"), l_ip + 0, l_ip + 1, l_ip + 2, l_ip + 3, l_port + 0, l_port + 1])) then
									js.break
								end
								if not js.loop_jumped then
									l_str.ce_gets (l_str + 1)
								end
							end
							js.reset
							if not (l_str[0]).to_boolean then
								sendf_inst.Curl_failf ([l_data, ce_string ("Couldn%'t interpret the 227-response")])
								Result := ({PC_CURLCODE}.CURLE_FTP_WEIRD_227_FORMAT)
								js.return
							end
							if not js.return_called then
								if l_data.item.set.ftp_skip_ip then
									sendf_inst.Curl_infof ([l_data, ce_string ("Skips %%d.%%d.%%d.%%d for data connection, uses %%s instead%N"), l_ip[0], l_ip[1], l_ip[2], l_ip[3], a_conn.item.ip_addr_str])
									if a_conn.item.bits.tunnel_proxy then
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
									elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 5 then
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
									elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 7 then
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
									elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 4 then
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
									elseif (l_data.item.set.proxytype).to_natural_32 = {NATURAL_32} 6 then
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.host.name]).do_nothing
									else
										mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%s"), a_conn.item.ip_addr_str]).do_nothing
									end
								else
									mprintf_inst.curl_msnprintf ([l_newhost, (Integer_8_bytes.to_natural_32 * (48).to_natural_32), ce_string ("%%d.%%d.%%d.%%d"), l_ip[0], l_ip[1], l_ip[2], l_ip[3]]).do_nothing
								end
								l_newport := (((((l_port[0]).bit_shift_left (8)) + l_port[1])).bit_and (65535)).to_natural_16
							end
						else
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:
						if l_ftpc.item.count1 = 0 then
							a_conn.item.bits.ftp_use_epsv := (0).to_boolean
							sendf_inst.Curl_infof ([l_data, ce_string ("disabling EPSV usage%N")])
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("PASV"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								l_ftpc.item.count1 := (l_ftpc.item.count1) + 1
								Result := l_result
								js.return
							end
						else
							sendf_inst.Curl_failf ([l_data, ce_string ("Bad PASV/EPSV response: %%03d"), a_ftpcode])
							Result := ({PC_CURLCODE}.CURLE_FTP_WEIRD_PASV_REPLY)
							js.return
						end
				when 7 then -- if done
						if js.return_called then
							l_pos := -1 + 12 -- skip if block
						end
				when 8 then -- if:
							if (l_data.item.set.str[16]).c_pointer = default_pointer then
								l_pos := -1 + 11 -- skip if block
							end
				when 9 then -- if:
							if ((l_data.item.set.str[16]).item).to_boolean then
								l_rc := hostip_inst.Curl_resolv (a_conn, a_conn.item.proxy.name, (a_conn.item.port).to_integer_32, l_addr.address)
								if l_rc = 1 then
									hostsyn_inst.Curl_wait_for_resolv (a_conn, l_addr.address).do_nothing
								end
								l_connectport := (a_conn.item.port).to_natural_16
								if (l_addr).c_pointer = default_pointer then
									sendf_inst.Curl_failf ([l_data, ce_string ("Can%'t resolve proxy host %%s:%%hu"), a_conn.item.proxy.name, l_connectport.to_integer_32])
									Result := ({PC_CURLCODE}.CURLE_FTP_CANT_GET_HOST)
									js.return
								end
							else
								l_pos := -1 + 11 -- goto _L___1
							end
				when 10 then -- if block done, skip else block:
							l_pos := -1 + 12
				when 11 then -- else: | _L___1:
							l_rc := hostip_inst.Curl_resolv (a_conn, l_newhost, l_newport.to_integer_32, l_addr.address)
							if l_rc = 1 then
								hostsyn_inst.Curl_wait_for_resolv (a_conn, l_addr.address).do_nothing
							end
							l_connectport := l_newport
							if (l_addr).c_pointer = default_pointer then
								sendf_inst.Curl_failf ([l_data, ce_string ("Can%'t resolve new host %%s:%%hu"), l_newhost, l_connectport.to_integer_32])
								Result := ({PC_CURLCODE}.CURLE_FTP_CANT_GET_HOST)
								js.return
							end
				when 12 then -- if done
							if not js.return_called then
								l_result := connect_inst.Curl_connecthost (a_conn, l_addr, a_conn.item.sock + 1, l_conninfo.address, create {CE_POINTER [BOOLEAN]}.make_shared ($l_connected))
								hostip_inst.Curl_resolv_unlock (l_data, l_addr)
								if l_result.to_boolean then
									if l_ftpc.item.count1 = 0 then
										if a_ftpcode = 229 then
											sendf_inst.Curl_infof ([l_data, ce_string ("got positive EPSV response, but can%'t connect. Disabling EPSV%N")])
											a_conn.item.bits.ftp_use_epsv := (0).to_boolean
											l_data.item.state.errorbuf := (0).to_boolean
											l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("PASV"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
											if l_result.to_natural_32 /= {NATURAL_32} 0 then
												Result := l_result
												js.return
											end
											if not js.return_called then
												l_ftpc.item.count1 := (l_ftpc.item.count1) + 1
												Result := l_result
												js.return
											end
										end
									end
								end
							end
							if not js.return_called and then (l_result.to_boolean) then
								Result := l_result
								js.return
							end
							if js.return_called then
								l_pos := -1 + 20 -- skip if block
							end
				when 13 then -- if:
								a_conn.item.bits.tcpconnect := l_connected
								if l_data.item.set.verbose then
									ftp_pasv_verbose (a_conn, l_conninfo, l_newhost, l_connectport.to_integer_32)
								end
								inspect l_data.item.set.proxytype -- switch 1 head
								when 5 then l_pos := -1 + 14
								when 7 then l_pos := -1 + 14
								when 4 then l_pos := -1 + 15
								when 6 then l_pos := -1 + 16
								when 0 then l_pos := -1 + 17
								when 1 then l_pos := -1 + 17
								else l_pos := -1 + 18
								end
				when 14 then -- switch 1, case 5 | switch 1, case 7
								l_result := socks_inst.Curl_SOCKS5 (a_conn.item.proxyuser, a_conn.item.proxypasswd, l_newhost, l_newport.to_integer_32, 1, a_conn)
								l_pos := -1 + 19 -- switch 1; break
				when 15 then -- switch 1, case 4
								l_result := socks_inst.Curl_SOCKS4 (a_conn.item.proxyuser, l_newhost, l_newport.to_integer_32, 1, a_conn, (0).to_boolean)
								l_pos := -1 + 19 -- switch 1; break
				when 16 then -- switch 1, case 6
								l_result := socks_inst.Curl_SOCKS4 (a_conn.item.proxyuser, l_newhost, l_newport.to_integer_32, 1, a_conn, (1).to_boolean)
								l_pos := -1 + 19 -- switch 1; break
				when 17 then -- switch 1, case 0 | switch 1, case 1
								l_pos := -1 + 19 -- switch 1; break
				when 18 then -- switch 1, default:
								sendf_inst.Curl_failf ([l_data, ce_string ("unknown proxytype option given")])
								l_result := {PC_CURLCODE}.CURLE_COULDNT_CONNECT
								l_pos := -1 + 19 -- switch 1; break
				when 19 then -- switch 1, done:
								if a_conn.item.bits.tunnel_proxy then
									if a_conn.item.bits.httpproxy then
										l_ftp_save.ce_gets (l_data.item.state.proto.ftp)
										memset (create {CE_POINTER [ANY]}.make_cast ((l_http_proxy.address)), 0, {PD_HTTP}.sizeof.to_natural_32).do_nothing
										l_data.item.state.proto.http := l_http_proxy.address
										l_result := http_inst.Curl_proxyCONNECT (a_conn, 1, l_newhost, l_newport)
										l_data.item.state.proto.ftp := l_ftp_save
										if {NATURAL_32} 0 /= l_result.to_natural_32 then
											Result := l_result
											js.return
										end
									end
								end
				when 20 then -- if done
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_STOP)
								Result := l_result
							end
				when 21 then
							l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
				end

	ftp_state_port_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_fcmd: INTEGER_32
			l_result: INTEGER_32
		do
			create l_data.make
			create l_ftpc.make

			l_data.ce_gets (a_conn.item.data)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_fcmd := l_ftpc.item.count1
			l_result := {PC_CURLCODE}.CURLE_OK
			if a_ftpcode /= 200 then
				if {NATURAL_32} 0 = l_fcmd.to_natural_32 then
					sendf_inst.Curl_infof ([l_data, ce_string ("disabling EPRT usage%N")])
					a_conn.item.bits.ftp_use_eprt := (0).to_boolean
				end
				l_fcmd := (l_fcmd.to_natural_32 + {NATURAL_32} 1).to_integer_32
				if l_fcmd.to_natural_32 = {NATURAL_32} 2 then
					sendf_inst.Curl_failf ([l_data, ce_string ("Failed to do PORT")])
					l_result := {PC_CURLCODE}.CURLE_FTP_PORT_FAILED
				else
					l_result := ftp_state_use_port (a_conn, l_fcmd)
				end
			else
				sendf_inst.Curl_infof ([l_data, ce_string ("Connect data stream actively%N")])
				state (a_conn, {PC_FTPSTATE}.FTP_STOP)
			end
			Result := l_result
		end

	ftp_state_mdtm_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_year: INTEGER_32
			l_month: INTEGER_32
			l_day: INTEGER_32
			l_hour: INTEGER_32
			l_minute: INTEGER_32
			l_second: INTEGER_32
			l_buf: CE_POINTER [INTEGER_8]
			l_secs: INTEGER_32
			l_tm: CE_POINTER [SD_TM]
			l_filetime: INTEGER_32
			l_buffer: SD_TM
			l_tmp___3: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make
			create l_buf.make
			create l_tm.make
			create l_buffer.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					inspect a_ftpcode -- switch 1 head
					when 213 then l_pos := -1 + 1
					when 550 then l_pos := -1 + 3
					else l_pos := -1 + 2
					end
				when 1 then -- switch 1, case 213
					l_buf.ce_gets (l_data.item.state.buffer)
					if 6 = (sscanf ([(l_buf + 4), ce_string ("%%04d%%02d%%02d%%02d%%02d%%02d"), create {CE_POINTER [INTEGER_32]}.make_shared ($l_year), create {CE_POINTER [INTEGER_32]}.make_shared ($l_month), create {CE_POINTER [INTEGER_32]}.make_shared ($l_day), create {CE_POINTER [INTEGER_32]}.make_shared ($l_hour), create {CE_POINTER [INTEGER_32]}.make_shared ($l_minute), create {CE_POINTER [INTEGER_32]}.make_shared ($l_second)])) then
						l_secs := (time (create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer)))
						mprintf_inst.curl_msnprintf ([l_buf, (Integer_8_bytes.to_natural_32 * (16385).to_natural_32), ce_string ("%%04d%%02d%%02d %%02d:%%02d:%%02d GMT"), l_year, l_month, l_day, l_hour, l_minute, l_second]).do_nothing
						l_data.item.info.filetime := (parsedate_inst.curl_getdate (l_buf, (create {CE_POINTER [INTEGER_32]}.make_shared ($l_secs))))
					end
					if l_data.item.set.opt_no_body then
						if ((l_ftpc.item.file).c_pointer /= default_pointer) then
							if l_data.item.set.get_filetime then
								if l_data.item.info.filetime >= 0 then
									l_filetime := l_data.item.info.filetime
									l_tm.ce_gets ((gmtime_r ((create {CE_POINTER [INTEGER_32]}.make_shared ($l_filetime)), (l_buffer.address))))
									if (l_tm.item.tm_wday).to_boolean then
										l_tmp___3 := l_tm.item.tm_wday - 1
									else
										l_tmp___3 := 6
									end
									mprintf_inst.curl_msnprintf ([l_buf, (16383).to_natural_32, ce_string ("Last-Modified: %%s, %%02d %%s %%4d %%02d:%%02d:%%02d GMT%R%N"), parsedate_inst.Curl_wkday[l_tmp___3], l_tm.item.tm_mday, parsedate_inst.Curl_month[l_tm.item.tm_mon], l_tm.item.tm_year + 1900, l_tm.item.tm_hour, l_tm.item.tm_min, l_tm.item.tm_sec]).do_nothing
									l_result := sendf_inst.Curl_client_write (a_conn, (1).bit_or (((1).bit_shift_left (1))), l_buf, (0).to_natural_32)
									if l_result.to_boolean then
										Result := l_result
										js.return
									end
								end
							end
						end
					end
					l_pos := -1 + 4 -- switch 1; break
				when 2 then -- switch 1, default:
					sendf_inst.Curl_infof ([l_data, ce_string ("unsupported MDTM reply format%N")])
					l_pos := -1 + 4 -- switch 1; break
				when 3 then -- switch 1, case 550
					sendf_inst.Curl_failf ([l_data, ce_string ("Given file does not exist")])
					l_result := {PC_CURLCODE}.CURLE_FTP_COULDNT_RETR_FILE
					l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
					if not (not js.return_called and then ((l_data.item.set.timecondition).to_boolean)) then
						l_pos := -1 + 14 -- skip if block
					end
				when 5 then -- if:
						if not (l_data.item.info.filetime > 0) then
							l_pos := -1 + 13 -- skip if block
						end
				when 6 then -- if:
						if not (l_data.item.set.timevalue > 0) then
							l_pos := -1 + 11 -- skip if block
						end
				when 7 then -- if:
						inspect l_data.item.set.timecondition -- switch 2 head
						when 2 then l_pos := -1 + 9
						else l_pos := -1 + 8
						end
				when 8 then -- switch 2, default:
						if l_data.item.info.filetime <= l_data.item.set.timevalue then
							sendf_inst.Curl_infof ([l_data, ce_string ("The requested document is not new enough%N")])
							l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_NONE
							l_data.item.info.timecond := (1).to_boolean
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
							Result := ({PC_CURLCODE}.CURLE_OK)
							js.return
						end
						l_pos := -1 + 10 -- switch 2; break
				when 9 then -- switch 2, case 2
						if l_data.item.info.filetime > l_data.item.set.timevalue then
							sendf_inst.Curl_infof ([l_data, ce_string ("The requested document is not old enough%N")])
							l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_NONE
							l_data.item.info.timecond := (1).to_boolean
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
							Result := ({PC_CURLCODE}.CURLE_OK)
							js.return
						end
						l_pos := -1 + 10 -- switch 2; break
				when 10 then -- switch 2, done: | if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						sendf_inst.Curl_infof ([l_data, ce_string ("Skipping time comparison%N")])
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 14
				when 13 then -- else:
						sendf_inst.Curl_infof ([l_data, ce_string ("Skipping time comparison%N")])
				when 14 then -- if done | if done
						if not js.return_called then
							if not l_result.to_boolean then
								l_result := ftp_state_post_mdtm (a_conn)
							end
							Result := l_result
						end
				when 15 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_type_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			js: JUMP_STATE
		do
			create l_data.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			if a_ftpcode // 100 /= 2 then
				sendf_inst.Curl_failf ([l_data, ce_string ("Couldn%'t set desired mode")])
				Result := ({PC_CURLCODE}.CURLE_FTP_COULDNT_SET_TYPE)
				js.return
			end
			if not js.return_called then
				if a_ftpcode /= 200 then
					sendf_inst.Curl_infof ([l_data, ce_string ("Got a %%03d response code instead of the assumed 200%N"), a_ftpcode])
				end
				if a_instate.to_natural_32 = {NATURAL_32} 19 then
					l_result := ftp_state_post_type (a_conn)
				elseif a_instate.to_natural_32 = {NATURAL_32} 20 then
					l_result := ftp_state_post_listtype (a_conn)
				elseif a_instate.to_natural_32 = {NATURAL_32} 21 then
					l_result := ftp_state_post_retrtype (a_conn)
				elseif a_instate.to_natural_32 = {NATURAL_32} 22 then
					l_result := ftp_state_post_stortype (a_conn)
				end
				Result := l_result
			end
		end

	ftp_state_post_retr_size (a_conn: CE_POINTER [PD_CONNECTDATA]; a_filesize: INTEGER_64): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			l_ftp.ce_gets (l_data.item.state.proto.ftp)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if (l_data.item.set.max_filesize).to_boolean then
				if a_filesize > l_data.item.set.max_filesize then
					sendf_inst.Curl_failf ([l_data, ce_string ("Maximum file size exceeded")])
					Result := ({PC_CURLCODE}.CURLE_FILESIZE_EXCEEDED)
					js.return
				end
			end
			if not js.return_called then
				l_ftp.item.downloadsize := a_filesize
				if (l_data.item.state.resume_from).to_boolean then
					if a_filesize = -{INTEGER_64} 1 then
						sendf_inst.Curl_infof ([l_data, ce_string ("ftp server doesn%'t support SIZE%N")])
					elseif l_data.item.state.resume_from < {INTEGER_64} 0 then
						if a_filesize < -l_data.item.state.resume_from then
							sendf_inst.Curl_failf ([l_data, ce_string ("Offset (%%lld) was beyond file size (%%lld)"), l_data.item.state.resume_from, a_filesize])
							Result := ({PC_CURLCODE}.CURLE_BAD_DOWNLOAD_RESUME)
							js.return
						end
						if not js.return_called then
							l_ftp.item.downloadsize := -l_data.item.state.resume_from
							l_data.item.state.resume_from := a_filesize - l_ftp.item.downloadsize
						end
					else
						if a_filesize < l_data.item.state.resume_from then
							sendf_inst.Curl_failf ([l_data, ce_string ("Offset (%%lld) was beyond file size (%%lld)"), l_data.item.state.resume_from, a_filesize])
							Result := ({PC_CURLCODE}.CURLE_BAD_DOWNLOAD_RESUME)
							js.return
						end
						if not js.return_called then
							l_ftp.item.downloadsize := a_filesize - l_data.item.state.resume_from
						end
					end
					if not js.return_called and then (l_ftp.item.downloadsize = {INTEGER_64} 0) then
						transfer_inst.Curl_setup_transfer (a_conn, -1, (-1).to_integer_64, (0).to_boolean, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer), -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
						sendf_inst.Curl_infof ([l_data, ce_string ("File already completely downloaded%N")])
						l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_NONE
						state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
					if not js.return_called then
						sendf_inst.Curl_infof ([l_data, ce_string ("Instructs server to resume from offset %%lld%N"), l_data.item.state.resume_from])
						l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("REST %%lld"), l_data.item.state.resume_from])
						if l_result.to_natural_32 /= {NATURAL_32} 0 then
							Result := l_result
							js.return
						end
					end
					if not js.return_called then
						state (a_conn, {PC_FTPSTATE}.FTP_RETR_REST)
					end
				else
					l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("RETR %%s"), l_ftpc.item.file])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
					if not js.return_called then
						state (a_conn, {PC_FTPSTATE}.FTP_RETR)
					end
				end
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_size_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_filesize: INTEGER_64
			l_buf: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_data.make
			create l_buf.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			l_buf.ce_gets (l_data.item.state.buffer)
			if a_ftpcode = 213 then
				l_filesize := (strtoll ((l_buf + 4), create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer), 0))
			else
				l_filesize := (-1).to_integer_64
			end
			if a_instate.to_natural_32 = {NATURAL_32} 23 then
				if -{INTEGER_64} 1 /= l_filesize then
					mprintf_inst.curl_msnprintf ([l_buf, (Integer_8_bytes.to_natural_32 * (16385).to_natural_32), ce_string ("Content-Length: %%lld%R%N"), l_filesize]).do_nothing
					l_result := sendf_inst.Curl_client_write (a_conn, (1).bit_or (((1).bit_shift_left (1))), l_buf, (0).to_natural_32)
					if l_result.to_boolean then
						Result := l_result
						js.return
					end
				end
				if not js.return_called then
					progress_inst.Curl_pgrsSetDownloadSize (l_data, l_filesize)
					l_result := ftp_state_post_size (a_conn)
				end
			elseif a_instate.to_natural_32 = {NATURAL_32} 24 then
				progress_inst.Curl_pgrsSetDownloadSize (l_data, l_filesize)
				l_result := ftp_state_post_retr_size (a_conn, l_filesize)
			elseif a_instate.to_natural_32 = {NATURAL_32} 25 then
				l_data.item.state.resume_from := l_filesize
				l_result := ftp_state_ul_setup (a_conn, (1).to_boolean)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_rest_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_buffer: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpc.make
			create l_buffer.make (<<24>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					inspect a_instate -- switch 1 head
					when 27 then l_pos := -1 + 2
					else l_pos := -1 + 1
					end
				when 1 then -- switch 1, default:
					if a_ftpcode = 350 then
						l_buffer[0] := (('A').code).to_integer_8
						l_buffer[1] := (('c').code).to_integer_8
						l_buffer[2] := (('c').code).to_integer_8
						l_buffer[3] := (('e').code).to_integer_8
						l_buffer[4] := (('p').code).to_integer_8
						l_buffer[5] := (('t').code).to_integer_8
						l_buffer[6] := (('-').code).to_integer_8
						l_buffer[7] := (('r').code).to_integer_8
						l_buffer[8] := (('a').code).to_integer_8
						l_buffer[9] := (('n').code).to_integer_8
						l_buffer[10] := (('g').code).to_integer_8
						l_buffer[11] := (('e').code).to_integer_8
						l_buffer[12] := (('s').code).to_integer_8
						l_buffer[13] := ((':').code).to_integer_8
						l_buffer[14] := ((' ').code).to_integer_8
						l_buffer[15] := (('b').code).to_integer_8
						l_buffer[16] := (('y').code).to_integer_8
						l_buffer[17] := (('t').code).to_integer_8
						l_buffer[18] := (('e').code).to_integer_8
						l_buffer[19] := (('s').code).to_integer_8
						l_buffer[20] := (('%R').code).to_integer_8
						l_buffer[21] := (('%N').code).to_integer_8
						l_buffer[22] := (('%/0/').code).to_integer_8
						l_buffer[23] := (0).to_integer_8
						l_result := sendf_inst.Curl_client_write (a_conn, (1).bit_or (((1).bit_shift_left (1))), l_buffer, (0).to_natural_32)
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
					end
					if not js.return_called then
						l_result := ftp_state_post_rest (a_conn)
					end
					l_pos := -1 + 3 -- switch 1; break
				when 2 then -- switch 1, case 27
					if a_ftpcode /= 350 then
						sendf_inst.Curl_failf ([a_conn.item.data, ce_string ("Couldn%'t use REST")])
						l_result := {PC_CURLCODE}.CURLE_FTP_COULDNT_USE_REST
					else
						l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("RETR %%s"), l_ftpc.item.file])
						if l_result.to_natural_32 /= {NATURAL_32} 0 then
							Result := l_result
							js.return
						end
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_RETR)
						end
					end
					l_pos := -1 + 3 -- switch 1; break
				when 3 then -- switch 1, done:
					if not js.return_called then
						Result := l_result
					end
				when 4 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_state_stor_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			js: JUMP_STATE
		do
			create l_data.make
			create l_ftp.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			l_ftp.ce_gets (l_data.item.state.proto.ftp)
			if a_ftpcode >= 400 then
				sendf_inst.Curl_failf ([l_data, ce_string ("Failed FTP upload: %%0d"), a_ftpcode])
				Result := ({PC_CURLCODE}.CURLE_UPLOAD_FAILED)
				js.return
			end
			if not js.return_called and then (l_data.item.set.ftp_use_port) then
				l_result := AllowServerConnect (a_conn)
				if l_result.to_boolean then
					Result := l_result
					js.return
				end
			end
			if not js.return_called and then (a_conn.item.ssl[1].use) then
				sendf_inst.Curl_infof ([l_data, ce_string ("Doing the SSL/TLS handshake on the data stream%N")])
				l_result := {PC_CURLCODE}.CURLE_FAILED_INIT
				if l_result.to_boolean then
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				(l_ftp.item.bytecountp).item := (0).to_integer_64
				progress_inst.Curl_pgrsSetUploadSize (l_data, l_data.item.set.infilesize)
				transfer_inst.Curl_setup_transfer (a_conn, -1, (-1).to_integer_64, (0).to_boolean, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer), 1, l_ftp.item.bytecountp)
				state (a_conn, {PC_FTPSTATE}.FTP_STOP)
				a_conn.item.proto.ftpc.pp.pending_resp := (1).to_boolean
				Result := l_result
			end
		end

	ftp_state_get_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_buf: CE_POINTER [INTEGER_8]
			l_size: INTEGER_64
			l_bytes: CE_POINTER [INTEGER_8]
			l_in: INTEGER_32
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: INTEGER_64
			l_tmp___3: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_buf.make
			create l_bytes.make
			create l_tmp___0.make
			create l_tmp___1.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_buf.ce_gets (l_data.item.state.buffer)
					if a_ftpcode /= 150 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___3
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 3 then -- else:
						if a_ftpcode /= 125 then
							l_pos := -1 + 15 -- skip if block
						end
				when 4 then -- if: | _L___3:
						l_size := (-1).to_integer_64
						if a_instate.to_natural_32 = {NATURAL_32} 31 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if not l_data.item.set.prefer_ascii then
							if l_ftp.item.downloadsize < {INTEGER_64} 1 then
								l_bytes.ce_gets (strstr (l_buf, ce_string (" bytes")))
								l_tmp___1.ce_gets (l_bytes)
								l_bytes.ce_gets (l_bytes - 1)
								if ((l_tmp___1).c_pointer /= default_pointer) then
									l_in := (l_bytes.distance (l_buf)).to_integer_32
									from
									until
										js.loop_left
									loop
										js.reset
										l_in := l_in - 1
										if not l_in.to_boolean then
											js.break
										end
										if not js.loop_jumped and then (40 = (l_bytes[0]).to_integer_32) then
											js.break
										end
										if not js.loop_jumped then
											if not ((((ctype_b_loc).item[((l_bytes[0]).to_natural_8).to_integer_32]).to_integer_32).bit_and (2048)).to_boolean then
												l_bytes.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
												js.break
											end
										end
										if not js.loop_jumped then
											l_bytes.ce_gets (l_bytes - 1)
										end
									end
									js.reset
									l_tmp___0.ce_gets (l_bytes)
									l_bytes.ce_gets (l_bytes + 1)
									if ((l_tmp___0).c_pointer /= default_pointer) then
										l_size := strtoll (l_bytes, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer), 0)
									end
								end
							else
								l_pos := -1 + 7 -- goto _L___0
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L___0:
						if l_ftp.item.downloadsize > -{INTEGER_64} 1 then
							l_size := l_ftp.item.downloadsize
						end
				when 8 then
						if l_data.item.set.ftp_use_port then
							l_result := AllowServerConnect (a_conn)
							if l_result.to_boolean then
								Result := l_result
								js.return
							end
						end
						if not js.return_called and then (a_conn.item.ssl[1].use) then
							sendf_inst.Curl_infof ([l_data, ce_string ("Doing the SSL/TLS handshake on the data stream%N")])
							l_result := {PC_CURLCODE}.CURLE_FAILED_INIT
							if l_result.to_boolean then
								Result := l_result
								js.return
							end
						end
						if js.return_called then
							l_pos := -1 + 14 -- skip if block
						end
				when 9 then -- if:
						if not (l_size > l_data.item.req.maxdownload) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_data.item.req.maxdownload > {INTEGER_64} 0 then
							l_tmp___2 := l_data.item.req.maxdownload
							l_data.item.req.size := l_tmp___2
							l_size := l_tmp___2
						else
							l_pos := -1 + 12 -- goto _L___1
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else: | _L___1:
						if a_instate.to_natural_32 /= {NATURAL_32} 31 then
							if l_data.item.set.prefer_ascii then
								l_size := (-1).to_integer_64
							end
						end
				when 13 then
						sendf_inst.Curl_infof ([l_data, ce_string ("Maxdownload = %%lld%N"), l_data.item.req.maxdownload])
						if a_instate.to_natural_32 /= {NATURAL_32} 31 then
							sendf_inst.Curl_infof ([l_data, ce_string ("Getting file with size: %%lld%N"), l_size])
						end
						transfer_inst.Curl_setup_transfer (a_conn, 1, l_size, (0).to_boolean, l_ftp.item.bytecountp, -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
						a_conn.item.proto.ftpc.pp.pending_resp := (1).to_boolean
						state (a_conn, {PC_FTPSTATE}.FTP_STOP)
				when 14 then -- if done | if block done, skip else block:
						l_pos := -1 + 19
				when 15 then -- else:
						if a_instate.to_natural_32 /= {NATURAL_32} 31 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if a_ftpcode = 450 then
							l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_NONE
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						else
							l_pos := -1 + 18 -- goto _L___2
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else: | _L___2:
						sendf_inst.Curl_failf ([l_data, ce_string ("RETR response: %%03d"), a_ftpcode])
						if a_instate.to_natural_32 = {NATURAL_32} 32 then
							if a_ftpcode = 550 then
								l_tmp___3 := 78
							else
								l_tmp___3 := 19
							end
						else
							l_tmp___3 := 19
						end
						Result := l_tmp___3
						js.return
				when 19 then -- if done | if done
						if not js.return_called then
							Result := l_result
						end
				when 20 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_state_loggedin (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			js: JUMP_STATE
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			if a_conn.item.ssl[0].use then
				l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("PBSZ %%d"), 0])
				if l_result.to_natural_32 /= {NATURAL_32} 0 then
					Result := l_result
					js.return
				end
				if not js.return_called then
					state (a_conn, {PC_FTPSTATE}.FTP_PBSZ)
				end
			else
				l_result := ftp_state_pwd (a_conn)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_state_user_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32; a_instate: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_tmp: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make
			create l_tmp.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_result := {PC_CURLCODE}.CURLE_OK
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					if a_ftpcode /= 331 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if (l_ftpc.item.state).to_natural_32 = {NATURAL_32} 3 then
							if ((l_ftp.item.passwd).c_pointer /= default_pointer) then
								l_tmp.ce_gets (l_ftp.item.passwd)
							else
								l_tmp.ce_gets (ce_string (""))
							end
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("PASS %%s"), l_tmp])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
							if not js.return_called then
								state (a_conn, {PC_FTPSTATE}.FTP_PASS)
							end
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						if a_ftpcode // 100 = 2 then
							l_result := ftp_state_loggedin (a_conn)
						elseif a_ftpcode = 332 then
							if ((l_data.item.set.str[7]).c_pointer /= default_pointer) then
								l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("ACCT %%s"), l_data.item.set.str[7]])
								if l_result.to_natural_32 /= {NATURAL_32} 0 then
									Result := l_result
									js.return
								end
								if not js.return_called then
									state (a_conn, {PC_FTPSTATE}.FTP_ACCT)
								end
							else
								sendf_inst.Curl_failf ([l_data, ce_string ("ACCT requested but none available")])
								l_result := {PC_CURLCODE}.CURLE_LOGIN_DENIED
							end
						elseif (((a_conn.item.data).item.set.str[8]).c_pointer /= default_pointer) then
							if not (a_conn.item.data).item.state.ftp_trying_alternative then
								l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("%%s"), (a_conn.item.data).item.set.str[8]])
								if l_result.to_natural_32 /= {NATURAL_32} 0 then
									Result := l_result
									js.return
								end
								if not js.return_called then
									(a_conn.item.data).item.state.ftp_trying_alternative := (1).to_boolean
									state (a_conn, {PC_FTPSTATE}.FTP_USER)
									l_result := {PC_CURLCODE}.CURLE_OK
								end
							else
								sendf_inst.Curl_failf ([l_data, ce_string ("Access denied: %%03d"), a_ftpcode])
								l_result := {PC_CURLCODE}.CURLE_LOGIN_DENIED
							end
						else
							sendf_inst.Curl_failf ([l_data, ce_string ("Access denied: %%03d"), a_ftpcode])
							l_result := {PC_CURLCODE}.CURLE_LOGIN_DENIED
						end
				when 4 then
					if not js.return_called then
						Result := l_result
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_state_acct_resp (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ftpcode: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
		do
			create l_data.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data.ce_gets (a_conn.item.data)
			if a_ftpcode /= 230 then
				sendf_inst.Curl_failf ([l_data, ce_string ("ACCT rejected by server: %%03d"), a_ftpcode])
				l_result := {PC_CURLCODE}.CURLE_FTP_WEIRD_PASS_REPLY
			else
				l_result := ftp_state_loggedin (a_conn)
			end
			Result := l_result
		end

	ftp_statemach_act (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			l_sock: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpcode: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_nread: NATURAL_32
			l_tmp___0: INTEGER_32
			l_ptr: CE_POINTER [INTEGER_8]
			l_dir: CE_POINTER [INTEGER_8]
			l_store: CE_POINTER [INTEGER_8]
			l_ptr___0: CE_POINTER [INTEGER_8]
			l_os: CE_POINTER [INTEGER_8]
			l_store___0: CE_POINTER [INTEGER_8]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [INTEGER_8]
			l_tmp___6: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftpc.make
			create l_pp.make
			create l_ptr.make
			create l_dir.make
			create l_store.make
			create l_ptr___0.make
			create l_os.make
			create l_store___0.make
			create l_tmp___3.make
			create l_tmp___4.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sock := a_conn.item.sock[0]
					l_data.ce_gets (a_conn.item.data)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
					l_nread := (0).to_natural_32
					if (l_pp.item.sendleft).to_boolean then
						Result := (pingpong_inst.Curl_pp_flushsend (l_pp))
						js.return
					end
					if not js.return_called then
						l_result := ftp_readresp (l_sock, l_pp, create {CE_POINTER [INTEGER_32]}.make_shared ($l_ftpcode), create {CE_POINTER [NATURAL_32]}.make_shared ($l_nread))
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
					end
					if not (not js.return_called and then (l_ftpcode.to_boolean)) then
						l_pos := -1 + 47 -- skip if block
					end
				when 1 then -- if:
						inspect l_ftpc.item.state -- switch 1 head
						when 1 then l_pos := -1 + 2
						when 2 then l_pos := -1 + 15
						when 3 then l_pos := -1 + 23
						when 4 then l_pos := -1 + 23
						when 5 then l_pos := -1 + 24
						when 6 then l_pos := -1 + 25
						when 7 then l_pos := -1 + 26
						when 8 then l_pos := -1 + 27
						when 9 then l_pos := -1 + 28
						when 10 then l_pos := -1 + 30
						when 11 then l_pos := -1 + 32
						when 12 then l_pos := -1 + 34
						when 15 then l_pos := -1 + 34
						when 13 then l_pos := -1 + 34
						when 14 then l_pos := -1 + 34
						when 16 then l_pos := -1 + 35
						when 17 then l_pos := -1 + 36
						when 18 then l_pos := -1 + 37
						when 19 then l_pos := -1 + 38
						when 20 then l_pos := -1 + 38
						when 21 then l_pos := -1 + 38
						when 22 then l_pos := -1 + 38
						when 23 then l_pos := -1 + 39
						when 24 then l_pos := -1 + 39
						when 25 then l_pos := -1 + 39
						when 26 then l_pos := -1 + 40
						when 27 then l_pos := -1 + 40
						when 29 then l_pos := -1 + 41
						when 30 then l_pos := -1 + 42
						when 28 then l_pos := -1 + 43
						when 31 then l_pos := -1 + 44
						when 32 then l_pos := -1 + 44
						when 33 then l_pos := -1 + 45
						else l_pos := -1 + 46
						end
				when 2 then -- switch 1, case 1
						if l_ftpcode /= 220 then
							sendf_inst.Curl_failf ([l_data, ce_string ("Got a %%03d ftp-server response when 220 was expected"), l_ftpcode])
							Result := ({PC_CURLCODE}.CURLE_FTP_WEIRD_SERVER_REPLY)
							js.return
						end
						if js.return_called then
							l_pos := -1 + 14 -- skip if block
						end
				when 3 then -- if:
						if not ((l_data.item.set.ftp_ssl).to_boolean) then
							l_pos := -1 + 13 -- skip if block
						end
				when 4 then -- if:
						if a_conn.item.ssl[0].use then
							l_pos := -1 + 11 -- skip if block
						end
				when 5 then -- if:
						l_ftpc.item.count3 := 0
						inspect l_data.item.set.ftpsslauth -- switch 2 head
						when 0 then l_pos := -1 + 6
						when 1 then l_pos := -1 + 6
						when 2 then l_pos := -1 + 7
						else l_pos := -1 + 8
						end
				when 6 then -- switch 2, case 0 | switch 2, case 1
						l_ftpc.item.count2 := 1
						l_ftpc.item.count1 := 0
						l_pos := -1 + 9 -- switch 2; break
				when 7 then -- switch 2, case 2
						l_ftpc.item.count2 := -1
						l_ftpc.item.count1 := 1
						l_pos := -1 + 9 -- switch 2; break
				when 8 then -- switch 2, default:
						sendf_inst.Curl_failf ([l_data, ce_string ("unsupported parameter to CURLOPT_FTPSSLAUTH: %%d"), l_data.item.set.ftpsslauth])
						Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
						js.return
				when 9 then -- switch 2, done:
						if not js.return_called then
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("AUTH %%s"), ftpauth[l_ftpc.item.count1]])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
						end
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_AUTH)
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_pos := -1 + 13 -- goto _L
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 14
				when 13 then -- else: | _L:
						l_result := ftp_state_user (a_conn)
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
				when 14 then -- if done
						l_pos := -1 + 47 -- switch 1; break
				when 15 then -- switch 1, case 2
						if l_ftpcode /= 234 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						l_pos := -1 + 19 -- goto _L___0
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else:
						if l_ftpcode /= 334 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L___0:
						l_result := {PC_CURLCODE}.CURLE_FAILED_INIT
						if {NATURAL_32} 0 = l_result.to_natural_32 then
							a_conn.item.protocol := (a_conn.item.protocol).bit_or (((1).bit_shift_left (3)).to_integer_32)
							a_conn.item.ssl[1].use := (0).to_boolean
							l_result := ftp_state_user (a_conn)
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						if l_ftpc.item.count3 < 1 then
							l_ftpc.item.count3 := (l_ftpc.item.count3) + 1
							l_ftpc.item.count1 := l_ftpc.item.count1 + (l_ftpc.item.count2)
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("AUTH %%s"), ftpauth[l_ftpc.item.count1]])
						elseif (l_data.item.set.ftp_ssl).to_natural_32 > {NATURAL_32} 1 then
							l_result := {PC_CURLCODE}.CURLE_USE_SSL_FAILED
						else
							l_result := ftp_state_user (a_conn)
						end
				when 22 then -- if done
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
						l_pos := -1 + 47 -- switch 1; break
				when 23 then -- switch 1, case 3 | switch 1, case 4
						l_result := ftp_state_user_resp (a_conn, l_ftpcode, l_ftpc.item.state)
						l_pos := -1 + 47 -- switch 1; break
				when 24 then -- switch 1, case 5
						l_result := ftp_state_acct_resp (a_conn, l_ftpcode)
						l_pos := -1 + 47 -- switch 1; break
				when 25 then -- switch 1, case 6
						if (l_data.item.set.ftp_ssl).to_natural_32 = {NATURAL_32} 2 then
							l_tmp___0 := ('C').code
						else
							l_tmp___0 := ('P').code
						end
						l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("PROT %%c"), l_tmp___0])
						if l_result.to_natural_32 /= {NATURAL_32} 0 then
							Result := l_result
							js.return
						end
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_PROT)
						end
						l_pos := -1 + 47 -- switch 1; break
				when 26 then -- switch 1, case 7
						if l_ftpcode // 100 = 2 then
							a_conn.item.ssl[1].use := ((l_data.item.set.ftp_ssl).to_natural_32 /= {NATURAL_32} 2)
						elseif (l_data.item.set.ftp_ssl).to_natural_32 > {NATURAL_32} 2 then
							Result := ({PC_CURLCODE}.CURLE_USE_SSL_FAILED)
							js.return
						end
						if not js.return_called then
							if (l_data.item.set.ftp_ccc).to_boolean then
								l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("CCC"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
								if l_result.to_natural_32 /= {NATURAL_32} 0 then
									Result := l_result
									js.return
								end
								if not js.return_called then
									state (a_conn, {PC_FTPSTATE}.FTP_CCC)
								end
							else
								l_result := ftp_state_pwd (a_conn)
								if l_result.to_boolean then
									Result := l_result
									js.return
								end
							end
						end
						l_pos := -1 + 47 -- switch 1; break
				when 27 then -- switch 1, case 8
						if l_ftpcode < 500 then
							l_result := {PC_CURLCODE}.CURLE_FAILED_INIT
							if l_result.to_boolean then
								sendf_inst.Curl_failf ([a_conn.item.data, ce_string ("Failed to clear the command channel (CCC)")])
								Result := l_result
								js.return
							end
						end
						if not js.return_called then
							l_result := ftp_state_pwd (a_conn)
							if l_result.to_boolean then
								Result := l_result
								js.return
							end
						end
						l_pos := -1 + 47 -- switch 1; break
				when 28 then -- switch 1, case 9
						if l_ftpcode = 257 then
							l_ptr.ce_gets (l_data.item.state.buffer + 4)
							l_dir.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([l_nread + {NATURAL_32} 1])))))
							if (l_dir).c_pointer = default_pointer then
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								js.return
							end
							if not js.return_called then
								if 34 = (l_ptr[0]).to_integer_32 then
									l_ptr.ce_gets (l_ptr + 1)
									l_store.ce_gets (l_dir)
									from
									until
										not (not js.loop_left and then ((l_ptr[0]).to_boolean))
									loop
										js.reset
										if 34 = (l_ptr[0]).to_integer_32 then
											if 34 = (l_ptr[1]).to_integer_32 then
												l_store[0] := l_ptr[1]
												l_ptr.ce_gets (l_ptr + 1)
											else
												l_store[0] := (('%/0/').code).to_integer_8
												js.break
											end
										else
											l_store[0] := l_ptr[0]
										end
										if not js.loop_jumped then
											l_store.ce_gets (l_store + 1)
											l_ptr.ce_gets (l_ptr + 1)
										end
									end
									js.reset
									if ((l_ftpc.item.entrypath).c_pointer /= default_pointer) then
										easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.entrypath)])
									end
									l_ftpc.item.entrypath := l_dir
									sendf_inst.Curl_infof ([l_data, ce_string ("Entry path is %'%%s%'%N"), l_ftpc.item.entrypath])
									l_data.item.state.most_recent_ftp_entrypath := l_ftpc.item.entrypath
									if (l_ftpc.item.server_os).c_pointer = default_pointer then
										if (l_ftpc.item.entrypath[0]).to_integer_32 /= 47 then
											l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("SYST"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
											if l_result.to_natural_32 /= {NATURAL_32} 0 then
												Result := l_result
												js.return
											end
											if not js.return_called then
												state (a_conn, {PC_FTPSTATE}.FTP_SYST)
											end
											l_pos := -1 + 47 -- switch 1; break
										end
									end
								else
									easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_dir)])
									sendf_inst.Curl_infof ([l_data, ce_string ("Failed to figure out path%N")])
								end
							end
						end
				when 29 then
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						end
						l_pos := -1 + 47 -- switch 1; break
				when 30 then -- switch 1, case 10
						if l_ftpcode = 215 then
							l_ptr___0.ce_gets (l_data.item.state.buffer + 4)
							l_os.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([l_nread + {NATURAL_32} 1])))))
							if (l_os).c_pointer = default_pointer then
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								js.return
							end
							if not js.return_called then
								from
								until
									(l_ptr___0[0]).to_integer_32 /= 32
								loop
									l_ptr___0.ce_gets (l_ptr___0 + 1)
								end
								l_store___0.ce_gets (l_os)
								from
								until
									js.loop_left
								loop
									js.reset
									if (l_ptr___0[0]).to_boolean then
										if (l_ptr___0[0]).to_integer_32 = 32 then
											js.break
										end
									else
										js.break
									end
									if not js.loop_jumped then
										l_tmp___3.ce_gets (l_store___0)
										l_store___0.ce_gets (l_store___0 + 1)
										l_tmp___4.ce_gets (l_ptr___0)
										l_ptr___0.ce_gets (l_ptr___0 + 1)
										l_tmp___3[0] := l_tmp___4[0]
									end
								end
								js.reset
								l_store___0[0] := (('%/0/').code).to_integer_8
								l_ftpc.item.server_os := l_os
								if (strequal_inst.curl_strequal (l_ftpc.item.server_os, ce_string ("OS/400"))).to_boolean then
									l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("SITE NAMEFMT 1"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
									if l_result.to_natural_32 /= {NATURAL_32} 0 then
										Result := l_result
										js.return
									end
									if not js.return_called then
										state (a_conn, {PC_FTPSTATE}.FTP_NAMEFMT)
									end
									l_pos := -1 + 47 -- switch 1; break
								end
							end
						end
				when 31 then
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						end
						l_pos := -1 + 47 -- switch 1; break
				when 32 then -- switch 1, case 11
						if l_ftpcode = 250 then
							ftp_state_pwd (a_conn).do_nothing
							l_pos := -1 + 47 -- switch 1; break
						end
				when 33 then
						state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						l_pos := -1 + 47 -- switch 1; break
				when 34 then -- switch 1, case 12 | switch 1, case 15 | switch 1, case 13 | switch 1, case 14
						if l_ftpcode >= 400 then
							if not (l_ftpc.item.count2).to_boolean then
								sendf_inst.Curl_failf ([a_conn.item.data, ce_string ("QUOT command failed with %%03d"), l_ftpcode])
								Result := ({PC_CURLCODE}.CURLE_QUOTE_ERROR)
								js.return
							end
						end
						if not js.return_called then
							l_result := ftp_state_quote (a_conn, (0).to_boolean, l_ftpc.item.state)
							if l_result.to_boolean then
								Result := l_result
								js.return
							end
						end
						l_pos := -1 + 47 -- switch 1; break
				when 35 then -- switch 1, case 16
						if l_ftpcode // 100 /= 2 then
							if ((a_conn.item.data).item.set.ftp_create_missing_dirs).to_boolean then
								if (l_ftpc.item.count1).to_boolean then
									if not (l_ftpc.item.count2).to_boolean then
										l_ftpc.item.count2 := (l_ftpc.item.count2) + 1
										l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("MKD %%s"), l_ftpc.item.dirs[(l_ftpc.item.count1 - 1)]])
										if l_result.to_natural_32 /= {NATURAL_32} 0 then
											Result := l_result
											js.return
										end
										if not js.return_called then
											state (a_conn, {PC_FTPSTATE}.FTP_MKD)
										end
									else
										sendf_inst.Curl_failf ([l_data, ce_string ("Server denied you to change to the given directory")])
										l_ftpc.item.cwdfail := (1).to_boolean
										Result := ({PC_CURLCODE}.CURLE_REMOTE_ACCESS_DENIED)
										js.return
									end
								else
									sendf_inst.Curl_failf ([l_data, ce_string ("Server denied you to change to the given directory")])
									l_ftpc.item.cwdfail := (1).to_boolean
									Result := ({PC_CURLCODE}.CURLE_REMOTE_ACCESS_DENIED)
									js.return
								end
							else
								sendf_inst.Curl_failf ([l_data, ce_string ("Server denied you to change to the given directory")])
								l_ftpc.item.cwdfail := (1).to_boolean
								Result := ({PC_CURLCODE}.CURLE_REMOTE_ACCESS_DENIED)
								js.return
							end
						else
							l_ftpc.item.count2 := 0
							l_ftpc.item.count1 := (l_ftpc.item.count1) + 1
							if l_ftpc.item.count1 <= l_ftpc.item.dirdepth then
								l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("CWD %%s"), l_ftpc.item.dirs[(l_ftpc.item.count1 - 1)]])
								if l_result.to_natural_32 /= {NATURAL_32} 0 then
									Result := l_result
									js.return
								end
							else
								l_result := ftp_state_post_cwd (a_conn)
								if l_result.to_boolean then
									Result := l_result
									js.return
								end
							end
						end
						l_pos := -1 + 47 -- switch 1; break
				when 36 then -- switch 1, case 17
						if l_ftpcode // 100 /= 2 then
							l_tmp___6 := l_ftpc.item.count3
							l_ftpc.item.count3 := (l_ftpc.item.count3) - 1
							if not l_tmp___6.to_boolean then
								sendf_inst.Curl_failf ([l_data, ce_string ("Failed to MKD dir: %%03d"), l_ftpcode])
								Result := ({PC_CURLCODE}.CURLE_REMOTE_ACCESS_DENIED)
								js.return
							end
						end
						if not js.return_called then
							state (a_conn, {PC_FTPSTATE}.FTP_CWD)
							l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("CWD %%s"), l_ftpc.item.dirs[(l_ftpc.item.count1 - 1)]])
							if l_result.to_natural_32 /= {NATURAL_32} 0 then
								Result := l_result
								js.return
							end
						end
						l_pos := -1 + 47 -- switch 1; break
				when 37 then -- switch 1, case 18
						l_result := ftp_state_mdtm_resp (a_conn, l_ftpcode)
						l_pos := -1 + 47 -- switch 1; break
				when 38 then -- switch 1, case 19 | switch 1, case 20 | switch 1, case 21 | switch 1, case 22
						l_result := ftp_state_type_resp (a_conn, l_ftpcode, l_ftpc.item.state)
						l_pos := -1 + 47 -- switch 1; break
				when 39 then -- switch 1, case 23 | switch 1, case 24 | switch 1, case 25
						l_result := ftp_state_size_resp (a_conn, l_ftpcode, l_ftpc.item.state)
						l_pos := -1 + 47 -- switch 1; break
				when 40 then -- switch 1, case 26 | switch 1, case 27
						l_result := ftp_state_rest_resp (a_conn, l_ftpcode, l_ftpc.item.state)
						l_pos := -1 + 47 -- switch 1; break
				when 41 then -- switch 1, case 29
						if l_ftpcode /= 200 then
							sendf_inst.Curl_failf ([l_data, ce_string ("PRET command not accepted: %%03d"), l_ftpcode])
							Result := ({PC_CURLCODE}.CURLE_FTP_PRET_FAILED)
							js.return
						end
						if not js.return_called then
							l_result := ftp_state_use_pasv (a_conn)
						end
						l_pos := -1 + 47 -- switch 1; break
				when 42 then -- switch 1, case 30
						l_result := ftp_state_pasv_resp (a_conn, l_ftpcode)
						l_pos := -1 + 47 -- switch 1; break
				when 43 then -- switch 1, case 28
						l_result := ftp_state_port_resp (a_conn, l_ftpcode)
						l_pos := -1 + 47 -- switch 1; break
				when 44 then -- switch 1, case 31 | switch 1, case 32
						l_result := ftp_state_get_resp (a_conn, l_ftpcode, l_ftpc.item.state)
						l_pos := -1 + 47 -- switch 1; break
				when 45 then -- switch 1, case 33
						l_result := ftp_state_stor_resp (a_conn, l_ftpcode)
						l_pos := -1 + 47 -- switch 1; break
				when 46 then -- switch 1, default:
						state (a_conn, {PC_FTPSTATE}.FTP_STOP)
						l_pos := -1 + 47 -- switch 1; break
				when 47 then -- switch 1, done: | if done
						if not js.return_called then
							Result := l_result
						end
				when 48 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_statemach_act_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_statemach_act)
		end

	ftp_multi_statemach (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_result: INTEGER_32
		do
			create l_ftpc.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_result := (pingpong_inst.Curl_pp_multi_statemach (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset)))
			a_done.item := ((l_ftpc.item.state).to_natural_32 = {NATURAL_32} 0)
			Result := l_result
		end

	ftp_multi_statemach_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_multi_statemach)
		end

	ftp_easy_statemach (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_result: INTEGER_32
			js: JUMP_STATE
		do
			create l_ftpc.make
			create l_pp.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
			l_result := {PC_CURLCODE}.CURLE_OK
			from
			until
				not (not js.loop_left and then ((l_ftpc.item.state).to_natural_32 /= {NATURAL_32} 0))
			loop
				js.reset
				l_result := pingpong_inst.Curl_pp_easy_statemach (l_pp)
				if l_result.to_boolean then
					js.break
				end
			end
			js.reset
			Result := l_result
		end

	ftp_init (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_ftp: CE_POINTER [PD_FTP]
			js: JUMP_STATE
		do
			create l_ftp.make

			if (0).to_natural_32 = (((a_conn.item.data).item.state.proto.ftp).c_pointer.to_integer_32).to_natural_32 then
				(a_conn.item.data).item.state.proto.ftp := create {CE_POINTER [PD_FTP]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([{PD_FTP}.sizeof.to_natural_32]))))
				if (0).to_natural_32 = (((a_conn.item.data).item.state.proto.ftp).c_pointer.to_integer_32).to_natural_32 then
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
			end
			if not js.return_called then
				l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
				l_ftp.item.bytecountp := create {CE_POINTER [INTEGER_64]}.make_shared ((a_conn.item.data).c_pointer + {PD_SESSIONHANDLE}.req_offset + {PD_SINGLEREQUEST}.bytecount_offset)
				l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_BODY
				l_ftp.item.downloadsize := (0).to_integer_64
				l_ftp.item.user := a_conn.item.user
				l_ftp.item.passwd := a_conn.item.passwd
				if 1 = (isBadFtpString (l_ftp.item.user)).to_integer then
					Result := ({PC_CURLCODE}.CURLE_URL_MALFORMAT)
					js.return
				end
			end
			if not js.return_called then
				if 1 = (isBadFtpString (l_ftp.item.passwd)).to_integer then
					Result := ({PC_CURLCODE}.CURLE_URL_MALFORMAT)
					js.return
				end
			end
			if not js.return_called then
				a_conn.item.proto.ftpc.known_filesize := (-1).to_integer_64
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	ftp_connect (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_http_proxy: PD_HTTP
			l_ftp_save: CE_POINTER [PD_FTP]
			js: JUMP_STATE
		do
			create l_ftpc.make
			create l_data.make
			create l_pp.make
			create l_http_proxy.make
			create l_ftp_save.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_data.ce_gets (a_conn.item.data)
			l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
			a_done.item := (0).to_boolean
			url_inst.Curl_reset_reqproto (a_conn)
			l_result := ftp_init (a_conn)
			if {NATURAL_32} 0 /= l_result.to_natural_32 then
				Result := l_result
				js.return
			end
			if not js.return_called then
				a_conn.item.bits.close := (0).to_boolean
				l_pp.item.response_time := 1800000
				l_pp.item.statemach_act := ftp_statemach_act_address
				l_pp.item.endofresp := ftp_endofresp_address
				l_pp.item.conn := a_conn
				if a_conn.item.bits.tunnel_proxy then
					if a_conn.item.bits.httpproxy then
						l_ftp_save.ce_gets (l_data.item.state.proto.ftp)
						memset (create {CE_POINTER [ANY]}.make_cast ((l_http_proxy.address)), 0, {PD_HTTP}.sizeof.to_natural_32).do_nothing
						l_data.item.state.proto.http := l_http_proxy.address
						l_result := http_inst.Curl_proxyCONNECT (a_conn, 0, a_conn.item.host.name, a_conn.item.remote_port)
						l_data.item.state.proto.ftp := l_ftp_save
						if {NATURAL_32} 0 /= l_result.to_natural_32 then
							Result := l_result
							js.return
						end
					end
				end
			end
			if not js.return_called and then (((a_conn.item.protocol).bit_and (((1).bit_shift_left (3)).to_integer_32)).to_boolean) then
				l_result := {PC_CURLCODE}.CURLE_FAILED_INIT
				if l_result.to_boolean then
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				pingpong_inst.Curl_pp_init (l_pp)
				state (a_conn, {PC_FTPSTATE}.FTP_WAIT220)
				if (l_data.item.state.used_interface).to_natural_32 = {NATURAL_32} 2 then
					l_result := ftp_multi_statemach (a_conn, a_done)
				else
					l_result := ftp_easy_statemach (a_conn)
					if not l_result.to_boolean then
						a_done.item := (1).to_boolean
					end
				end
				Result := l_result
			end
		end

	ftp_connect_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_connect)
		end

	ftp_done (a_conn: CE_POINTER [PD_CONNECTDATA]; a_status: INTEGER_32; a_premature: BOOLEAN): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_nread: INTEGER_32
			l_ftpcode: INTEGER_32
			l_result: INTEGER_32
			l_was_ctl_valid: BOOLEAN
			l_path: CE_POINTER [INTEGER_8]
			l_path_to_use: CE_POINTER [INTEGER_8]
			l_flen: NATURAL_32
			l_tmp___0: NATURAL_32
			l_dlen: NATURAL_32
			l_old_time: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make
			create l_pp.make
			create l_path.make
			create l_path_to_use.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
					l_result := {PC_CURLCODE}.CURLE_OK
					l_was_ctl_valid := l_ftpc.item.ctl_valid
					l_path_to_use.ce_gets (l_data.item.state.path)
					if (l_ftp).c_pointer = default_pointer then
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
					if js.return_called then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						inspect a_status -- switch 1 head
						when 36 then l_pos := -1 + 2
						when 13 then l_pos := -1 + 2
						when 30 then l_pos := -1 + 2
						when 17 then l_pos := -1 + 2
						when 19 then l_pos := -1 + 2
						when 25 then l_pos := -1 + 2
						when 9 then l_pos := -1 + 2
						when 63 then l_pos := -1 + 2
						when 78 then l_pos := -1 + 2
						when 23 then l_pos := -1 + 2
						when 0 then l_pos := -1 + 2
						else l_pos := -1 + 3
						end
				when 2 then -- switch 1, case 36 | switch 1, case 13 | switch 1, case 30 | switch 1, case 17 | switch 1, case 19 | switch 1, case 25 | switch 1, case 9 | switch 1, case 63 | switch 1, case 78 | switch 1, case 23 | switch 1, case 0
						if not a_premature then
							l_ftpc.item.ctl_valid := l_was_ctl_valid
							l_pos := -1 + 4 -- switch 1; break
						end
				when 3 then -- switch 1, default:
						l_ftpc.item.ctl_valid := (0).to_boolean
						l_ftpc.item.cwdfail := (1).to_boolean
						a_conn.item.bits.close := (1).to_boolean
						l_result := a_status
						l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
						if ((l_ftpc.item.prevpath).c_pointer /= default_pointer) then
							easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.prevpath)])
						end
						if l_data.item.set.wildcardmatch then
							if ((l_data.item.set.chunk_end).c_pointer /= default_pointer) then
								if ((l_ftpc.item.file).c_pointer /= default_pointer) then
									(l_data.item.set.chunk_end).call ([l_data.item.wildcard.customptr])
								end
							end
							l_ftpc.item.known_filesize := (-1).to_integer_64
						end
						l_path.ce_gets (escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (l_data), l_path_to_use, 0, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer)))
						if (l_path).c_pointer = default_pointer then
							l_ftpc.item.prevpath := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
						else
							if ((l_ftpc.item.file).c_pointer /= default_pointer) then
								l_tmp___0 := (strlen (l_ftpc.item.file))
							else
								l_tmp___0 := (0).to_natural_32
							end
							l_flen := l_tmp___0
							l_dlen := (strlen (l_path)) - l_flen
							if not l_ftpc.item.cwdfail then
								if l_dlen.to_boolean then
									if (l_data.item.set.ftp_filemethod).to_natural_32 /= {NATURAL_32} 2 then
										l_ftpc.item.prevpath := l_path
										if l_flen.to_boolean then
											l_ftpc.item.prevpath[l_dlen.to_integer_32] := (0).to_integer_8
										end
									else
										l_ftpc.item.prevpath := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([ce_string ("")]))
										easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_path)])
									end
								else
									l_ftpc.item.prevpath := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([ce_string ("")]))
									easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_path)])
								end
								if ((l_ftpc.item.prevpath).c_pointer /= default_pointer) then
									sendf_inst.Curl_infof ([l_data, ce_string ("Remembering we are in dir %"%%s%"%N"), l_ftpc.item.prevpath])
								end
							else
								l_ftpc.item.prevpath := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
								easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_path)])
							end
						end
						freedirs (l_ftpc)
						if a_conn.item.sock[1] /= -1 then
							if -1 /= a_conn.item.sock[1] then
								close (a_conn.item.sock[1]).do_nothing
								a_conn.item.sock[1] := -1
							end
						end
						if (l_ftp.item.transfer).to_natural_32 = {NATURAL_32} 0 then
							if l_ftpc.item.ctl_valid then
								if l_pp.item.pending_resp then
									if not a_premature then
										l_old_time := l_pp.item.response_time
										l_pp.item.response_time := 60000
										l_pp.item.response := timeval_inst.curlx_tvnow
										l_result := Curl_GetFTPResponse (create {CE_POINTER [INTEGER_32]}.make_shared ($l_nread), a_conn, create {CE_POINTER [INTEGER_32]}.make_shared ($l_ftpcode))
										l_pp.item.response_time := l_old_time
										if not l_nread.to_boolean then
											if {NATURAL_32} 28 = l_result.to_natural_32 then
												sendf_inst.Curl_failf ([l_data, ce_string ("control connection looks dead")])
												l_ftpc.item.ctl_valid := (0).to_boolean
												a_conn.item.bits.close := (1).to_boolean
											end
										end
										if l_result.to_boolean then
											Result := l_result
											js.return
										end
										if not js.return_called and then (not l_ftpc.item.dont_check) then
											if l_ftpcode /= 226 then
												if l_ftpcode /= 250 then
													sendf_inst.Curl_failf ([l_data, ce_string ("server did not report OK, got %%d"), l_ftpcode])
													l_result := {PC_CURLCODE}.CURLE_PARTIAL_FILE
												end
											end
										end
									end
								end
							end
						end
				when 5 then -- if done
					if js.return_called then
						l_pos := -1 + 16 -- skip if block
					end
				when 6 then -- if:
						if l_result.to_boolean then
							l_pos := -1 + 15 -- skip if block
						end
				when 7 then -- if:
						if a_premature then
							l_pos := -1 + 15 -- skip if block
						end
				when 8 then -- if:
						if not (l_data.item.set.upload) then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						if -{INTEGER_64} 1 /= l_data.item.set.infilesize then
							if l_data.item.set.infilesize /= (l_ftp.item.bytecountp).item then
								if not l_data.item.set.crlf then
									if (l_ftp.item.transfer).to_natural_32 = {NATURAL_32} 0 then
										sendf_inst.Curl_failf ([l_data, ce_string ("Uploaded unaligned file size (%%lld out of %%lld bytes)"), (l_ftp.item.bytecountp).item, l_data.item.set.infilesize])
										l_result := {PC_CURLCODE}.CURLE_PARTIAL_FILE
									end
								end
							end
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 11 then -- else:
						if -{INTEGER_64} 1 = l_data.item.req.size then
							l_pos := -1 + 14 -- skip if block
						end
				when 12 then -- if:
						if l_data.item.req.size /= (l_ftp.item.bytecountp).item then
							if l_data.item.req.size + l_data.item.state.crlf_conversions /= (l_ftp.item.bytecountp).item then
								if l_data.item.req.maxdownload /= (l_ftp.item.bytecountp).item then
									sendf_inst.Curl_failf ([l_data, ce_string ("Received only partial file: %%lld bytes"), (l_ftp.item.bytecountp).item])
									l_result := {PC_CURLCODE}.CURLE_PARTIAL_FILE
								else
									l_pos := -1 + 14 -- goto _L___1
								end
							else
								l_pos := -1 + 14 -- goto _L___1
							end
						else
							l_pos := -1 + 14 -- goto _L___1
						end
				when 13 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 14 then -- else: | _L___1:
						if not l_ftpc.item.dont_check then
							if not ((l_ftp.item.bytecountp).item).to_boolean then
								if l_data.item.req.size > {INTEGER_64} 0 then
									sendf_inst.Curl_failf ([l_data, ce_string ("No data was received!")])
									l_result := {PC_CURLCODE}.CURLE_FTP_COULDNT_RETR_FILE
								end
							end
						end
				when 15 then -- if done | if done | if done
						l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_BODY
						l_ftpc.item.dont_check := (0).to_boolean
						if not a_status.to_boolean then
							if not l_result.to_boolean then
								if not a_premature then
									if ((l_data.item.set.postquote).c_pointer /= default_pointer) then
										l_result := ftp_sendquote (a_conn, l_data.item.set.postquote)
									end
								end
							end
						end
						Result := l_result
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_done_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_done)
		end

	ftp_sendquote (a_conn: CE_POINTER [PD_CONNECTDATA]; a_quote: CE_POINTER [PD_CURL_SLIST]): INTEGER_32
		local
			l_item: CE_POINTER [PD_CURL_SLIST]
			l_nread: INTEGER_32
			l_ftpcode: INTEGER_32
			l_result: INTEGER_32
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_cmd: CE_POINTER [INTEGER_8]
			l_acceptfail: BOOLEAN
			js: JUMP_STATE
		do
			create l_item.make
			create l_ftpc.make
			create l_pp.make
			create l_cmd.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
			l_item.ce_gets (a_quote)
			from
			until
				not (not js.loop_left and then ((l_item).c_pointer /= default_pointer))
			loop
				if ((l_item.item.data).c_pointer /= default_pointer) then
					l_cmd.ce_gets (l_item.item.data)
					l_acceptfail := (0).to_boolean
					if (l_cmd[0]).to_integer_32 = 42 then
						l_cmd.ce_gets (l_cmd + 1)
						l_acceptfail := (1).to_boolean
					end
					l_result := Curl_ftpsendf ([a_conn, ce_string ("%%s"), l_cmd])
					if l_result.to_natural_32 /= {NATURAL_32} 0 then
						Result := l_result
						js.return
					end
					if not js.return_called then
						l_pp.item.response := timeval_inst.curlx_tvnow
						l_result := Curl_GetFTPResponse (create {CE_POINTER [INTEGER_32]}.make_shared ($l_nread), a_conn, create {CE_POINTER [INTEGER_32]}.make_shared ($l_ftpcode))
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
					end
					if not js.return_called and then (not l_acceptfail) then
						if l_ftpcode >= 400 then
							sendf_inst.Curl_failf ([a_conn.item.data, ce_string ("QUOT string not accepted: %%s"), l_cmd])
							Result := ({PC_CURLCODE}.CURLE_QUOTE_ERROR)
							js.return
						end
					end
				end
				if not js.return_called then
					l_item.ce_gets (l_item.item.next)
				end
			end
			if not js.return_called then
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	ftp_need_type (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ascii_wanted: BOOLEAN): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			if a_ascii_wanted then
				l_tmp := ('A').code
			else
				l_tmp := ('I').code
			end
			Result := ((a_conn.item.proto.ftpc.transfertype).to_integer_32 /= l_tmp).to_integer
		end

	ftp_nb_type (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ascii: BOOLEAN; a_newstate: INTEGER_32): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_result: INTEGER_32
			l_want: INTEGER_8
			l_tmp: INTEGER_32
			js: JUMP_STATE
		do
			create l_ftpc.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if a_ascii then
				l_tmp := ('A').code
			else
				l_tmp := ('I').code
			end
			l_want := l_tmp.to_integer_8
			if (l_ftpc.item.transfertype).to_integer_32 = l_want.to_integer_32 then
				state (a_conn, a_newstate)
				Result := (ftp_state_type_resp (a_conn, 200, a_newstate))
				js.return
			end
			if not js.return_called then
				l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset), ce_string ("TYPE %%c"), l_want.to_integer_32])
				if l_result.to_natural_32 /= {NATURAL_32} 0 then
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				state (a_conn, a_newstate)
				l_ftpc.item.transfertype := l_want
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	ftp_pasv_verbose (a_conn: CE_POINTER [PD_CONNECTDATA]; a_ai: CE_POINTER [PD_CURL_ADDRINFO]; a_newhost: CE_POINTER [INTEGER_8]; a_port: INTEGER_32)
		local
			l_buf: CE_ARRAY [INTEGER_8]
		do
			create l_buf.make (<<256>>)

			hostip_inst.Curl_printable_address (a_ai, l_buf, (Integer_8_bytes.to_natural_32 * (256).to_natural_32)).do_nothing
			sendf_inst.Curl_infof ([a_conn.item.data, ce_string ("Connecting to %%s (%%s) port %%d%N"), a_newhost, l_buf, a_port])
		end

	ftp_range (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_from: INTEGER_64
			l_to: INTEGER_64
			l_ptr: CE_POINTER [INTEGER_8]
			l_ptr2: CE_POINTER [INTEGER_8]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ptr.make
			create l_ptr2.make
			create l_data.make
			create l_ftpc.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					if not (l_data.item.state.use_range) then
						l_pos := -1 + 10 -- skip if block
					end
				when 1 then -- if:
						if (l_data.item.state.range).c_pointer = default_pointer then
							l_pos := -1 + 8 -- skip if block
						end
				when 2 then -- if:
						l_from := strtoll (l_data.item.state.range, (l_ptr.address), 0)
						from
						until
							js.loop_left
						loop
							js.reset
							if (l_ptr[0]).to_boolean then
								if not ((((ctype_b_loc).item[((l_ptr[0]).to_natural_8).to_integer_32]).to_integer_32).bit_and (8192)).to_boolean then
									if (l_ptr[0]).to_integer_32 /= 45 then
										js.break
									end
								end
							else
								js.break
							end
							if not js.loop_jumped then
								l_ptr.ce_gets (l_ptr + 1)
							end
						end
						js.reset
						l_to := strtoll (l_ptr, (l_ptr2.address), 0)
						if (l_ptr.c_pointer.to_integer_32).to_natural_32 = (l_ptr2.c_pointer.to_integer_32).to_natural_32 then
							l_to := (-1).to_integer_64
						end
						if -{INTEGER_64} 1 /= l_to then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						if l_from >= {INTEGER_64} 0 then
							l_data.item.state.resume_from := l_from
						else
							l_pos := -1 + 5 -- goto _L
						end
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 5 then -- else: | _L:
						if l_from < {INTEGER_64} 0 then
							l_data.item.req.maxdownload := -l_from
							l_data.item.state.resume_from := l_from
						else
							l_data.item.req.maxdownload := (l_to - l_from) + {INTEGER_64} 1
							l_data.item.state.resume_from := l_from
						end
				when 6 then
						l_ftpc.item.dont_check := (1).to_boolean
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_data.item.req.maxdownload := (-1).to_integer_64
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_data.item.req.maxdownload := (-1).to_integer_64
				when 11 then -- if done
					Result := ({PC_CURLCODE}.CURLE_OK)
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_nextconnect (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftpc.make
			create l_ftp.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_result := {PC_CURLCODE}.CURLE_OK
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					if not ((l_ftp.item.transfer).to_natural_32 <= {NATURAL_32} 1) then
						l_pos := -1 + 13 -- skip if block
					end
				when 1 then -- if:
						if not (l_data.item.set.upload) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_result := ftp_nb_type (a_conn, l_data.item.set.prefer_ascii, {PC_FTPSTATE}.FTP_STOR_TYPE)
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 4 then -- else:
						l_ftp.item.downloadsize := (-1).to_integer_64
						l_result := ftp_range (a_conn)
						if l_result.to_boolean then
							l_pos := -1 + 12 -- skip if block
						end
				when 5 then -- if:
						if not (l_data.item.set.ftp_list_only) then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if:
						l_pos := -1 + 9 -- goto _L
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 8 then -- else:
						if ((l_ftpc.item.file).c_pointer /= default_pointer) then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if: | _L:
						if (l_ftp.item.transfer).to_natural_32 = {NATURAL_32} 0 then
							l_result := ftp_nb_type (a_conn, (1).to_boolean, {PC_FTPSTATE}.FTP_LIST_TYPE)
							if l_result.to_boolean then
								Result := l_result
								js.return
							end
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_result := ftp_nb_type (a_conn, l_data.item.set.prefer_ascii, {PC_FTPSTATE}.FTP_RETR_TYPE)
						if l_result.to_boolean then
							Result := l_result
							js.return
						end
				when 12 then -- if done | if done | if done
						if not js.return_called then
							l_result := ftp_easy_statemach (a_conn)
						end
				when 13 then -- if done
					if not js.return_called then
						if l_result.to_natural_32 = {NATURAL_32} 0 then
							if (l_ftp.item.transfer).to_natural_32 /= {NATURAL_32} 0 then
								transfer_inst.Curl_setup_transfer (a_conn, -1, (-1).to_integer_64, (0).to_boolean, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer), -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
							end
						end
						Result := l_result
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_nextconnect_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_nextconnect)
		end

	ftp_perform (a_conn: CE_POINTER [PD_CONNECTDATA]; a_connected: CE_POINTER [BOOLEAN]; a_dophase_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			js: JUMP_STATE
		do
			create l_ftp.make

			l_result := {PC_CURLCODE}.CURLE_OK
			if (a_conn.item.data).item.set.opt_no_body then
				l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
				l_ftp.item.transfer := {PC_CURL_FTPTRANSFER}.FTPTRANSFER_INFO
			end
			a_dophase_done.item := (0).to_boolean
			l_result := ftp_state_quote (a_conn, (1).to_boolean, {PC_FTPSTATE}.FTP_QUOTE)
			if l_result.to_boolean then
				Result := l_result
				js.return
			end
			if not js.return_called then
				if ((a_conn.item.data).item.state.used_interface).to_natural_32 = {NATURAL_32} 2 then
					l_result := ftp_multi_statemach (a_conn, a_dophase_done)
				else
					l_result := ftp_easy_statemach (a_conn)
					a_dophase_done.item := (1).to_boolean
				end
				a_connected.item := a_conn.item.bits.tcpconnect
				if a_dophase_done.item then
				end
				Result := l_result
			end
		end

	wc_data_dtor (a_ptr: CE_POINTER [ANY])
		local
			l_tmp: CE_POINTER [PD_FTP_WC_TMPDATA]
		do
			create l_tmp.make

			l_tmp.ce_gets (create {CE_POINTER [PD_FTP_WC_TMPDATA]}.make_cast (a_ptr))
			if ((l_tmp).c_pointer /= default_pointer) then
				ftplistparser_inst.Curl_ftp_parselist_data_free (create {CE_POINTER [CE_POINTER [PD_FTP_PARSELIST_DATA]]}.make_shared (l_tmp.c_pointer + {PD_FTP_WC_TMPDATA}.parser_offset))
			end
			url_inst.Curl_safefree (create {CE_POINTER [ANY]}.make_cast (l_tmp))
		end

	wc_data_dtor_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent wc_data_dtor)
		end

	init_wc_data (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_last_slash: CE_POINTER [INTEGER_8]
			l_path: CE_POINTER [INTEGER_8]
			l_wildcard: CE_POINTER [PD_WILDCARDDATA]
			l_ret: INTEGER_32
			l_ftp_tmp: CE_POINTER [PD_FTP_WC_TMPDATA]
			js: JUMP_STATE
		do
			create l_last_slash.make
			create l_path.make
			create l_wildcard.make
			create l_ftp_tmp.make

			l_path.ce_gets ((a_conn.item.data).item.state.path)
			l_wildcard.ce_gets (create {CE_POINTER [PD_WILDCARDDATA]}.make_shared ((a_conn.item.data).c_pointer + {PD_SESSIONHANDLE}.wildcard_offset))
			l_ret := {PC_CURLCODE}.CURLE_OK
			l_last_slash.ce_gets (strrchr ((a_conn.item.data).item.state.path, ('/').code))
			if ((l_last_slash).c_pointer /= default_pointer) then
				l_last_slash.ce_gets (l_last_slash + 1)
				if (l_last_slash[0]).to_integer_32 = 0 then
					l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
					l_ret := ftp_parse_url_path (a_conn)
					Result := l_ret
					js.return
				else
					l_wildcard.item.pattern := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([l_last_slash]))
					if (l_wildcard.item.pattern).c_pointer = default_pointer then
						Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
						js.return
					end
					if not js.return_called then
						l_last_slash[0] := (('%/0/').code).to_integer_8
					end
				end
			elseif (l_path[0]).to_boolean then
				l_wildcard.item.pattern := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([l_path]))
				if (l_wildcard.item.pattern).c_pointer = default_pointer then
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
				if not js.return_called then
					l_path[0] := (('%/0/').code).to_integer_8
				end
			else
				l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
				l_ret := ftp_parse_url_path (a_conn)
				Result := l_ret
				js.return
			end
			if not js.return_called then
				l_ftp_tmp.ce_gets (create {CE_POINTER [PD_FTP_WC_TMPDATA]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([{PD_FTP_WC_TMPDATA}.sizeof.to_natural_32])))))
				if (l_ftp_tmp).c_pointer = default_pointer then
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
			end
			if not js.return_called then
				l_ftp_tmp.item.parser := ftplistparser_inst.Curl_ftp_parselist_data_alloc
				if (l_ftp_tmp.item.parser).c_pointer = default_pointer then
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftp_tmp)])
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
			end
			if not js.return_called then
				l_wildcard.item.tmp := create {CE_POINTER [ANY]}.make_cast (l_ftp_tmp)
				l_wildcard.item.tmp_dtor := wc_data_dtor_address
				if ((a_conn.item.data).item.set.ftp_filemethod).to_natural_32 = {NATURAL_32} 2 then
					(a_conn.item.data).item.set.ftp_filemethod := {PC_CURL_FTPFILE}.FTPFILE_MULTICWD
				end
				l_ret := ftp_parse_url_path (a_conn)
				if l_ret.to_boolean then
					Result := l_ret
					js.return
				end
			end
			if not js.return_called then
				l_ftp_tmp.item.backup.write_function := (a_conn.item.data).item.set.fwrite_func;
				(a_conn.item.data).item.set.fwrite_func := ftplistparser_inst.Curl_ftp_parselist_address
				l_ftp_tmp.item.backup.file_descriptor := create {CE_POINTER [SD_IO_FILE]}.make_cast ((a_conn.item.data).item.set.out2);
				(a_conn.item.data).item.set.out2 := create {CE_POINTER [ANY]}.make_cast (a_conn)
				l_wildcard.item.path := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([(a_conn.item.data).item.state.path]))
				if (l_wildcard.item.path).c_pointer = default_pointer then
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
			end
			if not js.return_called then
				sendf_inst.Curl_infof ([a_conn.item.data, ce_string ("Wildcard - Parsing started%N")])
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	wc_statemach (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_wildcard: CE_POINTER [PD_WILDCARDDATA]
			l_ftp_tmp: CE_POINTER [PD_FTP_WC_TMPDATA]
			l_tmp_path: CE_POINTER [INTEGER_8]
			l_ret: INTEGER_32
			l_userresponse: INTEGER_32
			l_finfo: CE_POINTER [PD_CURL_FILEINFO]
			l_tmp___1: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ftpc.make
			create l_wildcard.make
			create l_ftp_tmp.make
			create l_tmp_path.make
			create l_finfo.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_wildcard.ce_gets (create {CE_POINTER [PD_WILDCARDDATA]}.make_shared ((a_conn.item.data).c_pointer + {PD_SESSIONHANDLE}.wildcard_offset))
					l_ftp_tmp.ce_gets (create {CE_POINTER [PD_FTP_WC_TMPDATA]}.make_cast (l_wildcard.item.tmp))
					l_ret := {PC_CURLCODE}.CURLE_OK
					l_userresponse := 0
					inspect l_wildcard.item.state -- switch 1 head
					when 0 then l_pos := -1 + 1
					when 1 then l_pos := -1 + 3
					when 2 then l_pos := -1 + 4
					when 4 then l_pos := -1 + 10
					when 3 then l_pos := -1 + 11
					when 6 then l_pos := -1 + 12
					when 5 then l_pos := -1 + 12
					else l_pos := -1 + 13
					end
				when 1 then -- switch 1, case 0
					l_ret := init_wc_data (a_conn)
					if (l_wildcard.item.state).to_natural_32 = {NATURAL_32} 3 then
						l_pos := -1 + 13 -- switch 1; break
					elseif l_ret.to_boolean then
						l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_ERROR
					else
						l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_MATCHING
					end
				when 2 then
					l_pos := -1 + 13 -- switch 1; break
				when 3 then -- switch 1, case 1
					l_ftp_tmp.ce_gets (create {CE_POINTER [PD_FTP_WC_TMPDATA]}.make_cast (l_wildcard.item.tmp));
					(a_conn.item.data).item.set.fwrite_func := l_ftp_tmp.item.backup.write_function;
					(a_conn.item.data).item.set.out2 := create {CE_POINTER [ANY]}.make_cast (l_ftp_tmp.item.backup.file_descriptor)
					l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_DOWNLOADING
					if (ftplistparser_inst.Curl_ftp_parselist_geterror (l_ftp_tmp.item.parser)).to_boolean then
						l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
						Result := (wc_statemach (a_conn))
						js.return
					elseif (l_wildcard.item.filelist).item.size = {NATURAL_32} 0 then
						l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
						Result := ({PC_CURLCODE}.CURLE_REMOTE_FILE_NOT_FOUND)
						js.return
					end
					if not js.return_called then
						l_ret := wc_statemach (a_conn)
					end
					l_pos := -1 + 13 -- switch 1; break
				when 4 then -- switch 1, case 2
					l_finfo.ce_gets (create {CE_POINTER [PD_CURL_FILEINFO]}.make_cast (((l_wildcard.item.filelist).item.head).item.ptr))
					l_tmp___1 := strlen ((a_conn.item.data).item.state.path)
					l_tmp_path.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([(l_tmp___1 + (strlen (l_finfo.item.filename))) + {NATURAL_32} 1])))))
					if (l_tmp_path).c_pointer = default_pointer then
						Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
						js.return
					end
					if js.return_called then
						l_pos := -1 + 9 -- skip if block
					end
				when 5 then -- if:
						l_tmp_path[0] := (0).to_integer_8
						strcat (l_tmp_path, l_wildcard.item.path).do_nothing
						strcat (l_tmp_path, l_finfo.item.filename).do_nothing
						if (((a_conn.item.data).item.state.pathbuffer).c_pointer /= default_pointer) then
							easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast ((a_conn.item.data).item.state.pathbuffer)])
						end
						(a_conn.item.data).item.state.pathbuffer := l_tmp_path;
						(a_conn.item.data).item.state.path := l_tmp_path
						sendf_inst.Curl_infof ([a_conn.item.data, ce_string ("Wildcard - START of %"%%s%"%N"), l_finfo.item.filename])
						if ((a_conn.item.data).item.set.chunk_bgn).c_pointer = default_pointer then
							l_pos := -1 + 9 -- skip if block
						end
				when 6 then -- if:
						l_userresponse := ((a_conn.item.data).item.set.chunk_bgn).integer_32_item ([create {CE_POINTER [ANY]}.make_cast (l_finfo), l_wildcard.item.customptr, ((l_wildcard.item.filelist).item.size).to_integer_32])
						inspect l_userresponse.to_integer_32 -- switch 2 head
						when 2 then l_pos := -1 + 7
						when 1 then l_pos := -1 + 8
						else l_pos := -1 + 9
						end
				when 7 then -- switch 2, case 2
						sendf_inst.Curl_infof ([a_conn.item.data, ce_string ("Wildcard - %"%%s%" skipped by user%N"), l_finfo.item.filename])
						l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_SKIP
						Result := (wc_statemach (a_conn))
						js.return
				when 8 then -- switch 2, case 1
						if not js.return_called then
							Result := ({PC_CURLCODE}.CURLE_CHUNK_FAILED)
							js.return
						end
				when 9 then -- switch 2, done: | if done | if done
						if not js.return_called and then ((l_finfo.item.filetype).to_natural_32 /= {NATURAL_32} 0) then
							l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_SKIP
							Result := (wc_statemach (a_conn))
							js.return
						end
						if not js.return_called then
							if ((l_finfo.item.flags).bit_and (((1).bit_shift_left (6)).to_natural_32)).to_boolean then
								l_ftpc.item.known_filesize := l_finfo.item.size
							end
							l_ret := ftp_parse_url_path (a_conn)
							if l_ret.to_boolean then
								Result := l_ret
								js.return
							end
						end
						if not js.return_called then
							llist_inst.Curl_llist_remove (l_wildcard.item.filelist, (l_wildcard.item.filelist).item.head, create {CE_POINTER [ANY]}.make_shared (default_pointer)).do_nothing
							if (l_wildcard.item.filelist).item.size = {NATURAL_32} 0 then
								l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
								Result := ({PC_CURLCODE}.CURLE_OK)
								js.return
							end
						end
						l_pos := -1 + 13 -- switch 1; break
				when 10 then -- switch 1, case 4
						if (((a_conn.item.data).item.set.chunk_end).c_pointer /= default_pointer) then
							((a_conn.item.data).item.set.chunk_end).call ([(a_conn.item.data).item.wildcard.customptr])
						end
						llist_inst.Curl_llist_remove (l_wildcard.item.filelist, (l_wildcard.item.filelist).item.head, create {CE_POINTER [ANY]}.make_shared (default_pointer)).do_nothing
						if (l_wildcard.item.filelist).item.size = {NATURAL_32} 0 then
							l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_CLEAN
						else
							l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_DOWNLOADING
						end
						l_ret := wc_statemach (a_conn)
						l_pos := -1 + 13 -- switch 1; break
				when 11 then -- switch 1, case 3
						l_ret := {PC_CURLCODE}.CURLE_OK
						if ((l_ftp_tmp).c_pointer /= default_pointer) then
							l_ret := ftplistparser_inst.Curl_ftp_parselist_geterror (l_ftp_tmp.item.parser)
						end
						if l_ret.to_boolean then
							l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_ERROR
						else
							l_wildcard.item.state := {PC_CURL_WILDCARD_STATES}.CURLWC_DONE
						end
						l_pos := -1 + 13 -- switch 1; break
				when 12 then -- switch 1, case 6 | switch 1, case 5
						l_pos := -1 + 13 -- switch 1; break
				when 13 then -- switch 1, done:
						if not js.return_called then
							Result := l_ret
						end
				when 14 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_do (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_retcode: INTEGER_32
			js: JUMP_STATE
		do
			l_retcode := {PC_CURLCODE}.CURLE_OK
			a_done.item := (0).to_boolean
			url_inst.Curl_reset_reqproto (a_conn)
			l_retcode := ftp_init (a_conn)
			if l_retcode.to_boolean then
				Result := l_retcode
				js.return
			end
			if not js.return_called then
				if (a_conn.item.data).item.set.wildcardmatch then
					l_retcode := wc_statemach (a_conn)
					if ((a_conn.item.data).item.wildcard.state).to_natural_32 = {NATURAL_32} 4 then
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					elseif ((a_conn.item.data).item.wildcard.state).to_natural_32 = {NATURAL_32} 6 then
						Result := ({PC_CURLCODE}.CURLE_OK)
						js.return
					end
					if not js.return_called and then (l_retcode.to_boolean) then
						Result := l_retcode
						js.return
					end
				else
					l_retcode := ftp_parse_url_path (a_conn)
					if l_retcode.to_boolean then
						Result := l_retcode
						js.return
					end
				end
			end
			if not js.return_called then
				l_retcode := ftp_regular_transfer (a_conn, a_done)
				Result := l_retcode
			end
		end

	ftp_do_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_do)
		end

	Curl_ftpsendf (a_args: TUPLE [a_conn: CE_POINTER [PD_CONNECTDATA]; a_fmt: CE_POINTER [INTEGER_8]]): INTEGER_32
		local
			l_bytes_written: INTEGER_32
			l_s: CE_ARRAY [INTEGER_8]
			l_write_len: NATURAL_32
			l_sptr: CE_POINTER [INTEGER_8]
			l_res: INTEGER_32
			l_ap: CE_VA_LIST
			js: JUMP_STATE
		do
			create l_s.make (<<1024>>)
			create l_sptr.make

			l_sptr.ce_gets (l_s)
			l_res := {PC_CURLCODE}.CURLE_OK
			create l_ap.make (a_args, 3)
			mprintf_inst.curl_mvsnprintf (l_s, (1021).to_natural_32, a_args.a_fmt, l_ap).do_nothing
			strcat (l_s, ce_string ("%R%N")).do_nothing
			l_bytes_written := 0
			l_write_len := strlen (l_s)
			from
			until
				js.loop_left
			loop
				js.reset
				l_res := sendf_inst.Curl_write (a_args.a_conn, a_args.a_conn.item.sock[0], create {CE_POINTER [ANY]}.make_cast (l_sptr), l_write_len, create {CE_POINTER [INTEGER_32]}.make_shared ($l_bytes_written))
				if {NATURAL_32} 0 /= l_res.to_natural_32 then
					js.break
				end
				if not js.loop_jumped then
					if (a_args.a_conn.item.data).item.set.verbose then
						sendf_inst.Curl_debug (a_args.a_conn.item.data, {PC_CURL_INFOTYPE}.CURLINFO_HEADER_OUT, l_sptr, l_bytes_written.to_natural_32, a_args.a_conn).do_nothing
					end
					if l_bytes_written /= l_write_len.to_integer_32 then
						l_write_len := l_write_len - l_bytes_written.to_natural_32
						l_sptr.ce_gets (l_sptr + l_bytes_written)
					else
						js.break
					end
				end
			end
			js.reset
			Result := l_res
		end

	ftp_quit (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_result: INTEGER_32
			js: JUMP_STATE
		do
			l_result := {PC_CURLCODE}.CURLE_OK
			if a_conn.item.proto.ftpc.ctl_valid then
				l_result := pingpong_inst.Curl_pp_sendf ([create {CE_POINTER [PD_PINGPONG]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset + {PD_FTP_CONN}.pp_offset), ce_string ("QUIT"), create {CE_POINTER [ANY]}.make_shared (default_pointer)])
				if l_result.to_natural_32 /= {NATURAL_32} 0 then
					Result := l_result
					js.return
				end
				if not js.return_called then
					state (a_conn, {PC_FTPSTATE}.FTP_QUIT)
					l_result := ftp_easy_statemach (a_conn)
				end
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_disconnect (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_pp: CE_POINTER [PD_PINGPONG]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
		do
			create l_ftpc.make
			create l_pp.make
			create l_data.make

			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_pp.ce_gets (create {CE_POINTER [PD_PINGPONG]}.make_shared (l_ftpc.c_pointer + {PD_FTP_CONN}.pp_offset))
			ftp_quit (a_conn).do_nothing
			if ((l_ftpc.item.entrypath).c_pointer /= default_pointer) then
				l_data.ce_gets (a_conn.item.data)
				if ((l_data.item.state.most_recent_ftp_entrypath).c_pointer.to_integer_32).to_natural_32 = ((l_ftpc.item.entrypath).c_pointer.to_integer_32).to_natural_32 then
					l_data.item.state.most_recent_ftp_entrypath := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
				end
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.entrypath)])
				l_ftpc.item.entrypath := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			end
			freedirs (l_ftpc)
			if ((l_ftpc.item.prevpath).c_pointer /= default_pointer) then
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.prevpath)])
				l_ftpc.item.prevpath := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			end
			if ((l_ftpc.item.server_os).c_pointer /= default_pointer) then
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.server_os)])
				l_ftpc.item.server_os := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			end
			pingpong_inst.Curl_pp_disconnect (l_pp).do_nothing
			Result := ({PC_CURLCODE}.CURLE_OK)
		end

	ftp_disconnect_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_disconnect)
		end

	ftp_parse_url_path (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			l_slash_pos: CE_POINTER [INTEGER_8]
			l_path_to_use: CE_POINTER [INTEGER_8]
			l_cur_pos: CE_POINTER [INTEGER_8]
			l_filename: CE_POINTER [INTEGER_8]
			l_tmp___2: INTEGER_32
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_absolute_dir: BOOLEAN
			l_tmp___5: INTEGER_32
			l_len: INTEGER_32
			l_bigger: CE_POINTER [INTEGER_8]
			l_dlen: INTEGER_32
			l_path: CE_POINTER [INTEGER_8]
			l_tmp___13: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_ftp.make
			create l_ftpc.make
			create l_slash_pos.make
			create l_path_to_use.make
			create l_cur_pos.make
			create l_filename.make
			create l_tmp___3.make
			create l_bigger.make
			create l_path.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					l_ftp.ce_gets (l_data.item.state.proto.ftp)
					l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
					l_path_to_use.ce_gets (l_data.item.state.path)
					l_filename.ce_gets (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					l_cur_pos.ce_gets (l_path_to_use)
					l_ftpc.item.ctl_valid := (0).to_boolean
					l_ftpc.item.cwdfail := (0).to_boolean
					inspect l_data.item.set.ftp_filemethod -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 3 then l_pos := -1 + 2
					else l_pos := -1 + 11
					end
				when 1 then -- switch 1, case 2
					if ((l_data.item.state.path).c_pointer /= default_pointer) then
						if (l_data.item.state.path[0]).to_boolean then
							if (l_data.item.state.path[((strlen (l_data.item.state.path)) - {NATURAL_32} 1).to_integer_32]).to_integer_32 /= 47 then
								l_filename.ce_gets (l_data.item.state.path)
							end
						end
					end
					l_pos := -1 + 12 -- switch 1; break
				when 2 then -- switch 1, case 3
					if not (l_path_to_use[0]).to_boolean then
						l_ftpc.item.dirdepth := 0
						l_pos := -1 + 12 -- switch 1; break
					end
				when 3 then
					l_slash_pos.ce_gets ((strrchr (l_cur_pos, ('/').code)))
					if (l_slash_pos).c_pointer = default_pointer then
						l_pos := -1 + 6 -- skip if block
					end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if (l_cur_pos[0]).to_boolean then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						l_ftpc.item.dirs := create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_ccalloc.pointer_item ([(1).to_natural_32, Pointer_bytes.to_natural_32]))))
						if (l_ftpc.item.dirs).c_pointer = default_pointer then
							Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
							js.return
						end
						if not js.return_called then
							if ((l_slash_pos).c_pointer /= default_pointer) then
								l_tmp___2 := l_slash_pos.distance (l_cur_pos)
							else
								l_tmp___2 := 1
							end
							if ((l_slash_pos).c_pointer /= default_pointer) then
								l_tmp___3.ce_gets (l_cur_pos)
							else
								l_tmp___3.ce_gets (ce_string ("/"))
							end
							l_ftpc.item.dirs[0] := escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (a_conn.item.data), l_tmp___3, l_tmp___2, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer))
							if (l_ftpc.item.dirs[0]).c_pointer = default_pointer then
								freedirs (l_ftpc)
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								js.return
							end
						end
						if not js.return_called then
							l_ftpc.item.dirdepth := 1
							if ((l_slash_pos).c_pointer /= default_pointer) then
								l_filename.ce_gets (l_slash_pos + 1)
							else
								l_filename.ce_gets (l_cur_pos)
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_filename.ce_gets (l_cur_pos)
				when 10 then -- if done
						l_pos := -1 + 12 -- switch 1; break
				when 11 then -- switch 1, default:
						l_ftpc.item.dirdepth := 0
						l_ftpc.item.diralloc := 5
						l_ftpc.item.dirs := create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_ccalloc.pointer_item ([(l_ftpc.item.diralloc).to_natural_32, Pointer_bytes.to_natural_32]))))
						if (l_ftpc.item.dirs).c_pointer = default_pointer then
							Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
							js.return
						end
						if not js.return_called then
							if (strequal_inst.curl_strequal (l_path_to_use, ce_string ("/"))).to_boolean then
								l_cur_pos.ce_gets (l_cur_pos + 1)
								l_ftpc.item.dirs[0] := create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([ce_string ("/")]))
								l_ftpc.item.dirdepth := (l_ftpc.item.dirdepth) + 1
							else
								from
								until
									js.loop_left
								loop
									js.reset
									l_slash_pos.ce_gets ((strchr (l_cur_pos, ('/').code)))
									if (l_slash_pos.c_pointer.to_integer_32).to_natural_32 = (0).to_natural_32 then
										js.break
									end
									if not js.loop_jumped then
										if l_cur_pos.distance ((l_data.item.state.path)) > 0 then
											if l_ftpc.item.dirdepth = 0 then
												l_tmp___5 := 1
											else
												l_tmp___5 := 0
											end
										else
											l_tmp___5 := 0
										end
										l_absolute_dir := l_tmp___5.to_boolean
										if (l_slash_pos.distance (l_cur_pos)).to_boolean then
											l_len := (l_slash_pos.distance (l_cur_pos)) + l_absolute_dir.to_integer
											l_ftpc.item.dirs[l_ftpc.item.dirdepth] := escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (a_conn.item.data), l_cur_pos - l_absolute_dir.to_integer, l_len, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer))
											if (l_ftpc.item.dirs[l_ftpc.item.dirdepth]).c_pointer = default_pointer then
												sendf_inst.Curl_failf ([l_data, ce_string ("no memory")])
												freedirs (l_ftpc)
												Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
												js.return
											end
											if not js.return_called then
												if (isBadFtpString (l_ftpc.item.dirs[l_ftpc.item.dirdepth])) then
													easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.dirs[l_ftpc.item.dirdepth])])
													freedirs (l_ftpc)
													Result := ({PC_CURLCODE}.CURLE_URL_MALFORMAT)
													js.return
												end
											end
										else
											l_cur_pos.ce_gets (l_slash_pos + 1)
											js.continue
										end
									end
									if not js.loop_jumped then
										l_cur_pos.ce_gets (l_slash_pos + 1)
										l_ftpc.item.dirdepth := (l_ftpc.item.dirdepth) + 1
										if l_ftpc.item.dirdepth >= l_ftpc.item.diralloc then
											l_ftpc.item.diralloc := l_ftpc.item.diralloc * (2)
											l_bigger.ce_gets (create {CE_POINTER [INTEGER_8]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_crealloc.pointer_item ([create {CE_POINTER [ANY]}.make_cast (l_ftpc.item.dirs), (l_ftpc.item.diralloc).to_natural_32 * Pointer_bytes.to_natural_32])))))
											if (l_bigger).c_pointer = default_pointer then
												freedirs (l_ftpc)
												Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
												js.return
											end
											if not js.return_called then
												l_ftpc.item.dirs := create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_cast (l_bigger)
											end
										end
									end
								end
								js.reset
							end
						end
						if not js.return_called then
							l_filename.ce_gets (l_cur_pos)
						end
						l_pos := -1 + 12 -- switch 1; break
				when 12 then -- switch 1, done:
						if not js.return_called then
							if ((l_filename).c_pointer /= default_pointer) then
								if (l_filename[0]).to_boolean then
									l_ftpc.item.file := escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (a_conn.item.data), l_filename, 0, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer))
									if (0).to_natural_32 = ((l_ftpc.item.file).c_pointer.to_integer_32).to_natural_32 then
										freedirs (l_ftpc)
										sendf_inst.Curl_failf ([l_data, ce_string ("no memory")])
										Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
										js.return
									end
									if not js.return_called then
										if (isBadFtpString (l_ftpc.item.file)) then
											freedirs (l_ftpc)
											Result := ({PC_CURLCODE}.CURLE_URL_MALFORMAT)
											js.return
										end
									end
								else
									l_ftpc.item.file := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
								end
							else
								l_ftpc.item.file := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
							end
						end
						if not js.return_called and then (l_data.item.set.upload) then
							if (l_ftpc.item.file).c_pointer = default_pointer then
								if (l_ftp.item.transfer).to_natural_32 = {NATURAL_32} 0 then
									sendf_inst.Curl_failf ([l_data, ce_string ("Uploading to a URL without a file name!")])
									Result := ({PC_CURLCODE}.CURLE_URL_MALFORMAT)
									js.return
								end
							end
						end
						if not js.return_called then
							l_ftpc.item.cwddone := (0).to_boolean
							if ((l_ftpc.item.prevpath).c_pointer /= default_pointer) then
								l_path.ce_gets ((escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (a_conn.item.data), l_data.item.state.path, 0, create {CE_POINTER [INTEGER_32]}.make_shared ($l_dlen))))
								if (l_path).c_pointer = default_pointer then
									freedirs (l_ftpc)
									Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
									js.return
								end
								if not js.return_called then
									if ((l_ftpc.item.file).c_pointer /= default_pointer) then
										l_tmp___13 := (strlen (l_ftpc.item.file)).to_integer_32
									else
										l_tmp___13 := 0
									end
									l_dlen := l_dlen - l_tmp___13
									if l_dlen = (strlen (l_ftpc.item.prevpath)).to_integer_32 then
										if (strequal_inst.curl_strnequal (l_path, l_ftpc.item.prevpath, l_dlen.to_natural_32)).to_boolean then
											sendf_inst.Curl_infof ([l_data, ce_string ("Request has same path as previous transfer%N")])
											l_ftpc.item.cwddone := (1).to_boolean
										end
									end
									easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_path)])
								end
							end
						end
						if not js.return_called then
							Result := ({PC_CURLCODE}.CURLE_OK)
						end
				when 13 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	ftp_dophase_done (a_conn: CE_POINTER [PD_CONNECTDATA]; a_connected: BOOLEAN): INTEGER_32
		local
			l_result: INTEGER_32
			l_ftp: CE_POINTER [PD_FTP]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_ftp.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_ftp.ce_gets ((a_conn.item.data).item.state.proto.ftp)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			if a_connected then
				l_result := ftp_nextconnect (a_conn)
			end
			if l_result.to_boolean then
				if a_conn.item.sock[1] /= -1 then
					close (a_conn.item.sock[1]).do_nothing
					a_conn.item.sock[1] := -1
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				if (l_ftp.item.transfer).to_natural_32 /= {NATURAL_32} 0 then
					transfer_inst.Curl_setup_transfer (a_conn, -1, (-1).to_integer_64, (0).to_boolean, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer), -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
				elseif not a_connected then
					a_conn.item.bits.do_more := (1).to_boolean
				end
				l_ftpc.item.ctl_valid := (1).to_boolean
				Result := l_result
			end
		end

	ftp_doing (a_conn: CE_POINTER [PD_CONNECTDATA]; a_dophase_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_result: INTEGER_32
		do
			l_result := ftp_multi_statemach (a_conn, a_dophase_done)
			if a_dophase_done.item then
				l_result := ftp_dophase_done (a_conn, (0).to_boolean)
			end
			Result := l_result
		end

	ftp_doing_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_doing)
		end

	ftp_regular_transfer (a_conn: CE_POINTER [PD_CONNECTDATA]; a_dophase_done: CE_POINTER [BOOLEAN]): INTEGER_32
		local
			l_result: INTEGER_32
			l_connected: BOOLEAN
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ftpc: CE_POINTER [PD_FTP_CONN]
			js: JUMP_STATE
		do
			create l_data.make
			create l_ftpc.make

			l_result := {PC_CURLCODE}.CURLE_OK
			l_connected := (0).to_boolean
			l_data.ce_gets (a_conn.item.data)
			l_ftpc.ce_gets (create {CE_POINTER [PD_FTP_CONN]}.make_shared (a_conn.c_pointer + {PD_CONNECTDATA}.proto_offset + {PD_PROTO}.ftpc_offset))
			l_data.item.req.size := (-1).to_integer_64
			progress_inst.Curl_pgrsSetUploadCounter (l_data, (0).to_integer_64)
			progress_inst.Curl_pgrsSetDownloadCounter (l_data, (0).to_integer_64)
			progress_inst.Curl_pgrsSetUploadSize (l_data, (0).to_integer_64)
			progress_inst.Curl_pgrsSetDownloadSize (l_data, (0).to_integer_64)
			l_ftpc.item.ctl_valid := (1).to_boolean
			l_result := ftp_perform (a_conn, create {CE_POINTER [BOOLEAN]}.make_shared ($l_connected), a_dophase_done)
			if {NATURAL_32} 0 = l_result.to_natural_32 then
				if not a_dophase_done.item then
					Result := ({PC_CURLCODE}.CURLE_OK)
					js.return
				end
				if not js.return_called then
					l_result := ftp_dophase_done (a_conn, l_connected)
					if l_result.to_boolean then
						Result := l_result
						js.return
					end
				end
			else
				freedirs (l_ftpc)
			end
			if not js.return_called then
				Result := l_result
			end
		end

	ftp_setup_connection (a_conn: CE_POINTER [PD_CONNECTDATA]): INTEGER_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_type: CE_POINTER [INTEGER_8]
			l_command: INTEGER_8
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_data.make
			create l_type.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data.ce_gets (a_conn.item.data)
					if a_conn.item.bits.httpproxy then
						if not l_data.item.set.tunnel_thru_httpproxy then
							if ((a_conn.item.handler).c_pointer.to_integer_32).to_natural_32 = ((Curl_handler_ftp.address).c_pointer.to_integer_32).to_natural_32 then
								a_conn.item.handler := Curl_handler_ftp_proxy.address
							else
								sendf_inst.Curl_failf ([l_data, ce_string ("FTPS not supported!")])
								Result := ({PC_CURLCODE}.CURLE_UNSUPPORTED_PROTOCOL)
								js.return
							end
							if not js.return_called then
								a_conn.item.bits.close := (0).to_boolean
							end
						end
					end
					if js.return_called then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						l_data.item.state.path := (l_data.item.state.path) + 1
						l_data.item.state.slash_removed := (1).to_boolean
						l_type.ce_gets (strstr (l_data.item.state.path, ce_string (";type=")))
						if (l_type).c_pointer = default_pointer then
							l_type.ce_gets (strstr (a_conn.item.host.rawalloc, ce_string (";type=")))
						end
						if (l_type).c_pointer = default_pointer then
							l_pos := -1 + 6 -- skip if block
						end
				when 2 then -- if:
						l_type[0] := (0).to_integer_8
						l_command := rawstr_inst.Curl_raw_toupper (l_type[6])
						a_conn.item.bits.type_set := (1).to_boolean
						inspect l_command.to_integer_32 -- switch 1 head
						when 65 then l_pos := -1 + 3
						when 68 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 3 then -- switch 1, case 65
						l_data.item.set.prefer_ascii := (1).to_boolean
						l_pos := -1 + 6 -- switch 1; break
				when 4 then -- switch 1, case 68
						l_data.item.set.ftp_list_only := (1).to_boolean
						l_pos := -1 + 6 -- switch 1; break
				when 5 then -- switch 1, default:
						l_data.item.set.prefer_ascii := (0).to_boolean
						l_pos := -1 + 6 -- switch 1; break
				when 6 then -- switch 1, done: | if done
						Result := ({PC_CURLCODE}.CURLE_OK)
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	ftp_setup_connection_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ftp_setup_connection)
		end

end
